/*
 *  AbstractMatrix.h
 *  PLEX
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */

 /*
 This class is the abstract class of matrix,
 include matrix construction, deconstruction, fill, resize,
 get the rows, cols, swap the elements, 
 test the empty of matrix, and operator = and []
 commented by Wanjun
 */

#ifndef AbstractMatrix_h__
#define AbstractMatrix_h__

#include <algorithm>
#include "Vector.h"

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
class AbstractMatrix
{
	public:
				typedef	Vector<DataType>		RowType;
				typedef Vector<RowType>			RowVector;
				typedef	unsigned int			size_type;
				
												AbstractMatrix(void);
												AbstractMatrix(
													size_type			inRows,
													size_type			inColumns,
													const DataType&		inValue = DataType());
												AbstractMatrix(
													const AbstractMatrix&		inAbstractMatrix);
		virtual									~AbstractMatrix() {}
													
		virtual	void							fill(
													const DataType&		inVal) = 0;
				
		virtual	void							resize(
													size_type			inRows,
													size_type			inColumns,
													const DataType&		inVal = DataType()) = 0;
													
		virtual void							clear(void);

				bool							empty(void) const;
													
				size_type						rows(void) const;											
				size_type						cols(void) const;
				
		virtual	const RowType&					operator[](
													size_type			inRowIndex) const = 0;
		virtual	RowType&						operator[](
													size_type			inRowIndex) = 0;
													
//				const AbstractMatrix&			operator=(
//													const AbstractMatrix&	inAbstractMatrix);
//				const AbstractMatrix&			operator=(
//													DataType**				inAbstractMatrix);
													
		virtual	void							SwapRows(
													size_type			inRow1,
													size_type			inRow2) = 0;
		virtual	void							SwapColumns(
													size_type			inColumn1,
													size_type			inColumn2) = 0;
													
				void							swap(
													AbstractMatrix&				inAbstractMatrix);
															
	private:
				size_type						mNumRows;
				size_type						mNumColumns;
};

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
AbstractMatrix<DataType>::AbstractMatrix(void)
	:	mNumRows(0),
		mNumColumns(0)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
AbstractMatrix<DataType>::AbstractMatrix(
	size_type		inRows,
	size_type		inColumns,
	const DataType&	inValue)
	:	mNumRows(0),
		mNumColumns(0)
{
	resize(inRows, inColumns, inValue);
}

//---------------------------------------------------------------------------
template <typename DataType>
AbstractMatrix<DataType>::AbstractMatrix(
	const AbstractMatrix&	inAbstractMatrix)
	:	mNumRows(inAbstractMatrix.mNumRows),
		mNumColumns(inAbstractMatrix.mNumColumns)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
void
AbstractMatrix<DataType>::resize(
	size_type		inRows,
	size_type		inColumns,
	const DataType&	inVal)
{
	mNumRows	= inRows;
	mNumColumns = inColumns;
}

//---------------------------------------------------------------------------
template <typename DataType>
void
AbstractMatrix<DataType>::clear(void)
{
	mNumRows = 0;
	mNumColumns = 0;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
bool
AbstractMatrix<DataType>::empty(void) const
{
	return mNumRows == 0 || (mNumColumns == 0);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
typename AbstractMatrix<DataType>::size_type
AbstractMatrix<DataType>::rows(void) const
{
	return mNumRows;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
typename AbstractMatrix<DataType>::size_type
AbstractMatrix<DataType>::cols(void) const
{
	return mNumColumns;
}

//---------------------------------------------------------------------------
/*template <typename DataType>
inline
const AbstractMatrix<DataType>&
AbstractMatrix<DataType>::operator=(
	const AbstractMatrix&	inAbstractMatrix)
{
	AbstractMatrix	temp = inAbstractMatrix;
	
	swap(temp);
	return *this;
}
*/
//---------------------------------------------------------------------------
template <typename DataType>
inline
void
AbstractMatrix<DataType>::swap(
	AbstractMatrix&	inAbstractMatrix)
{
	std::swap(mNumRows, inAbstractMatrix.mNumRows);
	std::swap(mNumColumns, inAbstractMatrix.mNumColumns);
}

//---------------------------------------------------------------------------
#endif // AbstractMatrix_h__
