/*
 *  AbstractVector.h
 *  PLEX
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */

 /*
 This class is the abstract class of vector,
 include vector construction, deconstruction, fill, resize,
 get the rows, cols, and layers, swap the elements, 
 test the empty of vector, and operator = and []
 commented by Wanjun
 */

#ifndef AbstractVector_h__
#define AbstractVector_h__

#include <algorithm>
#include <cassert>

//---------------------------------------------------------------------------
template <typename DataType>
class AbstractVector
{
	public:
			typedef	unsigned int		size_type;
			
										AbstractVector(void);
										AbstractVector(
											size_type				inNumItems,
											const DataType&			inVal = DataType());
										AbstractVector(
											const AbstractVector&	inAbstractVector);
	virtual								~AbstractVector(void);
										
//			const AbstractVector&		operator=(
//											const AbstractVector&	inAbstractVector);
											
	virtual	bool						empty(void)		const;
	virtual	size_type					size(void)		const;
	virtual	void						resize(
											size_type		inNewSize,
											const DataType&	inVal = DataType()) = 0;
	virtual	void						fill(
											const DataType&	inVal) = 0;
											
	virtual	DataType&					operator[](
											size_type		inIndex) = 0;
	virtual	const DataType&				operator[](
											size_type		inIndex) const = 0;
											
			void						swap(
											AbstractVector&			ioAbstractVector);
	private:
			size_type					mNumItems;
};

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
AbstractVector<DataType>::AbstractVector(void)
	:	mNumItems(0)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
AbstractVector<DataType>::AbstractVector(
	size_type		inNumItems,
	const DataType&	inVal)
	:	mNumItems(inNumItems)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
AbstractVector<DataType>::AbstractVector(
	const AbstractVector&	inAbstractVector)
	:	mNumItems(inAbstractVector.mNumItems)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
AbstractVector<DataType>::~AbstractVector(void)
{
}

//---------------------------------------------------------------------------
/*template <typename DataType>
inline
const AbstractVector<DataType>&
AbstractVector<DataType>::operator=(
	const AbstractVector&	inAbstractVector)
{
	AbstractVector	temp(inAbstractVector);
	swap(temp);
	return *this;
}
*/

//---------------------------------------------------------------------------
template <typename DataType>
inline
bool
AbstractVector<DataType>::empty(void) const
{
	return mNumItems == 0;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
typename AbstractVector<DataType>::size_type
AbstractVector<DataType>::size(void) const
{
	return mNumItems;
}

//---------------------------------------------------------------------------
template <typename DataType>
void
AbstractVector<DataType>::resize(
	size_type		inNewSize,
	const DataType&	inVal)
{
	if (inNewSize != mNumItems)
		mNumItems = inNewSize;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
AbstractVector<DataType>::swap(
	AbstractVector&	inAbstractVector)
{
	std::swap(inAbstractVector.mNumItems, mNumItems);
}



#endif // AbstractVector_h__
