/*
 *  GTRModel.h
 *  PLEX
 *
 *  A.P. Jason de Koning, jason.de.koning@gmail.com
 *  http://jasondk.org
 *
 *  David Pollock lab, University of Colorado School of Medicine
 *  http://www.evolutionarygenomics.com
 *  All rights reserved.
 
 */


#ifndef GTRModel_h__
#define GTRModel_h__


#include "MixtureModel.h"

//---------------------------------------------------------------------------
class GTRModel : public MixtureModel
{
	public:
		GTRModel(
			const DoubleMatrix3D& inAlphaMatrix3D = DoubleMatrix3D(),
			const DoubleMatrix& inFrequencyMatrix = DoubleMatrix(),
			const IntMatrix& inCountMatrix = IntMatrix());

		GTRModel(
			const GTRModel&	inModel);
			
		virtual	~GTRModel(
			void);
									
	//	virtual const GTRModel& operator=(		//copy a GTR model
	//		const GTRModel&	inModel);
					
		DoubleMatrix3D* returnAlphaMatrix3D(void) {
			return &mAlphaMatrix3D;
		}
		
	
		virtual void swap(				//swap a GTR model
			GTRModel& inModel);
			
		virtual void PrintAlphaMatrix3D(		//print the alpha matrix for each model to outStream
			ostream& outStream, ObservedData& theData);
			
		void PrintCountMatrix(			//print the count matrix to outStream
			ostream& outStream, ObservedData& theData);

		virtual void PrintFrequencyMatrix(		//print the state frequency vector of each model to outStream
			ostream& outStream, ObservedData& theData);

		virtual void PrintRateMatrix3D(
				ostream&		outStream, ObservedData& theData);
			
		virtual void InitializeParameter(	//initialize the parameters of GTR model based on state changes count matrix
			const IntMatrix& inCountMatrix);
			
		virtual void UpdateParameter(		//update all the parameters for each model in mixture model,
			const int& m_nMethod);		//given the conditional pathway calculation method
		
	virtual void GTRModelParameterstoRates(		//transform the alpha matrix and frequency parameters for each model to rate matrix
										   void);
	
	
	//		DoubleMatrix& returnStateFrequencies(void) { return mFrequencyMatrix; }

	protected:
		
	private:
		virtual void InitializeFrequencyMatrix(		//initialize the frequency vector for each model
			const IntMatrix& inCountMatrix);
			
		virtual void InitializeAlphaMatrix3D(		//initialize the rate matrix for each model
			const IntMatrix& inCountMatrix);	
	
		virtual void UpdateFrequency(			//update the frequency vector for a single model
			DoubleVector& mFrequency);

		virtual void UpdateFrequencyMatrix(		//update the frequency vectors for all models
			void);
			
		virtual void UpdateAlphaMatrix(			//update the alpha matrix for a single model
			DoubleMatrix& mAlphaMatrix, 
			const DoubleVector& mFrequency, 
			const int& m_nMethod);
			
	void UpdateAlphaMatrix(int theModel, const int& m_nMethod);

		virtual void UpdateAlphaMatrix3D(		//update the alpha matrix for all models
			const int& m_nMethod);
			
		virtual void SetCountMatrix(			//set the count matrix for this model
			const IntMatrix& inCountMatrix);

		virtual void UpdateFrequencyParameter(		//update a single frequency parameter
			double& inParameter);

		virtual void UpdateAlphaParameter(		//update a single alpha parameter
			double& inParameter);
		
		int UpdateAlphaParameter( int theModel, int i, int j, int reversibleEvaluation);
	
//		DoubleMatrix	mFrequencyMatrix;	//frequency vector for each model
		IntMatrix	mCountMatrix;		//count matrix of state changes in alignment
};

#endif // GTRModel_h__
