/*
 *  KeyedRecyclery.h
 *  PLEX
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004-2005 Dr. David Pollock. All rights reserved.
 *
 */

#ifndef KeyedRecyclery_h__
#define KeyedRecyclery_h__

#include "Recyclery.h"

#include <map>

template <typename DataType, typename Initializer>
class KeyedRecyclery
{
	public:
													KeyedRecyclery(void);
													KeyedRecyclery(
														const Initializer&	inInitializer);
													KeyedRecyclery(
														const KeyedRecyclery&	inSrc);
													~KeyedRecyclery(void);
													
				const KeyedRecyclery&				operator=(
															const KeyedRecyclery& inSrc);

				DataType*							New(
														const Initializer&	inInitializer);
				void								Dispose(
														const Initializer&	inInitializer,
														DataType*&	ioPtr);
									
				
	protected:
	private:
				void								CreateNewRecycleryIfNeeded(
														const Initializer&	inInitializer);

		typedef std::map<Initializer, Recyclery<DataType, Initializer> >	RecycleryMap;
				RecycleryMap						mRecycleries;
};

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
KeyedRecyclery<DataType, Initializer>::KeyedRecyclery(void)
	:	mRecycleries()
{
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
KeyedRecyclery<DataType, Initializer>::KeyedRecyclery(
	const KeyedRecyclery&	inSrc)
	:	mRecycleries()
{
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
KeyedRecyclery<DataType, Initializer>::~KeyedRecyclery(void)
{
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
const KeyedRecyclery<DataType, Initializer>&
KeyedRecyclery<DataType, Initializer>::operator=(
	const KeyedRecyclery&	inSrc)
{
	mRecycleries.clear();
	return *this;
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
inline
DataType*
KeyedRecyclery<DataType, Initializer>::New(
	const Initializer&	inInitializer)
{
	CreateNewRecycleryIfNeeded(inInitializer);
	return mRecycleries[inInitializer].New(inInitializer);
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
inline
void
KeyedRecyclery<DataType, Initializer>::Dispose(
	const Initializer&	inInitializer,
	DataType*&			ioPtr)
{
	CreateNewRecycleryIfNeeded(inInitializer);
	mRecycleries[inInitializer].Dispose(ioPtr);
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
void
KeyedRecyclery<DataType, Initializer>::CreateNewRecycleryIfNeeded(
	const Initializer&	inInitializer)
{
	typename RecycleryMap::iterator i = mRecycleries.find(inInitializer);
	if (i == mRecycleries.end())
		mRecycleries[inInitializer] = Recyclery<DataType, Initializer>(inInitializer);
}


#endif // KeyedRecyclery_h__
