/*
 *  Matrix.h
 *  PLEX
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */

/*
 *  this class is general class for matrix
 *
 */

#ifndef Matrix_h__
#define Matrix_h__

#include <algorithm>
#include "AbstractMatrix.h"
#include "Vector.h"

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
class Matrix : public AbstractMatrix<DataType>
{
	public:
			typedef	Vector<DataType>								RowType;
			typedef Vector<RowType>									RowVector;
			typedef	typename AbstractMatrix<DataType>::size_type	size_type;
			
		struct Initializer
		{
			typedef typename Matrix<DataType>::size_type size_type;
			
			size_type	fRows;
			size_type	fCols;
			DataType	fInitValue;
			
						Initializer(
							size_type			inRows = 0,
							size_type			inCols = 0,
							const DataType&		inInitValue = DataType())
							:	fRows(inRows),
								fCols(inCols),
								fInitValue(inInitValue)
						{
						}
																	
			bool		operator<(
							const Initializer& inInitializer) const
						{
							if (fRows < inInitializer.fRows)
								return true;
								
							return fCols < inInitializer.fCols;
						}
		};
											Matrix(void);
											Matrix(
												size_type			inRows,
												size_type			inColumns,
												const DataType&		inValue = DataType());
											Matrix(
												const Matrix&		inMatrix);
											Matrix(
												const Initializer&	inMatrix);
												
	virtual									~Matrix(void) {}
	
			void							Initialize(
												const Initializer&	inMatrix);
												
	virtual	void							fill(
												const DataType&		inVal);
			
	virtual	void							clear(void);
			
	virtual	void							resize(
												size_type			inRows,
												size_type			inColumns,
												const DataType&		inVal = DataType());
												
	virtual	const RowType&					operator[](
												size_type			inRowIndex) const;
	virtual	RowType&						operator[](
												size_type			inRowIndex);
												
//			const Matrix&					operator=(
//												const Matrix&		inMatrix);
			const Matrix&					operator+(
												const Matrix&		inMatrix);
			const Matrix&					operator+=(
												const Matrix&		inMatrix);
//			Matrix&						operator=(
//												Matrix&		inMatrix);
//			const Matrix&					operator=(
//												DataType**			inMatrix);
												
	virtual	void							SwapRows(
												size_type			inRow1,
												size_type			inRow2);
	virtual	void							SwapColumns(
												size_type			inColumn1,
												size_type			inColumn2);
												
			void							swap(
												Matrix&				inMatrix);
															
			RowVector						mRows;
	private:
};

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
Matrix<DataType>::Matrix(void)
	:	AbstractMatrix<DataType>(),
		mRows()
{
}

//---------------------------------------------------------------------------
template <typename DataType>
Matrix<DataType>::Matrix(
	size_type		inRows,
	size_type		inColumns,
	const DataType&	inValue)
	:	AbstractMatrix<DataType>(),
		mRows()
{
	resize(inRows, inColumns, inValue);
}

//---------------------------------------------------------------------------
template <typename DataType>
Matrix<DataType>::Matrix(
	const Matrix&	inMatrix)
	:	AbstractMatrix<DataType>(inMatrix),
		mRows(inMatrix.mRows)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
Matrix<DataType>::Matrix(
	const Initializer&	inInitializer)
	:	AbstractMatrix<DataType>(inInitializer.fRows, inInitializer.fCols, inInitializer.fInitValue),
		mRows(inInitializer.fRows)
{
	resize(inInitializer.fRows, inInitializer.fCols, inInitializer.fInitValue);
}

//---------------------------------------------------------------------------
template <typename DataType>
void
Matrix<DataType>::Initialize(
	const Initializer&	inInitializer)
{
	fill(inInitializer.fInitValue);
	resize(inInitializer.fRows, inInitializer.fCols, inInitializer.fInitValue);
}

//---------------------------------------------------------------------------
template <typename DataType>
void
Matrix<DataType>::fill(
	const DataType&	inVal)
{
	size_type	numRows = this->rows();
	for (size_type i = 0; i < numRows; ++i)
	{
		size_type numCols = this->cols();
		for (size_type j = 0; j < numCols; ++j)
			mRows.mDataArray[i].mDataArray[j] = inVal;
	}
}

//---------------------------------------------------------------------------
template <typename DataType>
void
Matrix<DataType>::clear(void)
{
	size_type	numRows = this->rows();
	for (size_type i = 0; i < numRows; ++i)
		mRows[i].clear();

	AbstractMatrix<DataType>::clear();
}

//---------------------------------------------------------------------------
template <typename DataType>
void
Matrix<DataType>::resize(
	size_type		inRows,
	size_type		inColumns,
	const DataType&	inVal)
{
	mRows.resize(inRows);
	for (size_type i = 0; i < inRows; i++)
		mRows[i].resize(inColumns, inVal);
		
	AbstractMatrix<DataType>::resize(inRows, inColumns, inVal);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
const typename Matrix<DataType>::RowType&
Matrix<DataType>::operator[](
	size_type	inRowIndex) const
{
// Expensive bounds checking!
//	assert(inRowIndex < rows());
	return mRows[inRowIndex];
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
typename Matrix<DataType>::RowType&
Matrix<DataType>::operator[](
	size_type	inRowIndex)
{
// Expensive bounds checking!
//	if (inRowIndex >= rows()) {
//		std::cout << "Requested row " << inRowIndex << ", but only " << rows() << " are defined" << std::endl;
//		std::cout << "There is a bounds violation here..." << std::endl;
//		std::cout << "Put breakpoint here." << std::endl;
//	}
//	assert(inRowIndex < rows());
	return mRows[inRowIndex];
}

//---------------------------------------------------------------------------
/*
template <typename DataType>
inline
const Matrix<DataType>&
Matrix<DataType>::operator=(
	const Matrix&	inMatrix)
{
	Matrix	temp = inMatrix;
	
	swap(temp);
	return *this;
}
*/
//---------------------------------------------------------------------------
template <typename DataType>
inline
const Matrix<DataType>&
Matrix<DataType>::operator+(
	const Matrix&	inMatrix)
{
	const size_type	numRows = this->rows();
	const size_type numCols = this->cols();
	
	if (inMatrix.rows() == numRows && inMatrix.cols() == numCols)
	{
		for (size_type row = 0; row < numRows; ++row)
		{
			for (size_type column = 0; column < numCols; ++column)
				mRows[row][column] += inMatrix.mRows[row][column];
		}
	}

	return *this;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
const Matrix<DataType>&
Matrix<DataType>::operator+=(
	const Matrix&	inMatrix)
{
	return (*this + inMatrix);
}

//---------------------------------------------------------------------------
/*template <typename DataType>
inline
Matrix<DataType>&
Matrix<DataType>::operator=(
	Matrix&	inMatrix)
{
	Matrix	temp = inMatrix;
	
	swap(temp);
	return *this;
}
*/
//---------------------------------------------------------------------------
template <typename DataType>
inline
void
Matrix<DataType>::SwapRows(
	size_type	inRow1,
	size_type	inRow2)
{
	if (inRow1 != inRow2)
		mRows[inRow1].swap(mRows[inRow2]);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
Matrix<DataType>::SwapColumns(
	size_type	inColumn1,
	size_type	inColumn2)
{
	const size_type	numRows = this->rows();
	for (size_type row = 0; row < numRows; ++row)
		std::swap(mRows[row][inColumn1], mRows[row][inColumn2]);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
Matrix<DataType>::swap(
	Matrix&	inMatrix)
{
	mRows.swap(inMatrix.mRows);
	AbstractMatrix<DataType>::swap(inMatrix);
}

//---------------------------------------------------------------------------
#endif // Matrix_h__
