/*
 *  Matrix3D.h
 *  CLAMP
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */


/*
 *
 this class is a 3 dimensional matrix general class
 *
 */
 
#ifndef Matrix3D_h__
#define Matrix3D_h__

#include "AbstractMatrix3D.h"
#include "Matrix.h"

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
class Matrix3D : public AbstractMatrix3D<DataType>
{
	public:
			typedef	typename AbstractMatrix3D<DataType>::RowType	RowType;
			typedef typename AbstractMatrix3D<DataType>::RowVector	RowVector;
			typedef typename AbstractMatrix3D<DataType>::size_type	size_type;
			
		struct Initializer
		{
			typedef typename Matrix3D<DataType>::size_type size_type;
			
			size_type	fRows;
			size_type	fCols;
			size_type	fLayers;
			DataType	fInitValue;
			
						Initializer(
							size_type			inRows = 0,
							size_type			inCols = 0,
							size_type			inLayers = 0,
							const DataType&		inInitValue = DataType())
							:	fRows(inRows),
								fCols(inCols),
								fLayers(inLayers),
								fInitValue(inInitValue)
						{
						}
																	
			bool		operator<(
							const Initializer& inInitializer) const
						{
							if (fRows < inInitializer.fRows)
								return true;
								
							if (fCols < inInitializer.fCols)
								return true;
								
							return fLayers < inInitializer.fLayers;
						}
		};
			
											Matrix3D(void);
											Matrix3D(
												size_type			inRows,
												size_type			inColumns,
												size_type			inLayers,
												const DataType&		inValue = DataType());
											Matrix3D(
												const Matrix3D&		inMatrix);
											Matrix3D(
												const Initializer&	inMatrix);
	virtual									~Matrix3D(void) {}
	
			void							Initialize(
												const Initializer&	inMatrix);
												
	virtual	void							fill(
												const DataType&		inVal);
			
	virtual	void							resize(
												size_type			inRows,
												size_type			inColumns,
												size_type			inLayers,
												const DataType&		inVal = DataType());
												
	virtual	const RowType&					operator[](
												size_type			inRowIndex) const;
	virtual	RowType&						operator[](
												size_type			inRowIndex);
												
//			const Matrix3D&					operator=(
//												const Matrix3D&		inMatrix);
//			const Matrix3D&					operator=(
//												DataType***			inMatrix);
												
	virtual	void							SwapRows(
												size_type			inRow1,
												size_type			inRow2);
	virtual	void							SwapColumns(
												size_type			inColumn1,
												size_type			inColumn2);
	virtual	void							SwapLayers(
												size_type			inLayer1,
												size_type			inLayer2);
												
			void							swap(
												Matrix3D&			inMatrix);
									
			bool							empty(void) const;
						
			RowVector						mRows;
	private:
};

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
Matrix3D<DataType>::Matrix3D(void)
	:	AbstractMatrix3D<DataType>(),
		mRows()
{
}


template <typename DataType>
inline
bool
Matrix3D<DataType>::empty(void) const
{
	return (mRows.empty());
}

//---------------------------------------------------------------------------
template <typename DataType>
Matrix3D<DataType>::Matrix3D(
	size_type		inRows,
	size_type		inColumns,
	size_type		inLayers,
	const DataType&	inValue)
	:	AbstractMatrix3D<DataType>(),
		mRows()
{
	resize(inRows, inColumns, inLayers, inValue);
}

//---------------------------------------------------------------------------
template <typename DataType>
Matrix3D<DataType>::Matrix3D(
	const Matrix3D&	inMatrix)
	:	AbstractMatrix3D<DataType>(inMatrix),
		mRows(inMatrix.mRows)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
Matrix3D<DataType>::Matrix3D(
	const Initializer&	inInitializer)
	:	AbstractMatrix3D<DataType>(inInitializer.fRows, inInitializer.fCols, inInitializer.fLayers, inInitializer.fInitValue),
		mRows(inInitializer.fRows)
{
	resize(inInitializer.fRows, inInitializer.fCols, inInitializer.fLayers, inInitializer.fInitValue);
}


//---------------------------------------------------------------------------
template <typename DataType>
void
Matrix3D<DataType>::Initialize(
	const Initializer&	inInitializer)
{
	fill(inInitializer.fInitValue);
	resize(inInitializer.fRows, inInitializer.fCols, inInitializer.fLayers, inInitializer.fInitValue);
}
//---------------------------------------------------------------------------
template <typename DataType>
void
Matrix3D<DataType>::fill(
	const DataType&	inVal)
{
	size_type	numRows = this->rows();
	for (size_type i = 0; i < numRows; ++i)
		mRows.mDataArray[i].fill(inVal);
}

//---------------------------------------------------------------------------
template <typename DataType>
void
Matrix3D<DataType>::resize(
	size_type		inRows,
	size_type		inColumns,
	size_type		inLayers,
	const DataType&	inVal)
{
	mRows.resize(inRows);
	for (size_type i = 0; i < inRows; i++)
		mRows[i].resize(inColumns, inLayers, inVal);
		
	AbstractMatrix3D<DataType>::resize(inRows, inColumns, inLayers);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
const typename Matrix3D<DataType>::RowType&
Matrix3D<DataType>::operator[](
	size_type	inRowIndex) const
{
// Expensive
//	assert(inRowIndex < rows());
	return mRows[inRowIndex];
}

#include <iostream>

//---------------------------------------------------------------------------
template <typename DataType>
inline
typename Matrix3D<DataType>::RowType&
Matrix3D<DataType>::operator[](
	size_type	inRowIndex)
{
// Expensive
//	if (inRowIndex >= this->rows()) {
//		std::cout << "Problem!" << std::endl;
//		std::cout << "inrowindex is " << inRowIndex << ", num rows is " << rows() << std::endl;
//		std::cout << "STOP NOW" << std::endl;
//	}
//	assert(inRowIndex < this->rows());

	return mRows[inRowIndex];
}

//---------------------------------------------------------------------------
/*template <typename DataType>
inline
const Matrix3D<DataType>&
Matrix3D<DataType>::operator=(
	const Matrix3D&	inMatrix)
{
	Matrix3D	temp = inMatrix;
	
	swap(temp);
	return *this;
}
*/
//---------------------------------------------------------------------------
template <typename DataType>
inline
void
Matrix3D<DataType>::SwapRows(
	size_type	inRow1,
	size_type	inRow2)
{
	if (inRow1 != inRow2)
		mRows[inRow1].swap(mRows[inRow2]);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
Matrix3D<DataType>::SwapColumns(
	size_type	inColumn1,
	size_type	inColumn2)
{
	const size_type	numRows = this->rows();
	for (size_type row = 0; row < numRows; ++row)
		mRows[row][inColumn1].swap(mRows[row][inColumn2]);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
Matrix3D<DataType>::SwapLayers(
	size_type	inLayer1,
	size_type	inLayer2)
{
	const size_type	numRows = this->rows();
	for (size_type row = 0; row < numRows; ++row)
		mRows[row].SwapColumns(inLayer1, inLayer2);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
Matrix3D<DataType>::swap(
	Matrix3D&	inMatrix)
{
	mRows.swap(inMatrix.mRows);
	AbstractMatrix3D<DataType>::swap(inMatrix);
}

//---------------------------------------------------------------------------
#endif // Matrix3D_h__
