/*
 *  MixtureModel.h
 *  PLEX
 *
 *  A.P. Jason de Koning, jason.de.koning@gmail.com
 *  http://jasondk.org
 *
 *  David Pollock lab, University of Colorado School of Medicine
 *  http://www.evolutionarygenomics.com
 *  All rights reserved.
 
 */



/*
 *
 this class is a general class for mixture model
 *
 */
 
#ifndef MixtureModel_h__
#define MixtureModel_h__

#include <algorithm>
#include <iostream>

class PLEXApp;
//#include "PLEXApp.h"
//include "CLAMPTypes.h"
#include "Numerics.h"

class ObservedData;
//class Numerics;
using std::ostream;

//---------------------------------------------------------------------------
class MixtureModel
{
	public:
							MixtureModel(
								const DoubleMatrix3D& inRateMatrix3D = DoubleMatrix3D(),
								const int& inNumStates = 0,
								const IntVector& inStateIndex = IntVector(),
								const CharVector& inStates = CharVector(),
								const int& inMinState = 0,
								const int& inNumModels = 0,
								const DoubleVector& inFreqModels = DoubleVector(),
								const bool& inFixedFreq = 0,
								const double& inWindowsize = 0,
								const int& inNucldotides = 0);

							MixtureModel(
								const MixtureModel&	inMixtureModel);

			virtual				~MixtureModel(
								void);
										
	//		const MixtureModel&		operator=(		//copy of a mixture model
	//							const MixtureModel&	inMixtureModel);
											
											
			int updateUniformization(int theModel);
			int updateUniformization(void);
			int forceUpdateUniformization(int theModel);
			int forceUpdateUniformization(void);

			double returnOptimalUniformization(int theModel);

			void setUniformization(DoubleVector* vals);
			void setUniformization(int which, double val);
			double getUniformization(int which) { return uniformizationConstants.mDataArray[which]; }
	
			int enlargeUniformization(int theModel);
			int enlargeUniformization(void);
			
			void setReversible(int val);	// flag for whether the model is reversible or not
			int getReversible(void);
			void setRateScale(int which, double val) { rateScaleFactor[which] = val; }
			double getRateScale(int which) { return rateScaleFactor[which]; }
			double& returnRateScale(int which) { return rateScaleFactor[which]; }
			void setPredefinedMatrix(int val) { predefinedMatrix = val; }
			int  getPredefinedMatrix(void) { return predefinedMatrix; }
			const int getScaleMatrix(void) { return scaleMatrix; }
			void setScaleMatrix(int val) { scaleMatrix = val; }
			void initRateScale(double val);
			void initRateScale(int howMany, double val);
	

			const CharVector&		GetStates(		//get the state vector
								void) const;
								
			const int& 			GetMinState(		//get the minimum state
								void) const;
			
			const IntVector&		GetStateIndex(
								void) const;
			
			const int&			GetNumStates(
								void) const;

			const DoubleMatrix3D&		GetRateMatrix3D(
								void) const;
			
			const int&			GetNumModels(
								void) const;
			
			const DoubleVector&		GetFreqModels(
								void) const;
			
			const double& 			GetUpdateWindowsize(
								void) const;
			
			const int& 			GetModelNumNucleotides(
								void) const;

			CharVector&			GetStates(
								void);
								
			int& 				GetMinState(
								void);
			
			IntVector&			GetStateIndex(
								void);
			
			int&				GetNumStates(
								void);

			DoubleMatrix3D&			GetRateMatrix3D(
								void);
			
			int&				GetNumModels(
								void);
			
			DoubleVector&			GetFreqModels(
								void);
	
			double getRateSum( int which );
			double getRateSum( void );
			double scaleRateMatrices( int which, double scaleFactor );
			double scaleRateMatrices( double scaleFactor );
	
	
			
			double& 			GetUpdateWindowsize(
								void);

			int& 				GetModelNumNucleotides(
								void);

			void				SetStates(
								const CharVector& inStates);
								
			void				SetMinState(
								const int& inMinState);
			
			void				SetStateIndex(
								const IntVector& inStateIndex);
			
			void				SetNumStates(
								const int& inNumStates);
								
			void				SetNumModels(
								const int& inNumModels);
								
			void				SetFixedModels(
								const bool& inFixedFreq);
			
			void 				SetUpdateWindowsize(
								const double& inWindowsize);

			void				swap(
								MixtureModel&	inMixtureModel);
								
	//		void 				PrintDinucleotideModelRateMatrix3D(
	//							ostream& outStream);
			
			void				PrintRateMatrix3DLine(
								ostream& outStream);
								
			void				PrintFreqVector(
								ostream& outStream);
								
			virtual void 			InitializeParameter(
								const IntMatrix& m_ChangeMatrix);

			virtual void 			UpdateParameter(
								const int& m_nMethod);
		
			virtual void PrintRateParameters(ostream& outStream);		// for printing discrete rate parameters when applicable
		//	virtual void PrintCountMatrix(ostream& outStream);
		//	virtual void PrintFrequencyMatrix(ostream& outStream);

			virtual void PrintRateMatrix3D(ostream&		outStream, ObservedData& theData);

			DoubleMatrix3D* returnRateMatrix3D(void);

			void 				UpdateFreqModels(void);
			
			void PrintModelAssignments(ostream& outStream, IntVector& m_vecSiteAssignment);

//			virtual DoubleMatrix& returnStateFrequencies(void); // { return mFrequencyMatrix; }
			DoubleMatrix& returnStateFrequencies(void) { return mFrequencyMatrix; }

			DoubleMatrix& returnStationaryFrequencies(void) { return mStationaryFrequencyMatrix; }
			
			DoubleVector* returnUniformizationConstants(void) { return &uniformizationConstants; }

			void updateStationaryFreqs();
			void forceUpdateStationaryFreqs();
			void updateStationaryFreqs(int force);
	
			// Going to give external functions access to this
			DoubleMatrix3D	mAlphaMatrix3D;		//alpha matrix for each model

		protected:
			int reversibleModel;			// turn on reversible model (symmetric rates)

			int predefinedMatrix;
			int scaleMatrix; // scale each matrix by a parameter?
			double *rateScaleFactor;

			DoubleVector		uniformizationConstants; // in case we are using uniformization methods
			
			void				SetRateMatrix3D(
								const DoubleMatrix3D& inRateMatrix3D);
								
			void				SetFreqModels(
								const DoubleVector& inFreqModels);
							
			void				SetModelNumNucleotides(
								const int& inNucleotides);

			void 				InitializeFreqModels(
								void);
		
			DoubleMatrix3D	mRateMatrix3D;	//rate matrix for each model
			int				mNumStates;	//number of states in model
			IntVector		mStateIndex;	//state index
			CharVector		mStates;	//state vector
			int				mMinState;	//minimum state
			int				mNumModels;	//number of models in mixture model
			DoubleVector	mFreqModels;	//weight vector of each model (only used when we using weight scheme for mixtures)
			bool			mFixedFreq;	//fix the weight for each model or not
			double			mWindowsize;	//window size used to update the parameter in models
			int				mNucleotides;	//single nucleotide, dinucleotide or tri-nucleotide model

			PLEXApp			*m_oPLEXApp; // only need one of these...		
			Numerics		*num;		// [JdK] - an interface to accelerated numeric functions

			DoubleMatrix	mFrequencyMatrix;	//frequency vector for each model
	
			DoubleMatrix	mStationaryFrequencyMatrix;	//frequency vector for each model

	private:
			void 				UpdateFreqParameter(
								double& inParameter);

};

#endif // MixtureModel_h__
