/*
 *  Model.h
 *  PLEX
 *
 *  A.P. Jason de Koning, jason.de.koning@gmail.com
 *  http://jasondk.org
 *
 *  David Pollock lab, University of Colorado School of Medicine
 *  http://www.evolutionarygenomics.com
 *  All rights reserved.
 
 */



/*
 *
 *  this class is the general model for a single model
 *  it was only used when we simulate the mutation process in PLEX
 *  might be able to delete this one when we update the simulation process
 *
 */
 
#ifndef Model_h__
#define Model_h__

#include <algorithm>
#include <iostream>

#include "Numerics.h"
#include "CLAMPTypes.h"

class ObservedData;

using std::ostream;

//---------------------------------------------------------------------------
class Model
{
	public:
							Model(
								const DoubleMatrix& inRateMatrix = DoubleMatrix(),
								const int& inNumStates = 0,
								const IntVector& inStateIndex = IntVector(),
								const CharVector& inStates = CharVector(),
								const int& inMinState = 0);

							Model(
								const Model&	inModel);

			virtual				~Model(void);
										
			const Model&			operator=(
								const Model&	inModel);
											
			const CharVector&		GetStates(void) const;
								
			const int& 			GetMinState(void) const;
			
			const IntVector&		GetStateIndex(void) const;
			
			const int&			GetNumStates(void) const;

			const DoubleMatrix&		GetRateMatrix(void) const;
			
			CharVector&			GetStates(void);
								
			int& 				GetMinState(void);
			
			IntVector&			GetStateIndex(void);
			
			int&				GetNumStates(void);

			DoubleMatrix&			GetRateMatrix(void);
			
			void				SetStates(
								const CharVector& inStates);
								
			void				SetMinState(
								const int& inMinState);
			
			void				SetStateIndex(
								const IntVector& inStateIndex);
			
			void				SetNumStates(
								const int& inNumStates);
								
			void				swap(
								Model&	inModel);
								
			void 				PrintRateMatrix(
								ostream& outStream, ObservedData& theData );
								
			void				PrintRateMatrixLine(
								ostream& outStream);
								
			virtual void 			InitializeParameter(
								const IntMatrix& m_ChangeMatrix);

			virtual void 			UpdateParameter(
								const int& m_nMethod);
		
			virtual void 			SetUpdateWindowsize(
								const double& inWindowsize);

	protected:
			void				SetRateMatrix(
								const DoubleMatrix& inRateMatrix);

			DoubleMatrix			mRateMatrix;
			int				mNumStates;
			IntVector			mStateIndex;
			CharVector			mStates;
			int				mMinState;

			Numerics *num;			// [JdK] - an interface to accelerated numeric functions

	private:
};

#endif // Model_h__
