/*
 *  NewickTreeParser.h
 *  PLEX
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */

/*
 *
 *  this class includes all the functions to parse the tree in newick format
 *
 */

#ifndef NewickTreeParser_h__
#define NewickTreeParser_h__

#include "TreeParser.h"

#include "Tree.h"

#include <deque>

//---------------------------------------------------------------------------

class NewickTreeParser : public TreeParser
{
	public:
		typedef	 TreeParser::node_type			node_type;
		typedef  TreeParser::tree_type			tree_type;
		typedef  TreeParser::node_type::node_list	node_list;
		
										NewickTreeParser(void);
		virtual							~NewickTreeParser(void);
		
		virtual	bool					Read(
											std::istream&		inTreeStream,
											tree_type&			outTree);
		virtual	bool					Write(
											std::ostream&		outTreeStream,
											const tree_type&	inTree);

	private:
		friend class UTNewickTreeParser;
		
				bool					ReadNode(
											std::istream&		inTreeStream,
											node_type*&			outNode);
				bool					ReadNodeList(
											std::istream&		inTreeStream,
											bool				inLimitToOne,
											node_list&			outNodes);
											
		static	const char*				cDelimiters;
		static	const char*				cQuotes;
		static	const char*				cTreeTerminator;
		static	const char*				cNodeSeparator;
		static	const char*				cOpenParentNodeDelimiter;
		static	const char*				cCloseParentNodeDelimiter;
		static	const char*				cWhitespaceChars;
		
		static	const char*				cTerminatorNotFound;
		static	const char*				cMissingCommaErr;
		static	const char*				cMultiNodesAtTopErr;
};


#endif // NewickTreeParser_h__
