/*
 *  NodeDistanceMaker.cpp
 *  PLEX
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */

/*
 *  this class include functions to make a node distance and node name for building a tree
 *  
 */
 
#include "NodeDistanceMaker.h"

#include "TreeNode.h"

#include <cstdlib>
#include <cstring>

using std::atof;
using std::string;
using std::strncpy;
using std::max;

//---------------------------------------------------------------------------
unsigned int	NodeDistanceMaker::sNumNodes = 1;

const string	NodeDistanceMaker::cInitName;

#pragma mark -
//---------------------------------------------------------------------------
NodeDistanceMaker::~NodeDistanceMaker(void)
{
}

//---------------------------------------------------------------------------
NodeDistanceMaker::node_type*
NodeDistanceMaker::Make(
	const string&	inNodeDescription,
	node_type*		inNodeToSetFromDescription) const
{
	// if inNodeToSetFromDescription not NULL then it's id must be 0
	assert(inNodeToSetFromDescription == 0 || inNodeToSetFromDescription->GetData().id == 0);
	assert(inNodeToSetFromDescription == 0 || inNodeToSetFromDescription->GetData().name[0] == 0);
	
	double				distance		= GetInitDistance();
	string				name			= GetInitName();
	string::size_type	separatorLoc	= inNodeDescription.find(':');
	if (separatorLoc != string::npos)
	{
		name = inNodeDescription.substr(0, separatorLoc);
		distance = atof(inNodeDescription.substr(separatorLoc + 1).c_str());
	}
	
	TreeNode* node = inNodeToSetFromDescription
								? inNodeToSetFromDescription
								: new TreeNode();
	if (node)
	{
		string::size_type	nameLen = max(name.size(), static_cast<string::size_type>(MAXNAME));
		strncpy(node->GetData().name, name.c_str(), nameLen);
       
/*        if (distance == 0)
			node->GetData().distance = cSmallDistance;  // branch could not be zero, set a very small length
		else
			node->GetData().distance = distance;
*/
		
		node->GetData().distance = distance;

		if (distance <= 0)
			node->GetData().distance = cSmallDistance;  // branch could not be zero, set a very small length
		
		
		node->GetData().id = sNumNodes++;
	}
	
	return node;
}

//---------------------------------------------------------------------------
inline
double
NodeDistanceMaker::GetInitDistance(void) const
{
	return cInitDistance;
}

//---------------------------------------------------------------------------
inline
const string&
NodeDistanceMaker::GetInitName(void) const
{
	return cInitName;
}
