//
/*  ObservedData.h
*  PLEX
*
*  A.P. Jason de Koning, jason.de.koning@gmail.com
*  http://jasondk.org
*
*  David Pollock lab, University of Colorado School of Medicine
*  http://www.evolutionarygenomics.com
*  All rights reserved.

*/


#ifndef ObservedData_h__
#define ObservedData_h__

#include <string>
#include <map>
#include <vector>
#include "Matrix.h"
#include <iostream>
#include <fstream>
#include <sstream>

using namespace std;

class ObservedData {
    private:
    // primary stuff
    int numSites;
    int numTaxa;
    int tupleSize;
    int numStates;
    
    // PRIMARY
    vector<string>          taxonNames;               // The taxon names
    vector< vector<int> >   theStates;                // THE SEQUENCE DATA (by taxon)
    map <string, int>       stateIndex;               // return the number given the character state
    map <int, string>       reverseStateIndex;        // return the character state given the number
    
    map <int, int>          realDatasetColumn;        // given an encoded column, return the real column in the alignment
    map <int, int>          reverseRealDatasetColumn; // (other way around)
    
    // SECONDARY (internals)
    bool            _gapsAsCharacters;                 // include gap states or not?
    int             _currentState;                     // a counter for import
    map< int, bool > _excludedPositions;               // which positions to exclude?
    bool            _hasGap( string );                 // check if character is a gap character
    bool            _getStatesFromData;                // create a new state index or not
    bool            _overlappingTuples;                // do we let tuples overlap?
    void            _removeAllGapColumns(void);        // prune theStates and set realDataColumn data
    
    public:
    
    ObservedData();
    void usePresetStateIndex(int);
    void readData( string fileName );
    void encodeSequence( string, vector<int> &);

    void setNumStates(int);
    void setTupleSize(int);
    
    int  getAlignmentLength(void);
    int  getNumSequences(void);
    int  getNumStates(void);

	string getState(int);
    
    string& getTaxonName_ref(int);
    std::vector<int>& getSequence_ref(int);

};

#endif
