/*
 *  Options.h
 *  PLEX
 *
 *  A.P. Jason de Koning, jason.de.koning@gmail.com
 *  http://jasondk.org
 *
 *  David Pollock lab, University of Colorado School of Medicine
 *  http://www.evolutionarygenomics.com
 *  All rights reserved.
 
 */

#ifndef Options_h__
#define Options_h__

#include <vector>

class Options {
	public:
		Options();
		int randseed;
	
		int numThreads;		// number of threads to use (OpenMP)
	
		double __largestRate;  // a terrible global variable! oh no!
		int    __ASRVmode;	   // another one: which mode are we using for ASRV: 0=rates; 1=bls (only applies to B1u)
		double __globalLikelihoodTemp;
	
		int updatebls;
		double brlenupdatefreq;
	
		int outputSiteRates;
		int suppressFileOutput;
		// output the incomplete logL (and sample states) every 10,000 gens
		int outputIncompleteLogL;
		
		int capUniformizationRate;	// set it and leave it...
		bool capNumTreeNodes;
	
		int scaleByFrequencies;
	
		// New options: Aug 18...
		double gammaShapeASRV;
		int allowASRV;
		int updateASRVparms;
	
		int numCatASRV;
		int scaleRateMatrix;
	
		int numGenerations;
	
		// Coevolution:
		int coevol;
		int coevol_doPosteriorPredictiveNull;
		int coevol_minNumSubstitutions;
	
		// use the b1u* approximation (instead of b1u)...
		int doB1uStar;
		
		int reversibleModel;

		// Branch length discretization and/or subdivision...
		double maxBL;
		int numDiscreteBLs;
		std::vector<double> discreteBLs;
		int HwangGreen;
		
		// Ancestral state sampling...
		double ancStateUpdateFreq;
		int forceFullMarginalUpdate;
		int countExpectedSubstitutions;
	
	
		int datatype;
		// Protein RAR models...
		int doRAR;
		int numCat;
		int fixedRates;
		int freeFreqs;
		
		// Dependent rate assignment stuff...
		int DRA;
		double DRAprior;
	
		// File output stuff
		int outputFrequency;
    
        // LEGACY OPTIONS GO HERE
        int m_nDatatype, m_nGenerations, m_nBurnin, m_nSeqfreq, m_nMatfreq, m_nProbfreq, m_nCountfreq, m_nMethod, m_nWindowThreshold, m_nTotalWindow, m_nMaxWindow, m_nSimulationGeneration,  m_nNumModels;
        bool Debugging, m_bLoudness, m_bTreeflag, m_bSeqflag, m_bMatflag, m_bCountflag, m_bLikeflag, m_bUpdate, m_bFixedFreq, m_bSite;
        double m_fUpdatewindow;
        
		int m_nNumBranches;
        char* m_strTreefile;
        char* m_strSeqfile;
        char* m_strPositionfile;
        char* m_strLengthfile;
        char* m_strOLfile;
        char* m_strTreeOutfile;
        char* m_strSeqOutfile;
        char* m_strCountOutfile;
        char* m_strLikelihoodOutfile;
        char* m_strMatrixOutfile;
        
};

#endif