/*
 *  PLEXApp.cpp
 *  CLAMP
 *
 *  Created by Wanjun Gu.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */

#include "PLEXApp.h"
#include "RandomGenerator.h"
#include "NewickTreeParser.h"
#include "NodeDistanceMaker.h"

#include <ctype.h>

#include <algorithm>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <string>
#include <algorithm>

using std::min;
using std::cerr;
using std::endl;
using std::ifstream;
using std::setw;
using std::setprecision;
using std::flush;

const string cFileOpenErr = "Error opening file: ";
const string cFileReadErr = "Error reading file: ";

//---------------------------------------------------------------------------
PLEXApp::PLEXApp(void)
{
}

//---------------------------------------------------------------------------
void PLEXApp::SetSeed(int seed)
{
	RandomGenerator::GetGlobalGenerator().SetSeed(seed);
}


//---------------------------------------------------------------------------
double PLEXApp::rndu(void)
{
	return Numerics::getGlobalNumerics().Random();
}


//---------------------------------------------------------------------------
/*
 int PLEXApp::GetAlignLen(const SequenceVector& inSeqs)
{
	unsigned int m_nAlignlength = MAXLENGTH;
	
	for (int i=0; i<inSeqs.size(); i++)
		m_nAlignlength = min(m_nAlignlength, inSeqs[i].GetLen());

	return (m_nAlignlength);
}
*/
//---------------------------------------------------------------------------
inline
void PLEXApp::ReportError(
	const string&	inErrorMessage,
	bool		inAbort)
{
	cerr << inErrorMessage << endl;
	if (inAbort)
		throw(8);
}

//---------------------------------------------------------------------------
bool PLEXApp::GetTree(
	ifstream&		inTreeStream,
	Tree&	outTree)
{
	NewickTreeParser	parser;
	parser.SetNodeMaker(new NodeDistanceMaker);
	
	Tree	tree;

	bool success = parser.Read(inTreeStream, tree);
	
	if (success)
	{
		tree.MakeRootless();
		tree.swap(outTree);
	}
	
	return success;
}

//---------------------------------------------------------------------------
void PLEXApp::GetTree(
		const char*		m_strSequencefile,
		Tree&	outTree)
{
	ifstream treeFile(m_strSequencefile);

	if (treeFile.is_open())
	{
		ReportError("Reading tree file........", false);
		
		if (!GetTree(treeFile, outTree))
			ReportError(cFileReadErr + m_strSequencefile);
	}
	else 
		ReportError(cFileOpenErr + m_strSequencefile);
		
	ReportError("Done read tree file.", false);
	
	cerr << endl;
}

/*
//---------------------------------------------------------------------------
inline 
void PLEXApp::AddStates(
		CharVector& outState,
		char newstate)
{
	unsigned int  state;

	for(state=0; state < outState.size(); state++)
	{
		if (outState[state] == newstate)
		break;
	}
		
	if (state == outState.size())
		outState.push_back(newstate); 
}

//---------------------------------------------------------------------------
inline 
void PLEXApp::AddStates(
		std::vector<int>& outState,
		int newstate)
{
	unsigned int  state;

	for(state=0; state < outState.size(); state++)
	{
		if (outState[state] == newstate)
		break;
	}
		
	if (state == outState.size())
		outState.push_back(newstate); 
}


//---------------------------------------------------------------------------
void PLEXApp::DoStates(
		const SequenceVector&	inSeqs,
		const int& inLength,
		CharVector& outState)
{
	int t,s;
	char thischar;
	
	if (options.datatype == 2) { 
		std::cout << "ADDING SEQ STATES FOR PROTEINS..." << endl;
		char predefinedOrder[21] = {'M','T','N','I','R','K','S','H','P','L','V','E','F','D','A','W','G','C','Q','Y'};
		for (int i=0; i<20; i++) {
			AddStates( outState, predefinedOrder[i] );
		}
	} else {
		for (t = 0; t < inSeqs.size(); t++)
		{
			for (s = 0; s < inLength; s++)
			{
				thischar = inSeqs[t][s];
				
				if (thischar < 'a' && thischar > 'Z') 
					cerr << "Odd Character: " << thischar << "; seq " << t << " (" << inLength << "), site " << s << endl;
					
				if (!ispunct(thischar)) {
					AddStates(outState, thischar);
				}
			}
		}
	}
}

//---------------------------------------------------------------------------
void PLEXApp::DoSymmModelStates(
		CharVector& outState)
{
	std::string states = "ACGT"; //symmetrical model state lists
	
	char thischar;
	
	int t;
	
	for (t = 0; t < states.size(); t++)
	{
		thischar = states[t];
			
		if (!ispunct(thischar))
			AddStates(outState, thischar);
	}
}


//---------------------------------------------------------------------------
void PLEXApp::DoCodonModelStates(
		std::vector<int>& outState)
{
	std::string states = "ACGT"; //symmetrical model state lists
	
	int thisState;
	
	int i,j,k;
	// TEMP TEMP
	for (i = 0; i < states.size(); i++) {
		for (j = 0; j < states.size(); j++) {
			for (k = 0; k < states.size(); k++) {
				thisState = (i * 16) + (j * 4) + k;
				AddStates(outState, thisState);
			}
		}
	}
			
}


//---------------------------------------------------------------------------
void PLEXApp::MakeIndex(
		const CharVector& inState,
		IntVector& outIndex,
		int& minstate)
{
	int index;
	
	if (outIndex.empty())
	{
		minstate = (int) inState[0]; 
		
		int MaxState = (int) inState[0];
		
		unsigned int state;
		
		for(state=0; state < inState.size(); state++) {
			index=(int)inState[state];
			
			if (index < minstate) 
				minstate = index;
				
			if (index > MaxState)
				MaxState = index;
		}
		
		outIndex.resize(MaxState-minstate+1);
		
		for (int i=0; i<MaxState-minstate+1; i++)
			outIndex.mDataArray[i] = -1; 

		for(int state=0;state<inState.size();state++)
		{
			index = (int)inState[state];
			outIndex.mDataArray[index - minstate] = state;
		}
	}
}

//---------------------------------------------------------------------------
void PLEXApp::MakeIndex(
		const std::vector<int>& inState,
		IntVector& outIndex,
		int& minstate)
{
	int index;
	
	if (outIndex.empty())
	{
		minstate = (int) inState[0]; 
		
		int MaxState = (int) inState[0];
		
		unsigned int state;
		
		for( state=0; state < inState.size(); state++)
		{
			index=(int)inState[state];
			
			if (index < minstate) 
				minstate = index;
				
			if (index > MaxState)
				MaxState = index;
		}
		
		outIndex.resize(MaxState-minstate+1);
		
		for (int i=0; i<MaxState-minstate+1; i++)
			outIndex.mDataArray[i] = -1; 

		for(int state=0;state<inState.size();state++)
		{
			index = (int)inState[state];
			outIndex.mDataArray[index - minstate] = state;
		}
	}
}
//---------------------------------------------------------------------------
inline
unsigned int PLEXApp::SetIndex(
	const char& state,
	const IntVector& inStateindex,
	const int& minstate,
	const int& m_nNumstates)
{ 
	return (ispunct(state))
			?	m_nNumstates
			:	inStateindex[static_cast<int>(state) - minstate]; 
}



//---------------------------------------------------------------------------
void PLEXApp::CalculateSinglestateChanges(
	const vector< vector<int> > &m_vecSequences, 
	const IntVector& m_vecStateIndex, 
	const int& m_nMinstate, 
	const int& m_nNumStates,
	IntMatrix& m_gCountChanges)
{
	int i;
	
	i = 0;
	
	int sequencesize = m_vecSequences.size();
	
	if (m_gCountChanges.empty())
		m_gCountChanges.resize(m_nNumStates, m_nNumStates, 0);
	else
		m_gCountChanges.fill(0);

	Sequence mSequence1, mSequence2;
	
	while (i+1 <= sequencesize)
	{
		// read in the ancestral sequence;
		mSequence1 = m_vecSequences[i];
		
		// read in the leaf sequence;
		mSequence2 = m_vecSequences[i+1];
		
		// valid the number of ancestral and leaf sequence with same length;
		if (mSequence1.size() == mSequence2.size())
		{
			// get the count of the changes of different states;
			std::string str1 = mSequence1.str();
			
			std::string str2 = mSequence2.str();
			
			for (int j = 0; j < mSequence1.size(); j++)
			{
				unsigned int index1 = SetIndex(str1.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
			
				unsigned int index2 = SetIndex(str2.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
				
				if ((index1 < m_nNumStates) && (index2 < m_nNumStates))
					m_gCountChanges[index1][index2] += 1;
			}
		}
		
		i += 2;
	}
}

//---------------------------------------------------------------------------
inline
void PLEXApp::Dinucleocoding(
	const unsigned int& left, 
	const unsigned int& right,
	const unsigned int& m_nNumStates, 
	unsigned int& dinucleotidecode)
{
	if ((left < m_nNumStates) && (right < m_nNumStates))
		dinucleotidecode = left * m_nNumStates + right;
	else
		dinucleotidecode = m_nNumStates * m_nNumStates;
}

//---------------------------------------------------------------------------
inline
void PLEXApp::Trinucleocoding(
	const unsigned int& posA, 
	const unsigned int& posB,
	const unsigned int& posC,
	const unsigned int& m_nNumStates, 
	unsigned int& trinucleotidecode)
{
	if ((posA < m_nNumStates) && (posB < m_nNumStates) && (posC < m_nNumStates) )
		trinucleotidecode = posA * m_nNumStates* m_nNumStates + posB * m_nNumStates + posC;
	else
		trinucleotidecode = m_nNumStates * m_nNumStates * m_nNumStates;
}

//---------------------------------------------------------------------------
void PLEXApp::CalculateDoublestateChanges(
	const vector< vector<int> > &m_vecSequences, 
	const IntVector& m_vecStateIndex, 
	const int& m_nMinstate, 
	const int& m_nNumStates,
	IntMatrix& m_gCountChanges)
{
	int i;
	
	i = 0;
	
	int statesize = m_nNumStates * m_nNumStates;
	
	int sequencesize = m_vecSequences.size();
	
	if (m_gCountChanges.empty())
		m_gCountChanges.resize(statesize, statesize, 0);
	else
		m_gCountChanges.fill(0);
		
	Sequence mSequence1, mSequence2;
	
	while (i+1 < sequencesize)
	{
		// read in the ancestral sequence;
		mSequence1 = m_vecSequences[i];
		
		// read in the leaf sequence;
		mSequence2 = m_vecSequences[i+1];
		
		// valid the number of ancestral and leaf sequence with same length;
		if (mSequence1.size() == mSequence2.size())
		{
			// get the count of the changes of different states;
			std::string str1 = mSequence1.str();
			
			std::string str2 = mSequence2.str();
			
			for (int j = 0; j < mSequence1.size()-1; j++)
			{
				unsigned int index1left = SetIndex(str1.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
			
				unsigned int index1right = SetIndex(str1.at(j+1), m_vecStateIndex, m_nMinstate, m_nNumStates);
				
				unsigned int index1;
				
				Dinucleocoding(index1left, index1right, m_nNumStates, index1);

				unsigned int index2left = SetIndex(str2.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
			
				unsigned int index2right = SetIndex(str2.at(j+1), m_vecStateIndex, m_nMinstate, m_nNumStates);
				
				unsigned int index2;
				
				Dinucleocoding(index2left, index2right, m_nNumStates, index2);

				if ((index1 < statesize) && (index2 < statesize))
					m_gCountChanges[index1][index2] += 1;
			}
		}
		
		i += 2;
	}
}

//---------------------------------------------------------------------------
void PLEXApp::CalculateTriplestateChanges(
	const vector< vector<int> > &m_vecSequences, 
	const IntVector& m_vecStateIndex, 
	const int& m_nMinstate, 
	const int& m_nNumStates,
	IntMatrix& m_gCountChanges)
{
	int i;
	
	i = 0;
	
	int statesize = m_nNumStates * m_nNumStates * m_nNumStates;
	
	int sequencesize = m_vecSequences.size();
	
	if (m_gCountChanges.empty())
		m_gCountChanges.resize(statesize, statesize, 0);
	else
		m_gCountChanges.fill(0);
		
	Sequence mSequence1, mSequence2;
	
	while (i+1 < sequencesize)
	{
		// read in the ancestral sequence;
		mSequence1 = m_vecSequences[i];
		
		// read in the leaf sequence;
		mSequence2 = m_vecSequences[i+1];
		
		// valid the number of ancestral and leaf sequence with same length;
		if (mSequence1.size() == mSequence2.size())
		{
			// get the count of the changes of different states;
			std::string str1 = mSequence1.str();
			
			std::string str2 = mSequence2.str();
			
			for (int j = 0; j < mSequence1.size()-2; j++)
			{
				unsigned int index1a = SetIndex(str1.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);			
				unsigned int index1b = SetIndex(str1.at(j+1), m_vecStateIndex, m_nMinstate, m_nNumStates);
				unsigned int index1c = SetIndex(str1.at(j+2), m_vecStateIndex, m_nMinstate, m_nNumStates);				

				unsigned int index1;
				Trinucleocoding(index1a, index1b, index1c, m_nNumStates, index1);

				unsigned int index2a = SetIndex(str2.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);			
				unsigned int index2b = SetIndex(str2.at(j+1), m_vecStateIndex, m_nMinstate, m_nNumStates);
				unsigned int index2c = SetIndex(str2.at(j+2), m_vecStateIndex, m_nMinstate, m_nNumStates);				

				unsigned int index2;
				Trinucleocoding(index2a, index2b, index2c, m_nNumStates, index2);

				if ((index1 < statesize) && (index2 < statesize))
					m_gCountChanges[index1][index2] += 1;
			}
		}
		
		i += 3;
	}
}


//---------------------------------------------------------------------------
void PLEXApp::CalculateSitespecificDoublestateChanges(
	const vector< vector<int> > &m_vecSequences, 
	const IntVector& m_vecStateIndex, 
	const int& m_nMinstate, 
	const int& m_nNumStates,
	IntMatrix3D& m_gSiteCountChanges)
{
	int i;
	
	i = 0;
	
	int statesize = m_nNumStates * m_nNumStates;
	
	int sequencesize = m_vecSequences.size();
	
	Sequence mSequence1, mSequence2;
	
	int len = m_vecSequences[0].size();
		
	if (m_gSiteCountChanges.empty())
		m_gSiteCountChanges.resize(len-1, statesize, statesize, 0);
	else
		m_gSiteCountChanges.fill(0);
	
	while (i+1 < sequencesize)
	{
		// read in the ancestral sequence;
		mSequence1 = m_vecSequences[i];
		
		// read in the leaf sequence;
		mSequence2 = m_vecSequences[i+1];
		
		// valid the number of ancestral and leaf sequence with same length;
		if (mSequence1.size() == mSequence2.size())
		{
			// get the count of the changes of different states;
			std::string str1 = mSequence1.str();
			
			std::string str2 = mSequence2.str();
			
			for (int j = 0; j < mSequence1.size()-1; j++)
			{
				unsigned int index1left = SetIndex(str1.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
			
				unsigned int index1right = SetIndex(str1.at(j+1), m_vecStateIndex, m_nMinstate, m_nNumStates);
				
				unsigned int index1;
				
				Dinucleocoding(index1left, index1right, m_nNumStates, index1);

				unsigned int index2left = SetIndex(str2.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
			
				unsigned int index2right = SetIndex(str2.at(j+1), m_vecStateIndex, m_nMinstate, m_nNumStates);
				
				unsigned int index2;
				
				Dinucleocoding(index2left, index2right, m_nNumStates, index2);

				if ((index1 < statesize) && (index2 < statesize))
					m_gSiteCountChanges[j][index1][index2] += 1;
			}
		}
		
		i += 2;
	}
}

//---------------------------------------------------------------------------
void PLEXApp::CalculateSitespecificTriplestateChanges(
	const vector< vector<int> > &m_vecSequences, 
	const IntVector& m_vecStateIndex, 
	const int& m_nMinstate, 
	const int& m_nNumStates,
	IntMatrix3D& m_gSiteCountChanges)
{
	int i;
	
	i = 0;
	
	int statesize = m_nNumStates * m_nNumStates * m_nNumStates;
	
	int sequencesize = m_vecSequences.size();
	
	Sequence mSequence1, mSequence2;
	
	int len = m_vecSequences[0].size();
		
	if (m_gSiteCountChanges.empty())
		m_gSiteCountChanges.resize(len-1, statesize, statesize, 0);
	else
		m_gSiteCountChanges.fill(0);
	
	while (i+1 < sequencesize)
	{
		// read in the ancestral sequence;
		mSequence1 = m_vecSequences[i];
		
		// read in the leaf sequence;
		mSequence2 = m_vecSequences[i+1];
		
		// valid the number of ancestral and leaf sequence with same length;
		if (mSequence1.size() == mSequence2.size())
		{
			// get the count of the changes of different states;
			std::string str1 = mSequence1.str();
			
			std::string str2 = mSequence2.str();
			
			for (int j = 0; j < mSequence1.size()-2; j++)
			{
				unsigned int index1a = SetIndex(str1.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);			
				unsigned int index1b = SetIndex(str1.at(j+1), m_vecStateIndex, m_nMinstate, m_nNumStates);
				unsigned int index1c = SetIndex(str1.at(j+2), m_vecStateIndex, m_nMinstate, m_nNumStates);				

				unsigned int index1;
				Trinucleocoding(index1a, index1b, index1c, m_nNumStates, index1);

				unsigned int index2a = SetIndex(str2.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);			
				unsigned int index2b = SetIndex(str2.at(j+1), m_vecStateIndex, m_nMinstate, m_nNumStates);
				unsigned int index2c = SetIndex(str2.at(j+2), m_vecStateIndex, m_nMinstate, m_nNumStates);				

				unsigned int index2;
				Trinucleocoding(index2a, index2b, index2c, m_nNumStates, index2);

				if ((index1 < statesize) && (index2 < statesize))
					m_gSiteCountChanges[j][index1][index2] += 1;
			}
		}
		
		i += 3;
	}
}

//---------------------------------------------------------------------------
void PLEXApp::CalculateSitespecificSinglestateChanges(
	const vector< vector<int> > &m_vecSequences, 
	const IntVector& m_vecStateIndex, 
	const int& m_nMinstate, 
	const int& m_nNumStates,
	IntMatrix3D& m_gSiteCountChanges)
{
	int i;
	
	i = 0;
	
	int sequencesize = m_vecSequences.size();
	
	Sequence mSequence1, mSequence2;
	
	int len = m_vecSequences[0].size();
		
	if (m_gSiteCountChanges.empty())
		m_gSiteCountChanges.resize(len, m_nNumStates, m_nNumStates, 0);
	else
		m_gSiteCountChanges.fill(0);
	
	while (i+1 < sequencesize)
	{
		// read in the ancestral sequence;
		mSequence1 = m_vecSequences[i];
		
		// read in the leaf sequence;
		mSequence2 = m_vecSequences[i+1];
		
		// valid the number of ancestral and leaf sequence with same length;
		if (mSequence1.size() == mSequence2.size())
		{
			// get the count of the changes of different states;
			std::string str1 = mSequence1.str();
			
			std::string str2 = mSequence2.str();
			
			for (int j = 0; j < mSequence1.size(); j++)
			{
				unsigned int index1 = SetIndex(str1.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
				
				unsigned int index2 = SetIndex(str2.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
				
				if ((index1 < m_nNumStates) && (index2 < m_nNumStates))
					m_gSiteCountChanges[j][index1][index2] += 1;
			}
		}
		
		i += 2;
	}
}

//---------------------------------------------------------------------------
void PLEXApp::CalculateSequencespecificDoublestateChanges(
	const vector< vector<int> > &m_vecSequences, 
	const IntVector& m_vecStateIndex, 
	const int& m_nMinstate, 
	const int& m_nNumStates,
	IntMatrix3D& m_gSequenceCountChanges)
{
	int i;
	
	i = 0;
	
	int statesize = m_nNumStates * m_nNumStates;
	
	int sequencesize = m_vecSequences.size()/2;
	
	Sequence mSequence1, mSequence2;
	
//	int len = m_vecSequences[0].size();
		
	if (m_gSequenceCountChanges.empty())
		m_gSequenceCountChanges.resize(sequencesize, statesize, statesize, 0);
	else
		m_gSequenceCountChanges.fill(0);
	
	int k = 0;
	
	while (i+1 < 2 * sequencesize)
	{
		// read in the ancestral sequence;
		mSequence1 = m_vecSequences[i];
		
		// read in the leaf sequence;
		mSequence2 = m_vecSequences[i+1];
		
		// valid the number of ancestral and leaf sequence with same length;
		if (mSequence1.size() == mSequence2.size())
		{
			// get the count of the changes of different states;
			std::string str1 = mSequence1.str();
			
			std::string str2 = mSequence2.str();
			
			for (int j = 0; j < mSequence1.size()-1; j++)
			{
				unsigned int index1left = SetIndex(str1.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
			
				unsigned int index1right = SetIndex(str1.at(j+1), m_vecStateIndex, m_nMinstate, m_nNumStates);
				
				unsigned int index1;
				
				Dinucleocoding(index1left, index1right, m_nNumStates, index1);

				unsigned int index2left = SetIndex(str2.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
			
				unsigned int index2right = SetIndex(str2.at(j+1), m_vecStateIndex, m_nMinstate, m_nNumStates);
				
				unsigned int index2;
				
				Dinucleocoding(index2left, index2right, m_nNumStates, index2);

				if ((index1 < statesize) && (index2 < statesize))
					m_gSequenceCountChanges[k][index1][index2] += 1;
			}
		}
		
		i += 2;
		
		k++;
	}
}

//---------------------------------------------------------------------------
void PLEXApp::CalculateSequencespecificTriplestateChanges(
	const vector< vector<int> > &m_vecSequences, 
	const IntVector& m_vecStateIndex, 
	const int& m_nMinstate, 
	const int& m_nNumStates,
	IntMatrix3D& m_gSequenceCountChanges)
{
	// Added by Jason for codon models
	int i = 0;
	
	int statesize = m_nNumStates * m_nNumStates * m_nNumStates;
	int sequencesize = m_vecSequences.size()/3;
	
	Sequence mSequence1, mSequence2;
	
//	int len = m_vecSequences[0].size();
		
	if (m_gSequenceCountChanges.empty())
		m_gSequenceCountChanges.resize(sequencesize, statesize, statesize, 0);
	else
		m_gSequenceCountChanges.fill(0);
	
	int k = 0;
	
	while (i+1 < 3 * sequencesize)
	{
		// read in the ancestral sequence;
		mSequence1 = m_vecSequences[i];
		
		// read in the leaf sequence;
		mSequence2 = m_vecSequences[i+1];
		
		// valid the number of ancestral and leaf sequence with same length;
		if (mSequence1.size() == mSequence2.size())
		{
			// get the count of the changes of different states;
			std::string str1 = mSequence1.str();
			
			std::string str2 = mSequence2.str();
			
			for (int j = 0; j < mSequence1.size()-1; j++)
			{
				unsigned int index1a = SetIndex(str1.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
				unsigned int index1b = SetIndex(str1.at(j+1), m_vecStateIndex, m_nMinstate, m_nNumStates);
				unsigned int index1c = SetIndex(str1.at(j+2), m_vecStateIndex, m_nMinstate, m_nNumStates);
				
				unsigned int index1;
				Trinucleocoding(index1a, index1b, index1c, m_nNumStates, index1);

				unsigned int index2a = SetIndex(str2.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
				unsigned int index2b = SetIndex(str2.at(j+1), m_vecStateIndex, m_nMinstate, m_nNumStates);
				unsigned int index2c = SetIndex(str2.at(j+2), m_vecStateIndex, m_nMinstate, m_nNumStates);
				
				unsigned int index2;
				Trinucleocoding(index2a, index2b, index2c, m_nNumStates, index2);

				if ((index1 < statesize) && (index2 < statesize))
					m_gSequenceCountChanges[k][index1][index2] += 1;
			}
		}
		
		i += 3;
		
		k++;
	}
}

//---------------------------------------------------------------------------
void PLEXApp::CalculateSequencespecificSinglestateChanges(
	const vector< vector<int> > &m_vecSequences, 
	const IntVector& m_vecStateIndex, 
	const int& m_nMinstate, 
	const int& m_nNumStates,
	IntMatrix3D& m_gSequenceCountChanges)
{
	int i;
	
	i = 0;
	
	int sequencesize = m_vecSequences.size()/2;
	
	Sequence mSequence1, mSequence2;
	
//	int len = m_vecSequences[0].size();
		
	if (m_gSequenceCountChanges.empty())
		m_gSequenceCountChanges.resize(sequencesize, m_nNumStates, m_nNumStates, 0);
	else
		m_gSequenceCountChanges.fill(0);
		
	unsigned int k = 0;
	
	while (i+1 < 2 * sequencesize)
	{
		// read in the ancestral sequence;
		mSequence1 = m_vecSequences[i];
		
		// read in the leaf sequence;
		mSequence2 = m_vecSequences[i+1];
		
		// valid the number of ancestral and leaf sequence with same length;
		if (mSequence1.size() == mSequence2.size())
		{
			// get the count of the changes of different states;
			std::string str1 = mSequence1.str();
			
			std::string str2 = mSequence2.str();
			
			for (int j = 0; j < mSequence1.size(); j++)
			{
				unsigned int index1 = SetIndex(str1.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
				
				unsigned int index2 = SetIndex(str2.at(j), m_vecStateIndex, m_nMinstate, m_nNumStates);
				
				if ((index1 < m_nNumStates) && (index2 < m_nNumStates))
					m_gSequenceCountChanges[k][index1][index2] += 1;
			}
		}
		
		k++; 
		
		i += 2;
	}
}

*/

//---------------------------------------------------------------------------
inline
void PLEXApp::PrintSequenceStates(
	ostream&		outStream,
	ObservedData&	theData)
{
	for(unsigned int i=0; i < theData.getNumStates(); i++)
		outStream << setw(12) << theData.getState(i) << setw(0) << " ";

	outStream << endl;
}


//---------------------------------------------------------------------------
void PLEXApp::PrintSequenceStatesLine( ostream& outStream, ObservedData& theData ) {
	for(unsigned int i=0; i < theData.getNumStates(); i++)
	{
		for(unsigned int j=0; j < theData.getNumStates(); j++)	
			outStream << setw(0) << theData.getState(i) << theData.getState(j) << setw(0) << " ";
	}

	outStream << endl;
}

//---------------------------------------------------------------------------
void PLEXApp::PrintMatrix( ostream& outStream, DoubleMatrix& inMatrix, ObservedData& theData ) {

	if (outStream)
	{
		 
		outStream <<  "\t" << setw(0) << "\t";
		
		unsigned int numRows = inMatrix.rows();

		PrintSequenceStates(outStream, theData);
		
		unsigned int numCols = inMatrix.cols();
		
		for(DoubleMatrix::size_type i=0; i<numRows; i++)
		{
			outStream <<  theData.getState(i) << setw(0) << "\t";	
				
			for(unsigned int j=0; j<numCols; j++)
				outStream <<  setprecision(6) << inMatrix[i][j] << setprecision(0) << setw(0) << "\t";	
				
			outStream <<  theData.getState(i) << setw(0) << "\t" << endl;	
		}

		outStream <<  "\t" << setw(0) << "\t";
		
		PrintSequenceStates( outStream, theData );

		outStream << endl; 
	}
	
	outStream << flush;
}

/*//---------------------------------------------------------------------------
void PLEXApp::PrintMatrix(
	ostream&		outStream,
	const CharVector&	inStates,
	const DoubleMatrix&	inMatrix)
{

	if (outStream)
	{
		 
		outStream << setw(12) << "\t" << setw(0) << "\t";
		
		unsigned int numRows = inMatrix.rows();

		PrintSequenceStates(outStream, inStates);
		
		unsigned int numCols = inMatrix.cols();
		
		for(DoubleMatrix::size_type i=0; i<numRows; i++)
		{
			outStream << setw(12) << inStates[i] << setw(0) << "\t";	
				
			for(unsigned int j=0; j<numCols; j++)
				outStream << setw(12) << setprecision(6) << inMatrix[i][j] << setprecision(0) << setw(0) << "\t";	
				
			outStream << setw(12) << inStates[i] << setw(0) << "\t" << endl;	
		}

		outStream << setw(12) << "\t" << setw(0) << "\t";
		
		PrintSequenceStates(outStream, inStates);

		outStream << endl; 
	}
	
	outStream << flush;
}*/


//---------------------------------------------------------------------------
/*
void PLEXApp::PrintMatrix3D(
	ostream&		outStream,
	const CharVector&	inStates,
	const DoubleMatrix3D&	inMatrix3D)
{
	unsigned int numRows = inMatrix3D.rows();
	
	for (int k = 0; k< numRows; k++)
	{
		//DoubleMatrix tempMatrix = inMatrix3D[k];
		
		outStream << setw(8) << "Model" << setw(12) << k << endl; 

		PrintMatrix(outStream, inStates, inMatrix3D[k]);
	}
}

//---------------------------------------------------------------------------
void PLEXApp::PrintMatrix(
	ostream&		outStream,
	const StringVector&	inStates,
	const DoubleMatrix&	inMatrix)
{

	if (outStream)
	{
		 
		outStream <<  "\t" << setw(0) << "\t";
		
		unsigned int numRows = inMatrix.rows();

		PrintSequenceStates(outStream, inStates);
		
		unsigned int numCols = inMatrix.cols();
		
		for(DoubleMatrix::size_type i=0; i<numRows; i++)
		{
			outStream <<  inStates[i] << setw(0) << "\t";	
				
			for(unsigned int j=0; j<numCols; j++)
				outStream <<  setprecision(6) << inMatrix[i][j] << setprecision(0) << setw(0) << "\t";	
				
			outStream <<  inStates[i] << setw(0) << "\t" << endl;	
		}

		outStream <<  "\t" << setw(0) << "\t";
		
		PrintSequenceStates(outStream, inStates);

		outStream << endl; 
	}
	
	outStream << flush;
}


 
 //---------------------------------------------------------------------------
void PLEXApp::PrintMatrix(
	ostream&		outStream,
	const StringVector&	inStates,
	const DoubleMatrix&	inMatrix)
{

	if (outStream)
	{
		 
		outStream << setw(12) << "\t" << setw(0) << "\t";
		
		unsigned int numRows = inMatrix.rows();

		PrintSequenceStates(outStream, inStates);
		
		unsigned int numCols = inMatrix.cols();
		
		for(DoubleMatrix::size_type i=0; i<numRows; i++)
		{
			outStream << setw(12) << inStates[i] << setw(0) << "\t";	
				
			for(unsigned int j=0; j<numCols; j++)
				outStream << setw(12) << setprecision(6) << inMatrix[i][j] << setprecision(0) << setw(0) << "\t";	
				
			outStream << setw(12) << inStates[i] << setw(0) << "\t" << endl;	
		}

		outStream << setw(12) << "\t" << setw(0) << "\t";
		
		PrintSequenceStates(outStream, inStates);

		outStream << endl; 
	}
	
	outStream << flush;
}
*/

//---------------------------------------------------------------------------
void PLEXApp::PrintDoubleArray(
	ostream&		outStream,
	int numStates,
	const std::vector<double>* inMatrix)
{

	if (outStream)
	{
		 
		for(unsigned int j=0; j<numStates; j++)
			outStream << setw(12) << setprecision(6) << (*inMatrix)[j] << setprecision(0) << setw(0) << " ";	
				
		outStream << endl << endl; 
	}
	
	outStream << flush;
}

//---------------------------------------------------------------------------
void PLEXApp::PrintDoubleArray(
	ostream&		outStream,
	int numStates,
	const std::vector<double>* inMatrix, int offset)
{

	if (outStream)
	{
		 
		for(unsigned int j=offset; j<(numStates+offset); j++)
			outStream << setw(12) << setprecision(6) << (*inMatrix)[j] << setprecision(0) << setw(0) << " ";	
				
		outStream << endl; // << endl; 
	}
	
	outStream << flush;
}
//---------------------------------------------------------------------------
void PLEXApp::PrintMatrix3D(
	ostream&		outStream,
	const DoubleMatrix3D&	inMatrix3D,
	ObservedData& theData)
{
	unsigned int numRows = inMatrix3D.rows();
	
	for (int k = 0; k< numRows; k++)
	{
		DoubleMatrix tempMatrix = inMatrix3D[k];
		
		outStream << setw(8) << "Model" << setw(12) << k << endl; 

		PrintMatrix(outStream, tempMatrix, theData);
	}
}

//---------------------------------------------------------------------------
void PLEXApp::PrintMatrix(
	ostream&		outStream,
	IntMatrix&	inMatrix,
	ObservedData& theData )
{

	if (outStream)
	{
		 
		outStream <<  "\t" << setw(0) << "\t";

		unsigned int numRows = inMatrix.rows();

		PrintSequenceStates(outStream, theData);
		
		unsigned int numCols = inMatrix.cols();
		
		for(DoubleMatrix::size_type i=0; i<numRows; i++)
		{
			outStream <<  theData.getState(i) << setw(0) << "\t";	
				
			for(unsigned int j=0; j<numCols; j++)
				outStream <<  inMatrix[i][j] << setw(0) << "\t";	
				
			outStream <<  theData.getState(i) << setw(0) << "\t" << endl;	
		}

		outStream <<  "\t" << setw(0) << "\t";
		
		PrintSequenceStates(outStream, theData);

		outStream << endl; 
	}
	
	outStream << flush;
}

/*//---------------------------------------------------------------------------
void PLEXApp::PrintMatrix(
	ostream&		outStream,
	const StringVector&	inStates,
	const IntMatrix&	inMatrix)
{

	if (outStream)
	{
		 
		outStream << setw(12) << "\t" << setw(0) << "\t";

		unsigned int numRows = inMatrix.rows();

		PrintSequenceStates(outStream, inStates);
		
		unsigned int numCols = inMatrix.cols();
		
		for(DoubleMatrix::size_type i=0; i<numRows; i++)
		{
			outStream << setw(12) << inStates[i] << setw(0) << "\t";	
				
			for(unsigned int j=0; j<numCols; j++)
				outStream << setw(12) << inMatrix[i][j] << setw(0) << "\t";	
				
			outStream << setw(12) << inStates[i] << setw(0) << "\t" << endl;	
		}

		outStream << setw(12) << "\t" << setw(0) << "\t";
		
		PrintSequenceStates(outStream, inStates);

		outStream << endl; 
	}
	
	outStream << flush;
}
*/

/*//---------------------------------------------------------------------------
void PLEXApp::PrintMatrix(
	ostream&		outStream,
	const CharVector&	inStates,
	const IntMatrix&	inMatrix)
{

	if (outStream)
	{
		 
		outStream << setw(12) << "\t" << setw(0) << "\t";
		
		PrintSequenceStates(outStream, inStates);
		
		unsigned int numRows = inMatrix.rows();
		
		unsigned int numCols = inMatrix.cols();
		
		for(unsigned int i=0; i<numRows; i++)
		{
			//outStream << setw(12) << inStates[i] << setw(0) << " ";	
			outStream << setw(12) << inStates[i] << "\t";	
				
			for(unsigned int j=0; j<numCols; j++)
				outStream << setw(12) << inMatrix[i][j] << "\t";	
			//	outStream << setw(12) << inMatrix[i][j] << setw(0) << " ";	
				
//			outStream << setw(12) << inStates[i] << setw(0) << " " << endl;	
			outStream << setw(12) << inStates[i] << "\t " << endl;	
		}

//		outStream << setw(12) << " " << setw(0) << " ";
		outStream << setw(12) << "\t" << "\t";
		
		PrintSequenceStates(outStream, inStates);

		outStream << endl; 
	}
	
	outStream << flush;
}



//---------------------------------------------------------------------------
void PLEXApp::PrintSiteDoubleChangesMatrix(
			ostream&		outStream,
			const CharVector&	inStates,
			const IntVector& 	m_vecStateIndex, 
			const int& 		m_nMinstate, 
			const IntMatrix3D&	inMatrix3D,
			const Sequence& 	m_sequence)
{
	int m_nNumstates = inStates.size();
	
	StringVector tempStates(m_nNumstates*m_nNumstates);
	
	int k = 0;
	
	for (int i = 0; i< m_nNumstates; i++)
	{
		for (int j = 0; j < m_nNumstates; j++)
		{
			tempStates[k] = inStates[i];
			
			std::string tempstring(1, inStates[j]);
			
			tempStates[k] += tempstring;
			
			k++;
		}
	}
	
	outStream << setw(12) << "Site" << setw(12) << "state" << setw(0) << " ";

	PrintSequenceStates(outStream, tempStates);
		
	for (int j = 0; j< inMatrix3D.rows(); j++)
	{
		outStream << setw(12) << j << setw(0);
		
		IntMatrix sitechange = inMatrix3D[j];
		
		unsigned int numCols = sitechange.cols();
		
		unsigned int i;
		
		std::string str1 = m_sequence.str();

		unsigned int index1left = SetIndex(str1.at(j), m_vecStateIndex, m_nMinstate, m_nNumstates);
		
		unsigned int index1right = SetIndex(str1.at(j+1), m_vecStateIndex, m_nMinstate, m_nNumstates);
				
		Dinucleocoding(index1left, index1right, m_nNumstates, i);

		outStream << setw(12) << tempStates[i] << setw(0) << " ";	
				
		for(unsigned int k=0; k<numCols; k++)
			outStream << setw(12) << sitechange[i][k] << setw(0) << " ";	
				
		outStream << endl;	
	}

	outStream << flush;

}

//---------------------------------------------------------------------------
void PLEXApp::PrintSiteSingleChangesMatrix(
			ostream&		outStream,
			const CharVector&	inStates,
			const IntVector& 	m_vecStateIndex, 
			const int& 		m_nMinstate, 
			const IntMatrix3D&	inMatrix3D,
			const Sequence& 	m_sequence)
{
	int m_nNumstates = inStates.size();
	
	outStream << setw(12) << "Site" << setw(12) << "state" << setw(0) << " ";

	PrintSequenceStates(outStream, inStates);
		
	for (int j = 0; j< inMatrix3D.rows(); j++)
	{
		outStream << setw(12) << j << setw(0);
		
		IntMatrix sitechange = inMatrix3D[j];
		
		unsigned int numCols = sitechange.cols();
		
		std::string str1 = m_sequence.str();

		unsigned int i = SetIndex(str1.at(j), m_vecStateIndex, m_nMinstate, m_nNumstates);
		
		outStream << setw(12) << inStates[i] << setw(0) << " ";	
				
		for(unsigned int k=0; k<numCols; k++)
			outStream << setw(12) << sitechange[i][k] << setw(0) << " ";	
				
		outStream << endl;	
	}

	outStream << flush;

}

//---------------------------------------------------------------------------
void PLEXApp::PrintSequenceDoubleChangesMatrix(
			ostream&		outStream,
			const CharVector&	inStates,
			const IntVector& 	m_vecStateIndex, 
			const int& 		m_nMinstate, 
			const IntMatrix3D&	inMatrix3D)
{
	int m_nNumstates = inStates.size();
	
	StringVector tempStates(m_nNumstates*m_nNumstates);
	
	int k = 0;
	
	for (int i = 0; i< m_nNumstates; i++)
	{
		for (int j = 0; j < m_nNumstates; j++)
		{
			tempStates[k] = inStates[i];
			
			std::string tempstring(1, inStates[j]);
			
			tempStates[k] += tempstring;
			
			k++;
		}
	}
	
	for (int j = 0; j< inMatrix3D.rows(); j++)
	{
		outStream << setw(12) << "Sequence" << setw(12) << j << setw(0) << endl;
		
		IntMatrix sitechange = inMatrix3D[j];
		
		PrintMatrix(outStream, tempStates, sitechange);
	}

	outStream << flush;
}

//---------------------------------------------------------------------------
void PLEXApp::PrintSequenceSingleChangesMatrix(
			ostream&		outStream,
			const CharVector&	inStates,
			const IntVector& 	m_vecStateIndex, 
			const int& 		m_nMinstate, 
			const IntMatrix3D&	inMatrix3D) {
//	int m_nNumstates = inStates.size();
	
	for (int j = 0; j< inMatrix3D.rows(); j++) {
		outStream << setw(12) << "Sequence" << setw(12) << j << setw(0) << endl;
		
		IntMatrix sitechange = inMatrix3D[j];
		
		PrintMatrix(outStream, inStates, sitechange);
	}

	outStream << flush;
}
*/

//---------------------------------------------------------------------------
void PLEXApp::PrintMatrixLine(
	ostream&		outStream,
	const CharVector&	inStates,
	const DoubleMatrix&	inMatrix)
{

	if (outStream)
	{
		unsigned int numRows = inMatrix.rows();

		unsigned int numCols = inMatrix.cols();
		
		for(DoubleMatrix::size_type i=0; i<numRows; i++)
		{
			for(unsigned int j=0; j<numCols; j++)
				outStream << setprecision(6) << inMatrix[i][j] << "\t ";	
//				outStream << setw(12) << setprecision(6) << inMatrix[i][j] << setprecision(0) << setw(0) << " ";	
		}

		outStream << endl; 
	}
	
	outStream << flush;
}

//---------------------------------------------------------------------------
void PLEXApp::PrintMatrix3DLine(
	ostream&		outStream,
	const CharVector&	inStates,
	const DoubleMatrix3D&	inMatrix3D)
{
	unsigned int numRows = inMatrix3D.rows();
	
	for (int k = 0; k< numRows; k++)
	{
		DoubleMatrix tempMatrix = inMatrix3D[k];
		
		outStream << setw(12) << k; 

		PrintMatrixLine(outStream, inStates, tempMatrix);
	}
}

//---------------------------------------------------------------------------
void PLEXApp::PrintMatrixLine(
	ostream&		outStream,
	const CharVector&	inStates,
	const IntMatrix&	inMatrix)
{

	if (outStream)
	{
		unsigned int numRows = inMatrix.rows();
		
		unsigned int numCols = inMatrix.cols();
		
		for(unsigned int i=0; i<numRows; i++)
		{
			for(unsigned int j=0; j<numCols; j++)
				outStream << setw(12) << inMatrix[i][j] << setw(0) << " ";	
		}

		outStream << endl; 
	}
	
	outStream << flush;
}

//---------------------------------------------------------------------------
void PLEXApp::PrintVector(
			ostream&		outStream,
			ObservedData&	theData,
			const DoubleVector&	inVector)
{
	if (outStream)
	{
		outStream << setw(12) << " " << setw(0) << " ";
		
		PrintSequenceStates(outStream, theData);
		
		unsigned int numCols = inVector.size();
		
		outStream << setw(12) << " " << setw(0) << " ";	

		for(unsigned int i=0; i<numCols; i++)
			outStream << setw(12) << setprecision(6) << inVector[i] << setw(0) << setprecision(0) << " ";	
				
		outStream << endl; 
	}
	
	outStream << flush;
}

//---------------------------------------------------------------------------
void PLEXApp::PrintFreqVector(
			ostream&		outStream,
			const DoubleVector&	inVector)
{
	if (outStream)
	{
		unsigned int numCols = inVector.size();
		
		for(unsigned int i=0; i<numCols; i++)
			outStream << setw(8) << "Model" << setw(4) << i << setw(12) << setprecision(6) << inVector[i] << setprecision(0) << setw(0) << endl;	
				
		outStream << endl; 
	}
	
	outStream << flush;
}

//---------------------------------------------------------------------------
void PLEXApp::PrintVector(
			ostream&		outStream,
			ObservedData& theData,
			const IntVector&	inVector)
{
	if (outStream)
	{
		outStream << setw(12) << " " << setw(0) << " ";
		
		PrintSequenceStates(outStream, theData );
		
		unsigned int numCols = inVector.size();
		
		outStream << setw(12) << " " << setw(0) << " ";	
				
		for(unsigned int i=0; i<numCols; i++)
			outStream << setw(12) << inVector[i] << setw(0) << " ";	
				
		outStream << endl; 
	}
	
	outStream << flush;
}

//---------------------------------------------------------------------------
void PLEXApp::PrintIntVector(
			ostream&		outStream,
			const IntVector&	inVector)
{
	if (outStream)
	{
		unsigned int numCols = inVector.size();
		
		outStream << setw(12) << " " << setw(0) << " ";	
				
		for(unsigned int i=0; i<numCols; i++)
			outStream << setw(4) << inVector[i] << setw(0) << " ";	
				
		outStream << endl; 
	}
	
	outStream << flush;
}

//---------------------------------------------------------------------------
unsigned int
PLEXApp::SIndex(
	const char& state,
	const IntVector& inStateindex,
	const int& minstate)
{ 
	return (ispunct(state))
			?	inStateindex.size()
			:	inStateindex[static_cast<int>(state) - minstate]; 
}

/*
//---------------------------------------------------------------------------
void PLEXApp::PrintSamplingInternalnodes(
	ostream& outStream,
	const StringMatrix& InternalNodeSeq, 
	const IntVector& NodeID, 	
	const int& m_nSampleSequence, 
	ObservedData& theData,
	StringVector& AncestralNodeSeq)
{
	int len = InternalNodeSeq[0][0].size();
	int m_nNumStates = theData.getNumStates();
	
	for (unsigned int i = 0; i < NodeID.size(); i++) // each node
	{
		//output the node
		outStream << "Node#:" << setw(16) << i << setw(16) << " " << "NodeID" << setw(16) << NodeID[i] << endl;
		
		//output the state line
		outStream << setw(4) << " ";
		
		for (unsigned int j = 0; j < m_nNumStates; j++)
			outStream << setw(16) << theData.getState(j) << setw(0);
		
		outStream << endl;
		
		for (unsigned int j = 0; j < len; j++) //each site
		{
			IntVector count(m_nNumStates, 0);
			
			for (unsigned int k = 0; k < m_nSampleSequence; k++) //each sampling
			{
				int index = InternalNodeSeq[i][k][j];
				
				count[index]++;
			}
			
			outStream << setw(4) << j+1 << setw(0);
			
			double prob, maxprob = -1;
			
			int maxindex;
			
			for (unsigned int state = 0; state < m_nNumStates; state++)
			{
				//calculate the probability of each state on the site
				prob = (double) count[state] / (double) m_nSampleSequence;
				
				if (prob > maxprob)
				{
					maxprob = prob;
					maxindex = state;
				}
					
				//output the probability of each state on that site
				outStream << setw(16) << setprecision(3) << prob << setw(0) << setprecision(0);
			}
			
			if (maxprob >= 0.90)
			{
				AncestralNodeSeq[i][j] = inStates[maxindex];
				outStream << setw(16) << inStates[maxindex];
			}
			else
			{
				AncestralNodeSeq[i][j] = '?';
				outStream << setw(16) << "?";
			}
			
			outStream << endl;
		}
	}
}
 
 */

//---------------------------------------------------------------------------
void PLEXApp::PrintSitePosteriorProb(
	ostream& 		outStream, 
	const DoubleMatrix& 	m_matModelProb)
{
	unsigned int numSites = m_matModelProb.rows();
	
	unsigned int numModels = m_matModelProb.cols();
	
	outStream << setw(20) << "Site" << setw(0);
	
	for (unsigned int i = 0; i< numModels; i++)
		outStream << setw(20) << "Model"<< setw(0) << i+1;
		
	outStream << endl;

	for (unsigned int i = 0; i< numSites; i++)
	{
		outStream << setw(20) << i+1 << setw(0);
		
		for (unsigned int j = 0; j< numModels; j++)
			outStream << setw(20) << setprecision(3) << m_matModelProb[i][j] << setw(0) << setprecision(0);
			
		outStream << endl;
	}
}
		
//---------------------------------------------------------------------------
void PLEXApp::PrintSequencePosteriorProb(
	ostream& 		outStream, 
	const DoubleMatrix& 	m_matModelProb)
{
	unsigned int numSites = m_matModelProb.rows();
	
	unsigned int numModels = m_matModelProb.cols();
	
	outStream << setw(20) << "Sequence" << setw(0);
	
	for (unsigned int i = 0; i< numModels; i++)
		outStream << setw(20) << "Model"<< setw(0) << i+1;
		
	outStream << endl;

	for (unsigned int i = 0; i< numSites; i++)
	{
		outStream << setw(20) << i+1 << setw(0);
		
		for (unsigned int j = 0; j< numModels; j++)
			outStream << setw(20) << setprecision(3) << m_matModelProb[i][j] << setw(0) << setprecision(0);
			
		outStream << endl;
	}
}

//---------------------------------------------------------------------------
void PLEXApp::PrintModelPosteriorProb(
	ostream& 		outStream, 
	const IntMatrix& 	m_matModelAssignment,
	const int&		m_nNumSamples,
	const int& 		m_nNumModels,
	const bool& 		m_bSite)
{
	unsigned int m_nSequences = m_matModelAssignment.rows();
	
	DoubleMatrix m_matModelProb(m_nSequences, m_nNumModels, 0);

	for (unsigned int i = 0; i< m_nSequences; i++)
	{
		IntVector m_nModelCount(m_nNumModels, 0);
		
		for (unsigned int j = 0; j < m_nNumSamples; j++)
			m_nModelCount[m_matModelAssignment[i][j]]++;
			
		for (unsigned int j = 0; j < m_nNumModels; j++)
			m_matModelProb[i][j] = (double) m_nModelCount[j] / (double) m_nNumSamples;
	}
	
	if (options.m_bSite)
		outStream << setw(20) << "Site" << setw(0);
	else
		outStream << setw(20) << "Sequence" << setw(0);
	
	for (unsigned int i = 0; i< m_nNumModels; i++)
		outStream << setw(20) << "Model"<< setw(0) << i+1;
		
	outStream << endl;

	for (unsigned int i = 0; i< m_nSequences; i++)
	{
		outStream << setw(20) << i+1 << setw(0);
		
		for (unsigned int j = 0; j< m_nNumModels; j++)
			outStream << setw(20) << setprecision(3) << m_matModelProb[i][j] << setw(0) << setprecision(0);
			
		outStream << endl;
	}
}

//---------------------------------------------------------------------------
void PLEXApp::PrintSiteAssignment(
	ostream&		outStream, 
	const IntVector& 	m_vecAssignment)
{
	unsigned int numSites = m_vecAssignment.size();
	
	outStream << setw(20) << "Site" << setw(20) << "ModelAssignment" << setw(0) << endl;

	for (unsigned int i = 0; i< numSites; i++)
		outStream << setw(20) << i+1 << setw(20) << m_vecAssignment[i] << setw(0) << endl;
}

//---------------------------------------------------------------------------
void PLEXApp::PrintSequenceAssignment(
	ostream&		outStream, 
	const IntVector& 	m_vecAssignment)
{
	unsigned int numSites = m_vecAssignment.size();
	
	outStream << setw(20) << "Sequence" << setw(20) << "ModelAssignment" << setw(0) << endl;

	for (unsigned int i = 0; i< numSites; i++)
		outStream << setw(20) << i+1 << setw(20) << m_vecAssignment[i] << setw(0) << endl;
}

