/*
 *  RandomGenerator.cpp
 *  CLAMP
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */

#include "RandomGenerator.h"
#include <cmath>

#pragma mark -
//---------------------------------------------------------------------------
RandomGenerator::RandomGenerator(void)
	:	mW(cWDefault),
		mX(cXDefault),
		mY(cYDefault),
		mZ(cZDefault)
{
}

//---------------------------------------------------------------------------
double
RandomGenerator::Random(void)
{
	if (cFastRandomNumber)
	{
		mW *= 127773;
		return ldexp((double)mW, -32);
	}
	else
	{
		// U(0,1): AS 183: Appl. Stat. 31:188-190 
		// Wichmann BA & Hill ID.  1982.  An efficient and portable
		// pseudo-random number generator.  Appl. Stat. 31:188-190
		//
		//	x, y, z are any numbers in the range 1-30000.  Integer operation up
		//	to 30323 required.

		mX = 171 * (mX % 177) -  2 * (mX / 177);
		mY = 172 * (mY % 176) - 35 * (mY/ 176);
		mZ = 170 * (mZ % 178) - 63 * (mZ / 178);

		// if (mX<0)
		// 	mX+=30269;
		// if (mY<0)
		// 	mY+=30307;
		// if (mZ<0)
		// 	mZ+=30323;

		double r = mX / 30269.0 + mY / 30307.0 + mZ / 30323.0;
		return (r - static_cast<int>(r));
	}
}

//---------------------------------------------------------------------------
void
RandomGenerator::SetSeed(
	int	inSeed)
{
   mZ = 170 * (inSeed % 178) + 137;
   mW = inSeed;
}

//---------------------------------------------------------------------------
RandomGenerator&
RandomGenerator::GetGlobalGenerator(void)
{
	static RandomGenerator globalGenerator;
	return globalGenerator;
}

