/*
 *  ReadInputs.cpp
 *  CLAMP
 *
 *  Created by Wanjun Gu.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */

#include "ReadInputs.h"
#include "macrodefine.h"
#include "Options.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <fstream>
#include <iostream>
#include <string>
#include <iomanip>
#include <sstream>

using std::istream;
using std::ifstream;
using std::ofstream;
using std::cerr;
using std::endl;
using std::setw;
using std::setfill;
using std::setprecision;

#pragma mark -

extern Options options;
//---------------------------------------------------------------------------
ReadInputs::ReadInputs(void)
{
}

const int ReadInputs::m_nNucleotides = 4;
const int ReadInputs::m_nAminoacids = 20;
const int ReadInputs::m_nCodons = 64;
//---------------------------------------------------------------------------
void ReadInputs::SetParam(const char param[MAXNAME],const char value[MAXNAME]) {

	if (!strcmp(param, "datatype")) {
		options.m_nDatatype=atoi(value);
		options.datatype = atoi(value); // Jason - backup the datatype to global object
	} else if (!strcmp(param, "debug")) 
		options.Debugging=atoi(value);
	else if (!strcmp(param,"loudness")) 
		options.m_bLoudness=atoi(value);
	else if (!strcmp(param,"seed"))			// Jason
		options.randseed = atoi(value);
	else if (!strcmp(param,"treefile"))
		strcpy(options.m_strTreefile,value);
	else if (!strcmp(param,"seqfile"))
		strcpy(options.m_strSeqfile,value); 
	else if (!strcmp(param,"positionfile"))
		strcpy(options.m_strPositionfile,value); 
	else if (!strcmp(param,"seqlengthfile"))
		strcpy(options.m_strLengthfile,value); 
	else if (!strcmp(param,"olpositionfile"))
		strcpy(options.m_strOLfile,value); 
	else if (!strcmp(param,"treeoutfile"))
		strcpy(options.m_strTreeOutfile,value);
	else if (!strcmp(param,"seqoutfile"))
		strcpy(options.m_strSeqOutfile,value); 
	else if (!strcmp(param,"countoutfile"))
		strcpy(options.m_strCountOutfile,value);
	else if (!strcmp(param,"likelihoodoutfile"))
		strcpy(options.m_strLikelihoodOutfile,value); 
	else if (!strcmp(param,"matrixoutfile"))
		strcpy(options.m_strMatrixOutfile,value); 
	else if (!strcmp(param,"testupdate")) 
		options.m_bUpdate=atoi(value);
	else if (!strcmp(param,"updatewindow")) 
		options.m_fUpdatewindow=atof(value);
	else if (!strcmp(param,"outputincompletelogl"))			// Jason
		options.outputIncompleteLogL = atoi(value);
	else if (!strcmp(param,"dob1ustar"))			// Jason
		options.doB1uStar = atoi(value);
	else if (!strcmp(param,"hwang"))			// Jason
		options.HwangGreen = atoi(value);
	else if (!strcmp(param,"ASRV_update"))			// Jason
		options.updateASRVparms = atoi(value);
	else if (!strcmp(param,"coevolve"))			// Jason
		options.coevol = atoi(value);
	else if (!strcmp(param,"coevolveNull"))			// Jason
		options.coevol_doPosteriorPredictiveNull = atoi(value);
	else if (!strcmp(param,"minNumSubs"))			// Jason
		options.coevol_minNumSubstitutions = atoi(value);
	else if (!strcmp(param,"brlenupdatefreq"))			// Jason
		options.brlenupdatefreq = atof(value);
	else if (!strcmp(param,"updatebls"))			// Jason
		options.updatebls = atoi(value);
	else if (!strcmp(param,"liktemp"))			// Jason
		options.__globalLikelihoodTemp = atof(value);
	else if (!strcmp(param,"outputSiteRates"))			// Jason
		options.outputSiteRates = atoi(value);
	else if (!strcmp(param,"ASRV"))			// Jason
		options.allowASRV = atoi(value);
	else if (!strcmp(param,"ASRV_numCat"))			// Jason
		options.numCatASRV = atoi(value);	
	else if (!strcmp(param,"ASRV_gammaShape"))			// Jason
		options.gammaShapeASRV = atof(value);
	else if (!strcmp(param,"maxbl"))			// Jason
		options.maxBL=atof(value);
	else if (!strcmp(param,"numbls")) {			// Jason
		options.numDiscreteBLs=atoi(value);
		options.discreteBLs.resize(options.numDiscreteBLs);
	}
	else if (!strcmp(param,"ancstateupdatefreq"))			// Jason
		options.ancStateUpdateFreq=atof(value);
	else if (!strcmp(param,"forcefullupdater"))				// Jason
		options.forceFullMarginalUpdate=atoi(value);
	else if (!strcmp(param,"outputFrequency"))				// Jason
		options.outputFrequency=atoi(value);
	else if (!strcmp(param,"bls")) { 
        std::istringstream Result(value);
        string result;
        int i=0;
        while ( getline( Result, result, ',' ) ) {
			options.discreteBLs.at(i) = atof(result.c_str());
			i++;            
        }
	}
	else if (!strcmp(param,"rarmodel")) {			// Jason
		options.doRAR=atoi(value);
	}
	else if (!strcmp(param,"DRA")) {			// Jason
		options.DRA=atoi(value);
	}
	else if (!strcmp(param,"DRAprior")) {			// Jason
		options.DRAprior=atof(value);
	}
	else if (!strcmp(param,"numcat")) {				// Jason
		options.numCat=atoi(value);
	}
	else if (!strcmp(param,"freefreqs")) {			// Jason
		options.freeFreqs=atoi(value);
	}
	else if (!strcmp(param,"fixedrates")) {			// Jason
		if ( atoi(value) == 0 ) {
			options.fixedRates = -1;
		} else {
			options.fixedRates = atoi(value) - 1;
		}
	}
	else if (!strcmp(param,"generations")) { 
		options.m_nGenerations=atoi(value);
		options.numGenerations = options.m_nGenerations;
	} else if (!strcmp(param,"numthreads")) { 
		options.numThreads = atoi(value);
	} else if (!strcmp(param,"burnin")) 
		options.m_nBurnin=atoi(value);
	else if (!strcmp(param,"seqfreq")) 
		options.m_nSeqfreq=atoi(value);
	else if (!strcmp(param,"matfreq")) 
		options.m_nMatfreq=atoi(value);
	else if (!strcmp(param,"probfreq")) 
		options.m_nProbfreq=atoi(value);
	else if (!strcmp(param,"countfreq")) 
		options.m_nCountfreq=atoi(value);
	else if (!strcmp(param,"outtreeflag")) 
		options.m_bTreeflag=atoi(value);
	else if (!strcmp(param,"outseqsflag"))
		options.m_bSeqflag=atoi(value); 
	else if (!strcmp(param,"outmatsflag"))
		options.m_bMatflag=atoi(value);
	else if (!strcmp(param,"outcountflag")) 
		options.m_bCountflag=atoi(value);
	else if (!strcmp(param,"outlikesflag")) 
		options.m_bLikeflag=atoi(value);
	else if (!strcmp(param,"probmethod"))
		options.m_nMethod = atoi(value);
	else if (!strcmp(param,"updatepercentage")) 
		options.m_nWindowThreshold=atoi(value);
	else if (!strcmp(param,"totalwindowsize")) 
		options.m_nTotalWindow=atoi(value);
	else if (!strcmp(param,"maxwindowsize")) 
		options.m_nMaxWindow=atoi(value);
	else if (!strcmp(param,"simgeneration")) 
		options.m_nSimulationGeneration=atoi(value);
	else if (!strcmp(param, "fixedfreq"))
		options.m_bFixedFreq = atoi(value);
	else if (!strcmp(param, "classnumber"))
		options.m_nNumModels = atoi(value);
	else if (!strcmp(param,"site"))
		options.m_bSite = atoi(value); 
}

//---------------------------------------------------------------------------
void ReadInputs::readsets(CharVector& m_dParams, const char value[MAXNAME])
{
	//parameters are divided by ','
	int i = 0;
	
//	int j = 0;
		
//	int k = 0;
		
	while (value[i] != '\0')
	{
		if (value[i] != ',')
			m_dParams.push_back(value[i]);
			
		i++;
	}
}

//---------------------------------------------------------------------------
void ReadInputs::readsets(DoubleVector& m_dvParams, const char value[MAXNAME])
{
	//parameters are divided by ','
	char* pchTemp = (char*) malloc(sizeof(char) * 20);
	
	int i = 0;
	
	int j = 0;
		
	int k = 0;
		
	while (value[i] != '\0')
	{
		if (value[i] != ',')
			pchTemp[j] = value[i];
		else
		{
			pchTemp[j] = '\0';
			m_dvParams[k] = atof(pchTemp);
			j = -1;
			k++;
		}
		
		j++;
	
		i++;
	}
	
	pchTemp[j] = '\0';
	
	m_dvParams[k] = atof(pchTemp);
}

//---------------------------------------------------------------------------
void ReadInputs::readsets(DoubleVector& m_dvParams, const char value[MAXNAME], const int& m_numModels)
{
	if (m_dvParams.empty())
		m_dvParams.resize(m_numModels, -1);
		
	//parameters are divided by ','
	char* pchTemp = (char*) malloc(sizeof(char) * 20);
	
	int i = 0;
	
	int j = 0;
		
	int k = 0;
		
	while (value[i] != '\0')
	{
		if (value[i] != ',')
			pchTemp[j] = value[i];
		else
		{
			pchTemp[j] = '\0';
			m_dvParams[k] = atof(pchTemp);
			j = -1;
			k++;
		}
		
		j++;
	
		i++;
	}
	
	pchTemp[j] = '\0';
	
	m_dvParams[k] = atof(pchTemp);
}

//---------------------------------------------------------------------------
void ReadInputs::readsets(DoubleMatrix3D& m_dvParams, const char value[MAXNAME], const int& m_numModels, const int& m_numStates)
{
	if (m_dvParams.empty())
		m_dvParams.resize(m_numModels, m_numStates, m_numStates, -100);
	
	DoubleVector m_dvTemp(m_numModels * m_numStates * m_numStates, -100);
		
	//parameters are divided by ','
	char* pchTemp = (char*) malloc(sizeof(char) * 20);
	
	int i = 0;
	
	int j = 0;
		
	int k = 0;
		
	while (value[i] != '\0')
	{
		if (value[i] != ',')
			pchTemp[j] = value[i];
		else
		{
			pchTemp[j] = '\0';
			m_dvTemp[k] = atof(pchTemp);
			j = -1;
			k++;
		}
		
		j++;
	
		i++;
	}
	
	pchTemp[j] = '\0';
	
	m_dvTemp[k] = atof(pchTemp);
	
	unsigned int l = 0;
	
	for (i = 0; i < m_numModels; i++)
	{
		for (j = 0; j< m_numStates; j++)
		{
			for (k = 0; k < m_numStates; k++)
			{
				m_dvParams[i][j][k] = m_dvTemp[l];
				
				l++;
			}
		}
	}
}


//---------------------------------------------------------------------------
void ReadInputs::WriteParams(ostream& osparam) {
	osparam << "Writing Parameters." << endl;
	osparam << endl << "# debug and loudness controls #" << endl;
	osparam << "\tdebug" << setw(20) << options.Debugging << endl;
	osparam << "\tloudness" << setw(20) << options.m_bLoudness << endl;

	osparam << endl << "# control generations, frequency of sampling #" << endl;
	osparam << "\tgenerations" << setw(20) << options.m_nGenerations << endl;
	osparam << "\tburnin" << setw(20) << options.m_nBurnin << endl;
	osparam << "\tseqfreq" << setw(20) << options.m_nSeqfreq << endl;
	osparam << "\tmatfreq" << setw(20) << options.m_nMatfreq << endl;
	osparam << "\tprobfreq" << setw(20) << options.m_nProbfreq << endl;
	osparam << "\tcountfreq" << setw(20) << options.m_nCountfreq << endl;

	osparam << endl << "# options for simulation to get proposal step in MCMC #" << endl;
	
	osparam << "\ttestupdate" << setw(20) << options.m_bUpdate << endl;
	osparam << "\tupdatewindow" << setw(20) << options.m_fUpdatewindow << endl;
	
	osparam << "\tupdatepercentage" << setw(20) << options.m_nWindowThreshold << endl;
	osparam << "\ttotalwindowsize" << setw(20) << options.m_nTotalWindow << endl;
	osparam << "\tmaxwindowsize" << setw(20) << options.m_nMaxWindow << endl;
	osparam << "\tsimgeneration" << setw(20) << options.m_nSimulationGeneration << endl;

	osparam << endl << "# probablity calculation method #" << endl;
	osparam << "\tprobmethod" << setw(20) << options.m_nMethod << endl;

	osparam << endl << "# optional input file names #" << endl;
	osparam << "\ttreefile" << setw(20) << options.m_strTreefile << endl;
	osparam << "\tseqfile" << setw(20) << options.m_strSeqfile << endl;
	osparam << "\tpositionfile" << setw(20) << options.m_strPositionfile << endl;
	osparam << "\tseqlengthfile" << setw(20) << options.m_strLengthfile << endl;
	osparam << "\tolpositionfile" << setw(20) << options.m_strOLfile << endl;

	osparam << endl << "# optional output file names #" << endl;
	osparam << "\ttreeoutfile" << setw(20) << options.m_strTreeOutfile << endl;
	osparam << "\tseqoutfile" << setw(20) << options.m_strSeqOutfile << endl;
	osparam << "\tcountoutfile" << setw(20) << options.m_strCountOutfile << endl;
	osparam << "\tmatrixoutfile" << setw(20) << options.m_strMatrixOutfile << endl;
	osparam << "\tlikelihoodoutfile" << setw(20) << options.m_strLikelihoodOutfile << endl;

	osparam << endl << "# model setting #" << endl;
	osparam << "\tFixedFreq" << setw(20) << options.m_bFixedFreq << endl;
	osparam << "\tModelNum" << setw(20) << options.m_nNumModels << endl;
	osparam << "\tMixtureSite" << setw(20) << options.m_bSite << endl;

	osparam << endl << "# print to file flags #" << endl;
	osparam << "\touttreeflag" << setw(20) << options.m_bTreeflag << endl;
	osparam << "\toutseqsflag" << setw(20) << options.m_bSeqflag << endl;
	osparam << "\toutmatsflag" << setw(20) << options.m_bMatflag << endl;
	osparam << "\toutcountflag" << setw(20) << options.m_bCountflag << endl;
	osparam << "\toutlikesflag" << setw(20) << options.m_bLikeflag << endl;
}

/*
//---------------------------------------------------------------------------
void ReadInputs::WriteParams(ostream& osparam, const bool& Debugging, const bool& m_bLoudness, const char* m_strSeqfile, const char* m_strLikelihoodOutfile, const char* m_strMatrixOutfile,
	const bool& m_bUpdate, const double& m_fUpdatewindow, const int& m_nGenerations, const int& m_nBurnin, const int& m_nMatfreq, const int& m_nProbfreq, const int& m_nMethod, const int& m_nModel, 
	const bool& m_bMatflag, const bool& m_bLikeflag, const int& m_nWindowThreshold, const int& m_nTotalWindow, const int& m_nMaxWindow, const int&
	m_nSimulationGeneration, const int& m_nNumModels, const bool& m_bFixedFreq, const bool& m_bSite)
{
	osparam << "Writing Parameters." << endl;
	osparam << endl << "# debug and loudness controls #" << endl;
	osparam << "\tdebug" << setw(20) << Debugging << endl;
	osparam << "\tloudness" << setw(20) << m_bLoudness << endl;

	osparam << endl << "# control generations, frequency of sampling #" << endl;
	osparam << "\tgenerations" << setw(20) << m_nGenerations << endl;
	osparam << "\tburnin" << setw(20) << m_nBurnin << endl;
	osparam << "\tmatfreq" << setw(20) << m_nMatfreq << endl;
	osparam << "\tprobfreq" << setw(20) << m_nProbfreq << endl;

	osparam << endl << "# options for simulation to get proposal step in MCMC #" << endl;
	
	osparam << "\ttestupdate" << setw(20) << m_bUpdate << endl;
	osparam << "\tupdatewindow" << setw(20) << m_fUpdatewindow << endl;
	
	osparam << "\tupdatepercentage" << setw(20) << m_nWindowThreshold << endl;
	osparam << "\ttotalwindowsize" << setw(20) << m_nTotalWindow << endl;
	osparam << "\tmaxwindowsize" << setw(20) << m_nMaxWindow << endl;
	osparam << "\tsimgeneration" << setw(20) << m_nSimulationGeneration << endl;

	osparam << endl << "# probablity calculation method #" << endl;
	osparam << "\tprobmethod" << setw(20) << m_nMethod << endl;

	osparam << endl << "# model selection #" << endl;
	osparam << "\tModel" << setw(20) << m_nModel << endl;
	osparam << "\tFixedFreq" << setw(20) << m_bFixedFreq << endl;
	osparam << "\tModelNum" << setw(20) << m_nNumModels << endl;
	osparam << "\tMixtureSite" << setw(20) << m_bSite << endl;

	osparam << endl << "# optional input file names #" << endl;
	osparam << "\tpairwiseseqfile" << setw(20) << m_strSeqfile << endl;

	osparam << endl << "# optional output file names #" << endl;
	osparam << "\tmatrixoutfile" << setw(20) << m_strMatrixOutfile << endl;
	osparam << "\tlikelihoodoutfile" << setw(20) << m_strLikelihoodOutfile << endl;

	osparam << endl << "# print to file flags #" << endl;
	osparam << "\toutmatsflag" << setw(20) << m_bMatflag << endl;
	osparam << "\toutlikesflag" << setw(20) << m_bLikeflag << endl;
}

//---------------------------------------------------------------------------
void ReadInputs::WriteParams(ostream& osparam, const int& m_nDatatype, const DoubleVector& m_dvFrequencies, const DoubleVector& m_dvRates, const bool& m_bAncestralSet, 
	const char* m_strAncSeqfile, const char* m_strInputTreefile, const int& m_nLength, const bool& m_bWriteAncestral, const char* m_strOutSeqfile, const bool&
	m_bMutation, const char* m_strMutatinfile)
{
	osparam << "Writing Parameters." << endl;
	osparam << endl << "# type of sequence #" << endl;
	osparam << "\tdatatype" << setw(20) << m_nDatatype << endl;

	osparam << endl << "# frequencies of the characters #" << endl;

	osparam << "\tfrequencies";

	for (int i = 0; i < m_dvFrequencies.size(); i++)
		osparam << setprecision(3) << setw(10) << m_dvFrequencies[i];

	osparam << endl;
	
	osparam << "\trates";
	for (int i = 0; i < m_dvRates.size(); i++)
		osparam << setprecision(3) << setw(10) << m_dvRates[i];
	osparam << endl;

	osparam << endl << "# optional input file with ancestral sequence and tree file #" << endl;
	osparam << "\tancestral" << setw(20) << m_bAncestralSet << endl;
	
	if (m_bAncestralSet)
		osparam << "\tancseqfile" << setw(20) << m_strAncSeqfile << endl;
	else
		osparam << "\tlength" << setw(20) << m_nLength << endl;
		
	osparam << "\tsimtreefile" << setw(20) << m_strInputTreefile << endl;
	
	osparam << endl << "# options of output of the simulation #" << endl;

	osparam << "\twriteancestral" << setw(20) << m_bWriteAncestral << endl;
	osparam << "\toutseqfile" << setw(20) << m_strOutSeqfile << endl;
	osparam << "\twritemutation" << setw(20) << m_bMutation << endl;
	osparam << "\toutmutationfile" << setw(20) << m_strMutatinfile << endl;
}


//---------------------------------------------------------------------------
void ReadInputs::WriteParams(ostream& osparam, const bool& Debugging, const bool& m_bLoudness, const char* m_strTreefile, const char* m_strSeqfile, const int& m_nCodonposition,
			const char* m_strTreeOutfile, const char* m_strSeqOutfile, const char* m_strCountOutfile, const char* m_strLikelihoodOutfile, const char* m_strMatrixOutfile,
			const bool& m_bUpdate, const double& m_fUpdatewindow, const int& m_nGenerations, const int& m_nBurnin, const int& m_nSeqfreq, const int& m_nMatfreq, const int& m_nProbfreq, const int& m_nCountfreq,
			const int& m_nMethod, const bool& m_bTreeflag, const bool& m_bSeqflag, const bool& m_bMatflag, const bool& m_bCountflag, const bool& m_bLikeflag,
			const int& m_nWindowThreshold, const int& m_nTotalWindow, const int& m_nMaxWindow, const int& m_nSimulationGeneration, const int& m_nNumModels,
			const bool& m_bFixedFreq, const bool& m_bSite)
{
	osparam << "Writing Parameters." << endl;
	osparam << endl << "# debug and loudness controls #" << endl;
	osparam << "\tdebug" << setw(20) << Debugging << endl;
	osparam << "\tloudness" << setw(20) << m_bLoudness << endl;

	osparam << endl << "# control generations, frequency of sampling #" << endl;
	osparam << "\tgenerations" << setw(20) << m_nGenerations << endl;
	osparam << "\tburnin" << setw(20) << m_nBurnin << endl;
	osparam << "\tseqfreq" << setw(20) << m_nSeqfreq << endl;
	osparam << "\tmatfreq" << setw(20) << m_nMatfreq << endl;
	osparam << "\tprobfreq" << setw(20) << m_nProbfreq << endl;
	osparam << "\tcountfreq" << setw(20) << m_nCountfreq << endl;

	osparam << endl << "# options for simulation to get proposal step in MCMC #" << endl;
	
	osparam << "\ttestupdate" << setw(20) << m_bUpdate << endl;
	osparam << "\tupdatewindow" << setw(20) << m_fUpdatewindow << endl;
	
	osparam << "\tupdatepercentage" << setw(20) << m_nWindowThreshold << endl;
	osparam << "\ttotalwindowsize" << setw(20) << m_nTotalWindow << endl;
	osparam << "\tmaxwindowsize" << setw(20) << m_nMaxWindow << endl;
	osparam << "\tsimgeneration" << setw(20) << m_nSimulationGeneration << endl;

	osparam << endl << "# probablity calculation method #" << endl;
	osparam << "\tprobmethod" << setw(20) << m_nMethod << endl;

	osparam << endl << "# optional input file names #" << endl;
	osparam << "\ttreefile" << setw(20) << m_strTreefile << endl;
	osparam << "\tseqfile" << setw(20) << m_strSeqfile << endl;

	osparam << endl << "# model setting #" << endl;
	osparam << "\tFixedFreq" << setw(20) << m_bFixedFreq << endl;
	osparam << "\tModelNum" << setw(20) << m_nNumModels << endl;
	osparam << "\tMixtureSite" << setw(20) << m_bSite << endl;

	osparam << endl << "# optional output file names #" << endl;
	osparam << "\ttreeoutfile" << setw(20) << m_strTreeOutfile << endl;
	osparam << "\tseqoutfile" << setw(20) << m_strSeqOutfile << endl;
	osparam << "\tcountoutfile" << setw(20) << m_strCountOutfile << endl;
	osparam << "\tmatrixoutfile" << setw(20) << m_strMatrixOutfile << endl;
	osparam << "\tlikelihoodoutfile" << setw(20) << m_strLikelihoodOutfile << endl;

	osparam << endl << "# print to file flags #" << endl;
	osparam << "\touttreeflag" << setw(20) << m_bTreeflag << endl;
	osparam << "\toutseqsflag" << setw(20) << m_bSeqflag << endl;
	osparam << "\toutmatsflag" << setw(20) << m_bMatflag << endl;
	osparam << "\toutcountflag" << setw(20) << m_bCountflag << endl;
	osparam << "\toutlikesflag" << setw(20) << m_bLikeflag << endl;
	
	osparam << endl << "# codon position #" << endl;
	osparam << "\tcodonposition" << setw(20) << m_nCodonposition << endl;
}

//---------------------------------------------------------------------------
void ReadInputs::WriteParams(ostream& osparam, const char* m_strSeqfile, const char* m_strModelfile, const char* m_strAssignmentfile, const bool& m_bSite, const int& m_nMethod)
{
	osparam << endl << "# control parameters for model posterior probability calculation #" << endl;
	osparam << "\tpairwiseseqfile" << setw(20) << m_strSeqfile << endl;
	osparam << "\tmodelfile" << setw(20) << m_strModelfile << endl;
	osparam << "\tassignmentfile" << setw(20) << m_strAssignmentfile << endl;
	osparam << "\tsite" << setw(20) << m_bSite << endl;
	osparam << "\tprobmethod" << setw(20) << m_nMethod << endl;
}
 */

//---------------------------------------------------------------------------
bool ReadInputs::ReadControlfile(const char* controlfile) {
	int comment,stop,count,step=0;
	char word[MAXNAME]="";
	char lastword[MAXNAME]="";
	
//	std::cout << "THIS ONE!" << endl; exit(-1);
	options.m_nDatatype = -1;
	
	FILE* fp = fopen(controlfile, "rb");
	
	if (fp != NULL)
	{
		cerr << "Now reading control file......." << endl << endl;	
		
		for(fscanf(fp,"%s", word),comment=0,count=0,stop=0;stop==0;fscanf(fp,"%s", word))
		{
			count++;
			
			if(word[0]=='#')
			{
				comment=1-comment;

				if (comment==0)
					step=0;
			}
			else if (!comment)
			{
				if (step==0)
				{
					SetParam(lastword, word);
				} 
				if( ! strcmp(word,ENDWORD))
				{ 
					stop=1; 
				}
				else 
					strcpy(lastword,word);
			}

			step=1-step;
		} 

		WriteParams(cerr);
		
		cerr << "Read ancestral reconstruction control file done." << endl << endl;
		
		fclose(fp);
		
		return(true);
	}
	else
		return(false);
}


//---------------------------------------------------------------------------
/*
 bool ReadInputs::ReadControlfile(const char* controlfile, char* m_strSeqfile, char* m_strModelfile, char* m_strAssignmentfile, bool& m_bSite, int& m_nMethod)
{
	int comment,stop,count,step=0;
	char word[MAXNAME]="";
	char lastword[MAXNAME]="";
	
	FILE* fp = fopen(controlfile, "rb");
	
	if (fp != NULL)
	{
		cerr << "Now reading control file......." << endl << endl;	
		
		for(fscanf(fp,"%s", word),comment=0,count=0,stop=0;stop==0;fscanf(fp,"%s", word))
		{
			count++;
			
			if(word[0]=='#')
			{
				comment=1-comment;

				if (comment==0)
					step=0;
			}
			else if (!comment)
			{
				if (step==0)
				{
					SetParam(lastword, word, m_strSeqfile, m_strModelfile, m_strAssignmentfile, options.m_bSite, options.m_nMethod);
				} 
				if( ! strcmp(word,ENDWORD))
				{ 
					stop=1; 
				}
				else 
					strcpy(lastword,word);
			}

			step=1-step;
		} 

		WriteParams(cerr, m_strSeqfile, m_strModelfile, m_strAssignmentfile, options.m_bSite, options.m_nMethod);
		
		cerr << "Read probability calculation control file done." << endl << endl;
		
		fclose(fp);
		
		return(true);
	}
	else
		return(false);
}
*/

//---------------------------------------------------------------------------
char ReadInputs::GetMarkedLine(
	string&			outLine,
	string::size_type	max,
	FILE*			fp,
	char			inStartMarker,
	char			inEndMarker)
{
	char	marker = 'X';
	
	outLine.clear();
	outLine.reserve(max);
	
	int	temp; 
	
	while (EOF != (temp = getc(fp)))
	{
		if (temp != ELN && temp != ELN2)
		{
			if (outLine.size() == max)
				break;
				
			outLine.push_back(temp);
			if (temp == inStartMarker)
				marker = temp;
			else if (temp == inEndMarker)
			{
				marker = temp;
				return marker;
			}
		}
		else break;
	}
	
	if (temp == EOF)
		marker = EOF;
	return marker;
}

//---------------------------------------------------------------------------
// getln written similarly to fgets, but works on MACs
// problem with mixed carriage return characters going from PC/Unix to Mac
char ReadInputs::getln(
	string&			outLine,
	string::size_type	max,
	FILE*			fp)
{
	return GetMarkedLine(outLine, max, fp, NEWSEQ, ENDSEQ);
}

//---------------------------------------------------------------------------
char ReadInputs::getlnpos(
	string&			outLine,
	string::size_type	max,
	FILE*			fp)
{
	return GetMarkedLine(outLine, max, fp, NEWSEQ, -1);
}

//---------------------------------------------------------------------------
void ReadInputs::GetName(
	StringVector&		outNames,
	int			seq, 
	const string&		inLine, 
	int			type)
{
	char nameMarker = (type == PIR) ? PNAMESIG : FNAMESIG;
	string::size_type	startOfName = inLine.find(nameMarker);

	if (startOfName != string::npos)
	{
		if (++startOfName <= inLine.size())
		{
			string::size_type endOfName = inLine.find_first_of(cWhitespaceChars, startOfName);

			if (endOfName == string::npos)
				endOfName = inLine.size();

			string name = inLine.substr(startOfName, endOfName - startOfName);
			
			//convert the upper case character to lower case character
			/*
			for (int i = 1; i < name.size(); i++)
			{
				if ((name[i] <= 'Z') && (name[i] >= 'A') )
					name[i] += 32;
			}
			*/
			if (name.size())
				outNames.push_back(name);
		}
	}
}

void ReadInputs::FindName(
	const StringVector&	outNames,
	int&			seq, 
	const string&		inLine, 
	int			type)
{
	char nameMarker = (type == PIR) ? PNAMESIG : FNAMESIG;
	string::size_type	startOfName = inLine.find(nameMarker);

	if (startOfName != string::npos)
	{
		if (++startOfName <= inLine.size())
		{
			string::size_type endOfName = inLine.find_first_of(cWhitespaceChars, startOfName);

			if (endOfName == string::npos)
				endOfName = inLine.size();

			string name = inLine.substr(startOfName, endOfName - startOfName);
			
			if (name.size())
			{
				int i; 
				
				for (i = 0; i < outNames.size(); i++)
				{
					if (outNames[i] == name)
					{
						seq = i;
						break;
					}
				}
				
				if ( i == outNames.size())
					seq = -1;
			}
		}
	}
}


/*
//---------------------------------------------------------------------------
void ReadInputs::AddToSeqs(
	Sequence&		outSeq,
	const string&		inLine)
{
	string::size_type	lineSize = inLine.size();
	for (string::size_type i = 0; i != lineSize; ++i)
	{ 
		char c = inLine[i];
		
		// put the sequence as uppercase
		if( isalnum(c) || ispunct(c) )
		{
			if (c >= 'a' && c <= 'z')
				c -= 32;
				
			outSeq.push_back(c);
		}
	}
}

//---------------------------------------------------------------------------
void ReadInputs::AddToSeqs(
	Sequence&		outSeq,
	const string&		inLine,
	const bool&		transformed)
{
	string::size_type	lineSize = inLine.size();
	for (string::size_type i = 0; i != lineSize; ++i)
	{ 
		char c = inLine[i];
		
		if( isalnum(c) || ispunct(c) )
		{
			if (transformed)
			{
				if ( (c == 'c') || (c == 't') || (c == 'C') || (c == 'T') )
					c = 'Y';
					
				if ( (c == 'a') || (c == 'g') || (c == 'A') || (c == 'G') )
					c = 'R';
			}
			
			outSeq.push_back(c);
		}
	}
}
*/

//---------------------------------------------------------------------------
void ReadInputs::AddToPoss(
	IntMatrix&		outPoss,
	const int&		seq,
	const string&		inLine)
{
  	unsigned int s;	
	for(s=0; outPoss[seq][s]!=-1; s++);

	int temp=0;
	int	counter=0;
	string::size_type	lineSize = inLine.size();
	for (string::size_type i = 0; i != lineSize; ++i)
	{
		if ( isalnum(inLine[i]) || ispunct(inLine[i]) )
		{
			if (counter == 0)
				temp=atoi(inLine.substr(i).c_str());
				
			counter++;
		}
		else
		{
			if(s<MAXLENGTH-1)
				outPoss[seq][s]=temp; 
			else
			{
				cerr << "overshoot exit. " << s << endl;
				exit(0);
			}

			s++;
			temp=0;
			counter=0;
		}
	}
	
	if(s<MAXLENGTH-1)
		outPoss[seq][s]=-1;
}

//---------------------------------------------------------------------------
void ReadInputs::AddToOL(
	IntVector&		outOL,
	const int&		seq, 
	const string&		inLine)
{
	int temp=0;

	if ( isalnum(inLine[0]) || ispunct(inLine[0]) )
		temp=atoi(inLine.c_str());
	
	if (temp != 0)
		outOL[seq]=temp; 
}

/*
//---------------------------------------------------------------------------
int ReadInputs::Read_Seqs(
	FILE* 			m_fSeqs,
	SequenceVector&		outSeqs,
	StringVector&		outSeqNames,
	int 			type)
{
	cerr << endl << "Reading Sequence file......." << endl << endl; 
	
	unsigned int startline = (type == PIR) ? 1 : 0;

	unsigned int linenum = 0;

	bool inseq = false;
	char linesig;
	SequenceVector::size_type seq = 0;
	string line;
	
	while (0 != (linesig=getln(line, MAXLINE, m_fSeqs)))
	{
		if(linesig==NEWSEQ)  
		{
			if (inseq) 
				seq++; 

			linenum	= 0; 
			inseq = true;
		} 
		else 
			linenum++;
		
		if (linenum == 0)
			GetName(outSeqNames, seq, line, type);
		else if(linenum>startline && inseq)
		{
			assert(seq <= outSeqs.size());
			
			if (seq == outSeqs.size())
				outSeqs.push_back(Sequence());
				
			AddToSeqs(outSeqs[seq], line);
		}

		if (linesig==EOF || line[0]==EOF)
		{
			cerr << "EOF." << endl; 
			seq++; 
			break; 
		}
	}
	
	int m_nNumseqs = seq;
	
	cerr << endl << "Done Reading seqfile with " << m_nNumseqs << " seqs."
		<< endl << endl; 
		
	return (m_nNumseqs);
}

//---------------------------------------------------------------------------
int ReadInputs::Read_Seqs(
	FILE* 			m_fSeqs,
	SequenceVector&		outSeqs,
	int 			type)
{
	cerr << endl << "Reading Sequence file......." << endl << endl; 
	
	unsigned int startline = (type == PIR) ? 1 : 0;

	unsigned int linenum = 0;

	bool inseq = false;
	char linesig;
	SequenceVector::size_type seq = 0;
	string line;
	
	while (0 != (linesig=getln(line, MAXLINE, m_fSeqs)))
	{
		if(linesig==NEWSEQ)  
		{
			if (inseq) 
				seq++; 

			linenum	= 0; 
			inseq = true;
		} 
		else 
			linenum++;
		
		if ((linenum != 0) && (linenum>startline && inseq))
		{
			assert(seq <= outSeqs.size());
			
			if (seq == outSeqs.size())
				outSeqs.push_back(Sequence());
				
			AddToSeqs(outSeqs[seq], line);
		}

		if (linesig==EOF || line[0]==EOF)
		{
			cerr << "EOF." << endl; 
			seq++; 
			break; 
		}
	}
	
	int m_nNumseqs = seq;
	
	cerr << endl << "Done Reading seqfile with " << m_nNumseqs << " seqs."
		<< endl << endl; 
		
	return (m_nNumseqs);
}


//---------------------------------------------------------------------------
void ReadInputs::Read_Poss(
	FILE*			m_fPoss,
	IntMatrix&		outPoss,
	const StringVector&	outSeqNames,
	int			type)
{
	int seq = 0, linenum = 0, inseq = 0, startline;
	string	line;
	char linesig;
	int count = 0;

	cerr << endl << "Reading position file......" << endl << endl; 

	if(type==PIR)
		startline=1;
	else startline=0; 

	while (0 != (linesig = getlnpos(line, MAXLINE, m_fPoss)) )
	{
		if(linesig==NEWSEQ)
		{
			linenum=0; 
			inseq=1; 
		} 
		else 
			linenum++;

		if (linenum == 0)
			FindName(outSeqNames,seq,line,type);
		
		if (seq >= 0)
		{
			if ((linenum != 0) && (linenum>startline && inseq))
			{
				AddToPoss(outPoss, seq, line);
				
				if (linenum == startline + 1)
					count++;   	 
			}
		}
			
		if (linesig==ENDSEQ)
			inseq=0;

		if (linesig==EOF || line[0]==EOF)
		{
			cerr << "EOF." << endl; 
			break; 
		}
	 }
 
	if (count < outSeqNames.size())
		throw(6);
	else
		cerr << endl << "Done reading position file......" 
			<< endl << endl; 
}

//---------------------------------------------------------------------------
void ReadInputs::Read_OL(
	FILE*			m_fOL,
	IntVector&		outOL,
	const StringVector&	outSeqNames,
	int			type)
{
	int seq,linenum = 0, inseq, startline = 0;
	string	line;
	char linesig;
	seq=0;inseq=0;
	int count = 0;

	cerr << endl << "Reading OL file or size file......" 
		<< endl << endl; 			

	while (0 != (linesig=getln(line, MAXLINE, m_fOL)) )
	{
		if(linesig==NEWSEQ)
		{
			linenum=0; 
			inseq=1; 
		} 
		else 
			linenum++;

		if (linenum == 0)
			FindName(outSeqNames, seq, line, type);

		if (seq >= 0)
		{
			if ((linenum != 0) && ( linenum > startline && inseq))
			{
				AddToOL(outOL, seq, line);
			
				if (linenum == startline + 1)
					count++;
			}
		}

		if (linesig==ENDSEQ)

		inseq=0;
		
		if (linesig==EOF || line[0]==EOF)
		{
			cerr << "EOF." << endl; 
			break; 
		}
	}
	
	if (count < outSeqNames.size())
		throw(7);
	else
		cerr << endl << "Done reading OL file or size file......" 
			<< endl << endl; 			
}
*/
//---------------------------------------------------------------------------

/*
 // OLD VERSION
int ReadInputs::ReadInputfiles(void) {
//	**********************************************
//	Read the input files to get the data: Sequences,
//	**********************************************
   	FILE* m_fSequence = fopen(options.m_strSeqfile, "rb");
	
	int m_nNumseqs; 
	
	if (m_fSequence != NULL)
		m_nNumseqs = Read_Seqs(m_fSequence, options.m_vecSequences, m_vecSeqname, FASTA);
	else
		throw(2);
	
//	**********************************************
//	Read Positions of each sequences, Lengths, OL 
//	of eachtaxa, which will be used to calculate the DSSH
//	**********************************************

//	**********************************************
//	resize the vectors to store the input data
//	**********************************************
//	if (m_matPositions.empty())
		m_matPositions.resize(m_nNumseqs, MAXLENGTH, -1);
	
	if (m_vecSize.empty())
		m_vecSize.resize(m_nNumseqs, -1);
	
	if (m_vecOL.empty())
		m_vecOL.resize(m_nNumseqs, -1);

	FILE* m_fPosition = fopen(m_strPositionfile, "rb");
	
	if (m_fPosition != NULL)
		Read_Poss(m_fPosition, m_matPositions, m_vecSeqname, FASTA);
	else
		throw(3);
	
	FILE* m_fLength = fopen(m_strLengthfile, "rb");
	
	if (m_fLength != NULL)
		Read_OL(m_fLength, m_vecSize, m_vecSeqname, FASTA);
	else
		throw(4);
	
	FILE* m_fOL = fopen(m_strOLfile, "rb");
	
	if (m_fOL != NULL)
		Read_OL(m_fOL, m_vecOL, m_vecSeqname, FASTA);
	else
		throw(5);
	
	fclose(m_fPosition);
	fclose(m_fLength);
	fclose(m_fOL);
//	

	fclose(m_fSequence);
	return(m_nNumseqs);
}




//---------------------------------------------------------------------------
bool ReadInputs::Read_AncSeq(
	FILE* 			m_fSeqs,
	std::string&		outSeqs)
{
//	unsigned int linenum = 0;

	char linesig;
	string line;

	outSeqs = "";
		
	while (0 != (linesig=getln(line, MAXLINE, m_fSeqs)))
	{
		if(linesig != NEWSEQ)  
			outSeqs += line;

		if (linesig==EOF || line[0]==EOF)
			break; 
	}
	
	return (outSeqs.size());
}

//---------------------------------------------------------------------------
bool ReadInputs::ReadAncSequence(
	const char* m_strAncSeqfile, 
	std::string& m_strAncSeq)		//read the file to get the ancestral sequence
{
   	FILE* m_fAncSequence = fopen(m_strAncSeqfile, "rb");
	
	bool read = Read_AncSeq(m_fAncSequence, m_strAncSeq);
	
	fclose(m_fAncSequence);
	
	return(read);
}

//---------------------------------------------------------------------------
int ReadInputs::ReadPairwiseSequence(const char* m_strSeqfile, vector< vector<int> > &m_vecSequences)
{
//	**********************************************
//	Read the input files to get the data: Sequences,
//	**********************************************
   	FILE* m_fSequence = fopen(m_strSeqfile, "rb");
	
	int m_nNumseqs; 
	
	if (m_fSequence != NULL)
		m_nNumseqs = Read_Seqs(m_fSequence, m_vecSequences, FASTA);
	else
		throw(2);
	
	fclose(m_fSequence);

	return(m_nNumseqs);
}

//---------------------------------------------------------------------------
void ReadInputs::parse4foldsitetransformed(
	SequenceVector&		outSeqs)
{
	// 1, parse the four fold conserved sites,
	// 2, transform the nucleotides on that site to R(purine) or Y (pyrimidine)

	assert( outSeqs[0].size() %3 == 0);
	
	bool conserved = true;
	
	StringVector tempStrings(outSeqs.size());
	
	for (string::size_type j = 0; j < tempStrings.size(); j++)
		tempStrings[j].clear();
	
	for (string::size_type j = 0; j < outSeqs[0].size()/3; j++)
	{
		conserved = true;
		
		for (string::size_type i = 0; i< outSeqs.size() && conserved; i++)
		{
			char first = outSeqs[i][3*j];
			
			char second = outSeqs[i][3*j+1];
			
			// four fold sites
			if ( (first == 'C' && second == 'C') || (first == 'c' && second == 'C') || (first == 'C' && second == 'c') || (first == 'c' && second == 'c')		// Pro
				|| (first == 'C' && second == 'G') || (first == 'c' && second == 'G') || (first == 'C' && second == 'g') || (first == 'c' && second == 'g') 	//Arg
				|| (first == 'A' && second == 'C') || (first == 'a' && second == 'C') || (first == 'A' && second == 'c') || (first == 'a' && second == 'c')	//Thr
				|| (first == 'G' && second == 'T') || (first == 'g' && second == 'T') || (first == 'G' && second == 't') || (first == 'g' && second == 't')	//Val
				|| (first == 'G' && second == 'C') || (first == 'g' && second == 'C') || (first == 'G' && second == 'c') || (first == 'g' && second == 'c')	//Ala
				|| (first == 'G' && second == 'G') || (first == 'g' && second == 'G') || (first == 'G' && second == 'g') || (first == 'g' && second == 'g') )	//Gly
				conserved = true;
			else
				conserved = false;
		}
		
		if (conserved)	//output the site to sequencevector;
		{
			bool valid = true;
			
			for (string::size_type i = 0; i< outSeqs.size() && valid; i++)
			{
				char c = outSeqs[i][3*j+2];
		
				if( isalnum(c) || ispunct(c) )
				{
					if ( (c == 'c') || (c == 't') || (c == 'C') || (c == 'T') || (c == 'a') || (c == 'g') || (c == 'A') || (c == 'G') )
						valid = true;
					else
						valid = false;
				}
			}
		
			if (valid)
			{
				for (string::size_type i = 0; i< outSeqs.size(); i++)
				{
					char c = outSeqs[i][3*j+2];
		
					if( isalnum(c) || ispunct(c) )
					{
						if ( (c == 'c') || (c == 't') || (c == 'C') || (c == 'T') )
						{
							c = 'Y';
							tempStrings[i].push_back(c);
						}
						else if ( (c == 'a') || (c == 'g') || (c == 'A') || (c == 'G') )
						{
							c = 'R';
							tempStrings[i].push_back(c);
						}
					}
				}
			}
		}
	}
	
	outSeqs.clear();
	
	for (int i = 0; i< tempStrings.size(); i++)
	{
		outSeqs.push_back(Sequence());

		AddToSeqs(outSeqs[i], tempStrings[i]);
	}
}

//---------------------------------------------------------------------------
void ReadInputs::parsecodositetransformed(
			SequenceVector&		outSeqs,
			const int& 		m_nCodonposition)
{
	// 1, parse the four fold conserved sites,
	// 2, transform the nucleotides on that site to R(purine) or Y (pyrimidine)

	assert( outSeqs[0].size() %3 == 0);
	
	bool valid = true;
	
	StringVector tempStrings(outSeqs.size());
	
	for (int i = 0; i < tempStrings.size(); i++)
		tempStrings[i].clear();
	
	if (m_nCodonposition != 0)	//1st or 2nd codon position
	{
		for (string::size_type j = 0; j < outSeqs[0].size()/3; j++)
		{
			valid = true;
			
			for (string::size_type i = 0; i< outSeqs.size() && valid; i++)
			{
				char c = outSeqs[i][3*j + (m_nCodonposition-1)];

				if( isalnum(c) || ispunct(c) )
				{
					if ( (c == 'c') || (c == 't') || (c == 'C') || (c == 'T') || (c == 'a') || (c == 'g') || (c == 'A') || (c == 'G') )
						valid = true;
					else
						valid = false;
				}
			}
			
			if (valid)
			{
				for (string::size_type i = 0; i< outSeqs.size(); i++)
				{
					char c = outSeqs[i][3*j + (m_nCodonposition-1)];

					if( isalnum(c) || ispunct(c) )
					{
						if ( (c == 'c') || (c == 't') || (c == 'C') || (c == 'T') )
						{
							c = 'Y';
							tempStrings[i].push_back(c);
						}
						else if ( (c == 'a') || (c == 'g') || (c == 'A') || (c == 'G') )
						{
							c = 'R';
							tempStrings[i].push_back(c);
						}
					}	
				}
			}
		}
	}
	else	//1st +2nd codon position
	{
		for (string::size_type j = 0; j < outSeqs[0].size()/3; j++)
		{
			valid = true;
		
			for (string::size_type i = 0; i< outSeqs.size() && valid; i++)
			{
				char c = outSeqs[i][3*j];

				if( isalnum(c) || ispunct(c) )
				{
					if ( (c == 'c') || (c == 't') || (c == 'C') || (c == 'T') || (c == 'a') || (c == 'g') || (c == 'A') || (c == 'G') )
						valid = true;
					else
						valid = false;
				}
			}
			
			if (valid)
			{
				for (string::size_type i = 0; i< outSeqs.size(); i++)
				{
					char c = outSeqs[i][3*j];

					if( isalnum(c) || ispunct(c) )
					{
						if ( (c == 'c') || (c == 't') || (c == 'C') || (c == 'T') )
						{
							c = 'Y';
							tempStrings[i].push_back(c);
						}	
						else if ( (c == 'a') || (c == 'g') || (c == 'A') || (c == 'G') )
						{
							c = 'R';
							tempStrings[i].push_back(c);
						}
					}	
				}
			}
	
			valid = true;
		
			for (string::size_type i = 0; i< outSeqs.size() && valid; i++)
			{
				char c = outSeqs[i][3*j + 1];

				if( isalnum(c) || ispunct(c) )
				{
					if ( (c == 'c') || (c == 't') || (c == 'C') || (c == 'T') || (c == 'a') || (c == 'g') || (c == 'A') || (c == 'G') )
						valid = true;
					else	
						valid = false;
				}
			}
			
			if (valid)
			{
				for (string::size_type i = 0; i< outSeqs.size(); i++)
				{
					char c = outSeqs[i][3*j + 1];

					if( isalnum(c) || ispunct(c) )
					{
						if ( (c == 'c') || (c == 't') || (c == 'C') || (c == 'T') )
						{
							c = 'Y';
							tempStrings[i].push_back(c);
						}
						else if ( (c == 'a') || (c == 'g') || (c == 'A') || (c == 'G') )
						{
							c = 'R';
							tempStrings[i].push_back(c);
						}
					}	
				}
			}
		}
	}

	outSeqs.clear();
	
	for (int i = 0; i< tempStrings.size(); i++)
	{
		outSeqs.push_back(Sequence());

		AddToSeqs(outSeqs[i], tempStrings[i]);
	}
}

//---------------------------------------------------------------------------
int ReadInputs::Read_Seqs(
	FILE* 			m_fSeqs,
	SequenceVector&		outSeqs,
	StringVector&		outSeqNames,
	int 			type,
	const bool&		transformed,
	const int& 		m_nCodonposition)
{
	cerr << endl << "Reading Sequence file......." << endl << endl; 
	
	unsigned int startline = (type == PIR) ? 1 : 0;

	unsigned int linenum = 0;

	bool inseq = false;
	char linesig;
	SequenceVector::size_type seq = 0;
	string line;
	
	while (0 != (linesig=getln(line, MAXLINE, m_fSeqs)))
	{
		if(linesig==NEWSEQ)  
		{
			if (inseq) 
				seq++; 

			linenum	= 0; 
			inseq = true;
		} 
		else 
			linenum++;
		
		if (linenum == 0)
			GetName(outSeqNames, seq, line, type);
		else if(linenum>startline && inseq)
		{
			assert(seq <= outSeqs.size());
			
			if (seq == outSeqs.size())
				outSeqs.push_back(Sequence());
				
			AddToSeqs(outSeqs[seq], line);
		}

		if (linesig==EOF || line[0]==EOF)
		{
			cerr << "EOF." << endl; 
			seq++; 
			break; 
		}
	}
	
	int m_nNumseqs = seq;
	
	cerr << endl << "Done Reading seqfile with " << m_nNumseqs << " seqs."
		<< endl << endl; 
		
	if (transformed)
	{
		if (m_nCodonposition == 3)
			parse4foldsitetransformed(outSeqs);
		else if ((m_nCodonposition == 1) || (m_nCodonposition == 2) || (m_nCodonposition == 0))
			parsecodositetransformed(outSeqs, m_nCodonposition);
	}
		
	cerr << endl << "Done Transforming seqfile with " << m_nNumseqs << " seqs to 4 fourfold conserved sites."
		<< endl << endl; 

	return (m_nNumseqs);
}

//---------------------------------------------------------------------------
int ReadInputs::ReadSequence(const char* m_strSeqfile, vector< vector<int> > &m_vecSequences, StringVector& m_vecSeqNames, const int& m_nCodonposition)
{
//	**********************************************
//	Read the input files to get the data: Sequences,
//	**********************************************
   	FILE* m_fSequence = fopen(m_strSeqfile, "rb");
	
	int m_nNumseqs; 
	
	if (m_fSequence != NULL)
		m_nNumseqs = Read_Seqs(m_fSequence, m_vecSequences, m_vecSeqNames, FASTA, 1, m_nCodonposition);
	else
		throw(2);
	
	fclose(m_fSequence);

	return(m_nNumseqs);
}


//---------------------------------------------------------------------------
int ReadInputs::Identify3rdcodonPositionFold(
			const SequenceVector&		outSeqs,
			const StringVector&		outSeqNames)
{
	int numSeq = outSeqs.size();
	
	int numSites = outSeqs[0].size()/3;
	
	int species1, species2;
	
	for (int i = 0; i < numSeq; i++)
	{
		if (outSeqNames[i] == "Xenagama_unicolor")
			species1 = i;
		else if (outSeqNames[i] == "Pogona_vitticepes")
			species2 = i;
	}
	
	ofstream outStream("fold.csv");
	
	outStream << "Site" << "\t" << "percentage2fold" << "\t" << "percentage4fold" << "\t" << "Xenagama_unicolor" << "\t" << "Pogona_vitticepes" << endl;

	int num2fold = 0;
	
	int num4fold = 0;
	
	int species1fold;
	
	int species2fold;
	
	double percentage2fold, percentage4fold;
	
	for (string::size_type j = 0; j < numSites; j++)
	{
		num2fold = 0;
	
		num4fold = 0;

		species1fold = 0;
			
		species2fold = 0;
					
		for (string::size_type i = 0; i< numSeq; i++)
		{
			char first = outSeqs[i][3*j];
			
			char second = outSeqs[i][3*j+1];
			
			// four fold sites
			if ( (first == 'C' && second == 'C') || (first == 'c' && second == 'C') || (first == 'C' && second == 'c') || (first == 'c' && second == 'c')		// Pro
				|| (first == 'C' && second == 'G') || (first == 'c' && second == 'G') || (first == 'C' && second == 'g') || (first == 'c' && second == 'g') 	//Arg
				|| (first == 'C' && second == 'T') || (first == 'c' && second == 'T') || (first == 'C' && second == 't') || (first == 'c' && second == 't') 	//Arg
				|| (first == 'T' && second == 'C') || (first == 't' && second == 'C') || (first == 'T' && second == 'c') || (first == 't' && second == 'c') 	//Arg
				|| (first == 'A' && second == 'C') || (first == 'a' && second == 'C') || (first == 'A' && second == 'c') || (first == 'a' && second == 'c')	//Thr
				|| (first == 'G' && second == 'T') || (first == 'g' && second == 'T') || (first == 'G' && second == 't') || (first == 'g' && second == 't')	//Val
				|| (first == 'G' && second == 'C') || (first == 'g' && second == 'C') || (first == 'G' && second == 'c') || (first == 'g' && second == 'c')	//Ala
				|| (first == 'G' && second == 'G') || (first == 'g' && second == 'G') || (first == 'G' && second == 'g') || (first == 'g' && second == 'g') )	//Gly
			{
				num4fold++;
				
				if (i == species1)
					species1fold = 4;
				else if (i == species2)
					species2fold = 4;
			}
			else if ((first != '-') && (second != '-'))
			{
				num2fold++;
				
				if (i == species1)
					species1fold = 2;
				else if (i == species2)
					species2fold = 2;
			}	
		}
		
		//print the output of each site as the 2 or 4 fold
		if (num4fold + num2fold >= 30)
		{
			percentage2fold = (double) num2fold / (double) (num4fold + num2fold);
			
			percentage4fold = 1 - percentage2fold;
			
			outStream << j+1 << "\t" << setprecision(2) << percentage2fold << "\t"<< setprecision(2) << percentage4fold << setprecision(0) << "\t";
		}
		else
			outStream << j+1 << "\t" << 0 << "\t" << 0 << "\t";
			
		outStream << species1fold << "\t" << species2fold << endl;
	}
	
	return(1);
}

//---------------------------------------------------------------------------
int ReadInputs::Read_SeqFold(
	const char* 		m_strSeqfile,
	SequenceVector&		outSeqs,
	StringVector&		outSeqNames)
{
	cerr << endl << "Reading Sequence file......." << endl << endl; 
	
   	FILE* m_fSeqs = fopen(m_strSeqfile, "rb");
	
	int type = FASTA;
	
	unsigned int startline = (type == PIR) ? 1 : 0;

	unsigned int linenum = 0;

	bool inseq = false;

	char linesig;

	SequenceVector::size_type seq = 0;

	string line;
	
	while (0 != (linesig=getln(line, MAXLINE, m_fSeqs)))
	{
		if(linesig==NEWSEQ)  
		{
			if (inseq) 
				seq++; 

			linenum	= 0; 
			inseq = true;
		} 
		else 
			linenum++;
		
		if (linenum == 0)
			GetName(outSeqNames, seq, line, type);
		else if(linenum>startline && inseq)
		{
			assert(seq <= outSeqs.size());
			
			if (seq == outSeqs.size())
				outSeqs.push_back(Sequence());
				
			AddToSeqs(outSeqs[seq], line);
		}

		if (linesig==EOF || line[0]==EOF)
		{
			cerr << "EOF." << endl; 
			seq++; 
			break; 
		}
	}
	
	int m_nNumseqs = seq;
	
	cerr << endl << "Done Reading seqfile with " << m_nNumseqs << " seqs."
		<< endl << endl; 
		
	Identify3rdcodonPositionFold(outSeqs, outSeqNames);

	return (m_nNumseqs);
}
*/
/*
//---------------------------------------------------------------------------
int ReadInputs::Read_Modelfile(const char* m_strModelfile, CharVector& m_vecState, DoubleMatrix3D& m_matModelRates, DoubleVector& m_vecModelFreq, bool& m_bDinuModel)
{
	int comment,stop,count,step=0;
	char word[MAXNAME]="";
	char lastword[MAXNAME]="";
	
	FILE* fp = fopen(m_strModelfile, "rb");
	
	int numModels = 0;
	
	if (fp != NULL)
	{
		for(fscanf(fp,"%s", word),comment=0,count=0,stop=0;stop==0;fscanf(fp,"%s", word))
		{
			count++;
			
			if(word[0]=='#')
			{
				comment=1-comment;

				if (comment==0)
					step=0;
			}
			else if (!comment)
			{
				if (step==0)
					SetParam(lastword, word, m_vecState, m_matModelRates, m_vecModelFreq, numModels, m_bDinuModel);
			
				if( ! strcmp(word,ENDWORD))
					stop=1; 
				else 
					strcpy(lastword,word);
			}

			step=1-step;
		} 

		fclose(fp);
		
		return(numModels);
	}
	else
		return(0);
}
*/