/*
 *  ReadInputs.h
 *  PLEX
 *
 *  Created by Wanjun Gu.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */

/*
 *  
 *  this class includes all the functions in reading files for trees, sequences, ol, lengths
 *  also functions to read control files
 *
 *
 */

#ifndef ReadInputs_h__
#define ReadInputs_h__

#include "CLAMPTypes.h"
#include "Options.h"

using std::string;
using std::ostream;

extern Options options;

class ReadInputs {
	public:
		ReadInputs(
			void);
		
		bool ReadControlfile(
			const char* controlfile, 
			int& m_nInitseed, 
			int&  programTask);			//read control parameters of task and random seed

    
    bool ReadControlfile( const char* controlfile );			//read the control parameters of ancestral sequence reconstruction
        
    /*
    int ReadInputfiles();		// read the input files 
    
		bool ReadAncSequence(
			const char* m_strAncSeqfile, 
			std::string& m_strAncSeq);	//read the file to get the ancestral sequence

		int ReadPairwiseSequence(
			const char* m_strSeqfile, 
			vector< vector<int> > &m_vecSequences); // read the pairwise sequences into sequence vectors

		int ReadSequence(
			const char* m_strSeqfile,
			vector< vector<int> > &m_vecSequences, 
			StringVector& m_vecSeqNames, 
			const int& m_nCodonposition);	//read coding sequences on the specific codon position

		int Read_SeqFold(
			const char* m_strSeqfile, 
			SequenceVector& outSeqs, 
			StringVector& outSeqNames);	//read sequence folds in the sequence position 
	
		int Read_Modelfile(
			const char* m_strModelfile, 
			CharVector& m_vecState, 
			DoubleMatrix3D& m_matModelRates, 
			DoubleVector& m_vecModelFreq, 
			bool& m_bDinuModel);		//read the model file for simulation
*/
	protected:
	
	private:
		void readsets(
			DoubleVector& m_dvParams, 
			const char value[MAXNAME]);	//read the sets of double parameters from a character array

		void readsets(
			DoubleVector& m_dvParams, 
			const char value[MAXNAME], 
			const int& m_numModels);	//read sets of double parameters for different number of models from a character array

		void readsets(
			CharVector& m_vParams, 
			const char value[MAXNAME]);	//read sets of character parameters from a character array

		void readsets(
			DoubleMatrix3D& m_dvParams, 
			const char value[MAXNAME], 
			const int& m_numModels, 
			const int& m_numStates);	//read sets of rate matrices for all models from a character array

		char GetMarkedLine( 
			string& outLine, 
			string::size_type max, 
			FILE* fp, 
			char inStartMarker, 
			char inEndMarker);		//get marked line from each line string

		char getln(
			string& outLine, 
			string::size_type max, 
			FILE* fp);			//get line characters from each line string

		char getlnpos(
			string& outLine, 
			string::size_type max, 
			FILE* fp);

		void GetName(
			StringVector& outNames, 
			int seq, 
			const string& inLine, 
			int type);			//get sequence name from line string

		void FindName(
			const StringVector& outNames, 
			int& seq, 
			const string& inLine, 
			int type);			//find string names in string name vector

		void AddToSeqs(
			Sequence& outSeq, 
			const string& inLine);		//add a sequence into sequence vector

		void AddToSeqs(Sequence& outSeq, 
			const string& inLine, 
			const bool& transformed);	//add a transformed sequence into sequence vector (transversion only.etc)

		void AddToPoss(
			IntMatrix& outPoss, 
			const int& seq, 
			const string& inLine);		//add a position information to position matrix

		void AddToOL(
			IntVector& outOL, 
			const int& seq, 
			const string& inLine);		//add ol information to ol vector

/*
    int Read_Seqs(
			FILE* m_fSeqs, 
			SequenceVector& outSeqs, 
			StringVector& outSeqNames, 
			int type);			//read sequences file to sequence vector and sequence name vector

		int Read_Seqs(
			FILE* m_fSeqs, 
			SequenceVector& outSeqs, 
			int type);			//read sequence file to sequence vector
		
		int Read_Seqs(
			FILE* m_fSeqs, 
			SequenceVector& outSeqs, 
			StringVector& outSeqNames, 
			int type, 
			const bool& transformed, 
			const int& m_nCodonposition);	//read sequence file to sequence vector with sequences selected according to codon position

		void Read_Poss(
			FILE* m_fPoss, 
			IntMatrix& outPoss, 
			const StringVector& outSeqNames, 
			int type);			//read position file into position matrix with sequences ordered by sequence name

		void Read_OL(
			FILE* m_fOL, 
			IntVector& outOL, 
			const StringVector& outSeqNames, 
			int type);			//read OL file into OL vector with sequences ordered by sequence name

		bool Read_AncSeq(
			FILE* m_fSeqs, 
			std::string& outSeqs);		//read ancestral sequence file into ancestral sequence
*/	
    void SetParam( const char param[MAXNAME], const char value[MAXNAME] );			//set parameters in ancestral sequence reconstruction, rate estimation using plex

		void WriteParams( ostream& osparam );		//write parameters in ancestral sequence reconstruction, rate estimation using plex

    /*

    void parse4foldsitetransformed(
			SequenceVector& outSeqs);	//parse four fold sites
		
		void parsecodositetransformed(
			SequenceVector& outSeqs, 
			const int& m_nCodonposition);	//parse codon sites based on codon position

		int Identify3rdcodonPositionFold(
			const SequenceVector& outSeqs, 
			const StringVector& outSeqNames);//identify the site fold information for each 3rd codon position in sequences
		*/
		static const int m_nNucleotides;	//number of nucleotides
		static const int m_nAminoacids;		//number of amino acids
		static const int m_nCodons;			//number of nucleotides

};

#endif // ReadInputs_h__
