/*
 *  RecycleMatrix.h
 *  PLEX
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004-2005 Dr. David Pollock. All rights reserved.
 *
 */

/*
 *
 * recycle the memory cost for matrix, not used in current release
 *
 */

#ifndef RecycleMatrix_h__
#define RecycleMatrix_h__

#include <algorithm>
#include "KeyedRecyclery.h"
#include "Matrix.h"

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
class RecycleMatrix : public AbstractMatrix<DataType>
{
	public:
			typedef	typename AbstractMatrix<DataType>::RowType		RowType;
			typedef	typename AbstractMatrix<DataType>::RowVector	RowVector;
			typedef	typename AbstractMatrix<DataType>::size_type	size_type;
			
											RecycleMatrix(void);
											RecycleMatrix(
												size_type			inRows,
												size_type			inColumns,
												const DataType&		inValue = DataType());
											RecycleMatrix(
												const RecycleMatrix&		inRecycleMatrix);
												
	virtual									~RecycleMatrix(void);
												
	virtual	void							fill(
												const DataType&		inVal);
			
	virtual	void							resize(
												size_type			inRows,
												size_type			inColumns,
												const DataType&		inVal = DataType());
												
	virtual	const RowType&					operator[](
												size_type			inRowIndex) const;
	virtual	RowType&						operator[](
												size_type			inRowIndex);
												
			const RecycleMatrix&			operator=(
												const RecycleMatrix&		inRecycleMatrix);
			const RecycleMatrix&			operator=(
												DataType**			inRecycleMatrix);
												
	virtual	void							SwapRows(
												size_type			inRow1,
												size_type			inRow2);
	virtual	void							SwapColumns(
												size_type			inColumn1,
												size_type			inColumn2);
												
			void							swap(
												RecycleMatrix&				inRecycleMatrix);
															
	private:
		typedef typename Matrix<DataType>::Initializer Initializer;
		
				Matrix<DataType>*						mMatrix;
				Initializer								mInitializer;
				
		typedef KeyedRecyclery<Matrix<DataType>, Initializer>	RecycleryType;
		static	RecycleryType							sRecyclery;
};

template <typename DataType>
typename RecycleMatrix<DataType>::RecycleryType	RecycleMatrix<DataType>::sRecyclery;

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
RecycleMatrix<DataType>::RecycleMatrix(void)
	:	AbstractMatrix<DataType>(),
		mMatrix(0),
		mInitializer(0, 0, DataType())
{
	mMatrix = sRecyclery.New(mInitializer);
}

//---------------------------------------------------------------------------
template <typename DataType>
RecycleMatrix<DataType>::RecycleMatrix(
	size_type		inRows,
	size_type		inColumns,
	const DataType&	inValue)
	:	AbstractMatrix<DataType>(inRows, inColumns, inValue),
		mMatrix(0),
		mInitializer(inRows, inColumns, inValue)
{
	mMatrix = sRecyclery.New(mInitializer);
}

//---------------------------------------------------------------------------
template <typename DataType>
RecycleMatrix<DataType>::RecycleMatrix(
	const RecycleMatrix&	inRecycleMatrix)
	:	AbstractMatrix<DataType>(inRecycleMatrix),
		mMatrix(0),
		mInitializer(inRecycleMatrix.mInitializer)
{
	mMatrix = sRecyclery.New(mInitializer);
	*mMatrix = *(inRecycleVector.mMatrix);
}

//---------------------------------------------------------------------------
template <typename DataType>
RecycleMatrix<DataType>::~RecycleMatrix(void)
{
	sRecyclery.Dispose(mInitializer, mMatrix);
}

//---------------------------------------------------------------------------
template <typename DataType>
void
RecycleMatrix<DataType>::fill(
	const DataType&	inVal)
{
	mMatrix->fill(inVal);
}

//---------------------------------------------------------------------------
template <typename DataType>
void
RecycleMatrix<DataType>::resize(
	size_type		inRows,
	size_type		inColumns,
	const DataType&	inVal)
{
	mMatrix->resize(inRows, inColumns, inVal);
	mInitializer.fRows	= inRows;
	mInitializer.fCols	= inColumns;
	mInitializer.fInitValue	= inVal;
	AbstractMatrix<DataType>::resize(inRows, inColumns, inVal);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
const typename RecycleMatrix<DataType>::RowType&
RecycleMatrix<DataType>::operator[](
	size_type	inRowIndex) const
{
	return (*mMatrix)[inRowIndex];
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
typename RecycleMatrix<DataType>::RowType&
RecycleMatrix<DataType>::operator[](
	size_type	inRowIndex)
{
	return (*mMatrix)[inRowIndex];
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
const RecycleMatrix<DataType>&
RecycleMatrix<DataType>::operator=(
	const RecycleMatrix&	inRecycleMatrix)
{
	RecycleMatrix	temp = inRecycleMatrix;
	
	swap(temp);
	return *this;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
RecycleMatrix<DataType>::SwapRows(
	size_type	inRow1,
	size_type	inRow2)
{
	mMatrix->SwapRows(inRow1, inRow2);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
RecycleMatrix<DataType>::SwapColumns(
	size_type	inColumn1,
	size_type	inColumn2)
{
	mMatrix->SwapColumns(inColumn1, inColumn2);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
RecycleMatrix<DataType>::swap(
	RecycleMatrix&	inRecycleMatrix)
{
	std::swap(inRecycleMatrix.mInitializer, mInitializer);
	std::swap(inRecycleMatrix.mMatrix, mMatrix);
	AbstractMatrix<DataType>::swap(inRecycleMatrix);
}

//---------------------------------------------------------------------------
#endif // RecycleMatrix_h__
