/*
 *  RecycleMatrix3D.h
 *  CLAMP
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */

/*
 *
 * recycle the memory cost for matrix3D, not used in current release
 *
 */

#ifndef RecycleMatrix3D_h__
#define RecycleMatrix3D_h__

#include <algorithm>
#include "KeyedRecyclery.h"
#include "Matrix3D.h"

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
class RecycleMatrix3D : public AbstractMatrix3D<DataType>
{
	public:
			typedef	typename AbstractMatrix3D<DataType>::RowType	RowType;
			typedef typename AbstractMatrix3D<DataType>::RowVector	RowVector;
			typedef typename AbstractMatrix3D<DataType>::size_type	size_type;
			
											RecycleMatrix3D(void);
											RecycleMatrix3D(
												size_type			inRows,
												size_type			inColumns,
												size_type			inLayers,
												const DataType&		inValue = DataType());
											RecycleMatrix3D(
												const RecycleMatrix3D&		inMatrix);
	virtual									~RecycleMatrix3D(void);
												
	virtual	void							fill(
												const DataType&		inVal);
			
	virtual	void							resize(
												size_type			inRows,
												size_type			inColumns,
												size_type			inLayers,
												const DataType&		inVal = DataType());
												
	virtual	const RowType&					operator[](
												size_type			inRowIndex) const;
	virtual	RowType&						operator[](
												size_type			inRowIndex);
												
			const RecycleMatrix3D&			operator=(
												const RecycleMatrix3D&		inMatrix);
			const RecycleMatrix3D&			operator=(
												DataType***			inMatrix);
												
	virtual	void							SwapRows(
												size_type			inRow1,
												size_type			inRow2);
	virtual	void							SwapColumns(
												size_type			inColumn1,
												size_type			inColumn2);
	virtual	void							SwapLayers(
												size_type			inLayer1,
												size_type			inLayer2);
												
			void							swap(
												RecycleMatrix3D&			inMatrix);
															
	private:
		typedef typename Matrix3D<DataType>::Initializer Initializer;
		
				Matrix3D<DataType>*						mMatrix;
				Initializer								mInitializer;
				
		typedef KeyedRecyclery<Matrix3D<DataType>, Initializer>	RecycleryType;
		static	RecycleryType							sRecyclery;
};

template <typename DataType>
typename RecycleMatrix3D<DataType>::RecycleryType	RecycleMatrix3D<DataType>::sRecyclery;

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
RecycleMatrix3D<DataType>::RecycleMatrix3D(void)
	:	AbstractMatrix3D<DataType>(),
		mMatrix(0),
		mInitializer(0, 0, 0, DataType())
{
	mMatrix = sRecyclery.New(mInitializer);
}

//---------------------------------------------------------------------------
template <typename DataType>
RecycleMatrix3D<DataType>::RecycleMatrix3D(
	size_type		inRows,
	size_type		inColumns,
	size_type		inLayers,
	const DataType&	inValue)
	:	AbstractMatrix3D<DataType>(inRows, inColumns, inLayers, inValue),
		mMatrix(0),
		mInitializer(inRows, inColumns, inLayers, inValue)
{
	mMatrix = sRecyclery.New(mInitializer);
}

//---------------------------------------------------------------------------
template <typename DataType>
RecycleMatrix3D<DataType>::RecycleMatrix3D(
	const RecycleMatrix3D&	inMatrix)
	:	AbstractMatrix3D<DataType>(inMatrix),
		mMatrix(0),
		mInitializer(inMatrix.mInitializer)
{
	mMatrix = sRecyclery.New(mInitializer);
	*mMatrix = *(inMatrix.mMatrix);
}

//---------------------------------------------------------------------------
template <typename DataType>
RecycleMatrix3D<DataType>::~RecycleMatrix3D(void)
{
	sRecyclery.Dispose(mInitializer, mMatrix);
}

//---------------------------------------------------------------------------
template <typename DataType>
void
RecycleMatrix3D<DataType>::fill(
	const DataType&	inVal)
{
	mMatrix->fill(inVal);
}

//---------------------------------------------------------------------------
template <typename DataType>
void
RecycleMatrix3D<DataType>::resize(
	size_type		inRows,
	size_type		inColumns,
	size_type		inLayers,
	const DataType&	inVal)
{
	mMatrix->resize(inRows, inColumns, inLayers, inVal);
	mInitializer.fRows	= inRows;
	mInitializer.fCols	= inColumns;
	mInitializer.fLayers= inLayers;
	mInitializer.fInitValue	= inVal;
	AbstractMatrix3D<DataType>::resize(inRows, inColumns, inLayers, inVal);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
const typename RecycleMatrix3D<DataType>::RowType&
RecycleMatrix3D<DataType>::operator[](
	size_type	inRowIndex) const
{
	return (*mMatrix)[inRowIndex];
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
typename RecycleMatrix3D<DataType>::RowType&
RecycleMatrix3D<DataType>::operator[](
	size_type	inRowIndex)
{
	return (*mMatrix)[inRowIndex];
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
const RecycleMatrix3D<DataType>&
RecycleMatrix3D<DataType>::operator=(
	const RecycleMatrix3D&	inMatrix)
{
	RecycleMatrix3D	temp = inMatrix;
	
	swap(temp);
	return *this;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
RecycleMatrix3D<DataType>::SwapRows(
	size_type	inRow1,
	size_type	inRow2)
{
	mMatrix->SwapRows(inRow1, inRow2);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
RecycleMatrix3D<DataType>::SwapColumns(
	size_type	inColumn1,
	size_type	inColumn2)
{
	mMatrix->SwapColumns(inColumn1, inColumn2);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
RecycleMatrix3D<DataType>::SwapLayers(
	size_type	inLayer1,
	size_type	inLayer2)
{
	mMatrix->SwapLayers(inLayer1, inLayer2);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
RecycleMatrix3D<DataType>::swap(
	RecycleMatrix3D&	inMatrix)
{
	std::swap(inMatrix.mInitializer, mInitializer);
	std::swap(inMatrix.mMatrix, mMatrix);
	AbstractMatrix3D<DataType>::swap(inMatrix);
}

//---------------------------------------------------------------------------
#endif // RecycleMatrix3D_h__
