/*
 *  RecycleVector.h
 *  PLEX
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */

/*
 *
 * recycle the memory cost for vector, not used in current release
 *
 */

#ifndef RecycleVector_h__
#define RecycleVector_h__

#include <algorithm>
#include <cassert>

#include "KeyedRecyclery.h"
#include "Vector.h"

//---------------------------------------------------------------------------
template <typename DataType>
class RecycleVector : public AbstractVector<DataType>
{
	public:
		typedef typename AbstractVector<DataType>::size_type	size_type;
		
											RecycleVector(void);
											RecycleVector(
												size_type		inNumItems,
												const DataType&	inVal = DataType());
											RecycleVector(
												const RecycleVector&	inRecycleVector);
		virtual								~RecycleVector(void);
											
				const RecycleVector&				operator=(
												const RecycleVector&	inRecycleVector);
												
		virtual	void						resize(
												size_type		inNewSize,
												const DataType&	inVal = DataType());
		virtual	void						fill(
												const DataType&	inVal);
												
		virtual	DataType&					operator[](
												size_type		inIndex);
		virtual	const DataType&				operator[](
												size_type		inIndex) const;
												
				void						swap(
												RecycleVector&			ioRecycleVector);
	private:
		typedef typename Vector<DataType>::Initializer Initializer;
		
				Vector<DataType>*						mVector;
				Initializer								mInitializer;
				
		typedef KeyedRecyclery<Vector<DataType>, Initializer>	RecycleryType;
		static	RecycleryType							sRecyclery;
};

template <typename DataType>
typename RecycleVector<DataType>::RecycleryType	RecycleVector<DataType>::sRecyclery;

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
RecycleVector<DataType>::RecycleVector(void)
	:	AbstractVector<DataType>(),
		mVector(0),
		mInitializer(0, DataType())
{
	mVector = sRecyclery.New(mInitializer);
}

//---------------------------------------------------------------------------
template <typename DataType>
RecycleVector<DataType>::RecycleVector(
	size_type		inNumItems,
	const DataType&	inVal)
	:	AbstractVector<DataType>(inNumItems, inVal),
		mVector(0),
		mInitializer(inNumItems, inVal)
{
	mVector = sRecyclery.New(mInitializer);
}

//---------------------------------------------------------------------------
template <typename DataType>
RecycleVector<DataType>::RecycleVector(
	const RecycleVector&	inRecycleVector)
	:	AbstractVector<DataType>(inRecycleVector),
		mVector(0),
		mInitializer(inRecycleVector.mInitializer)
{
	mVector = sRecyclery.New(mInitializer);
	*mVector = *(inRecycleVector.mVector);
}

//---------------------------------------------------------------------------
template <typename DataType>
RecycleVector<DataType>::~RecycleVector(void)
{
	sRecyclery.Dispose(mInitializer, mVector);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
const RecycleVector<DataType>&
RecycleVector<DataType>::operator=(
	const RecycleVector&	inRecycleVector)
{
	RecycleVector	temp(inRecycleVector);
	swap(temp);
	return *this;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
DataType&
RecycleVector<DataType>::operator[](
	size_type	inIndex)
{
	return (*mVector)[inIndex];
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
const DataType&
RecycleVector<DataType>::operator[](
	size_type	inIndex) const
{
	return (*mVector)[inIndex];
}

//---------------------------------------------------------------------------
template <typename DataType>
void
RecycleVector<DataType>::resize(
	size_type		inNewSize,
	const DataType&	inVal)
{
	mVector->resize(inNewSize, inVal);
	mInitializer.fNumItems	= inNewSize;
	mInitializer.fInitValue	= inVal;
	AbstractVector<DataType>::resize(inNewSize, inVal);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
RecycleVector<DataType>::fill(
	const DataType&	inVal)
{
	mVector->fill(inVal);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
RecycleVector<DataType>::swap(
	RecycleVector&	inRecycleVector)
{
	AbstractVector<DataType>::swap(inRecycleVector);
	std::swap(inRecycleVector.mVector, mVector);
	std::swap(inRecycleVector.mInitializer, mInitializer);
}



#endif // RecycleVector_h__
