/*
 *  Recyclery.h
 *  CLAMP
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004-2005 Dr. David Pollock. All rights reserved.
 *
 */

/*
 *
 * recycle the memory cost for a data type, not used in current release
 *
 */

#ifndef Recyclery_h__
#define Recyclery_h__

#include <algorithm>
#include <stack>

template <typename DataType, typename Initializer>
class Recyclery
{
	public:
													Recyclery(void);
													Recyclery(
														const Initializer&	inInitializer);
													Recyclery(
														const Recyclery&	inSrc);
													~Recyclery(void);
													
				const Recyclery&					operator=(
															const Recyclery& inSrc);

				DataType*							New(void);
				DataType*							New(
														const Initializer&	inInitializer);
				void								Dispose(
														DataType*&	ioPtr);
									
				
	protected:
	private:
				std::stack<DataType*>				mRecycleBin;
				Initializer							mInitializer;
};

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
Recyclery<DataType, Initializer>::Recyclery(void)
	:	mRecycleBin(),
		mInitializer()
{
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
Recyclery<DataType, Initializer>::Recyclery(
	const Recyclery&	inSrc)
	:	mRecycleBin(),
		mInitializer(inSrc.mInitializer)
{
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
Recyclery<DataType, Initializer>::Recyclery(
	const Initializer&	inInitializer)
	:	mRecycleBin(),
		mInitializer(inInitializer)
{
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
Recyclery<DataType, Initializer>::~Recyclery(void)
{
	while(mRecycleBin.size())
	{
		delete mRecycleBin.top();
		mRecycleBin.pop();
	}
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
const Recyclery<DataType, Initializer>&
Recyclery<DataType, Initializer>::operator=(
	const Recyclery&	inSrc)
{
	mInitializer = inSrc.mInitializer;
	
	while(mRecycleBin.size())
		mRecycleBin.pop();
		
	return *this;
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
DataType*
Recyclery<DataType, Initializer>::New(void)
{
	return New(mInitializer);
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
DataType*
Recyclery<DataType, Initializer>::New(
	const Initializer&	inInitializer)
{
	DataType*	ptr;
	if (mRecycleBin.size())
	{
		ptr = mRecycleBin.top();
		mRecycleBin.pop();
		ptr->Initialize(inInitializer);
	}
	else ptr = new DataType(inInitializer);
	return ptr;
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
void
Recyclery<DataType, Initializer>::Dispose(
	DataType*&	ioPtr)
{
	mRecycleBin.push(ioPtr);
	ioPtr = 0;
}

#endif // Recyclery_h__
