/*
 *  Sequence.cpp
 *  CLAMP
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */

#include "Sequence.h"
#include "RandomGenerator.h"

#include <algorithm>
#include <cassert>
#include "Numerics.h"

using std::min;
using std::string;
using std::swap;
/*
//---------------------------------------------------------------------------
class Sequence_Impl
{
	public:
		typedef unsigned int						RefCount;
		typedef unsigned int						size_type;
		
													Sequence_Impl(
														const std::string&			inSeqenceStr = "");
													Sequence_Impl(
														const Sequence_Impl&	inSrc);
													~Sequence_Impl(void);
													
				bool								IsShared(void) const;
				void								Retain(void) const;
				void								Release(void) const;
				
				void								resize(
														size_type				inNewSize);
				size_type							size(void) const;
				
				size_type							GetLen(void) const;
				
				void								SetAtIndex(
														size_type				inIndex,
														char					inVal);
				void								push_back(
														char					inVal);
														
	//			const std::string&						str(void) const;
				
				std::string							str(void) const;
				
				void								SetString(const std::string& inString);
				
				int								GetAssignment(void);
				
				void								SetAssignment(const int& inAssignment);
				
				void								Mix(
														Sequence*	inSeq);
				
				const char							operator[](
														size_type				inIndex) const;
				bool								operator==(
														const Sequence_Impl&	inSrc) const;
														
				void								BeginSharedChange(void);
				void								EndSharedChange(void);
				std::string							mSequenceStr;

				Numerics *num;			// [JdK] - an interface to accelerated numeric functions

	protected:
	private:
				mutable	RefCount						mRefCount;
				RefCount							mSharedChangeCount;
				int								mModelAssignment;
};


#pragma mark -
//---------------------------------------------------------------------------
inline
Sequence_Impl::Sequence_Impl(
	const string&	inSequenceStr)
	:	mSequenceStr(),
		mRefCount(1),
		mSharedChangeCount(0),
		mModelAssignment(-1)
{
	num = &Numerics::getGlobalNumerics();
	string::size_type	endSeq = inSequenceStr.find(Sequence::cEndSeq);
	mSequenceStr = inSequenceStr.substr(0, endSeq);
}

//---------------------------------------------------------------------------
inline
Sequence_Impl::Sequence_Impl(
	const Sequence_Impl& inSrc)
	:	mSequenceStr(inSrc.mSequenceStr),
		mRefCount(1),
		mSharedChangeCount(0),
		mModelAssignment(-1)
{
	num = &Numerics::getGlobalNumerics();
}

//---------------------------------------------------------------------------
inline
Sequence_Impl::~Sequence_Impl(void)
{
}

//---------------------------------------------------------------------------
inline
bool
Sequence_Impl::IsShared(void) const
{
	return (mRefCount > 1) && (mSharedChangeCount == 0);
}

//---------------------------------------------------------------------------
inline
void
Sequence_Impl::Retain(void) const
{
	++mRefCount;
}

//---------------------------------------------------------------------------
inline
void
Sequence_Impl::Release(void) const
{
	assert(mRefCount >= 1);
	--mRefCount;
	
	if (mRefCount == 0)
		delete this;
}

//---------------------------------------------------------------------------
inline
void
Sequence_Impl::resize(
	size_type	inNewSize)
{
	mSequenceStr.resize(inNewSize, Sequence::cEndSeq);
}

//---------------------------------------------------------------------------
inline
Sequence_Impl::size_type
Sequence_Impl::size(void) const
{
	return mSequenceStr.size();
}

//---------------------------------------------------------------------------
inline
Sequence_Impl::size_type
Sequence_Impl::GetLen(void) const
{
	string::size_type	endSequence = mSequenceStr.find(Sequence::cEndSeq);
	return ((endSequence == string::npos)
				?	size()
				:	endSequence);
}

#include <iostream>
//---------------------------------------------------------------------------
inline
void
Sequence_Impl::SetAtIndex(
	size_type	inIndex,
	char		inVal)
{
	if (inIndex >= size()) {
		std::cout << "ERROR! bounds exceeded. requested index of " << inIndex << std::endl;
		std::cout << "but size was " << size() << ", endseq is " << ENDSEQ << std::endl;
		std::cout << "breakpoint here" << std::endl;
	}
	
	assert(inIndex < size());
	
//	if (inIndex >= size()) {
//		std::cout << "resizing sequence to " << inIndex+1 << std::endl;
//		resize(inIndex + 1);
//	}
		
	mSequenceStr[inIndex] = inVal;
}

//---------------------------------------------------------------------------
void
Sequence_Impl::push_back(
	char		inValue)
{
	mSequenceStr.push_back(inValue);
}

//---------------------------------------------------------------------------
//const string&
//Sequence_Impl::str(void) const
//{
//	return mSequenceStr;
//}

//---------------------------------------------------------------------------
std::string 
Sequence_Impl::str(void) const
{
	return mSequenceStr;
}

//---------------------------------------------------------------------------
void
Sequence_Impl::SetString(const std::string& inString)
{
	mSequenceStr = inString;
}

//---------------------------------------------------------------------------
void
Sequence_Impl::Mix(
	Sequence*	inSeq)
{
	size_type	len = min(inSeq->size(), size());
	for (size_type i = 0; i != len; ++i)
	{
		if (num->Random() > 0.5)
			mSequenceStr[i] = (*inSeq)[i];
	}
}

//---------------------------------------------------------------------------
int
Sequence_Impl::GetAssignment(
	void)
{
	return(mModelAssignment);
}
				
//---------------------------------------------------------------------------
void
Sequence_Impl::SetAssignment(
	const int& inAssignment)
{
	mModelAssignment = inAssignment;
}
				
//---------------------------------------------------------------------------
inline
const char
Sequence_Impl::operator[](
	size_type	inIndex) const
{
	return mSequenceStr[inIndex];
}

//---------------------------------------------------------------------------
inline
bool
Sequence_Impl::operator==(
	const Sequence_Impl&	inSrc) const
{
	return mSequenceStr == inSrc.mSequenceStr;
}

//---------------------------------------------------------------------------
inline
void
Sequence_Impl::BeginSharedChange(void)
{
	++mSharedChangeCount;
}

//---------------------------------------------------------------------------
inline
void
Sequence_Impl::EndSharedChange(void)
{
//March 7, 2007
//[BLOCK] - UNNECESSARY!
//	assert(mSharedChangeCount > 0);
	--mSharedChangeCount;
}

#pragma mark -
//---------------------------------------------------------------------------
Sequence::Sequence(void)
	:   mImpl(new Sequence_Impl)
{
	//num = &Numerics::getGlobalNumerics();
}

//---------------------------------------------------------------------------
Sequence::Sequence(
	const string&	inSrc)
	:   mImpl(new Sequence_Impl(inSrc))
{
	//num = &Numerics::getGlobalNumerics();
}

//---------------------------------------------------------------------------
Sequence::Sequence(
	const Sequence& inSrc)
	:   mImpl(inSrc.mImpl)
{
	//num = &Numerics::getGlobalNumerics();
	mImpl->Retain();
}

//---------------------------------------------------------------------------
Sequence::~Sequence(void)
{
	mImpl->Release();
}

//---------------------------------------------------------------------------
const Sequence&
Sequence::operator=(
	const Sequence& inSrc)
{
	Sequence	temp(inSrc);
	Swap(temp);
	return *this;
}

//---------------------------------------------------------------------------
bool
Sequence::operator==(
	const Sequence& inSrc) const
{
	return mImpl->operator==(*inSrc.mImpl);
}

//---------------------------------------------------------------------------
int
Sequence::GetAssignment(
	void)
{
	return(mImpl->GetAssignment());
}
				
//---------------------------------------------------------------------------
void
Sequence::SetAssignment(
	const int& inAssignment)
{
	mImpl->SetAssignment(inAssignment);
}
				
//---------------------------------------------------------------------------
const char
Sequence::operator[](
	size_type	inIndex) const
{
	// Modified by Jason: TOO MUCH TIME IS WASTED DOING THE BOUNDS CHECK!
	//	if (inIndex >= size())
	//		return Sequence::cEndSeq;
		
	return (*mImpl)[inIndex];
}

//---------------------------------------------------------------------------
void
Sequence::Swap(
	Sequence&		ioSwapee)
{
	swap(mImpl, ioSwapee.mImpl);
}

//---------------------------------------------------------------------------
void
Sequence::resize(
	size_type	inNewSize)
{
	PrepareToModify();
	mImpl->resize(inNewSize);
}

//---------------------------------------------------------------------------
int
Sequence::size(void) const
{
	return mImpl->size();
}

//---------------------------------------------------------------------------
int
Sequence::GetLen(void) const
{
	return mImpl->GetLen();
}

//---------------------------------------------------------------------------
void
Sequence::clear(void)
{
	Sequence	temp;
	Swap(temp);
}

//---------------------------------------------------------------------------
bool
Sequence::empty(void) const
{
	return size() == 0;
}

//---------------------------------------------------------------------------
void
Sequence::SetAtIndex(
	size_type	inIndex,
	char		inValue)
{
	PrepareToModify();
	if (inValue == cEndSeq && inIndex < size())
		resize(inIndex);
	else mImpl->SetAtIndex(inIndex, inValue);
}

//---------------------------------------------------------------------------
void
Sequence::push_back(
	char		inValue)
{
	PrepareToModify();
	
	if (inValue != cEndSeq)
		mImpl->push_back(inValue);
}

//---------------------------------------------------------------------------
//const string&
//Sequence::str(void) const
//{
//	return mImpl->str();
//}

//---------------------------------------------------------------------------
std::string
Sequence::str(void) const
{
	return mImpl->str();
}

//---------------------------------------------------------------------------
void
Sequence::SetString(const std::string& inString)
{
	return mImpl->SetString(inString);
}

//---------------------------------------------------------------------------
void
Sequence::Mix( std::vector< std::vector<int> >*	inSeq1, std::vector< std::vector<int> >* inSeq2)
{
	if (inSeq1->size()) {
		*this = *inSeq1;
		PrepareToModify();
		mImpl->Mix(inSeq2);
	} else {
		*this = *inSeq2;
		PrepareToModify();
	}
}

//---------------------------------------------------------------------------
//	Sequence::PrepareToModify
//
//	This method makes a copy of our internal data if we're sharing it, so we
//	we can modify a copy own solely by us.  Important part of "copy on write"
//	idiom.
inline
void
Sequence::PrepareToModify(void)
{
	if (mImpl->IsShared())
	{
		Sequence_Impl*	temp = new Sequence_Impl(*mImpl);
		mImpl->Release();
		mImpl = temp;
	}
}

//---------------------------------------------------------------------------
void
Sequence::BeginSharedChange(void)
{
	mImpl->BeginSharedChange();
}

//---------------------------------------------------------------------------
void
Sequence::EndSharedChange(void)
{
	mImpl->EndSharedChange();
}
*/
