/*
 *  SiteCategory.cpp
 *  PLEX
 *
 *  A.P. Jason de Koning, jason.de.koning@gmail.com
 *  http://jasondk.org
 *
 *  David Pollock lab, University of Colorado School of Medicine
 *  http://www.evolutionarygenomics.com
 *  All rights reserved.
 */

#include "Options.h"
#include "SiteCategory.h"
#include "dcdflib.h"
#include "Numerics.h"


extern Options options;

SiteCategory::SiteCategory(int numSites) {
	if (options.allowASRV == 1) {
		std::cout << std::endl << "	[ INITIALIZING GAMMA DISTRIBUTION FOR ASRV; NUMSITES IS " << numSites << ", gammaShape = " << options.gammaShapeASRV << " ]" << std::endl;
	}
	
	siteAssignments.resize( numSites, 0 );
	backupAssignments.resize( numSites, 0 );
	setGammaDistribution(options.gammaShapeASRV);
}

void SiteCategory::setGammaDistribution(double shapeParm) {
	gammaShapeParameter_ASRV = shapeParm;
	setGammaDistribution();
}

void SiteCategory::setGammaDistribution(void) {
	// Code adapted from PIDAMEBL, A.P. Jason de Koning
	int which, status;
	double p,q,x,a,b, bound;
	// SET ALL THE MIXTURE COMPONENTS TO USE THE SAME GAMMA DIST (#0)...

	if ( siteRateDistribution.empty() ) {
		siteRateDistribution.resize( options.numCatASRV, 1.0 );
	}

	double sum = 0.0;
	for (int k=0; k<options.numCatASRV; k++) {
		a = b = gammaShapeParameter_ASRV;
		//a = gammaShapeParameter[0];
		//b = gammaScaleParameter[0];
		
		which = 2;											// (calculate P and Q)
		p = (2.0*k+1.0) / (2.0 * options.numCatASRV);	// set region to integrate over
		q = (1-p);
		
		// Do the inverseCDF search for gamma distributions...
		cdfgam( &which, &p, &q, &x, &a, &b, &status, &bound);
		
		if ( x < 1e-10 ) { x = 1e-10; }
		if ( x > 100   ) { x = 100; }
		
		siteRateDistribution.mDataArray[k] = x;
		sum += x;
	}
	
	// Scale by the largest!
	// sum = x;
	
	for (int k=0; k<options.numCatASRV; k++) {
//		siteRateDistribution.mDataArray[k] /= sum; 
		siteRateDistribution.mDataArray[k] *= options.numCatASRV/sum; 

		//	siteRateDistribution.mDataArray[k] *= 1.0/sum; 
	//	std::cout << "			Gamma dist cat " << k << ": " << siteRateDistribution.mDataArray[k] << std::endl;
	}

	
	// Set the largest rate (global variable)
	options.__largestRate = siteRateDistribution.mDataArray[ (options.numCatASRV - 1)];
	
	
}
