/*
 *  SiteCategory.h
 *  PLEX
 *
 *  A.P. Jason de Koning, jason.de.koning@gmail.com
 *  http://jasondk.org
 *
 *  David Pollock lab, University of Colorado School of Medicine
 *  http://www.evolutionarygenomics.com
 *  All rights reserved.
 
 */


#ifndef SiteCategory__h__
#define SiteCategory__h__

#include "CLAMPTypes.h"

class SiteCategory {
	public:
		double		 gammaShapeParameter_ASRV;
		DoubleVector siteRateDistribution;
		DoubleVector scalingFactors;
	
		IntVector	 siteAssignments;	// for each site, which category is assigned?
		IntVector	 backupAssignments;	// for each site, which category is assigned?
		
	void setGammaDistribution(void);
	void setGammaDistribution(double shapeParm);

	// Constructor:
	SiteCategory(int numSites);

	// Member functions:
	int getBackupAssignment(int site) { return backupAssignments.mDataArray[site]; }
	int getAssignment(int site) { return siteAssignments.mDataArray[site]; }
	void setAssignment( int site, int state ) { siteAssignments[site] = state; }

	double getRate(int which) { return siteRateDistribution.mDataArray[which]; }
	double getShapeParm(void) { return gammaShapeParameter_ASRV; }
	void   setShapeParm(double val) { gammaShapeParameter_ASRV = val; }
	
	void backupSiteAssignments(void) { for (int i=0; i<siteAssignments.size(); i++) { backupAssignments[i] = siteAssignments[i]; } };
	void restoreSiteAssignments(void) { for (int i=0; i<siteAssignments.size(); i++) { siteAssignments[i] = backupAssignments[i]; } };
};

#endif