/*
 *  Tokenizer.h
 *  PLEX
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2005 Dr. David Pollock. All rights reserved.
 *
 */

 /*
 *
 *  set the token inside the tree file of newick format
 */

#ifndef Tokenizer_h__
#define Tokenizer_h__

#include <iosfwd>
#include <memory>
#include <string>

class Tokenizer_Impl;

class Tokenizer
{
	public:
														Tokenizer(void);
														Tokenizer(
															const Tokenizer&	inSrc);
		virtual											~Tokenizer(void);
		
				const Tokenizer&						operator=(
															const Tokenizer&	inSrc);
															
				void									SetStream(
															std::istream&		inStream);
				bool									HasStream(void) const;
		
				bool									ReadToken(
															std::string&		outToken);
				void									PutBackToken(
															const std::string&	inToken);
															
				void									Reset(void);
				
				void									EatWhitespace(void);

				void									SetDelimiters(
															const std::string&	inDelimiters);
				const std::string&						GetDelimiters(void) const;
				bool									IsDelimiter(
															const std::string&	inToken) const;
				
				void									SetWhitespaceChars(
															const std::string&	inWhitespaceChars);
				const std::string&						GetWhitespaceChars(void) const;
				
				void									SetQuoteChars(
															const std::string&	inQuoteChars);
				const std::string&						GetQuoteChars(void) const;
				
				void									SkipWhitespace(
															bool				inSkipWhitespace = true);
				bool									SkippingWhitespace(void) const;
				
				void									RemoveQuotes(
															bool				inRemoveQuotes = true);
				bool									RemovingQuotes(void) const;

	protected:
	private:
				void									Swap(
															Tokenizer&		ioSwapee);
														
				std::auto_ptr<Tokenizer_Impl>	mImpl;
};

#endif // Tokenizer_h__
