/*
 *  Tree.cpp
 *  PLEXlite
 *
 *  Created by A.P. Jason de Koning on 5/3/11.
 *  Copyright 2011 University of Colorado Denver, School of Medicine. All rights reserved.
 *
 */


#include "Tree.h"
#include "MixtureModel.h"
#include "TreeNodeData.h"

//---------------------------------------------------------------------------
#pragma mark -
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------

void
Tree::calculatetreelnlikelihood(
                                          const int& m_nMethod,
                                          const bool& m_bSite,
                                          double& m_Lnlikelihood)
{
	DoubleVector tempFreqModels = GetModelFrequencyVector();
    
	mRoot->CalTreeLnLikelihood(options.m_nMethod, options.m_bSite, tempFreqModels, m_Lnlikelihood);
}


//---------------------------------------------------------------------------

void
Tree::calculatetreelnlikelihooddeltab1(
                                                 int m_nMethod,
                                                 const bool& m_bSite,
                                                 double& m_Lnlikelihood, int theModel, int row)
{
	DoubleVector tempFreqModels = GetModelFrequencyVector();
    
	mRoot->CalTreeLnLikelihoodDeltaB1(options.m_nMethod, options.m_bSite, tempFreqModels, m_Lnlikelihood, theModel, row);
}

//---------------------------------------------------------------------------

/*
void
Tree::setrootsequence(const vector<int>& m_strAncSeq)
{
	mRoot->SetRootSequence(m_strAncSeq);
}


//---------------------------------------------------------------------------

void
Tree::evolvetree(
                           const DoubleMatrix& inRates,
                           const IntVector& inStateindex,
                           const std::vector<char>& inStateVector,
                           const int& minstate,
                           const int& numStates)
{
	mRoot->EvolveTree(inRates, inStateindex, inStateVector, minstate, numStates);
}

//---------------------------------------------------------------------------

void
Tree::evolvetree(
                           ofstream& outStream,
                           const DoubleMatrix& inRates,
                           const IntVector& inStateindex,
                           const std::vector<char>& inStateVector,
                           const int& minstate,
                           const int& numStates)
{
	mRoot->EvolveTree(outStream, inRates, inStateindex, inStateVector, minstate, numStates);
}

//---------------------------------------------------------------------------
*/

void
Tree::updateBranchLength(
                                   const double& m_fWindowsize)
{
	mRoot->UpdateBranchLength(m_fWindowsize);
}

//---------------------------------------------------------------------------
 int Tree::KEEP_updateBranchLength(int startNode,
                                                                         const double& windowSize,
                                                                         MixtureModel& theModel,
                                                                         ObservedData& theData,
                                                                         const IntVector& m_vecSiteAssignment,
                                                                         int version) {
    
    int found=0;
	mRoot->GetData().treeLength = 0;
	
	int dirtyTree = 0;
	mRoot->KEEP_UpdateBranchLength_fromStartNode(startNode, *this, found, 0, windowSize, theModel, theData, m_vecSiteAssignment, version, dirtyTree);
    
	if ( (dirtyTree > 0) && (options.m_nMethod == 6) && (!options.capNumTreeNodes) && (num->Random() < 0.2) ) {
		std::cout << "		[ Re-parsing the tree structure to keep branch-lengths short enough for B1u; there were " << dirtyTree << " segments that exceeded " << options.maxBL << " ]" << endl;
		subdivideLongBranches( mRoot, options.maxBL, theModel.returnUniformizationConstants(), 1 ); // recursive version
		
		// count
		
		// should now recount changes too probably...
	}
    
    return dirtyTree;
}
//---------------------------------------------------------------------------

void
Tree::printbranchlength(
                                  ofstream& outStream)
{
	mRoot->PrintBranchLength(outStream);
}

//---------------------------------------------------------------------------

void
Tree::printbranchlabel(
                                 ofstream& outStream)
{
	mRoot->PrintBranchLabel(outStream);
}

//---------------------------------------------------------------------------

void
Tree::treecopy(
                         Tree& inTarget)
{
	node_type* temp = new node_type;
	
	inTarget.mRoot = temp; 
	
	mRoot->CopyTree(inTarget.mRoot);
}

//---------------------------------------------------------------------------

void
Tree::treedatacopy(
                             Tree& inTarget)
{
	mRoot->CopyTreeData(inTarget.mRoot);
}

//---------------------------------------------------------------------------

inline
void
Tree::printtreechanges(
                                 ofstream& outStream) const
{
	mRoot->PrintTreeChanges(outStream);
}

//---------------------------------------------------------------------------

inline
void
Tree::printtreechangescount(
                                      ofstream& outStream)
{
	mRoot->PrintBranchChangecount(outStream);
}

//---------------------------------------------------------------------------

inline
void
Tree::printtreesitechangescount(
                                          ofstream& outStream)
{
	mRoot->PrintSiteChangecount(outStream);
}

//---------------------------------------------------------------------------

inline
double
Tree::rndu(void)
{
	return Numerics::getGlobalNumerics().Random();
}


//---------------------------------------------------------------------------

inline
int
Tree::calculateDwellTimeOverBranches(int s,
                                               double siteRate,
                                               MixtureModel& theModel,
											   ObservedData& theData,
                                               IntVector& m_vecSiteAssignment)
{
	if ( mRoot->GetData().tempTreeDepth.empty() ) {
		mRoot->GetData().tempTreeDepth.resize( theModel.GetNumModels(), 0.0 );
	} else {
		mRoot->GetData().tempTreeDepth.fill( 0.0 );
	}
	
	double returnVal = mRoot->calculateDwellTimeOverBranches( s, siteRate, theModel, theData, m_vecSiteAssignment );
	
	DoubleMatrix* marginalDwell = mRoot->getMarginalDwell( mRoot );
	DoubleVector* treeDepth = &mRoot->GetData().tempTreeDepth;
	
    //	for (int k=0; k<numModels; k++) {
    for (int i=0; i<theData.getNumStates(); i++) {
        marginalDwell->mRows.mDataArray[ 0 ].mDataArray[ i ] /= (*treeDepth)[0];
        //			std::cout << "Model " << k << ", site " << s << ", state " << i << ", marginal dwell time (depth normalized): " << ( marginalDwell->mRows.mDataArray[ k ].mDataArray[ i ] / treeDepth) << endl;
    }
    //	}
	
	return returnVal;
}

//---------------------------------------------------------------------------

inline
void
Tree::calculateJointDwellTimeOverBranches( int s1, int s2,
                                                    double siteRate1, double siteRate2,
													MixtureModel& theModel,
													ObservedData& theData,
													IntVector& m_vecSiteAssignment)
{
	mRoot->calculateJointDwellTimeOverBranches( s1, s2, siteRate1, siteRate2, theModel, theData, m_vecSiteAssignment );
	
    //	DoubleMatrix* jointDwell = mRoot->getJointDwell( mRoot );
    //	DoubleVector* treeDepth = &mRoot->GetData().tempTreeDepth;
	
    //	for (int k=0; k<numModels; k++) {
    //	std::cout << "DIAG OUTPUT: TREE DEPTH IS " << (*treeDepth)[0] << endl;
	
    //		for (int i=0; i<numStates; i++) {
    //			std::cout << i << ": before scaled " << jointDwell->mRows.mDataArray[ 0 ].mDataArray[ i ]  << " and after ";
    //			jointDwell->mRows.mDataArray[ 0 ].mDataArray[ i ] /= ( (*treeDepth)[0] * 2.0 );
    //			std::cout << jointDwell->mRows.mDataArray[ 0 ].mDataArray[ i ] << endl;
    
    // BLOCKING RESCALE BY TREE DEPTH FOR NOW...
    
    //			std::cout << "Model " << k << ", site " << s << ", state " << i << ", marginal dwell time (depth normalized): " << ( marginalDwell->mRows.mDataArray[ k ].mDataArray[ i ] / treeDepth) << endl;
    //		}
    //	}
}

// --- Uniformized method here... ------

//---------------------------------------------------------------------------

inline
void
Tree::KEEP_CountFullChangesOverTree(
                                              MixtureModel& theModel,
                                              ObservedData& theData,
                                              const IntVector& m_vecSiteAssignment)
{
	KEEP_CountFullChangesOverTree(theModel, theData, m_vecSiteAssignment, 0);
}

//---------------------------------------------------------------------------

inline
void
Tree::KEEP_CountFullChangesOverTree(
                                              MixtureModel& theModel,
                                              ObservedData& theData,
                                              const IntVector& m_vecSiteAssignment,
                                              int version)
{
	// version 0: do all the counting
	// version 1: resample / calculate the virtual subs
	int numStates = theData.getNumStates();
	int numModels = theModel.GetNumModels();
	
	if (options.m_nMethod != 6) std::cout << "ERROR: CountFullChangesOverTreeUni() called for non-uniformized method" << endl;
	mRoot->countChangesAlongBranchUni( theModel, theData, m_vecSiteAssignment, version);
    
	if ( ( options.doB1uStar == 0 ) && (options.m_nMethod == 6) && (version == 0) ) { 
		DoubleMatrix   *augmentedLogSum;
		DoubleMatrix3D *ChangeCount3D;
		DoubleMatrix3D *ChangeCount3D_scaling;
		DoubleMatrix *augmentedLogSum_scaling;
        
		augmentedLogSum			= root()->getAugmentedLogSum( mRoot );
		ChangeCount3D			= root()->getChangeMatrix3Duni( mRoot ); //&node2->GetData().countModelchange3D; //[jdk]
		augmentedLogSum_scaling	= root()->getAugmentedLogSum_scaling( mRoot );
		ChangeCount3D_scaling	= root()->getChangeMatrix3D_scaling( mRoot ); //&node2->GetData().countModelchange3D; //[jdk]
        
		// Take logarithms and subtract off the scaling factors...
		for (int k=0; k<numModels; k++) {
			for (int i=0; i<numStates; i++) {
				for (int j=0; j<numStates; j++) {
					if ((*ChangeCount3D).mRows.mDataArray[k].mRows.mDataArray[i].mDataArray[j] != 0) {
						(*ChangeCount3D).mRows.mDataArray[k].mRows.mDataArray[i].mDataArray[j] = num->logarithm( (*ChangeCount3D).mRows.mDataArray[k].mRows.mDataArray[i].mDataArray[j] ) - (*ChangeCount3D_scaling).mRows.mDataArray[k].mRows.mDataArray[i].mDataArray[j];		
					} else {
						(*ChangeCount3D).mRows.mDataArray[k].mRows.mDataArray[i].mDataArray[j] = num->logarithm( 1e-70 ) - (*ChangeCount3D_scaling).mRows.mDataArray[k].mRows.mDataArray[i].mDataArray[j];		
					}
				}
				if ((*augmentedLogSum).mRows.mDataArray[k].mDataArray[i] != 0) {
					(*augmentedLogSum).mRows.mDataArray[k].mDataArray[i] = num->logarithm( (*augmentedLogSum).mRows.mDataArray[k].mDataArray[i] ) - (*augmentedLogSum_scaling).mRows.mDataArray[k].mDataArray[i];		
				} else {
					(*augmentedLogSum).mRows.mDataArray[k].mDataArray[i] = num->logarithm( 1e-70 ) - (*augmentedLogSum_scaling).mRows.mDataArray[k].mDataArray[i];		
				}
			}
		}
	}
    
}
// ----


//---------------------------------------------------------------------------

inline
int
Tree::KEEP_ResampleSiteAssignments_ASRV_rates(
                                                        MixtureModel& theModel,
                                                        ObservedData& theData,
                                                        IntVector& m_vecSiteAssignment)
{
	int numUpdated = 0;
	int doFullCount = 0;
	//std::cout << "doing gibbs assignment" << endl;
	
    //	setprobmatrixalongbranch(options.m_nMethod, numStates, RateMatrix3D, uniformizationConstants, stateFreqs, 1);
	
	int numStates = theData.getNumStates();
	int numModels = theModel.GetNumModels();
	int numRateModels = options.numCatASRV;
	
	unsigned int i, m;
	
	static DoubleVector LnLikelihoodVector(numRateModels, 1.0);
	static DoubleVector LnLikelihoodVector2(numRateModels, 1.0);
	static DoubleVector m_fLikelihoodRatio(numRateModels, 0);
	static DoubleVector m_fCummulativeRatio(numRateModels, 0);
	
	IntVector* siteAssignments = &root()->GetData().siteCats->siteAssignments;
	
	LnLikelihoodVector.fill(1.0);
	LnLikelihoodVector2.fill(1.0);
	m_fLikelihoodRatio.fill(0);
	m_fCummulativeRatio.fill(0);
	
	if (options.m_bSite)	//assign different models for different sites
	{
		if (siteAssignments->empty()) {
			siteAssignments->resize(theData.getAlignmentLength(), 0);
			KEEP_CountFullChangesOverTree( theModel, theData, m_vecSiteAssignment );
            
		} else {
			// siteAssignments->fill(-1);
		}
		
		if (options.numCatASRV <= 1) {
			return 0;
		}
		
		// Set the P matrix for each model on each branch
		if (options.m_nMethod != 6) {
			int tempCalcProbDist = mRoot->getCalculateFullProbabilityDistribution();
			mRoot->setCalculateFullProbabilityDistribution(1); //override
			setprobmatrixalongbranch( theModel, 1);
			mRoot->setCalculateFullProbabilityDistribution(tempCalcProbDist);				
		} else {
			// We don't need the branch-wise substitution prob matrices for B1u...
			int tempCalcProbDist = mRoot->getCalculateFullProbabilityDistribution();
			mRoot->setCalculateFullProbabilityDistribution(0); // '1' should not be necessary here for B1u
			setprobmatrixalongbranch( theModel, theModel.returnStateFrequencies(), 1 );
			mRoot->setCalculateFullProbabilityDistribution(tempCalcProbDist);				
		}
		
		// THIS STEP IS QUESTIONABLE: BOTH HERE AND IN STANDARD ResampleSiteAssignments_MixtureModel()...
        /*		if (options.m_nMethod == 6) {
         // mModel.updateUniformization();
         // Need to recalculate all the prob matricies...
         int tempCalcProbDist = mRoot->getCalculateFullProbabilityDistribution();
         mRoot->setCalculateFullProbabilityDistribution(1);
         setprobmatrixalongbranch(options.m_nMethod, numStates, RateMatrix3D, uniformizationConstants, stateFreqs, 1);
         mRoot->setCalculateFullProbabilityDistribution(tempCalcProbDist);
         } 
         */
		
        //	std::cout << "assignments: ";
        
		// IS THIS NEEDED?
        //		if (options.m_nMethod == 6) KEEP_CountFullChangesOverTree(options.m_nMethod, m_vecSiteAssignment, numModels, uniformizationConstants, inStateindex, minstate, numStates, RateMatrix3D);	
        //	else KEEP_CountFullChangesOverTree(options.m_nMethod, m_vecSiteAssignment, numModels, inStateindex, minstate, numStates);
        
        
		for (i = 0; i < theData.getAlignmentLength(); i++)
		{
			for (int j=0; j<numRateModels; j++) { LnLikelihoodVector[j] = 1.0; }
			
			// calculate the site specific likelihood (rather than log-likelihood) for each model
			
			// NOTE: THIS IS OVERRIDING THE USE OF THE B1U SPECIFIC FUNCTION FOR CalculateSiteSpecificLnLikelihoodUni_ASRV(), WHICH DOESN'T SEEM TO BE WORKING...
			if (options.m_nMethod != 6) {
				mRoot->CalculateSiteSpecificLnLikelihood_ASRV( i, theModel, theData, m_vecSiteAssignment, LnLikelihoodVector );
			} else {
                // *******************************************************************************************************************************************************
                // The first method uses a site-specific calc that is INTEGRATED OVER VIRTUAL SUBSTITUTIONS...
                // *******************************************************************************************************************************************************
                /*	int tempCalcProbDist = mRoot->getCalculateFullProbabilityDistribution();
                 mRoot->setCalculateFullProbabilityDistribution(1);
                 setprobmatrixalongbranch(options.m_nMethod, numStates, RateMatrix3D, uniformizationConstants, stateFreqs, 1);
                 mRoot->setCalculateFullProbabilityDistribution(tempCalcProbDist);				
                 mRoot->CalculateSiteSpecificLnLikelihood_ASRV(i, theModel, theData, m_vecSiteAssignment, LnLikelihoodVector );
                 double lik1, lik2;
                 lik1 = 0.0;
                 std::cout << "SITE " << i << "..." << endl;
                 for (int ii=0; ii<LnLikelihoodVector.size(); ii++) {
                 lik1 += LnLikelihoodVector[ mRoot->GetData().siteCats->getAssignment(i) ];
                 std::cout << "First method, site lik for " << ii << " is: " << LnLikelihoodVector[ii] << ", vs 0: " << exp( LnLikelihoodVector[ii] - LnLikelihoodVector[0]) << endl;
                 }
                 mRoot->setCalculateFullProbabilityDistribution(0);
                 
                 for (int kk=0; kk<10; kk++) {	
                 setprobmatrixalongbranch(options.m_nMethod, numStates, RateMatrix3D, uniformizationConstants, stateFreqs, 1);
                 // *******************************************************************************************************************************************************
                 
                 
                 KEEP_CountFullChangesOverTree(options.m_nMethod, m_vecSiteAssignment, numModels, uniformizationConstants, inStateindex, minstate, numStates, RateMatrix3D, 1);	
                 */	
				// *******************************************************************************************************************************************************
				// This method uses a delta-logL that is NOT integrated over virtual substitutions: we are keeping them fixed...
                mRoot->CalculateSiteSpecificLnLikelihoodUni_ASRV( i, theModel, theData, m_vecSiteAssignment, LnLikelihoodVector );
                /*		lik2 = 0.0;
                 for (int ii=0; ii<LnLikelihoodVector.size(); ii++) {
                 std::cout << "Second method, site lik for " << ii << " is: " << LnLikelihoodVector[ii]  << ", vs 0: " << exp( LnLikelihoodVector[ii] - LnLikelihoodVector[0]) << endl;
                 lik2+= LnLikelihoodVector[ii];
                 }
                 std::cout << "site lik 1: " << lik1 << endl;
                 std::cout << "site lik 2: " << lik2 << endl;
                 
                 // *******************************************************************************************************************************************************
                 } //exit(-1);
                 */
			}
			
			// Gibbs sampling using the likelihood space of each model
			double logLikelihoodModelMax;
			
			for (m = 0; m < numRateModels; m++)
			{
				if (m== 0)
					logLikelihoodModelMax = LnLikelihoodVector[m];
				else if (LnLikelihoodVector[m] > logLikelihoodModelMax)
					logLikelihoodModelMax = LnLikelihoodVector[m];
			}
			
			m_fLikelihoodRatio.fill(0);
			
			double sum = 0.0;
			
			//[jdk]
			//	std::cout << "logLikelihoodModelMax is " << logLikelihoodModelMax << std::endl;
			for (m = 0; m < numRateModels; m++)
			{
				//[jdk]
                //	if (i==321) std::cout << "	site " << i << "; model likelihood " << m << " is " << LnLikelihoodVector[m] << std::endl;
				//[jdk]		if (LnLikelihoodVector[m] - logLikelihoodModelMax >= -22) {
				// Need the actual likelihood ratio...
				//	TEMP DISABLE
				//		m_fLikelihoodRatio[m] = (LnLikelihoodVector[m] / logLikelihoodModelMax);
				//[jdk]		
				m_fLikelihoodRatio[m] = exp( LnLikelihoodVector[m] - logLikelihoodModelMax);
				//[jdk]		} else
				//[jdk]			m_fLikelihoodRatio[m] = 0;
				
				sum += m_fLikelihoodRatio[m];
			}
			
			for (m = 0; m < numRateModels; m++) m_fCummulativeRatio[m] = 0.0;
			
			for (m = 0; m < numRateModels; m++)
			{
				if (m == 0)
					m_fCummulativeRatio[m] = m_fLikelihoodRatio[m] / sum;
				else
					m_fCummulativeRatio[m] = m_fCummulativeRatio[m-1] + m_fLikelihoodRatio[m] / sum;
				
				//	std::cout << "m_fCummulativeRatio[" << m << "] = " << m_fCummulativeRatio[m] << endl;
			}
			
			// assign each site with the model to the assignment vector
			double rannum = rndu();
			
			int lastAssignment = siteAssignments->mDataArray[i]; //mRoot->GetData().siteCats->getAssignment( i ); // 
            //		std::cout <<  "last assignment at site " << i << " is " << lastAssignment << endl;
			int itWasSet=0;
			for (m = 0; m < numRateModels; m++)
			{
				if (rannum <= m_fCummulativeRatio[m])
				{
					siteAssignments->mDataArray[i] = m;
					//std::cout << m << " ";
					itWasSet = 1;
					break;
				}
			}
			numUpdated++;
			//std::cout << " +++ Chose state " << m << endl;
            
			if (itWasSet == 0) { std::cout << "ERROR! ResampleSiteAssignments_MixtureModel not mode!!!  rannum was " << rannum << "; cum[numRateModels] = " << m_fCummulativeRatio[(numRateModels-1)] << ", numRateModels = " << numRateModels << "; sum = " << sum << endl; }
			
			if ( ( options.m_nMethod == 6) && ( siteAssignments->mDataArray[i] != lastAssignment ) ) {
				// Re-allocate the counts from one category to another...
				if ( lastAssignment < 0 ) {
					doFullCount = 1;
				} else {
					doFullCount = 0;
					mRoot->reassignSiteCountsBetweenRateCategories_Uni(i, lastAssignment, siteAssignments->mDataArray[i], m_vecSiteAssignment, options.m_nMethod, numRateModels, numStates);
				}
			}
            
			//override
			//doFullCount = 1;
            
		}
        //		std::cout << endl;
		
		//itWasSet = 1;
		// calculate the changes for each model in each branches
		//	KEEP_CountFullChangesOverTree(options.m_nMethod, m_vecSiteAssignment, numModels, inStateindex, minstate, numStates);
		if (options.m_nMethod != 6) {
			KEEP_CountFullChangesOverTree(theModel, theData, m_vecSiteAssignment );
		}
		
		// Handling counts separately...
		//else {
		if ( (options.m_nMethod == 6) && (doFullCount == 1) ) {
            // [block]	std::cout << "	[ Rebuilding full substitution count index, since last counts were invalid or unset. ]" << endl;
			KEEP_CountFullChangesOverTree(theModel, theData, m_vecSiteAssignment );
		}
		
	}
	else			//assign different models for different lineages(sequences)
	{
		std::cout << "	[ERROR: lineage-based mixtures not implemented yet for ResampleSiteAssignments_ASRV_rates]" << endl;
		exit(-1);
		// calculate the likelihood of each model in each branch, sampling from the likelihood of each branch
		// and assign each branch to each model
		mRoot->GibbsSamplingLineageAssignment();
	}
	
    
	return numUpdated;
}


//---------------------------------------------------------------------------

void
Tree::calculateCompleteDataLogL(
                                          MixtureModel& theModel,
                                          const IntVector& m_vecSiteAssignment,
                                          double& m_Lnlikelihood, int version)
{
	
	// Argh - something got messed up...
	//mRoot->UpdateBranchLength(numModels, options.m_nMethod, options.m_bSite, m_vecSiteAssignment, m_Lnlikelihood, uniformizationConstants, version);
    
	mRoot->CalcLogLikelihood_branch( theModel, m_vecSiteAssignment, m_Lnlikelihood, version);
}

//---------------------------------------------------------------------------

void
Tree::calculateCompleteDataLogLdeltab1(
                                                 int m_nMethod,
                                                 const bool& m_bSite,
                                                 const IntVector& m_vecSiteAssignment,
                                                 double& m_Lnlikelihood, int theModel, int x)
{
	mRoot->UpdateBranchLengthDeltaB1(options.m_nMethod, options.m_bSite, m_vecSiteAssignment, m_Lnlikelihood, theModel, x);
}


//---------------------------------------------------------------------------

double
Tree::calculateCompleteDataLogLdeltab1uni(
                                                    DoubleMatrix3D *rateMat,
                                                    DoubleVector *uniformizationConstants,
                                                    int m_nMethod,
                                                    const bool& m_bSite,
                                                    const IntVector& m_vecSiteAssignment,
                                                    double& m_Lnlikelihood, int theModel, int x, int y)
{
	return mRoot->UpdateBranchLengthDeltaB1uni(rateMat, uniformizationConstants, options.m_nMethod, options.m_bSite, m_vecSiteAssignment, m_Lnlikelihood, theModel, x, y);
}


//---------------------------------------------------------------------------

int
Tree::UpdateGibbsSampling_B1xM_branchlengths(
                                                       const int& rannode,
                                                       const DoubleMatrix& stateFreqMatrix,
                                                       const IntVector& m_vecSiteAssignment,
                                                       const bool&	m_bSite,
                                                       const IntVector& inStateindex,
                                                       const std::vector<char>& inStateVector,
                                                       const int& minstate,
                                                       const int& numStates)
{
	int found=0;
    //	std::cout << "	call to lookupgibbssampling_B1xM_branchLengths (tree.h); calling mRoot->ResampleAncestralAndTransientStates_local_fromNode..." << endl;
	
	//---------------------------------------------------------------------------		
	// Looking up the random internal node which serves as the start of updation
	//---------------------------------------------------------------------------		
	return mRoot->lookupgibbssampling_B1xM_branchLengths(rannode, found, stateFreqMatrix, m_vecSiteAssignment, options.m_bSite, inStateindex, inStateVector, minstate, numStates); 
}

//---------------------------------------------------------------------------

int
Tree::ResampleAncestralAndTransientStates_local(
														  int& rannode,
														  MixtureModel& theModel,
														  ObservedData& theData,
														  IntVector& m_vecSiteAssignment,
														  DoubleMatrix& stateFreqMatrix) {
	int found=0;
    //	std::cout << "	call to ResampleAncestralAndTransientStates_local (tree.h); calling mRoot->ResampleAncestralAndTransientStates_local_fromNode..." << endl;
	
	//---------------------------------------------------------------------------		
	// Looking up the random internal node which serves as the start of updation
	//---------------------------------------------------------------------------		
	return mRoot->ResampleAncestralAndTransientStates_local_fromNode( rannode, found, theModel, theData, m_vecSiteAssignment, stateFreqMatrix ); 
}

//---------------------------------------------------------------------------

void
Tree::MarkNodesBelow(
                               const int& theNode,
                               const IntVector& m_vecSiteAssignment,
                               const bool&	m_bSite,
                               const IntVector& inStateindex,
                               const std::vector<char>& inStateVector,
                               const int& minstate,
                               const int& numStates)
{
	int found=0;
    //	std::cout << "	call to ResampleAncestralAndTransientStates_local (tree.h); calling mRoot->ResampleAncestralAndTransientStates_local_fromNode..." << endl;
	
	//---------------------------------------------------------------------------		
	// Looking up the random internal node which serves as the start of updation
	//---------------------------------------------------------------------------		
	mRoot->markNodesBelow(theNode, found, m_vecSiteAssignment, options.m_bSite, inStateindex, inStateVector, minstate, numStates); 
}



//---------------------------------------------------------------------------

int
Tree::AssignAncestralStatesByMarginalProbability(MixtureModel& theModel,
														   ObservedData& theData,
														   IntVector& m_vecSiteAssignment,
														   DoubleMatrix& stateFreqMatrix)
{
	// Default to sampling from posterior (mode 1)...
	return AssignAncestralStatesByMarginalProbability( theModel, theData, m_vecSiteAssignment, stateFreqMatrix, 1);
}

 int Tree::AssignAncestralStatesByMarginalProbability(MixtureModel& mModel,
																							ObservedData& theData,
																							IntVector& m_vecSiteAssignment,
																							DoubleMatrix& stateFreqMatrix,
																							int assignMode) {
	
	// When assignMode = 0, only do the complete conditional posterior calculation...
    int numStates = theData.getNumStates();
	int updateCount = 0;
	
	if (!options.m_bSite) {
		std::cout << "ERROR.  Ancestral state reconstruction only checked/implemented for integration by site, for now." << endl;
		exit(-1);
	}
	
	// Initialize the site likelihood data structure
	initSiteLikelihoods( m_vecSiteAssignment, numStates);
    
	// Do the post-order traversal
	mRoot->setSiteLikelihoods_downPass(mRoot, mModel, theData, m_vecSiteAssignment, 0);
    
	/*
     // Old way: calculated marginal ancestral-state probabilities and sample:
     mRoot->setSiteLikelihoods_upPass(mRoot, stateFreqs, GetModelFrequencyVector(), m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates); 
     mRoot->setSiteLikelihoods_doAssignment(mRoot, stateFreqs, GetModelFrequencyVector(), m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates, assignMode); 
     */
	
	// New way: Nielsen (2002) - sampling from posterior dist
	updateCount = mRoot->setSiteLikelihoods_doSimpleAssignment(mRoot, mModel, theData, m_vecSiteAssignment, GetModelFrequencyVector(), assignMode);
    
	double lnL = 0.0;
	double temp;
	int theModel;
	for (int i=0; i<m_vecSiteAssignment.size();i++) {
		theModel = m_vecSiteAssignment.mDataArray[i];
		temp = 0.0;
		for (int j=0; j<numStates; j++) {
			// [?? Using stateFreqs here instead of stationaryFreqs... ??]
			temp += mRoot->GetData().siteLikelihoodBelow[(i*numStates) + j]* mModel.returnStateFrequencies().mRows.mDataArray[theModel].mDataArray[j]; 
		}
		
		if (temp <= 0) temp = 1e-70;
		lnL += num->logarithm(temp);
	}
    
	incompleteDataLnL = lnL;
	return updateCount;
}


double
Tree::simulatePosteriorPredictiveTestStatistic_multinomialLikelihood(
                                                                               const DoubleMatrix& stateFreqs,
                                                                               const IntVector& m_vecSiteAssignment,
                                                                               const bool&	m_bSite,
                                                                               const IntVector& inStateindex,
                                                                               const std::vector<char>& inStateVector,
                                                                               const int& minstate,
                                                                               const int& numStates)
{
	std::map<std::vector<int>,int> sitePatternCounts;
	
	std::vector<int> tipStates;
	tipStates.resize( leaves() );
    
	for (int i=0; i< m_vecSiteAssignment.size(); i++) {
		mRoot->posteriorPredictive_doSimpleAssignment(mRoot, stateFreqs, GetModelFrequencyVector(), m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates, tipStates);
		sitePatternCounts[tipStates]++;
	}
    
	// Now do the test statistic...
	std::map<std::vector<int>,int>::iterator iter;   
	double testStat = 0.0;
	for( iter = sitePatternCounts.begin(); iter != sitePatternCounts.end(); iter++ ) {
		testStat += iter->second * log( iter->second );
	}
	
	testStat -= m_vecSiteAssignment.size() * log( m_vecSiteAssignment.size() );
	//testStat *= m_vecSiteAssignment.size() /5000.0;
	
	std::cout << "Posterior predictive T(X) = " << testStat << " (over " << m_vecSiteAssignment.size() << " sites, " << sitePatternCounts.size() << " unique site patterns)" << endl;
	return testStat;
}


double
Tree::calculateObservedSequenceTestStatistic(
                                                       const DoubleMatrix& stateFreqs,
                                                       const IntVector& m_vecSiteAssignment,
                                                       const bool&	m_bSite,
                                                       const IntVector& inStateindex,
                                                       const std::vector<char>& inStateVector,
                                                       const int& minstate,
                                                       const int& numStates)
{
	std::map<std::vector<int>,int> sitePatternCounts;
	
	std::vector<int> tipStates;
	tipStates.resize( leaves() );
    
	for (int i=0; i< m_vecSiteAssignment.size(); i++) {
		mRoot->posteriorPredictive_getSite(i, mRoot, stateFreqs, GetModelFrequencyVector(), m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates, tipStates);
		sitePatternCounts[tipStates]++;
	}
    
	// Now do the test statistic...
	std::map<std::vector<int>,int>::iterator iter;   
	double testStat = 0.0;
	for( iter = sitePatternCounts.begin(); iter != sitePatternCounts.end(); iter++ ) {
		testStat += iter->second * log( iter->second );
	}
	
	testStat -= m_vecSiteAssignment.size() * log( m_vecSiteAssignment.size() );
	
	//std::cout << "Observed T(X) = " << testStat << " (over " << m_vecSiteAssignment.size() << " sites, " << sitePatternCounts.size() << " unique site patterns)" << endl;
    
	return testStat;
}


void
Tree::AssignFromPosteriorPredictive(
                                              const DoubleMatrix& stateFreqs,
                                              const IntVector& m_vecSiteAssignment,
                                              const bool&	m_bSite,
                                              const IntVector& inStateindex,
                                              const std::vector<char>& inStateVector,
                                              const int& minstate,
                                              const int& numStates, std::vector<int>& tipStates)
{
	if (!m_bSite) {
		std::cout << "ERROR.  Posterior predictive sampling only implemented for integration over sites, for now." << endl;
		exit(-1);
	}
	
	return mRoot->posteriorPredictive_doSimpleAssignment(mRoot, stateFreqs, GetModelFrequencyVector(), m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates, tipStates);
	
}

//---------------------------------------------------------------------------

double 
Tree::getIncompleteDataLikelihood(
                                            const DoubleMatrix& stateFreqs,
                                            const IntVector& m_vecSiteAssignment,
                                            const bool&	m_bSite,
                                            const IntVector& inStateindex,
                                            const std::vector<char>& inStateVector,
                                            const int& minstate,
                                            const int& numStates)
{
	// Likelihood integrated over all ancestral states...
	if (!m_bSite) {
		std::cout << "ERROR.  Ancestral state reconstruction only checked/implemented for integration by site, for now." << endl;
		exit(-1);
	}
	
	return mRoot->incompleteDataLikelihood(mRoot, stateFreqs, GetModelFrequencyVector(), m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates); 
}

//---------------------------------------------------------------------------

void
Tree::initSiteLikelihoods(
                                    IntVector& m_vecSiteAssignment,
                                    int numStates)
{
	mRoot->initSiteLikelihoods(mRoot, m_vecSiteAssignment, numStates, 0, m_vecSiteAssignment.size() );
	
	if (mRoot->GetData().scalingFactor.size() == 0) {
		mRoot->GetData().scalingFactor.resize( m_vecSiteAssignment.size(), 0.0 );
	} else {
		for (int i=0; i< m_vecSiteAssignment.size(); i++) { mRoot->GetData().scalingFactor[i] = 0.0; }
	}
    
	// After the first init, we have set node scaling.  Need to turn off resetting it.
	mRoot->GetData().doScaling = 1; // this means don't reassign scaling to nodes
	
}

//---------------------------------------------------------------------------

void
Tree::initSiteLikelihoods(
									IntVector& m_vecSiteAssignment,
									int numStates,
                                    int startSite,
                                    int endSite)
{
	mRoot->initSiteLikelihoods(mRoot, m_vecSiteAssignment, numStates, startSite, endSite); //m_vecSiteAssignment, inStateindex, inStateVector, minstate,
	if (mRoot->GetData().scalingFactor.size() == 0) {
		std::cout << "WARNING!  SCALINGFACTOR VECTOR AT ROOT MUST BE ALLOCATED BEFORE CALLING THREADED initSiteLikelihoods()" << endl;
	} else {
		for (int i=startSite; i< endSite; i++) { mRoot->GetData().scalingFactor[i] = 0.0; }
	}
	mRoot->GetData().doScaling = 1; // this means don't reassign scaling to nodes
}




void
Tree::insertNodeTest(void) {
	TreeNode* theNewNode[10];
	int numNodes = leaves() + internalnodes() + 1;
	double bl = mRoot->Left()->Left()->GetData().distance;
    
	bl /= 10;
	
	mRoot->Left()->Left()->GetData().distance = bl;
	
    //	std::cout << "inserting a string of nodes above " << mRoot->Left()->Left()->Up()->GetData().id << ".." << mRoot->Left()->Left()->GetData().id << endl;
	for (int i=0; i<9; i++) {
		theNewNode[i] = new TreeNode();
		mRoot->Left()->Left()->insertNodeAbove( theNewNode[i] );
        //		std::cout <<"	Adding node with id " << numNodes << endl;
		theNewNode[i]->GetData().id = numNodes++;
		theNewNode[i]->GetData().distance = bl; // / 10 * rndu();
	}
}


/*
 double
 Tree::discreteBranchLengthApproximation(TreeNode* theNode, double fixedBL) {
 // Use this in conjunction with B1xm (probmethod=3) to represent each branch as an integral number of short branch segments....
 TreeNode* theNewNode[1000];
 // Need an array of segment sizes;
 const int numSegSizes = 3;
 double theSizes[ numSegSizes ];
 
 theSizes[0] = fixedBL;
 //theSizes[1] = 0.01;
 //theSizes[2] = 0.005;
 theSizes[1] = 0.0100246;
 theSizes[2] = 0.000575393;
 
 // Set the global info for discrete branchlengths...
 mRoot->numDiscreteBLs = numSegSizes;
 mRoot->discreteBLs = malloc( sizeof(double) * numSegSizes );
 for (int i=0; i<numSegSizes; i++) { mRoot->discreteBLs[i] = theSizes[i]; }
 
 //	std::cout << "NumNodes at start: leaves = " << leaves() << ", numinternal = " << internalnodes() << endl;
 int numNodes;
 static double loss = 0.0;
 
 double bl = theNode->GetData().distance;
 
 if ( (theNode != NULL) ) {
 if (theNode->Left() != NULL) {
 discreteBranchLengthApproximation(theNode->Left(), fixedBL);
 }
 
 if (theNode->Right() != NULL) {
 discreteBranchLengthApproximation(theNode->Right(), fixedBL);
 }
 
 numNodes = leaves() + internalnodes() + 1;
 if ( (theNode->GetData().id != mRoot->GetData().id) && ( theNode->Up() != theNode->Up()->Up() ) ) {
 int totalNodeCount = 1;
 double newsize = theNode->GetData().distance;
 // [want to not go over, for now] totalNodeCount = round( newsize / fixedBL );
 totalNodeCount = floor( newsize / fixedBL );
 // [blocking this - want the minSize if anything] if (totalNodeCount == 0) { totalNodeCount = 1; }
 
 std::cout << "Subdividing branch: " << theNode->Up()->GetData().id << ".." << theNode->GetData().id << " using " << totalNodeCount << " segments; remainder is: " << (newsize -  (totalNodeCount * fixedBL)) << endl;
 if (numSegSizes <= 1) loss += (newsize -  (totalNodeCount * fixedBL)) * (newsize -  (totalNodeCount * fixedBL));
 
 theNode->GetData().distance = fixedBL;
 if (totalNodeCount == 0) theNode->GetData().distance = 0.0; //totalNodeCount=1; // 
 
 if (totalNodeCount > 1000) { std::cout << "WARNING!  Hard-coded limit violated on the number of node pointers!  Fix this." << endl; exit(-1); }
 
 for (int i=0; i<(totalNodeCount-1); i++) {
 theNewNode[i] = new TreeNode();
 theNewNode[i]->GetData().id = numNodes++;
 theNewNode[i]->GetData().distance = fixedBL;
 theNewNode[i]->GetData().branchLengthIndex = 0;
 theNewNode[i]->GetData().sequence.Mix(&theNode->Up()->GetData().sequence, &theNode->GetData().sequence);
 theNode->insertNodeAbove( theNewNode[i] );
 }
 
 int lastNodeCount = 0;
 // Now need to add extra branches of different sizes...
 double remainder = (newsize -  (totalNodeCount * fixedBL));
 for (int j=1; j<numSegSizes; j++) {
 //std::cout << "---SEGSIZE " << j << "---" << endl;
 
 lastNodeCount = totalNodeCount;
 //std::cout << "	Last orig full-bl was " << newsize << "; remainder to work with is " << remainder << std::endl;
 
 newsize = remainder;
 // [want to not go over, for now] totalNodeCount = round( newsize / fixedBL );
 totalNodeCount = floor( newsize / theSizes[j] );
 //std::cout << "	remainder is " << newsize << ", the segsize is " << theSizes[j] << endl;
 
 // [blocking this - want the minSize if anything] if (totalNodeCount == 0) { totalNodeCount = 1; }
 
 remainder = (remainder -  (totalNodeCount * theSizes[j]));
 
 //" = " << theNode->GetData().name << ";
 std::cout << "	Further subdividing branch (using " << theSizes[j] << "): " << theNode->Up()->GetData().id << ".." << theNode->GetData().id << " using " << totalNodeCount << " more segments; new remainder is: " << remainder << endl;
 if (j == (numSegSizes-1)) { 
 loss += (remainder * remainder);
 }
 
 theNode->GetData().distance = theSizes[j];
 if (totalNodeCount == 0) theNode->GetData().distance = 0.0; //totalNodeCount=1; // 
 
 if (totalNodeCount > 1000) { std::cout << "WARNING!  Hard-coded limit violated on the number of node pointers!  Fix this." << endl; exit(-1); }
 
 if (lastNodeCount == 0) { lastNodeCount = 1; }
 
 for (int i=lastNodeCount-1; i<((lastNodeCount-1)+(totalNodeCount)); i++) {
 //		std::cout << "	-- STARTING SUBDIV NODE ADDING AT " << (lastNodeCount-1) << endl;
 theNewNode[i] = new TreeNode();
 theNewNode[i]->GetData().id = numNodes++;
 theNewNode[i]->GetData().distance = theSizes[j];
 theNewNode[i]->GetData().branchLengthIndex = j;
 theNewNode[i]->GetData().sequence.Mix(&theNode->Up()->GetData().sequence, &theNode->GetData().sequence);
 theNode->insertNodeAbove( theNewNode[i] );
 //		std::cout << "			adding secondary subdivision of len " << theSizes[j] << "; id is " << (numNodes-1) << endl;
 }
 
 
 }
 // Debug: output node info...
 
 } // not root
 else {
 // Root
 theNode->GetData().distance = 0.0; //fixedBL;
 }
 } // node not null
 
 return loss;
 }
 */


double
Tree::returnLossFromBLApproximation(TreeNode* theNode, int numSizes, double *theSizes) {
	// This has to be called BEFORE doing the subdivision...
	for (int i=0; i<numSizes; i++) { if (theSizes[i] <= 0.005) { theSizes[i] = 0.005; } }
	for (int i=0; i<numSizes; i++) { for (int j=i+1; j<numSizes; j++) { if (theSizes[j] > theSizes[i]) { theSizes[j] = theSizes[i] - 0.0001; } } }
	
	int numNodes;
	static double loss = 0.0;
	static int segCount = 0;
	if (theNode == mRoot) { segCount = 0; loss = 0.0; }
	
	double bl = theNode->GetData().distance;
    
	if ( (theNode != NULL) ) {
		if (theNode->Left() != NULL) {
			returnLossFromBLApproximation(theNode->Left(), numSizes, theSizes);
		}
		
		if (theNode->Right() != NULL) {
			returnLossFromBLApproximation(theNode->Right(), numSizes, theSizes);
		}
        
		numNodes = leaves() + internalnodes() + 1;
		if ( (theNode->GetData().id != mRoot->GetData().id) && ( theNode->Up() != theNode->Up()->Up() ) ) {
			int totalNodeCount = 1;
			double newsize = theNode->GetData().distance;
            
			totalNodeCount = floor( newsize / theSizes[0] );
			
			if (numSizes <= 1) loss += (newsize -  (totalNodeCount * theSizes[0])) * (newsize -  (totalNodeCount * theSizes[0]));
			segCount++;
			
			int lastNodeCount = 0;
			// Now need to add extra branches of different sizes...
			double remainder = (newsize -  (totalNodeCount * theSizes[0]));
			for (int j=1; j<numSizes; j++) {
				segCount++;
				lastNodeCount = totalNodeCount;
				newsize = remainder;
				totalNodeCount = floor( newsize / theSizes[j] );
				remainder = (remainder -  (totalNodeCount * theSizes[j]));
                
				if (j == (numSizes-1)) { 
					loss += (remainder * remainder);
				}
				
				if (lastNodeCount == 0) { lastNodeCount = 1; }
			}
		} // not root
		else {
			// Root
			// do Nothing!
		}
	} // node not null
    
    //	if ( (theNode->Left() == NULL) && (theNode->Right() == NULL) ) std::cout << "Loss: " << loss << "; segCount: " << segCount << endl;
	return (loss*segCount);
}




inline
int
Tree::ResampleSiteAssignments_MixtureModel(MixtureModel& mModel,
                                                     ObservedData& theData,
                                                     IntVector& m_vecSiteAssignment
                                                     ) {
    
    int numModels = mModel.GetNumModels();
    int numStates = theData.getNumStates();
	int m_nAlignLength = theData.getAlignmentLength();
	
	// Optimized by Jason de Koning (Sep 21, 2007)
	int numUpdated = 0;
	// set the P matrix for each model on each branch
	
	//std::cout << "[NEEDS FIX] GIBBS ASSIGNMENT caling setProbMatrix.." << endl;
	if (options.m_nMethod != 6) setprobmatrixalongbranch(mModel, mModel.returnStateFrequencies(), 1);
    
	// int numModels = RateMatrix3D.rows();
	unsigned int i, m;
    
	static DoubleVector LnLikelihoodVector(numModels, 1.0);
	static DoubleVector LnLikelihoodVector2(numModels, 1.0);
	static DoubleVector m_fLikelihoodRatio(numModels, 0);
	static DoubleVector m_fCummulativeRatio(numModels, 0);
    
	LnLikelihoodVector.fill(1.0);
	LnLikelihoodVector2.fill(1.0);
	m_fLikelihoodRatio.fill(0);
	m_fCummulativeRatio.fill(0);
	
	if (options.m_bSite)	//assign different models for different sites
	{
        // There seems to be a memory problem with the allocation of this vector.  Forcing resize:
		if (m_vecSiteAssignment.empty()) {
			m_vecSiteAssignment.resize(m_nAlignLength, 0);
			KEEP_CountFullChangesOverTree( mModel, theData, m_vecSiteAssignment );
		}
	} else {
		// [not needed]		m_vecSiteAssignment.fill(-1);
	}
	
	if (numModels <= 1) {
		return -1;
	}
	
	if (options.m_nMethod == 6) {
		// mModel.updateUniformization();
		// Need to recalculate all the prob matricies...
		int tempCalcProbDist = mRoot->getCalculateFullProbabilityDistribution();
		mRoot->setCalculateFullProbabilityDistribution(1);
		setprobmatrixalongbranch(mModel, mModel.returnStateFrequencies(), 1);
		mRoot->setCalculateFullProbabilityDistribution(tempCalcProbDist);
	} 
	
	for (i = 0; i < theData.getAlignmentLength(); i++)
	{
		for (int j=0; j<numModels; j++) { LnLikelihoodVector[j] = 1.0; }
		
		// calculate the site specific likelihood (rather than log-likelihood) for each model
		
		int count1; 
		
		if (options.m_nMethod != 6) {
			mRoot->CalculateSiteSpecificLnLikelihood(options.m_nMethod, numModels, i, LnLikelihoodVector, theData );
		} else {
			count1 = mRoot->CalculateSiteSpecificLnLikelihood(options.m_nMethod, numModels, i, LnLikelihoodVector, theData );
			//				count2 = mRoot->CalculateSiteSpecificLnLikelihood_B1u(options.m_nMethod, numModels, i, &LnLikelihoodVector, &inStateindex, minstate, numStates, &RateMatrix3D, stateFreqs, uniformizationConstants);
		}
		//			std::cout << "MODEL " << j << ", Likelihoods for site " << i << ": " <<  ( LnLikelihoodVector[0] - LnLikelihoodVector[1]) << " and " << (LnLikelihoodVector2[0] - LnLikelihoodVector2[1]) << endl;
		
		// gibbs sampling using the likelihood space of each model
		double logLikelihoodModelMax;
		
		for (m = 0; m < numModels; m++)
		{
			if (m== 0)
				logLikelihoodModelMax = LnLikelihoodVector[m];
			else if (LnLikelihoodVector[m] > logLikelihoodModelMax)
				logLikelihoodModelMax = LnLikelihoodVector[m];
		}
		
		m_fLikelihoodRatio.fill(0);
		
		double sum = 0.0;
		
		for (m = 0; m < numModels; m++) {
			m_fLikelihoodRatio[m] = exp( LnLikelihoodVector[m] - logLikelihoodModelMax);
			sum += m_fLikelihoodRatio[m];
		}
		
		for (m = 0; m < numModels; m++) m_fCummulativeRatio[m] = 0.0;
		
		for (m = 0; m < numModels; m++) {
			if (m == 0)
				m_fCummulativeRatio[m] = m_fLikelihoodRatio[m] / sum;
			else
				m_fCummulativeRatio[m] = m_fCummulativeRatio[m-1] + m_fLikelihoodRatio[m] / sum;
		}
		
		// assign each site with the model to the assignment vector
		double rannum = rndu();
		
		//		int lastAssignment = m_vecSiteAssignment[i];
		
		int itWasSet=0;
		for (m = 0; m < numModels; m++) {
			if (rannum <= m_fCummulativeRatio[m]) {
				m_vecSiteAssignment[i] = m;
				itWasSet = 1;
				break;
			}
		}
		numUpdated++;
		
		if (itWasSet == 0) { std::cout << "ERROR! ResampleSiteAssignments_MixtureModel not mode!!!  rannum was " << rannum << "; cum[numModels] = " << m_fCummulativeRatio[(numModels-1)] << ", numModels = " << numModels << "; sum = " << sum << endl; }
	}
	
	KEEP_CountFullChangesOverTree(mModel, theData, m_vecSiteAssignment );
    
	return numUpdated;
}

//---------------------------------------------------------------------------

void
Tree::setprobmatrixalongbranch(MixtureModel& mModel,
                                         const DoubleVector& inModelFreqVector) {
	setprobmatrixalongbranch( mModel, inModelFreqVector, -1, -1);
}


void
Tree::setprobmatrixalongbranch(
                                         MixtureModel& mModel,
                                         const DoubleVector& inModelFreqVector, 
                                         int theModel, 
                                         int row)
{
    int numStates = mModel.GetNumStates();
    
	SetModelFrequencyVector(inModelFreqVector);
    
	if (options.m_nMethod == 0) {
		if (theModel != -1) {
			std::cout << "ERROR!  setprobmatrixalongbranch not implemented for row specific calcs in BL yet." << endl; exit(-1);
		} else {
			mRoot->SetTreeBLProb(numStates, mModel.returnRateMatrix3D() );
		}
	} else if ( (options.m_nMethod == 1) || (options.m_nMethod == 3)) {
		if (theModel != -1) {
			mRoot->SetTreeB1Prob(options.m_nMethod, numStates, mModel.returnRateMatrix3D() , theModel, row);
		} else {
			mRoot->SetTreeB1Prob(options.m_nMethod, numStates, mModel.returnRateMatrix3D() , -1, -1);
		}
	} else if (options.m_nMethod == 6) {
        if (theModel != -1) {
            mRoot->SetTreeB1uProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D() , mModel.returnUniformizationConstants(), theModel, row);
        } else {
            mRoot->SetTreeB1uProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D() , mModel.returnUniformizationConstants(), -1, -1);
        }
 	} else if (options.m_nMethod == 2) {
		if (theModel != -1) {
			std::cout << "ERROR!  setprobmatrixalongbranch not implemented for row specific calcs in B2 yet." << endl; exit(-1);
		} else {
			mRoot->SetTreeB2Prob(numStates, mModel.returnRateMatrix3D());
		}
	} else if ( (options.m_nMethod == 4) || (options.m_nMethod == 5) ) {
		// 4 = Binf, 5 = BinfxM
		if (theModel != -1) {
			mRoot->SetTreeBinfProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnStateFrequencies(), theModel);
		} else {
			mRoot->SetTreeBinfProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnStateFrequencies(), -1);
		}	
	}
	else
		throw(9);
}

//---------------------------------------------------------------------------

void
Tree::setprobmatrixalongbranch(MixtureModel& mModel,
                                         const DoubleVector& inModelFreqVector, 
                                         int dummyGibbs) {	
    
	setprobmatrixalongbranch( mModel, inModelFreqVector, dummyGibbs, -1, -1 );
    
}


void
Tree::setprobmatrixalongbranch(
                                         MixtureModel& mModel,
                                         const DoubleVector& inModelFreqVector, 
                                         int dummyGibbs, 
                                         int theModel, 
                                         int row)
{	
	int numStates = mModel.GetNumStates();
	
	SetModelFrequencyVector(inModelFreqVector);
    
	// set probability calculation method here
	if (options.m_nMethod == 0) {
		if (theModel != -1) {
			std::cout << "ERROR!  setprobmatrixalongbranch not implemented for row specific calcs in BL yet." << endl; exit(-1);
		} else {
			mRoot->SetTreeBLProb(numStates, mModel.returnRateMatrix3D(), dummyGibbs);
		}
	} else if ( (options.m_nMethod == 1) || (options.m_nMethod == 3)) {
		if (theModel != -1) {
			mRoot->SetTreeB1Prob(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), dummyGibbs, theModel, row);
		} else {
			mRoot->SetTreeB1Prob(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), dummyGibbs);
		}
	} else if ( (options.m_nMethod == 6)) {
        //	if ((mRoot->getCalculateFullProbabilityDistribution() == 0) ) {
        if (theModel != -1) {
            mRoot->SetTreeB1uProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnUniformizationConstants(), dummyGibbs, theModel, row);
        } else {
            mRoot->SetTreeB1uProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnUniformizationConstants(), dummyGibbs);
        }
 	} else if (options.m_nMethod == 2) {
		if (theModel != -1) {
			std::cout << "ERROR!  setprobmatrixalongbranch not implemented for row specific calcs in B2 yet." << endl; exit(-1);
		} else {
			mRoot->SetTreeB2Prob(numStates, mModel.returnRateMatrix3D(), dummyGibbs);
		}
	} else if ( (options.m_nMethod == 4) || (options.m_nMethod == 5) ) {
		// 4 = Binf, 5 = BinfxM
		if (theModel != -1) {
			mRoot->SetTreeBinfProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnStateFrequencies(), theModel);
		} else {
			mRoot->SetTreeBinfProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnStateFrequencies(), -1);
		}	
	}
	else
		throw(9);
}

//---------------------------------------------------------------------------

void
Tree::setprobmatrixalongbranch(MixtureModel& theModel) {
	setprobmatrixalongbranch( theModel, -1, -1 );
}


void
Tree::setprobmatrixalongbranch(MixtureModel& mModel,
                                         int theModel,
                                         int row) {
    //	std::cout << "setprobmatrixalongbranch" << endl;
    //	std::cout << "[CALL TO setprobmatrixalongbranch 2]" << endl;
    int numStates = mModel.GetNumStates();
	
	if (options.m_nMethod == 0) {
		if (theModel != -1) {
			std::cout << "ERROR!  setprobmatrixalongbranch not implemented for row specific calcs in BL yet." << endl; exit(-1);
		} else {
			mRoot->SetTreeBLProb(numStates, mModel.returnRateMatrix3D());
		}
	} else if ( (options.m_nMethod == 1) || (options.m_nMethod == 3) )  {
		if (theModel != -1) {
			mRoot->SetTreeB1Prob(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), theModel, row);
		} else {
			mRoot->SetTreeB1Prob(options.m_nMethod, numStates, mModel.returnRateMatrix3D());
		}
	} else if ( (options.m_nMethod == 6) )  {
		//		std::cout << "ERROR: This version of setprobmatrixalongbranch requries uniformization constants for probmethod #6" << endl;
		//		exit(-1);
        //	if ((mRoot->getCalculateFullProbabilityDistribution() == 0) ) {
        if (theModel != -1) {
            mRoot->SetTreeB1uProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnUniformizationConstants(), theModel, row);
        } else {
            mRoot->SetTreeB1uProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnUniformizationConstants());
        }
        /*	} else {
         std::cout << "CALCULATING FULL PROBABILITY DISTRIBUTION USING STANDARD B1..." << endl;
         if (theModel != -1) {
         mRoot->SetTreeB1Prob(options.m_nMethod, numStates, inRateMatrix3D, theModel, row);
         } else {
         mRoot->SetTreeB1Prob(options.m_nMethod, numStates, inRateMatrix3D);
         }
         }
         */
	} else if (options.m_nMethod == 2) {
		if (theModel != -1) {
			std::cout << "ERROR!  setprobmatrixalongbranch not implemented for row specific calcs in B2 yet." << endl; exit(-1);
		} else {
			mRoot->SetTreeB2Prob(numStates, mModel.returnRateMatrix3D());
		}
	} else if ( (options.m_nMethod == 4) || (options.m_nMethod == 5) ) {
		// 4 = Binf, 5 = BinfxM
		if (theModel != -1) {
			mRoot->SetTreeBinfProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnStateFrequencies(), theModel);
		} else {
			mRoot->SetTreeBinfProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnStateFrequencies(), -1);
		}	
	}
	else
		throw(9);
}

//---------------------------------------------------------------------------

void
Tree::setprobmatrixalongbranch(MixtureModel& mModel,
                                         int dummyGibbs) {
	setprobmatrixalongbranch( mModel, dummyGibbs, -1, -1);
}


void
Tree::setprobmatrixalongbranch(MixtureModel& mModel,
                                         int dummyGibbs,
                                         int theModel,
                                         int row) {
    
	int numStates = mModel.GetNumStates();
	
	if (options.m_nMethod == 0) {
		if (theModel != -1) {
			std::cout << "ERROR!  setprobmatrixalongbranch not implemented for row specific calcs in BL yet." << endl; exit(-1);
		} else {
			mRoot->SetTreeBLProb(numStates, mModel.returnRateMatrix3D(), dummyGibbs);
		}
	} else if ((options.m_nMethod == 1) || (options.m_nMethod == 3)) {
		if (theModel != -1) {
			mRoot->SetTreeB1Prob(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), dummyGibbs, theModel, row);
		} else {
			mRoot->SetTreeB1Prob(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), dummyGibbs);
		}
	} else if ((options.m_nMethod == 6)) {
        if (theModel != -1) {
            mRoot->SetTreeB1uProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnUniformizationConstants(), dummyGibbs, theModel, row);
        } else {
            mRoot->SetTreeB1uProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnUniformizationConstants(), dummyGibbs);
        }
	} else if (options.m_nMethod == 2) {
		if (theModel != -1) {
			std::cout << "ERROR!  setprobmatrixalongbranch not implemented for row specific calcs in B2 yet." << endl; exit(-1);
		} else {
			mRoot->SetTreeB2Prob(numStates, mModel.returnRateMatrix3D(), dummyGibbs);
		}
	} else if ( (options.m_nMethod == 4) || (options.m_nMethod == 5) ) {
		// 4 = Binf, 5 = BinfxM
		if (theModel != -1) {
			mRoot->SetTreeBinfProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnStateFrequencies(), theModel);
		} else {
			mRoot->SetTreeBinfProb(options.m_nMethod, numStates, mModel.returnRateMatrix3D(), mModel.returnStateFrequencies(), -1);
		}	
	}
	else
		throw(9);
}

