/*
 *  TreeNodeData_.h
 *  PLEX
 *
 *  A.P. Jason de Koning, jason.de.koning@gmail.com
 *  http://jasondk.org
 *
 *  David Pollock lab, University of Colorado School of Medicine
 *  http://www.evolutionarygenomics.com
 *  All rights reserved.
 
 */


/*
 *  This class defines the data type of each tree node
 */

#ifndef TreeNodeData__h__
#define TreeNodeData__h__

//include "Sequence.h"
#include "SiteCategory.h"
#include "CLAMPTypes.h"
#include "macrodefine.h"

class TreeNodeData_ {
	public:
		char name[MAXNAME];		/* node name */
		double distance;		/* associated with up branch */
		
		int id; //,tips;	  	/* node identification; number of tips below */
		std::vector<int> sequence;		/* sequence information */
		IntMatrix countchange;		/* for change of each states with up branch*/
		
		DoubleMatrix3D probmatrix3D;	/* the probability matrix of the up node to current node*/
		DoubleMatrix3D *minimalBLprobmatrix3D;	// this allows each mixture component to have a minimal (but non-zero) model for B1xM
		
		DoubleMatrix3D countModelchange3D;	/* the changes of sites assigned to different models */
		DoubleMatrix   augmentedCounts;
	
		IntVector	 totalCount_actualSubs;

	// [don't need this]	DoubleMatrix expectedVirtualSubsByRateCat;		// for rate, state
	
		DoubleMatrix augmentedLogSum;
		DoubleMatrix3D countModelchange3Duni;	// partial change count path stuff
	
		DoubleMatrix augmentedLogSum_scaling;
		DoubleMatrix3D ChangeCount3D_scaling;

		DoubleVector tempTreeDepth;
		DoubleMatrix marginalDwellTimes;
		DoubleMatrix observedFreqs;
		DoubleMatrix jointDwellTimes;
	
		double C0, C1;
		DoubleMatrix Cxx; // this will hold counts of Cxx events for each model type
	
		double treeLength;
	
		Vector<DoubleMatrix> countModelchange_perSite;	/* the changes of sites assigned to different models */

		// Need these for ancestral state reconstructions...
		double *siteLikelihoodBelow, *siteLikelihoodAbove, *siteLikelihoodBelow_andAbove; //, *siteLikelihoodL, *siteLikelihoodR;
		int siteLallocatedFlag;
		Vector<int> seqBackup;
		
		int posteriorPredictiveAssignment;	// so we can simulate a site from the posterior predictive distribution
		int branchLengthIndex;
		int modelIndex;
		
		int UVRallocated;
		double *U, *V, *Root; // these are to be used at the root only
		
		SiteCategory *siteCats;
	
			// int marked;	// temp for temporal
		
		int doScaling;
		DoubleVector scalingFactor;
		
	void allocateForSiteLikelihoods(int numSites, int numStates) { if (siteLallocatedFlag ==1) { return; } siteLallocatedFlag = 1; siteLikelihoodBelow = new double[numStates * numSites];  } 
		// siteLikelihoodBelow_andAbove = new double[numStates * numSites]; siteLikelihoodAbove =  new double[numStates * numSites];
		// siteLikelihoodL =  new double[numStates * numSites]; siteLikelihoodR =  new double[numStates * numSites]; }

	void deallocateSiteLikelihoods(void) { delete [] siteLikelihoodBelow; } 
		//, siteLikelihoodAbove, siteLikelihoodBelow_andAbove; } //, siteLikelihoodL, siteLikelihoodR; }
		
		TreeNodeData_(void);

		TreeNodeData_(
					const TreeNodeData_& inSrc);

		const TreeNodeData_&	operator=(
					const TreeNodeData_& inSrc);

	private:
} ;

#endif // TreeNodeData__h__
