/*
 *  Vector.h
 *  PLEX
 *
 *  Created by Chip Jarred.
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 *
 */

/*
 *  this class the vector class and manipulation of vector
 */

#ifndef Vector_h__
#define Vector_h__

#include <algorithm>
#include <cassert>

#include "AbstractVector.h"

//---------------------------------------------------------------------------
template <typename DataType>
class Vector : public AbstractVector<DataType>
{
	public:
		typedef typename AbstractVector<DataType>::size_type	size_type;
		
		struct Initializer
		{
			typedef typename AbstractVector<DataType>::size_type size_type;
			
			size_type	fNumItems;
			DataType	fInitValue;
			
						Initializer(
							size_type			inNumItems = 0,
							const DataType&		inInitValue = DataType())
							:	fNumItems(inNumItems),
								fInitValue(inInitValue)
						{
						}
																	
			bool		operator<(
							const Initializer& inInitializer) const
						{
							return fNumItems < inInitializer.fNumItems;
						}
		};
		
										Vector(void);
										Vector(
											size_type		inNumItems,
											const DataType&	inVal = DataType());
										Vector(
											const Vector&	inVector);
										Vector(
											const Initializer&	inInitializer);
	virtual								~Vector(void);
	
			const Vector&				operator=(
											const Vector&	inVector);
											
			const Vector&					operator+(
												const Vector&		inVector);
			const Vector&					operator+=(
												const Vector&		inVector);
	virtual	void						resize(
											size_type		inNewSize,
											const DataType&	inVal = DataType());
	virtual	void						fill(
											const DataType&	inVal);
											
	virtual	void						clear(void);

	virtual	DataType&					operator[](
											size_type		inIndex);
											
	virtual	const DataType&				operator[](
											size_type		inIndex) const;
											
			void						swap(
											Vector&			ioVector);
		
			void						Initialize(
											const Initializer&	inInitializer);
											
//			DataType* returnHandle(void);
			DataType*					mDataArray;
			
	private:
										
			size_type					mNumItems;
};

//template <typename DataType> DataType* Vector<DataType>::returnHandle(void) {
//	return &mDataArray;
//}

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
Vector<DataType>::Vector(void)
	:	AbstractVector<DataType>(),
		mDataArray(0)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
Vector<DataType>::Vector(
	size_type		inNumItems,
	const DataType&	inVal)
	:	AbstractVector<DataType>(),
		mNumItems(0),
		mDataArray(0)
{
	resize(inNumItems, inVal);
}

//---------------------------------------------------------------------------
template <typename DataType>
Vector<DataType>::Vector(
	const Vector&	inVector)
	:	AbstractVector<DataType>(inVector),
		mNumItems(0),
		mDataArray(0)
{
	DataType*	temp = 0;
	size_type	numItems = inVector.size();
	
	try
	{
		temp = new DataType[numItems];
		std::copy(&inVector.mDataArray[0], &inVector.mDataArray[numItems], &temp[0]);
	}
	catch(...)
	{
		delete [] temp;
	}
	
	std::swap(temp, mDataArray);
	delete [] temp;
}

//---------------------------------------------------------------------------
template <typename DataType>
Vector<DataType>::Vector(
	const Initializer&	inInitializer)
	:	AbstractVector<DataType>(),
		mNumItems(0),
		mDataArray(0)
{
	resize(inInitializer.fNumItems, inInitializer.fInitValue);
}

#include <iostream>
//---------------------------------------------------------------------------
template <typename DataType>
Vector<DataType>::~Vector(void)
{
//	std::cout << "Vector desctructor called" << std::endl;
//	std::cout << "Break here" << std::endl;
	delete [] mDataArray;	// Note: C++ standard says this is OK to do on NULL
}

//---------------------------------------------------------------------------
template <typename DataType>
void 
Vector<DataType>::clear(void)
{
	mNumItems = 0;
	delete [] mDataArray;	// Note: C++ standard says this is OK to do on NULL
}

//---------------------------------------------------------------------------
template <typename DataType>
void
Vector<DataType>::Initialize(
	const Initializer&	inInitializer)
{
	fill(inInitializer.fInitValue);
	resize(inInitializer.fNumItems, inInitializer.fInitValue);
}

//---------------------------------------------------------------------------

template <typename DataType> inline const Vector<DataType>& Vector<DataType>::operator=( const Vector&	inVector) {
	//Vector	temp(inVector);
	//swap(temp);
	
	if ( this->size() != inVector.size()) {
		resize( inVector.size() );
	}
	
	for (int i=0; i < this->size(); i++) {
		mDataArray[i] = inVector.mDataArray[i];
	}

	return *this;
}

#include<iostream>
//---------------------------------------------------------------------------
template <typename DataType>
inline
DataType&
Vector<DataType>::operator[](
	size_type	inIndex)
{
// JdK: blocking this - this check is extremely costly!

//	if (inIndex > size()) {
//		std::cout << "ERROR! vector[] bounds exceeded" << std::endl;
//		std::cout << "Stop now" << std::endl;
//	}
	
//	assert(inIndex < size());
//	std::cout << "DEPRECATING THIS" << std::endl;
//	std::cout << "BREAK" << std::endl;
	
	return mDataArray[inIndex];
}

#include <iostream>
//---------------------------------------------------------------------------
template <typename DataType>
inline
const DataType&
Vector<DataType>::operator[](
	size_type	inIndex) const
{

//	if (inIndex > size()) {
//		std::cout << "OH OH! inIndex is " << inIndex << ", size is " << size() << std::endl;
//		exit(-1);
//	}
//	assert(inIndex < size());
//	std::cout << "DEPRECATING THIS" << std::endl;
//	std::cout << "BREAK" << std::endl;
	
	return mDataArray[inIndex];
}

//---------------------------------------------------------------------------
template <typename DataType> inline const Vector<DataType>& Vector<DataType>::operator+( const Vector&	inVector) {
	const size_type	numItems = this->size();
	
	if (inVector.size() == numItems)
	{
		for (size_type row = 0; row < numItems; ++row)
			mDataArray[row] += inVector.mDataArray[row];
	}

	return *this;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
const Vector<DataType>&
Vector<DataType>::operator+=(
	const Vector&	inVector)
{
	return (*this + inVector);
}

//---------------------------------------------------------------------------
template <typename DataType>
void
Vector<DataType>::resize(
	size_type		inNewSize,
	const DataType&	inVal)
{
	size_type numItems = this->size();
	if (inNewSize != numItems)
	{
		//  Only re-allocate if we need more space
		if (inNewSize > numItems)
		{
			DataType*	newArray = 0;
			
			try
			{
				newArray = new DataType[inNewSize];
				
				size_type	endCopyIndex = std::min(inNewSize, numItems);
				std::copy(&mDataArray[0], &mDataArray[endCopyIndex], &newArray[0]);
				
				if (inNewSize > endCopyIndex)
					std::fill(&newArray[endCopyIndex], &newArray[inNewSize], inVal);
			}
			catch(...)
			{
				delete newArray;
			}
			
			std::swap(newArray, mDataArray);
			delete [] newArray;
		}
		
		AbstractVector<DataType>::resize(inNewSize);
	}
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
Vector<DataType>::fill(
	const DataType&	inVal)
{
	// Max entry was using mNumItems, but this appears to sometimes not be correct..not sure why.  replacing with size()...
	std::fill(&mDataArray[0], &mDataArray[this->size()], inVal);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
Vector<DataType>::swap(
	Vector&	inVector)
{
	AbstractVector<DataType>::swap(inVector);
	std::swap(inVector.mDataArray, mDataArray);
}



#endif // Vector_h__
