/*
 *  TreeNode.h
 *  PLEX
 *
 *  A.P. Jason de Koning, jason.de.koning@gmail.com
 *  http://jasondk.org
 *
 *  David Pollock lab, University of Colorado School of Medicine
 *  http://www.evolutionarygenomics.com
 *  All rights reserved.
 
 */

// As in other files, this contains a variety of old and experimental code.

#ifndef TreeNode_h__
#define TreeNode_h__

#include "macrodefine.h"
#include "CLAMPTypes.h"
#include "Numerics.h"
#include "TreeNodeData.h"
#include "Options.h"

#include "Model.h"

#include <vector>
#include <cassert>
#include <deque>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <cmath>
#include <algorithm>
#include <string>
#include <limits>
#include "SequenceAddressing.h"
#include "ObservedData.h"

// For OpenMP...
// #include <omp.h>

using std::ofstream;
using std::flush;
using std::setfill;
using std::setprecision;
using std::dec;
using std::fixed;
using std::hex;
using std::endl;
using std::ostringstream;
using std::min;
using std::cerr;
using std::setw;
using std::setprecision;

extern Options options;
#include "MixtureModel.h"

class Tree;

// For pthreads experimental implementation
struct thread_data {
	//	void *initSiteLikelihoods;
	//	void *setSiteLikelihoods_downPass;
	//	void *setSiteLikelihoods_doSimpleAssignment;
	void (*theFxn)(void *);
	
    int thread_id;
    int updateCount;
	int m_nMethod;
	IntVector *m_vecStateIndex; 
	int numModels;
	DoubleMatrix *stateFreqs;
	IntVector *m_vecSiteAssignment;
	bool	m_bSite;
	IntVector *inStateindex;
	CharVector *inStateVector;
	int minstate;
	int numStates;
	int assignMode;
	
	int startSite;
	int endSite;
};

//---------------------------------------------------------------------------

class TreeNode
{
public:
	typedef std::deque<TreeNode*>	node_list;
	
	TreeNode(
			 void);
	
	~TreeNode(
			  void);
	
	void propogateRootLink( TreeNode*	inFrom);
	
	TreeNode*			Prev(				//get the previous node of current node		
							 void);
		
	void				SetPrev(			//set the previous node of current node
								TreeNode* inSrc);
	
	TreeNode*			Next(				//get the next node of current node
							 void);
	
	void				SetNext(			//set the next node of current node
								TreeNode* inSrc);
	
	TreeNode*			Up(				//get the up node of current node
						   void);
	
	void insertNodeAbove(TreeNode* newNode);
	
	void				SetUp(				//set the up node of current node
							  TreeNode* inSrc);
	
	TreeNode*			Left(				//get the left node of current node
							 void);
	
	TreeNode*			Sibling(void);		// return the sibling node
	
	void				SetLeft(			//set the left node of current node
								TreeNode* inSrc);
	
	TreeNode*			Right(				//get the right node of current node
							  void);
	void				SetRight(			//set the right node of current node
								 TreeNode* inSrc);
	
	TreeNodeData_&			GetData(			//get the node data of current node
									void) ;
	
	bool				IsLeaf(				//test if this is a leaf node
							   void) ;
	
	unsigned int			CountNodes(			//count the number of nodes of the tree
									   void) ;
	
	unsigned int			CountInternalNodes(		//count the number of internal node of the tree
											   void) ;
	
	unsigned int			CountLeaves(			//count the number of leaves in the tree
										void) ;
	
	unsigned int			CountBranches(			//count the number of branches in the tree
										  void) ;
	
	unsigned int			GetNumNulls(			//get number of nulls
										TreeNode*& outNode1,
										TreeNode*& outNode2,
										TreeNode*& outNode3);
	
	void				PrintTree(ofstream& outStream, 	//print the tree file with scale distances or sequences sites
								  int& site,
								  double& scale) ;
	
	//	void				subdivideLongBranches(TreeNode* theNode, double maxBL, int recurseTree);
	
#if 0
	void				UnlinkFromTree(
									   TreeNode*&	ioRoot);
	
	void				UnlinkFromList(
									   ThreeNode*&	ioHead);
#endif
	
	void				DisconnectFrom(			//disconnect current node from input node
									   TreeNode*	inFrom);
	
	void				AssignChildren(			//assign the input code as the child node of current node
									   node_list&	ioChildren);
	
	template <typename Functor>
	void				Visit(				//visit the node of the whole tree to find a node meet function
							  Functor&	ioFunctor);
	
	template <typename Functor>
	TreeNode*			RemoveFreeLeaves(		//remove free leaves given function
										 Functor&	ioFunctor);
	
	vector<int>* 		SetInternalNodes(		//set sequence of current internal node based on up, left and right node sequence
											 void);
	
	void				PrintTreeSequence(		//print tree sequences on each node
										  ofstream& outStream) ;
	
	void 				PrintTreeContent(		//print sequence content at each node of the tree
										 ofstream& outStream, 
										 IntVector& inStateindex,
										 int& minstate,	
										 int& numStates) ;
	
	void 				CalculateNodeContent(		//calculate the content of nodes at internal and leaf nodes
											 DoubleVector& internalContent,
											 DoubleVector& leafContent,
											 int& numInternalnode,
											 int& numLeafnode,							
											 IntVector& inStateindex,
											 int& minstate,
											 int& numStates) ;
	
	void				PrintInternalNodeSequence(	//print the sequences on internal nodes only
												  ofstream& outStream) ;
	
	void				PrintLeafSequence(		//print the sequences on leaf nodes only
										  ofstream& outStream) ;
	
	void				PrintNodeSequence(		//print sequence on a specific node
										  ofstream& outStream, 
										  int& node) ;
	
	void				PrintNodeSequence(		//print sequence on a specific node of a specific node name
										  ofstream& outStream, 
										  char* nodename) ;
	
	void				PrintInternalNodeSequence(	//print sequences on internal nodes with the sampling count number
												  ofstream& outStream,
												  int& count) ;
	
	void				PrintNodeSequence(		//print sequence on specific node (nodeID) with the sampling count number
										  ofstream& outStream, 
										  int& node,
										  int& count) ;
	
	void				PrintNodeSequence(		//print sequence on specific node (node name) with the sampling count number
										  ofstream& outStream, 
										  char* nodename,
										  int& count) ;
	
	void				PrintInternalNodeSequence(	//print sequence on internal nodes with the sampling count number to a string matrix
												  StringMatrix& InternalnodeSeq, 
												  int& m_nSampleSequence, 
												  IntVector& NodeID) ;
	
	void				CountSimpleChangesOnSubtree(		//count the changes along each branch
													ObservedData &theData);
	
	void				SumChangesOverSubtree(		//count the changes along the whole tree
											  IntMatrix& m_matCountChange);
	
	void				SetUniqueModelFlags(void);
	void				SetTreeB1xm_bl(double bl);
	
	double				_calculateMarginalDwell( double siteRate, int x, int y, int whichState, int theModel, DoubleMatrix3D& inRateMatrix3D, double bl );
	double				_calculateJointDwellTime( double siteRate1, double siteRate2, int x1, int x2, int y1, int y2, int state1, int state2, int theModel1, int theModel2, MixtureModel& mModel, double bl );
	
	
	double				BranchSpecificLikelihood(MixtureModel& theModel,
												 ObservedData& theData,
												 IntVector& m_vecSiteAssignment);
	
	void				SetTreeBLProb(			//set the probability of each branch with BL method
									  int numStates,
									  DoubleMatrix3D& inRateMatrix3D);
	
	void				SetTreeB1Prob(			//set the probability of each branch with B1 method
									  int numStates,
									  DoubleMatrix3D& inRateMatrix3D);
	
	void				SetTreeB1Prob(			//set the probability of each branch with B1 method
									  int numStates,
									  DoubleMatrix3D& inRateMatrix3D, int theModel, int x);
	
	void				SetTreeB1uProb(			//set the probability of each branch with B1 method
									   int numStates,
									   DoubleMatrix3D& inRateMatrix3D, DoubleVector* uniformizationants);
	
	void				SetTreeB1uProb(			//set the probability of each branch with B1 method
									   int numStates,
									   DoubleMatrix3D& inRateMatrix3D, DoubleVector* uniformizationants, int theModel, int x);
	
	void				SetTreeBinfProb(			//set the probability of each branch with B1 method
										int numStates,
										DoubleMatrix3D& inRateMatrix3D, DoubleMatrix *stateFreqs);
	
	void				SetTreeBLProb(			//set the probability of each branch with BL method
									  int numStates,
									  DoubleMatrix3D& inRateMatrix3D, int dummyGibbs);
	
	void				SetTreeB1Prob(			//set the probability of each branch with B1 method
									  int numStates,
									  DoubleMatrix3D& inRateMatrix3D, int dummyGibbs);
	
	void				SetTreeB1Prob(			//set the probability of each branch with B1 method
									  int numStates,
									  DoubleMatrix3D& inRateMatrix3D, int dummyGibbs, int theModel, int x);
	
	void				SetTreeB1uProb(			//set the probability of each branch with B1 method
									   int numStates,
									   DoubleMatrix3D& inRateMatrix3D, DoubleVector* uniformizationants, int dummyGibbs);
	
	void				SetTreeB1uProb(			//set the probability of each branch with B1 method
									   int numStates,
									   DoubleMatrix3D& inRateMatrix3D, DoubleVector* uniformizationants, int dummyGibbs, int theModel, int x);
	
	void				SetTreeBinfProb(			//set the probability of each branch with B1 method
										int numStates,
										DoubleMatrix3D& inRateMatrix3D, DoubleMatrix *stateFreqs, int modelIndex);
	
	void				SetTreeB2Prob(			//set the probability of each branch with B2 method
									  int numStates,
									  DoubleMatrix3D& inRateMatrix3D);
	
	void				SetTreeB2Prob(			// no checking for inferred subs here
									  int numStates,
									  DoubleMatrix3D& inRateMatrix3D, int dummyGibbs);
	
	
	void				CalTreeLnLikelihood(		//set the log likelihood of whole tree given the data
											DoubleVector& inFreqVector,
											double& logLikelihood);
	
	void				CalTreeLnLikelihoodDeltaB1(		//set the log likelihood of whole tree given the data
												  DoubleVector& inFreqVector,
												   double& logLikelihood, int theModel, int x);				
    
	void				findNodeForLocalUpdateOfAncestralAndTransientStates(			//look up a specific node to start update the internal states on the tree
																			int&	rannode,
																			int&		found,
																			ObservedData& theData,
																			DoubleVector& inFreqModels);
	
	void 				SetRootSequence(		//set the sequence on root node
										vector<int>& m_strAncSeq);
	
	void 				EvolveTree(			//mapping the substitutions on the branches along the whole tree
								   DoubleMatrix& inRates,
								   IntVector& inStateindex,
								   CharVector& inStateVector,
								   int& minstate,
								   int& numStates);
	
	void 				EvolveTree(			//mapping the substitutions on the branches along the tree, with output of mapping
								   ofstream& outStream,
								   DoubleMatrix& inRates,
								   IntVector& inStateindex,
								   CharVector& inStateVector,
								   int& minstate,
								   int& numStates);
	
	int				KEEP_UpdateBranchLength(
											TreeNode* inFrom,
											Tree& mTree,
											double& m_fWindowsize,
											MixtureModel& theModel,
											ObservedData& theData,
											IntVector& m_vecSiteAssignment,
											int version);
	
	
	void				KEEP_UpdateBranchLength_fromStartNode( 
															  int startNode, 
															  Tree& mTree,
															  int found,
															  TreeNode* inFrom,
															  double& m_fWindowsize,
															  MixtureModel& theModel,
															  ObservedData& theData,
															  IntVector& m_vecSiteAssignment,
															  int version, int& dirtyTree);
	
	void				PrintBranchLength(		//print the branch lengths all over the tree
										  ofstream& outStream);
	
	void				PrintBranchLabel(		//print branch labels all over the tree, if leaf node, node name, if not, nodeID
										 ofstream& outStream);
	
	void				CopyTree(			//make a copy of current tree
								 TreeNode* inTarget);
	
	void				CopyTreeData(			//make a copy of tree data of current tree
									 TreeNode* inTarget);
	
	void				PrintTreeChanges(		//print changes along the tree
										 ofstream& outStream) ;
	
	void				PrintBranchChangecount(		//print the changes of each branch along the tree
											   ofstream& outStream);
	
	void				PrintSiteChangecount(		//print the number of changes along the tree for each sequence site
											 ofstream& outStream);
	
	void				GibbsSamplingAssignment(
												//assignment of each model to each sequence/site using gibbs sampling 
												IntVector& m_vecSiteAssignment, 
												DoubleMatrix3D& RateMatrix3D, 
												int& m_nMethod, 
												bool& m_bSite);
	
	double				CalculateSiteSpecificLnLikelihood(//calculate the site specific log likelihood of each model along the tree
														  int& numModels,
														  int& site,
														  DoubleVector& LnLikelihoodVector,
														  ObservedData& theData);
	
	double				CalculateSiteSpecificLnLikelihood_ASRV(//calculate the site specific log likelihood of each model along the tree
															   int& site,
															   MixtureModel& theModel,
															   ObservedData& theData,
															   IntVector& m_vecSiteAssignment,
															   DoubleVector& LnLikelihoodVector);
	
	void			reassignSiteCountsBetweenRateCategories_Uni(  int site,
																int lastAssignment,
																int newAssignment,
																ObservedData& theData,
																IntVector& m_vecSiteAssignment
);
	
	void			CalculateSiteSpecificLnLikelihoodUni_ASRV(
															  int& site,
															  MixtureModel& theModel,
															  ObservedData& theData,
															  IntVector& m_vecSiteAssignment,
															  DoubleVector& LnLikelihoodVector);
	
	double				CalculateSiteSpecificLnLikelihood_B1u(
															  MixtureModel& mModel,
															  int numModels,
															  int site,
															  DoubleVector* LnLikelihoodVector,
															  ObservedData& theData );
															//  DoubleMatrix3D* inRateMatrix3D,
		  													//  DoubleMatrix* stateFreqMatrix,
															//  DoubleVector *uniformizationants);
	
	void				CalculateSiteSpecificLikelihood(//calculate the site specific log likelihood of each model along the tree
														int& numModels,
														int& site,
														DoubleVector& LnLikelihoodVector,
														IntVector& inStateindex,
														int& minstate,
														int& numStates);
	
	void				countChangesAlongBranch(
												MixtureModel& theModel,
												ObservedData& theData,
												IntVector& m_vecSiteAssignment);
	
	/*				void				countChangesAlongBranch(
	 MixtureModel& theModel,
	 ObservedData& theData,
	 IntVector& m_vecSiteAssignment,
	 int version); */
	
	void				CalculateBranchModelExpectedSubsUni(
															//calculate the changes on each branch of different sites assigned to each model 
															MixtureModel& theModel,
															ObservedData& theData,
															IntVector& m_vecSiteAssignment);
	
	void calculateJointDwellTimeOverBranches(int s1, int s2,
											 double siteRate1, double siteRate2,
											 MixtureModel& theModel,
											 ObservedData& theData,
											 IntVector& m_vecSiteAssignment);
	
	int				calculateDwellTimeOverBranches(
												   int s,
												   double siteRate,
												   MixtureModel& theModel,
												   ObservedData& theData,
												   IntVector& m_vecSiteAssignment);
	
	void				countChangesAlongBranchUni(
												   //calculate the changes on each branch of different sites assigned to each model 
												   MixtureModel& theModel,
												   ObservedData& theData,
												   IntVector& m_vecSiteAssignment,
												   int version);
	
	void				initSiteLikelihoods(
											TreeNode*	theNode,
											IntVector& m_vecSiteAssignment,
											int& numStates);
	
	void				initSiteLikelihoods(
											TreeNode*	theNode,
											IntVector& m_vecSiteAssignment,
											int& numStates,
											int startSite,
											int endSite);
	
	void				setSiteLikelihoods_downPass(
													TreeNode*	theNode,
													MixtureModel& theModel,
													ObservedData& theData,
													IntVector& m_vecSiteAssignment,
                                                    DoubleVector& modelFreqVector,
													int& theMode);
	
	void				setSiteLikelihoods_downPass(
													TreeNode*	theNode,
													MixtureModel& theModel,
													ObservedData& theData,
													IntVector& m_vecSiteAssignment,
                                                    DoubleVector& modelFreqVector,
													int& theMode,
													int startSite,
													int endSite);
	
	void				restoreLastSequenceStates(
												  IntVector& m_vecSiteAssignment,
												  CharVector& inStateVector,
												  int& minstate,
												  int &numStates);
	
	void assignSequencesToLeaves( ObservedData &theData );
	
	void backupSeqSite(int theSite,   IntVector& inStateindex, int& minstate);
	void restoreSeqSite(int theSite,  CharVector& inStateVector, int& minstate);
	
	void _mixSequences( std::vector<int>& result,  std::vector<int>& seq1, std::vector<int>& seq2 );
    
	DoubleMatrix3D *getProbMatrix3D( TreeNode *theNode );
	DoubleMatrix3D *getChangeMatrix3D( TreeNode *theNode );
	IntVector *getTotalSubCountVector( TreeNode *theNode );
	
	DoubleMatrix *getAugmentedCountMatrix( TreeNode *theNode );
	DoubleMatrix *getAugmentedLogSum( TreeNode *theNode );
	DoubleMatrix *getAugmentedLogSum_scaling( TreeNode *theNode );
	DoubleMatrix3D *getChangeMatrix3D_scaling( TreeNode *theNode );
	
	DoubleMatrix *getChangeMatrix3D( TreeNode *theNode, int theSite );
	IntMatrix   *getChangeMatrix( TreeNode *theNode );
	DoubleMatrix3D *getChangeMatrix3Duni( TreeNode *theNode );
	//DoubleMatrix *getChangeMatrixUni( TreeNode *theNode );
	DoubleMatrix *getMarginalDwell( TreeNode *theNode );
	DoubleMatrix *getJointDwell( TreeNode *theNode );
	
	void setCalculateFullProbabilityDistribution(int val) { calculateFullProbabilityDistribution = val; }
	int  getCalculateFullProbabilityDistribution(void) { return calculateFullProbabilityDistribution; }
	
	int numDiscreteBLs;
	double *discreteBLs;
	
	void setSiteLikelihoods_upPass(
								   TreeNode*	theNode,
								   DoubleMatrix& stateFreqMatrix,
								   DoubleVector& modelFreqVector,
								   IntVector& m_vecSiteAssignment,
								   IntVector& inStateindex,
								   CharVector& inStateVector,
								   int& minstate,
								   int& numStates,
								   int startSite,
								   int endSite);
	
	void setSiteLikelihoods_doAssignment(
										 TreeNode*	theNode,
										 MixtureModel& theModel,
										 ObservedData& theData,
										 IntVector& m_vecSiteAssignment,
                                         DoubleVector& modelFreqVector
                                         );
	
	void setSiteLikelihoods_doAssignment(
										 TreeNode*	theNode,
										 MixtureModel& theModel,
										 ObservedData& theData,
										 IntVector& m_vecSiteAssignment, 
                                         DoubleVector& modelFreqVector,
										 int theMode, int startsite, int endsite);
	
	void posteriorPredictive_doSimpleAssignment(
												TreeNode*	theNode,
												DoubleMatrix& stateFreqMatrix,
												DoubleVector& modelFreqVector,
												IntVector& m_vecSiteAssignment,
												IntVector& inStateindex,
												CharVector& inStateVector,
												int& minstate,
												int& numStates, std::vector<int>& tipStates);
	
	int setSiteLikelihoods_doSimpleAssignment(
											  TreeNode*	theNode,
											  MixtureModel& theModel,
											  ObservedData& theData,
											  IntVector& m_vecSiteAssignment,
											  DoubleVector& modelFreqVector,
											  int theMode,
											  int startSite, int endSite);
	
	void posteriorPredictive_getSite(
									 int theSite,
									 TreeNode*	theNode,
									 DoubleMatrix& stateFreqMatrix,
									 DoubleVector& modelFreqVector,
									 IntVector& m_vecSiteAssignment,
									 IntVector& inStateindex,
									 CharVector& inStateVector,
									 int& minstate,
									 int& numStates, std::vector<int>& tipStates);
	
	int setSiteLikelihoods_doSimpleAssignment(
											  TreeNode*	theNode,
											  MixtureModel& theModel,
											  ObservedData& theData,
											  IntVector& m_vecSiteAssignment,
											  DoubleVector& modelFreqVector,
											  int theMode);
	
	
	void				GibbsSamplingLineageAssignment(
													   //use gibbs sampling to sample the branch assignment of each model
													   void);
	
	void				CalcLogLikelihood_branch(//calculate the log likelihood given the mixture of site or lineage
												 MixtureModel& theModel,
												 IntVector& m_vecSiteAssignment,
												 double& logLikelihood);
	
	void				CalcLogLikelihood_branch(
												 MixtureModel& theModel,
												 IntVector& m_vecSiteAssignment,
												 double& logLikelihood,
												 int version);
	
	void				CalcLogLikelihood_branchDeltaB1(MixtureModel& mModel,
														IntVector& m_vecSiteAssignment,
														double& logLikelihood, int theModel, int x);
	
	double				CalcLogLikelihood_branchDeltaB1uni(
                                                           MixtureModel& mModel,
														  // DoubleMatrix3D *rateMat,
														  // DoubleVector *uniformizationants,
														   IntVector& m_vecSiteAssignment,
														   double& logLikelihood, int theModel, int x, int y);
	
	double				CalcLogLikelihood_branchB1uni_bySite(
															 MixtureModel& mModel,
															 double* logLikelihood, int theModel, int site );
	
	int					ResampleAncestralAndTransientStates_local_fromNode(	//look up the node and update the internal states when we use gibbs sampling to do mixture
																		   int&	rannode,
																		   int&		found,
																		   MixtureModel& theModel,
																		   ObservedData& theData,
																		   IntVector& m_vecSiteAssignment,
																		    DoubleMatrix& stateFreqs
																);
	
	void				markNodesBelow(
									   int&	theNode,
									   int&		found,
									   IntVector& m_vecSiteAssignment,
									   bool&	m_bSite,
									   IntVector& inStateindex,
									   CharVector& inStateVector,
									   int& minstate,
									   int& numStates);
	
	int					ResampleAncestralAndTransientStates_local_fromNode2(	//look up the node and update the internal states when we use gibbs sampling to do mixture
																			int&	rannode,
																			int&		found,
																			DoubleMatrix& stateFreqMatrix,
																			IntVector& m_vecSiteAssignment,
																			bool&	m_bSite,
																			IntVector& inStateindex,
																			CharVector& inStateVector,
																			int& minstate,
																			int& numStates, DoubleMatrix3D* rates, DoubleVector* uni);
	
	double				incompleteDataLikelihood(
												 TreeNode*	theNode,
												 DoubleMatrix& stateFreqs,
												 DoubleVector& inFreqVector,
												 IntVector& m_vecSiteAssignment,
                                                 ObservedData &theData);
    
	int					getUniqueModelFlag(void) { return uniqueModelFlag; }
	void				setUniqueModelFlag(int theVal) { uniqueModelFlag = theVal; }
	
	void				TransformBranchLength_B1u(
												  TreeNode* inFrom,
												  double rateFactor, double mu);
	
private:
	friend class			UTTree;
	friend class			UTTreeNode;
	friend class			Tree;
	
	int uniqueModelFlag;
	int calculateFullProbabilityDistribution; // only used on the root node
	
	Numerics *num;	// an interface to accelerated numeric functions
	
	// Forbid copying
	TreeNode(
			 TreeNode& inSrc);
	TreeNode&			operator=(
									  TreeNode&	inSrc);
	
	void				ConnectUp(
								  TreeNode*		inTo);
	void				ConnectRight(
									 TreeNode*		inTo);
	void				ConnectLeft(
									TreeNode*		inTo);
	void				ConnectRoot( TreeNode* inTo);
	
	
	template <typename Functor>
	void				Visit(
							  Functor&		ioFunctor,
							  TreeNode*	inFrom);
	
	unsigned int			CountNodes(
									   TreeNode*	inFrom) ;
	
	unsigned int			CountLeaves(
										TreeNode*	inFrom) ;
	
	unsigned int			CountBranches(
										  TreeNode*	inFrom) ;
	
	unsigned int			CountInternalNodes(
											   TreeNode*	inFrom) ;
	
	void				PrintTree(
								  ofstream& outStream, 
								  int& site,
								  double& scale,
								  TreeNode* inFrom) ;
	
	void				PrintTreeSequence(
										  ofstream& outStream, 
										  TreeNode* inFrom) ;
	
	void 				PrintTreeContent(
										 ofstream& outStream, 
										 TreeNode* inFrom,
										 IntVector& inStateindex,
										 int& minstate,	
										 int& numStates) ;
	
	void				PrintNodeSequence(
										  ofstream& outStream,
										  TreeNode* inFrom,
										  int& node) ;
	
	void				PrintNodeSequence(
										  ofstream& outStream,
										  TreeNode* inFrom,
										  char* nodename) ;
	
	void				PrintInternalNodeSequence(
												  ofstream& outStream,
												  TreeNode* inFrom) ;
	
	void				PrintLeafSequence(
										  ofstream& outStream,
										  TreeNode* inFrom) ;
	
	void				PrintNodeSequence(
										  ofstream& outStream,
										  TreeNode* inFrom,
										  int& node,
										  int& count) ;
	
	void				EvolveTree(
								   TreeNode* inFrom,
								   DoubleMatrix& inRates,
								   IntVector& inStateindex,
								   CharVector& inStateVector,
								   int& minstate,
								   int& numStates);
	
	void				EvolveNode(
								   std::string& inSequence,
								   double& inBranch,
								   DoubleMatrix& inRates,
								   IntVector& inStateindex,	
								   CharVector& inStateVector,
								   int& minstate,
								   int& numStates,
								   std::string& outSequence);
	
	void				EvolveTree(
								   TreeNode* inFrom,
								   ofstream& outStream,
								   DoubleMatrix& inRates,
								   IntVector& inStateindex,
								   CharVector& inStateVector,
								   int& minstate,
								   int& numStates);
	
	void				EvolveNode(
								   ofstream& outStream,
								   std::string& inSequence,
								   double& inBranch,
								   DoubleMatrix& inRates,
								   IntVector& inStateindex,	
								   CharVector& inStateVector,
								   int& minstate,
								   int& numStates,
								   std::string& outSequence);
	
	void				PrintNodeSequence(
										  ofstream& outStream,
										  TreeNode* inFrom,
										  char* nodename,
										  int& count) ;
	
	void				PrintInternalNodeSequence(
												  ofstream& outStream,
												  TreeNode* inFrom,
												  int& count) ;
	
	void				PrintInternalNodeSequence(
												  StringMatrix& InternalnodeSeq, 
												   TreeNode* inFrom,
												  int& m_nSampleSequence, 
												  IntVector& NodeID) ;
	
	DoubleVector			calculatenodecontent(
												 vector<int>	inSeq,
												 IntVector& inStateindex,
												 int& minstate,
												 int& numStates) ;
	
	void 				CalculateNodeContent(
											 DoubleVector& internalContent,
											 DoubleVector& leafContent,
											 int& numInternalnode,
											 int& numLeafnode,							
											 TreeNode* inFrom,
											 IntVector& inStateindex,
											 int& minstate,
											 int& numStates) ;
	
	void				PrepareAChildForAssignment(
												   TreeNode*		inChild,
												   node_list&		ioChildren);
	
	template <typename Functor>
	TreeNode*			RemoveFreeLeaves(
										 Functor&		ioFunctor,
										 TreeNode*	inFrom);
	
	template <typename Functor>
	void				RemoveFreeLeavesForChild(
												 Functor&		ioFunctor,
												 TreeNode*		inChild);
	
	double 				getdist(
								TreeNode*	node) ;
	
	void				printdist(
								  ofstream& outStream,
								  double& scale,
								  TreeNode* node) ;
	
	int				changeinmatrix(
								   IntMatrix& countMatrix) ;
	
	double 				getchange(
								  TreeNode*	node) ;
	
	void				printchange(
									ofstream& outStream,
									TreeNode* node);
	
	vector<int>*			CallSetINodesForAdjacentNode(
														 TreeNode*	adjNode,
														 TreeNode*	inFrom);
	vector<int>* 		SetInternalNodes(
										 TreeNode*	inFrom);
	static	unsigned int		AssignNumNulls(
											   TreeNode*		inNode,
											   TreeNode*&		outNode1, 
											   TreeNode*&		outNode2,
											   TreeNode*&		outNode3,
											   unsigned int	inAssignCount);
	void 				printseq(
								 ofstream& 	outStream,
								 vector<int>&	inSeq,
								 char*	name,
								 int		id,
								 int		type) ;
	
	void 				printseq(
								 ofstream& 	outStream,
								 vector<int>&	inSeq,
								 char*	name,
								 int		id,
								 int		type,
								 int& 	count) ;
	
	void 				printseqcontent(
										ofstream& 	outStream,
										vector<int>&	inSeq,
										IntVector& inStateindex,
										int& minstate,
										int& numStates) ;
	
	unsigned int			SIndex(
								   char& state,
								   IntVector& inStateindex,
								   int& minstate);
	
	void				CountSimpleChangesOnBranch(
												   TreeNode*		node1,
												   TreeNode*		node2,
												   ObservedData& theData);
	
	void				SetBranchBLProb(
										TreeNode*	node,
										int numStates,
										DoubleMatrix3D& inRateMatrix3D);
	
	void				SetBranchB1Prob(
										TreeNode*	node,
										int numStates,
										DoubleMatrix3D& inRateMatrix3D);
	
	void				SetBranchB1uProb(
										 TreeNode*	node,
										 int numStates,
										 DoubleMatrix3D& inRateMatrix3D, DoubleVector* uniformizationants);
	
	void				SetBranchBinfProb(
										  TreeNode*	node,
										  int numStates,
										  DoubleMatrix3D& inRateMatrix3D, DoubleMatrix *stateFreqs);
	
	void				SetBranchB1Prob(
										TreeNode*	node,
										int numStates,
										DoubleMatrix3D& inRateMatrix3D, int theModel, int row);
	
	void				SetBranchB1uProb(
										 TreeNode*	node,
										 int numStates,
										 DoubleMatrix3D& inRateMatrix3D, DoubleVector* uniformizationants, int theModel, int row);
	
	void				SetBranchBLProb(
										TreeNode*	node,
										int numStates,
										DoubleMatrix3D& inRateMatrix3D, int dummyGibbs);
	
	void				SetBranchB1Prob(
										TreeNode*	node,
										int numStates,
										DoubleMatrix3D& inRateMatrix3D, int dummyGibbs);
	
	void				SetBranchB1uProb(
										 TreeNode*	node,
										 int numStates,
										 DoubleMatrix3D& inRateMatrix3D, DoubleVector* uniformizationants, int dummyGibbs);
	
	void				SetBranchB1Prob(
										TreeNode*	node,
										int numStates,
										DoubleMatrix3D& inRateMatrix3D, int dummyGibbs, int theModel, int row);
	
	void				SetBranchB1uProb(
										 TreeNode*	node,
										 int numStates,
										 DoubleMatrix3D& inRateMatrix3D, DoubleVector* uniformizationants, int dummyGibbs, int theModel, int row);
	
	void				SetBranchBinfProb(
										  TreeNode*	node,
										  int numStates,
										  DoubleMatrix3D& inRateMatrix3D, DoubleMatrix *stateFreqs, int dummyGibbs);
	
	void				SetBranchB2Prob(
										TreeNode*	node,
										int numStates,
										DoubleMatrix3D& inRateMatrix3D); // this version skips transitions not seen in counts
	
	void				SetBranchB2Prob(
										TreeNode*	node,
										int numStates,
										DoubleMatrix3D& inRateMatrix3D, int dummyGibbs);  // this version does all transition probabiltiies
	
	void				UpdateAncestralAndTransientStatesForSubtree_localUpdater(
																				 TreeNode*	head,
																				 TreeNode*	from,
																				 ObservedData& theData,
																				 DoubleVector& inFreqModels);
	
	int UpdateAncestralAndTransientStatesForSubtree_localUpdaterForB1xM(
																		TreeNode*	head,
																		TreeNode*	from,
																		ObservedData& theData,
																		IntVector& m_vecSiteAssignment,
																		DoubleMatrix& stateFreqMatrix
																		);
	
	double  			rndu(void);
	
	void				SetRootSequence(
										vector<int>& m_strAncSeq,
										TreeNode* inFrom);
	
	void				PrintBranchLength(
										  TreeNode* inFrom,
										  ofstream& outStream);
	
	void				PrintBranchLabel(
										 TreeNode* inFrom,
										 ofstream& outStream);
	
	void				CopyTree(
								 TreeNode* inFrom,
								 TreeNode* inTarget);
	
	void				CopyTreeData(
									 TreeNode* inFrom,
									 TreeNode* inTarget);
	
	void				PrintTreeChanges(
										 ofstream& outStream, 
										 TreeNode* inFrom) ;
	
	void				PrintBranchChangecount(
											   TreeNode* inFrom,
											   ofstream& outStream);
	
	void				CountSiteChangecount(
											 TreeNode*  inFrom,
											 TreeNode*  inTo,
											 IntVector& SiteCount);
	
	void				CountTreeSiteChangecount(
												 IntVector& SiteCount);
	
	void 				NormalizeMatrix(
										DoubleMatrix& m_probMatrix);
	
	void				NormalizeMatrix(	DoubleMatrix& m_probMatrix, int row);	
    
	void				_SetChangesAlongBranch(
											   TreeNode*	node1,
											   TreeNode* node2,
											   MixtureModel& theModel,
											   ObservedData& theData,
											   IntVector& m_vecSiteAssignment
											   ); 
	
	int					_CalculateDwellTimeBranch(
												  int s,
												  double siteRate,
												  TreeNode*	node1,
												  TreeNode* node2,
												  MixtureModel& theModel,
												  ObservedData& theData,
												  IntVector& m_vecSiteAssignment );
	
	void				_CalculateJointDwellTimeBranch(
													   int s1, int s2,
													   double siteRate1, double siteRate2,
													   TreeNode*	node1,
													   TreeNode* node2,
													   MixtureModel& theModel,
													   ObservedData& theData,
													   IntVector& m_vecSiteAssignment); 
	
	void				_SetChangesAlongBranchUni(
												  TreeNode*	node1,
												  TreeNode* node2,
												  MixtureModel& theModel,
												  ObservedData& theData,
												  IntVector& m_vecSiteAssignment); 
	
	void RecalcVirtualSubsAlongBranchUni(
										 TreeNode*	node1,
										 TreeNode* node2,
										 MixtureModel& theModel,
										 ObservedData& theData,
										 IntVector& m_vecSiteAssignment); 
	
	void				SetBranchModelExpectedSubsUni(
													  TreeNode*	node1,
													  TreeNode* node2,
													  MixtureModel& theModel,
													  ObservedData& theData,
													  IntVector& m_vecSiteAssignment,
													  double *siteLikelihoods); 	
	
	int				_resampleAncestralAndTransientStates_local_fromNode(
																		TreeNode*	head,
																		TreeNode*	from,
																		MixtureModel& theModel,
																		ObservedData& theData,
																		IntVector& m_vecSiteAssignment,
																		DoubleMatrix& stateFreqMatrix);
	
	
	TreeNodeData_			mData;
	TreeNode*				mLeft;
	TreeNode*				mRight;
	TreeNode*				mUp;
	TreeNode*				mPrev;
	TreeNode*				mNext;
	
	TreeNode*				mRoot;
	Vector<int>				conditionalState;	// for Pupko et al type joint reruction
};

//---------------------------------------------------------------------------

inline
TreeNode::TreeNode(void)
:	mData(),
mLeft(0),
mRight(0),
mUp(0),
mPrev(0),
mNext(0),
mRoot(0)
{
	num = &Numerics::getGlobalNumerics();
	setUniqueModelFlag(0);
	numDiscreteBLs=0;
	calculateFullProbabilityDistribution = 0;
	GetData().UVRallocated=0;
    //	std::cout << "TEST: num->exponential(1) = " << num->exponential(1.0) << ", approx = " << num->exponential(1.0) << endl;
}

//---------------------------------------------------------------------------

// inline TreeNode::doEigenSolution(int which) {
//	num->PAML_eigenQREV (double Q[], double pi[], int n, double Root[], double U[], double V[]);
//}


//---------------------------------------------------------------------------

inline
TreeNode::~TreeNode(void)
{
	//TreeNode*	next	= mNext;
	//TreeNode*	prev	= mPrev;
	
	mNext = 0;
	mPrev = 0;
	
	if (mRight && mRight != mUp)
	{
		TreeNode*	right = mRight;
		DisconnectFrom(right);
		delete right;
	}
    
	if (mLeft && mLeft != mUp)
	{
		TreeNode*	left = mLeft;
		DisconnectFrom(left);
		delete left;
	}
    
	// Note: deleting mUp must be last (that is deleting left link and right link must be before deleting up link)
	if (mUp)
	{
		TreeNode* up = mUp;
		DisconnectFrom(up);
		delete up;
	}
    
    
	//delete	next;
	//delete	prev;
}

//---------------------------------------------------------------------------

inline
TreeNodeData_&
TreeNode::GetData(void)
{
	return mData;
}


//---------------------------------------------------------------------------

//---------------------------------------------------------------------------

inline void
TreeNode::DisconnectFrom(
                         TreeNode*	inFrom)
{
	if (inFrom)
	{
		if (Left() == inFrom)
		{
			SetLeft(0);
			inFrom->DisconnectFrom(this);
		}
		
		if (Right() == inFrom)
		{
			SetRight(0);
			inFrom->DisconnectFrom(this);
		}
		
		if (Up() == inFrom)
		{
			SetUp(0);
			inFrom->DisconnectFrom(this);
		}
	}
}


inline void
TreeNode::PrepareAChildForAssignment(
                                     TreeNode*	theChild,
                                     node_list&	ioChildren)
{
	assert(theChild == 0 || theChild == Right() || theChild == Left());
	
	if(theChild)
	{
		theChild->DisconnectFrom(this);
		ioChildren.push_front(theChild);
	}
}

//---------------------------------------------------------------------------

inline void
TreeNode::AssignChildren(
                         node_list&	ioChildren)
{
	// If there are existing children in this node, disconnect them and prepend them to the child list.
	PrepareAChildForAssignment(Right(), ioChildren);
	PrepareAChildForAssignment(Left(), ioChildren);
	
	TreeNode*	child = 0;
	
	node_list::size_type	numChildren = ioChildren.size();
	
	if (numChildren <= 3)
	{
		// If we have 3 or fewer children, just connect them directly.
		if (numChildren > 0)
		{
			// for 1, 2 or 3 nodes, we can connect the left node directly
			child = ioChildren.front();
			ioChildren.pop_front();
			ConnectLeft(child);
			
			if (numChildren > 1)
			{
				if (numChildren < 3)
				{
					// For 2 nodes, we can connect the right node directly too
					child = ioChildren.front();
					ioChildren.pop_front();
				}
				else
				{
					// for three nodes, we need to recursively call with 
					child = new TreeNode;
					child->AssignChildren(ioChildren);
				}
				ConnectRight(child);
			}
		}
	}
	else
	{
		// We have more than 3 children, so we need to divide the child list evenly
		node_list::size_type	midPoint = numChildren / 2;
		{
			node_list	leftChildren(ioChildren.begin(), ioChildren.begin() + midPoint);
			
			child = new TreeNode;
			child->AssignChildren(leftChildren);
			ConnectLeft(child);
		}
		
		{
			node_list	rightChildren(ioChildren.begin() + midPoint, ioChildren.end());
			
			child = new TreeNode;
			child->AssignChildren(rightChildren);
			ConnectRight(child);
		}
		
		ioChildren.clear();
	}
}

//---------------------------------------------------------------------------
// Connects this->mUp to ioNodeToConnect, BUT NOTICE it does NOT set
// ioNodeToConnect->mUp to anything.  This because it is used within the
// scope of ConnectRight() and ConnectLeft().

inline
void
TreeNode::ConnectUp(
                    TreeNode*		ioNodeToConnect)
{
	assert(Up() == 0);
	assert(ioNodeToConnect != 0);
	
	SetUp(ioNodeToConnect);
}

//---------------------------------------------------------------------------
// Set the root node linkage...


inline
void
TreeNode::ConnectRoot(
                      TreeNode*		ioNodeToConnect)
{
	mRoot = ioNodeToConnect;
}

//---------------------------------------------------------------------------
// Inserts a node between the current node and its parent
inline void TreeNode::insertNodeAbove(TreeNode* newNode) {
	// Used for inserting 'virtual nodes' to break up long branches...
	TreeNode* tempHandle = Up();
    
	if (tempHandle->Left() == this) {
		tempHandle->SetLeft(newNode);
	} else if (tempHandle->Right() == this) {
		tempHandle->SetRight(newNode);	
	} else {
		std::cout << "ERROR - the nodes aren't connected!?" << endl;
		exit(-1);
	}
    
	newNode->SetLeft(this);
	newNode->SetRight(NULL); // set right to self??
	newNode->SetUp( tempHandle );
	
	SetUp( newNode );
}

//---------------------------------------------------------------------------
// Connects this->mRight to ioNodeToConnect, and ioNodeToConnect->mUp to this

inline
void
TreeNode::ConnectRight(
                       TreeNode*		ioNodeToConnect)
{
	assert(Right() == 0);
	assert(ioNodeToConnect != 0);
	
	SetRight(ioNodeToConnect);
	ioNodeToConnect->ConnectUp(this);
}

//---------------------------------------------------------------------------
// Connects this->mLeft to ioNodeToConnect, and ioNodeToConnect->mUp to this

inline
void
TreeNode::ConnectLeft(
                      TreeNode*		ioNodeToConnect)
{
	assert(Left() == 0);
	assert(ioNodeToConnect != 0);
	
	SetLeft(ioNodeToConnect);
	ioNodeToConnect->ConnectUp(this);
}

//---------------------------------------------------------------------------
#if 0

void
TreeNode::UnlinkFromTree(
                         TreeNode*&	ioRoot)
{
}

//---------------------------------------------------------------------------

void
TreeNode::UnlinkFromList(
                         TreeNode*&	ioHead)
{
}
#endif

//---------------------------------------------------------------------------

inline
TreeNode*
TreeNode::Prev(void)
{
	return mPrev;
}


//---------------------------------------------------------------------------

inline
void
TreeNode::SetPrev(
                  TreeNode*	inSrc)
{
	mPrev = inSrc;
}

//---------------------------------------------------------------------------

inline
TreeNode*
TreeNode::Next(void)
{
	return mNext;
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetNext(
                  TreeNode*	inSrc)
{
	mNext = inSrc;
}

//---------------------------------------------------------------------------

inline
TreeNode*
TreeNode::Up(void)
{
	return mUp;
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetUp(
                TreeNode*	inSrc)
{
	mUp = inSrc;
}

//---------------------------------------------------------------------------

inline
TreeNode*
TreeNode::Left(void)
{
	return mLeft;
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetLeft(
                  TreeNode*	inSrc)
{
	mLeft = inSrc;
}

//---------------------------------------------------------------------------

inline
TreeNode*
TreeNode::Sibling(void)
{
	if (this == Up()->Left()) { return Up()->Right(); }
	else if (this == Up()->Right()) { return Up()->Left(); }
	else {
		std::cout << "ERROR! The sibling couldn't be found." << endl;
	}
}

//---------------------------------------------------------------------------

inline
TreeNode*
TreeNode::Right(void)
{
	return mRight;
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetRight(
                   TreeNode*	inSrc)
{
	mRight = inSrc;
}

//---------------------------------------------------------------------------

template <typename Functor>
inline
void
TreeNode::Visit(
                Functor&	ioFunctor)
{
	Visit(ioFunctor, 0);
}

//---------------------------------------------------------------------------

template <typename Functor>
void
TreeNode::Visit(
                Functor&		ioFunctor,
                TreeNode*	inFrom)
{
	ioFunctor(GetData());
	
	if ((Right() && Right() != inFrom) && (Right() != Up()))
		Right()->Visit(ioFunctor, this);
    
	if ((Left() && Left() != inFrom) && (Left() != Up()))
		Left()->Visit(ioFunctor, this);
    
	if ((Up() && Up() != inFrom) && (Up()->Up() != this))
		Up()->Visit(ioFunctor, this);
}

//---------------------------------------------------------------------------

inline
bool
TreeNode::IsLeaf(void) 
{
    //	std::cout << "About to do count leave fxn (this is " << this << ")" << endl;
	return ( (Right() == 0) && (Left() == 0) );
}

//---------------------------------------------------------------------------

inline unsigned int
TreeNode::CountNodes(
                     TreeNode*	inFrom) 
{
	unsigned int count; 
	
	if (Up()->Up() != this)
		count = 1;		// this node counts as 1.
	else
		count = 0;
	
	if (Right() && Right() != inFrom)
		count += Right()->CountNodes(this);
	
	if (Left() && Left() != inFrom)
		count += Left()->CountNodes(this);
    
	if (Up() && Up() != inFrom)
		count += Up()->CountNodes(this);
	
	return count;
}

//---------------------------------------------------------------------------

inline
unsigned int
TreeNode::CountNodes(void) 
{
	return CountNodes(0);
}

//---------------------------------------------------------------------------

inline unsigned int
TreeNode::CountInternalNodes(
                             TreeNode*	inFrom) 
{
	unsigned int count;
	
	if (Up()->Up() == this)
		count = 0;
	else if (Left() ==NULL && Right()==NULL)
		count = 0;
	else
		count = 1;
    
	if (Right() && Right() != inFrom)
		count += Right()->CountInternalNodes(this);
	
	if (Left() && Left() != inFrom)
		count += Left()->CountInternalNodes(this);
    
	if (Up() && Up() != inFrom)
		count += Up()->CountInternalNodes(this);
	
	return count;
}

//---------------------------------------------------------------------------

inline
unsigned int
TreeNode::CountInternalNodes(void) 
{
	return CountInternalNodes(0);
}

//---------------------------------------------------------------------------

inline unsigned int
TreeNode::CountLeaves(
                      TreeNode*	inFrom) 
{
    //	std::cout << "Going to countleaves for node " << this << endl;
	unsigned int count = IsLeaf() ? 1 : 0;		// this node counts as 1 if it is a leaf.
    //	std::cout << "Counted leaves on node " << this << ", in from " << inFrom << endl;
    //	std::cout << "TESTS:" << endl;
    //	std::cout << "Right() is " << Right() << endl;
    //	std::cout << "Left() is " << Left() << endl;
    //	std::cout << "Up() is " << Up() << endl;
    //	std::cout << "inFrom is " << inFrom << endl;
    
	if (Right() && Right() != inFrom) {
        //		std::cout << "About to call right->countleaves; count is " << count << endl;
		count += Right()->CountLeaves(this);
	}
    //std::cout << "right passed" << endl;	
	if (Left() && Left() != inFrom)
		count += Left()->CountLeaves(this);
    //std::cout << "left passed" << endl;	
    
	if (Up() && Up() != inFrom)
		count += Up()->CountLeaves(this);
    //std::cout << "up passed" << endl;	
    
	return count;
}

//---------------------------------------------------------------------------


inline void
TreeNode::propogateRootLink(
                            TreeNode*	inFrom) 
{
    
	if(Up()->Up()->Up() == Up()) {		// Root
        //	std::cout << "Setting root!" << endl;
		ConnectRoot( (TreeNode*) inFrom);
	}
    
	if (Up() && Up() != inFrom) {
        //	std::cout << "Doing connectRoot..." << endl;
		ConnectRoot(Up()->mRoot);
	}
    
	if (Right() && Right() != inFrom)
		Right()->propogateRootLink(Right());
	
	if (Left() && Left() != inFrom)
		Left()->propogateRootLink(Left());
    
}


//---------------------------------------------------------------------------

inline
unsigned int
TreeNode::CountLeaves(void) 
{
	return CountLeaves(0);
}


//---------------------------------------------------------------------------

inline unsigned int
TreeNode::CountBranches(
                        TreeNode*	inFrom) 
{
	unsigned int count = 0;
	
	if (Right())
	{
		++count;
		
		if (Right() != inFrom)
			count += Right()->CountBranches(this);
	}
	
	if (Left() && Left() != Up())
	{
		++count;
		
		if (Left() != inFrom)
			count += Left()->CountBranches(this);
	}
    
	if (Up() && Up() != inFrom)
		count += Up()->CountBranches(this);
	
	return count;
}

//---------------------------------------------------------------------------

inline
unsigned int
TreeNode::CountBranches(void) 
{
	// revise here.
	// due to unrooted tree included here, so we get the number of branches 
	// equal to branch of rooted tree minus 1
	
	return CountBranches(0) - 1;
}

//---------------------------------------------------------------------------

inline
void
TreeNode::PrintTree(ofstream& outStream, int& site, double& scale) 
{
	PrintTree(outStream, site, scale, 0);
}

//---------------------------------------------------------------------------

inline
void
TreeNode::PrintTreeChanges(ofstream& outStream) 
{
	PrintTreeChanges(outStream, 0);
}

//---------------------------------------------------------------------------

inline unsigned int
TreeNode::AssignNumNulls(
                         TreeNode*		inNode,
                         TreeNode*&		outNode1,
                         TreeNode*&		outNode2,
                         TreeNode*&		outNode3,
                         unsigned int	inAssignCount)
{
	unsigned int	returnCount = 0;
	if (inNode)
	{
		switch(inAssignCount)
		{
			case 1:
				outNode1 = inNode;
				break;
                
			case 2:
				outNode2 = inNode;
				break;
                
			case 3:
				outNode3 = inNode;
				break;
				
			default:
				assert(false);	// This should never happen.
		};
	}
	else returnCount = 1;
	
	return returnCount;
}

//---------------------------------------------------------------------------
// Gets number of nulls, but also gets the first 1, 2, or 3 non-null nodes

inline unsigned int
TreeNode::GetNumNulls(
                      TreeNode*&	outNode1,
                      TreeNode*&	outNode2,
                      TreeNode*&	outNode3)
{
	outNode1 = 0;
	outNode2 = 0;
	outNode3 = 0;
	
	unsigned int nullCount=0;
	nullCount += AssignNumNulls(Left(), outNode1, outNode2, outNode3, 1-nullCount);
	nullCount += AssignNumNulls(Right(), outNode1, outNode2, outNode3, 2-nullCount);
	nullCount += AssignNumNulls(Up(), outNode1, outNode2, outNode3, 3-nullCount);
    
	return nullCount;
}

//---------------------------------------------------------------------------

template <typename Functor>
inline
TreeNode*
TreeNode::RemoveFreeLeaves(
                           Functor&	ioFunctor)
{
	return RemoveFreeLeaves(ioFunctor, 0);
}

//---------------------------------------------------------------------------

template <typename Functor>
inline void
TreeNode::RemoveFreeLeavesForChild(
                                   Functor&		ioFunctor,
                                   TreeNode*	inChild)
{
	if (inChild)
	{
		inChild->RemoveFreeLeaves(ioFunctor, this);
		if (inChild->IsLeaf())
		{
			if (ioFunctor(inChild->GetData()))
			{
				DisconnectFrom(inChild);
				delete inChild;
			}
		}
	}
}

//---------------------------------------------------------------------------

template <typename Functor>
inline TreeNode*
TreeNode::RemoveFreeLeaves(
                           Functor&		ioFunctor,
                           TreeNode*	inFrom)
{
	RemoveFreeLeavesForChild(ioFunctor, Left());
	RemoveFreeLeavesForChild(ioFunctor, Right());
	
	bool isLeaf = IsLeaf();
	
	TreeNode*	newHead = 0;
	if (Up() && Up() != inFrom)
		newHead = Up()->RemoveFreeLeaves(ioFunctor, this);
	else newHead = this;
    
	return isLeaf ? newHead : this;
}

//---------------------------------------------------------------------------

inline
vector<int>*
TreeNode::CallSetINodesForAdjacentNode(
                                       TreeNode*	adjNode,
                                       TreeNode*	inFrom)
{
    // OBSOLETE
    
	//static Sequence cEmptySequence;
    
	assert(this != NULL);

	/*
	if ((adjNode) != (inFrom)) {
		return adjNode->SetInternalNodes(this);
	} else {
        cout << " at node " << adjNode->GetData().id << ", infrom " << inFrom->GetData().id << endl;
        return &inFrom->GetData().sequence;
//		return (NULL); //(Sequence*) &cEmptySequence;
	}
    */
    if ( !adjNode->IsLeaf() ) {
		return adjNode->SetInternalNodes(this);
	} else {
        cout << " at node " << adjNode->GetData().id << ", infrom " << inFrom->GetData().id << endl;
        return &adjNode->GetData().sequence;
        //		return (NULL); //(Sequence*) &cEmptySequence;
	}

    //	return ((adjNode) != (inFrom))
    //				?	adjNode->SetInternalNodes(this)
    //				:	cEmptySequence;
}


// New function: randomly mix two sequences, return as result...

inline void TreeNode::_mixSequences( std::vector<int>& result,  std::vector<int>& seq1, std::vector<int>& seq2 ) {
	result = seq1;
	
	for (int i=0; i<seq1.size(); i++) {
		if (num->Random() > 0.5) {
			result[i] = seq2[i];
		}
	}
	
}


//---------------------------------------------------------------------------

inline
vector<int>* 
TreeNode::SetInternalNodes( TreeNode*	inFrom) {
//	static Sequence	cEmptySequence;
	vector<int> *seqleft, *seqright;
    
	if (this != NULL)
	{
		if (Left() != NULL || Right() != NULL)
		{
			if (Left() != NULL) seqleft = Left()->SetInternalNodes( this ); //CallSetINodesForAdjacentNode(Left(), inFrom);
            
			if (Right() != NULL) seqright = Right()->SetInternalNodes( this ); //CallSetINodesForAdjacentNode(Right(), inFrom);
            
         //   cout << "Test: node " << GetData().id << ", seqleft is: ";
         //   for (int ii=0; ii< (*seqleft).size(); ii++) {
         //       cout << (*seqleft)[ii];
         //   } 
         //   cout << endl;
            
			if ( (this != Up()->Up() ) ) { // && (GetData().sequence.empty()) )
				if ( !IsLeaf() ) { //(GetData().sequence.empty()) ) {
					if (Left() != NULL && Right() != NULL) {
                        //			GetData().sequence.resize( seqleft->size() );
						_mixSequences(GetData().sequence, *seqleft, *seqright);
                        //                        GetData().sequence.Mix(seqleft, seqright); 
					} else if (Left() == NULL) {
                        //			GetData().sequence.resize( seqleft->size() );
                        //						GetData().sequence.Mix(seqright, seqright); 
						_mixSequences(GetData().sequence, *seqright, *seqright);
					} else if (Right() == NULL) {
                        //			GetData().sequence.resize( seqleft->size() );
                        //						GetData().sequence.Mix(seqleft, seqleft); 
						_mixSequences(GetData().sequence, *seqleft, *seqleft);
					}
                    
                 //   cout << "MIXED SEQ for NODE " << GetData().id << " IS: ";
                 //   for (int ii=0; ii<GetData().sequence.size(); ii++) {
                 //       cout << GetData().sequence[ii];
                 //   } cout << endl;
                    
				}				
            }
			return &GetData().sequence;
		}
		else {
			return &GetData().sequence;
		}
	}
	else
		return( NULL ); //)&cEmptySequence);
}

//---------------------------------------------------------------------------

inline
vector<int>* 
TreeNode::SetInternalNodes(void)
{
	return SetInternalNodes(0);
}

//---------------------------------------------------------------------------

inline
double
TreeNode::getdist(
                  TreeNode*	node) 
{
	double distance = cInitDistance;
	
	if (node == NULL)
		cerr << "Error: no distance when a node is null (" << hex << node << "--" << this << dec << ")" << endl;
	else
	{
		if(Up() == node && GetData().distance > cInitDistance+1)
			distance = GetData().distance;
		else if (node->Up() == this && node->GetData().distance > cInitDistance+1)
			distance = node->GetData().distance;
	}
	
	return distance;
}

//---------------------------------------------------------------------------
// print distance between two nodes

inline
void
TreeNode::printdist(
                    ofstream& outStream,
                    double& scale,
                    TreeNode* node) 
{
	if(node!=NULL)
	{
		double distance=getdist(node);
		
		if (distance > cInitDistance + 1)
			outStream << ":" << fixed << setprecision(5) << distance * scale;
        
		outStream << flush;
	}
}

//---------------------------------------------------------------------------

inline
int
TreeNode::changeinmatrix(
                         IntMatrix& countMatrix) 
{
	int count = 0;
	
	for (int i = 0; i < countMatrix.rows(); i++)
	{
		for (int j = 0; j < countMatrix.cols(); j++)
		{
			if (i != j)
				count += countMatrix[i][j];		 
		}
	}
	
	return(count);
}								

//---------------------------------------------------------------------------

inline
double
TreeNode::getchange(
                    TreeNode*	node) 
{
	double change = cInitDistance;
	
	if (node == NULL)
		cerr << "Error: no distance when a node is null (" << hex << node << "--" << this << dec << ")" << endl;
	else
	{
		if(Up() == node && GetData().distance > cInitDistance+1)
			change = (double) changeinmatrix(GetData().countchange) / (double) GetData().sequence.size();
		else if (node->Up() == this && node->GetData().distance > cInitDistance+1)
			change = (double) changeinmatrix(node->GetData().countchange) / (double) node->GetData().sequence.size();
	}
	
	return change;
}

//---------------------------------------------------------------------------
// print distance between two nodes

inline void TreeNode::printchange( ofstream& outStream, TreeNode* node) {
    
	// Modified to output branchlengths by Jason...
	
	if(node!=NULL)
	{
		double change = GetData().distance;
        //		double change = getchange(node);
		
		if (change > cInitDistance + 1)
			outStream << ":" << fixed << setprecision(5) << change;
        
		outStream << flush;
	}
}

//---------------------------------------------------------------------------

inline
void
TreeNode::PrintTree(
                    ofstream& outStream, 
                    int& site, 
                    double& scale,
                    TreeNode* inFrom) 
{
	int noiselevel;
    
	if(IsLeaf())
	{
		// it is a terminal node.
		if(inFrom==NULL && Up()!=NULL)
		{	
			// don't want to begin with a terminal node.
			Up()->PrintTree(outStream, site, scale, 0);
		}	
		else
		{
			if (site < 0)
				outStream << GetData().name;
			else
			{
				ostringstream sitename;
				sitename << GetData().sequence[site];
				outStream << sitename.str() << "\t" << GetData().name;
			}
		}
	}
	else
	{	   /*  it is an internal node.  */
		outStream << "(";
        
		if (Up() != inFrom && Up() != NULL && Up()->Up() != this)
		{
			Up()->PrintTree(outStream, site, scale, this);
			Up()->printdist(outStream, scale, this);
		}
		
		if (Left() != inFrom && Left() != NULL)
		{
			if (Up() != inFrom && Up() !=NULL && Up()->Up() != this)
				outStream << ",";
            
			Left()->PrintTree(outStream, site, scale, this);
			printdist(outStream, scale, Left());
		}
		
		if (Right() != inFrom && Right() !=NULL)
		{
			if ((Up()!=inFrom && Up() !=NULL && Up()->Up() != this) || (Left()!=inFrom && Left()!=NULL)) 
				outStream << ",";
            
			Right()->PrintTree(outStream, site, scale, this);
			printdist(outStream, scale, Right());
		}
		
		outStream << ")";
        
		if(inFrom==NULL)
		{
			outStream << ";" << endl;
		}
	}
	
	outStream << flush;
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SumChangesOverSubtree(
                                IntMatrix& m_matCountChange)
{
	if (!GetData().countchange.empty())
	{
		if (m_matCountChange.empty()) {
            m_matCountChange.resize(GetData().countchange.rows(), GetData().countchange.cols(), 0);
		}
        
		IntMatrix* handle = &GetData().countchange;
		
       // int total = 0;
		for (int i=0;i<(GetData().countchange.cols());i++) {
			for (int j=0; j<(GetData().countchange.rows()); j++) {
				//total += 
                m_matCountChange.mRows.mDataArray[i].mDataArray[j] += handle->mRows.mDataArray[i].mDataArray[j];
			}
		}
        // cout << "   [at node " << GetData().id << "] TOTAL: " << total << endl;
		//m_matCountChange += GetData().countchange;
	}
    
	if (Left() != NULL)
		Left()->SumChangesOverSubtree(m_matCountChange); 
    
	if (Right() != NULL) 
		Right()->SumChangesOverSubtree(m_matCountChange);  
}

// NEW function: if there is a node name defined, check it against ObservedData and assign a reference to the sequence as necessary

inline void TreeNode::assignSequencesToLeaves( ObservedData &theData ) {
 //   cout << "Assigning taxon name " << GetData().name << "..." << endl;
	
	// Resize the sequence vector...
	GetData().sequence.resize( theData.getNumSequences() );
	
    for (int i=0; i < theData.getNumSequences(); i++) {
     //   cout << "   Comparing to name " << i << ": " << theData.getTaxonName_ref(i) << endl;
        if ( ( GetData().name == theData.getTaxonName_ref(i) ) && ( GetData().name != "" ) ) {
            if ( theData.getSequence_ref(i).size() == 0 ) {
				GetData().sequence.resize( theData.getAlignmentLength() );
			} else {
				GetData().sequence = theData.getSequence_ref(i);
			}
		}
    }
    
    if ( Left() != NULL )
        Left()->assignSequencesToLeaves( theData );
    
    if ( Right() != NULL )
        Right()->assignSequencesToLeaves( theData );
    
}


inline
DoubleMatrix *
TreeNode::getAugmentedCountMatrix( TreeNode *theNode ) {
	if (theNode != NULL) {
		if (theNode->getUniqueModelFlag()) {
            //			std::cout << "returning node changecountmat" << endl;
			return &theNode->GetData().augmentedCounts;
		} else {
            //			std::cout << "returning ROOT changecountmat" << endl;
			return &mRoot->GetData().augmentedCounts;
		}
	}
    return NULL;
    //else {
	//	return &mRoot->GetData().countModelchange3D;
	//}
}


inline
DoubleMatrix *
TreeNode::getChangeMatrix3D( TreeNode *theNode, int theSite ) {
	// Use this version if we are caching the site-specific change counts...
    //	if (theNode != NULL) {
    //		if (theNode->getUniqueModelFlag()) {
    //			std::cout << "returning node changecountmat" << endl;
    //			return &theNode->GetData().countModelchange3D_perSite.mDataArray[theSite];
    //		} else {
    //			std::cout << "returning ROOT changecountmat" << endl;
    return &mRoot->GetData().countModelchange_perSite.mDataArray[theSite];
    //		}
    ///	} //else {
	//	return &mRoot->GetData().countModelchange3D;
	//}
}


inline
DoubleMatrix3D *
TreeNode::getChangeMatrix3D( TreeNode *theNode ) {
	if (theNode != NULL) {
		if (theNode->getUniqueModelFlag()) {
            //			std::cout << "returning node changecountmat" << endl;
			return &theNode->GetData().countModelchange3D;
		} else {
            //			std::cout << "returning ROOT changecountmat" << endl;
			return &mRoot->GetData().countModelchange3D;
		}
	} 
    return NULL;
    //else {
	//	return &mRoot->GetData().countModelchange3D;
	//}
}



inline
IntMatrix *
TreeNode::getChangeMatrix( TreeNode *theNode ) {
	if (theNode != NULL) {
		if (theNode->getUniqueModelFlag()) {
            //			std::cout << "returning node changecountmat" << endl;
			return &theNode->GetData().countchange;
		} else {
            //	std::cout << "returning root aliased countchange matrix..." << endl;
            //			std::cout << "returning ROOT changecountmat" << endl;
			return &mRoot->GetData().countchange;
		}
	}
    
    return NULL;
    //else {
	//	return &mRoot->GetData().countModelchange3D;
	//}
}

inline DoubleMatrix * TreeNode::getJointDwell( TreeNode *theNode ) {
	// Overriding branch-specific stuff for now...
	
	if (theNode != NULL) {
		if (theNode->getUniqueModelFlag()) {
			//			std::cout << "returning node changecountmat" << endl;
			return &mRoot->GetData().jointDwellTimes;
		} else {
			//			std::cout << "returning ROOT changecountmat" << endl;
			return &mRoot->GetData().jointDwellTimes;
		}
	}
    
    return NULL;
    //else {
	//	return &mRoot->GetData().countModelchange3D;
	//}
}


inline
DoubleMatrix *
TreeNode::getMarginalDwell( TreeNode *theNode ) {
	// Overriding this for now, just returning root matrix
	if (theNode != NULL) {
		if (theNode->getUniqueModelFlag()) {
			//			std::cout << "returning node changecountmat" << endl;
			return &mRoot->GetData().marginalDwellTimes;
		} else {
			//			std::cout << "returning ROOT changecountmat" << endl;
			return &mRoot->GetData().marginalDwellTimes;
		}
	}
    return NULL;
    //else {
	//	return &mRoot->GetData().countModelchange3D;
	//}
}

inline void
TreeNode::reassignSiteCountsBetweenRateCategories_Uni(  int site,
                                                      int lastAssignment,
                                                      int newAssignment,
                                                      ObservedData& theData,
                                                      IntVector& m_vecSiteAssignment
                                                      ) {
	// Subtract the site-specific virtual substitutions from one category and assign to another...
    //	int numCat = options.numCatASRV;
	int modelassignment = m_vecSiteAssignment.mDataArray[site];
	
	int numStates = theData.getNumStates();
	
	DoubleMatrix	   *changeCountsPerSite;
	DoubleMatrix      *augmentedCount;
    
	IntVector   *totalCount_actualSubs;
	
	totalCount_actualSubs   = getTotalSubCountVector( mRoot );
	augmentedCount		= getAugmentedCountMatrix( mRoot );
	changeCountsPerSite = getChangeMatrix3D( mRoot, site );
    
	// We seem to be having off by 1 errors!
	for (unsigned int i=0; i<numStates; i++) {
		(*augmentedCount).mRows.mDataArray[num->returnModelIndex( modelassignment, lastAssignment, 0)].mDataArray[i] -= (*changeCountsPerSite).mRows.mDataArray[i].mDataArray[i];
		if ((*augmentedCount).mRows.mDataArray[num->returnModelIndex( modelassignment, lastAssignment, 0)].mDataArray[i] < 0) {
			std::cout << "ERROR!  WE REMOVED MORE THAN EXISTED!  State " << i << ", site " << site << " curent Count is " << (*augmentedCount).mRows.mDataArray[num->returnModelIndex( modelassignment, lastAssignment, 0)].mDataArray[i] << endl; 
			std::cout << "changecountspersite for state " << i << " said " << (*changeCountsPerSite).mRows.mDataArray[i].mDataArray[i] << ", while augmentedCount for lastAssignment " << lastAssignment << " said " << (*augmentedCount).mRows.mDataArray[num->returnModelIndex( modelassignment, lastAssignment, 0)].mDataArray[i] << " (after subtraction)" << endl;
		}
		(*augmentedCount).mRows.mDataArray[num->returnModelIndex( modelassignment, newAssignment, 0)].mDataArray[i]  += (*changeCountsPerSite).mRows.mDataArray[i].mDataArray[i];
		for (unsigned int j=0; j<numStates; j++) {
			if (j != i) {
				(*totalCount_actualSubs).mDataArray[ lastAssignment ] -= (*changeCountsPerSite).mRows.mDataArray[i].mDataArray[j];
				(*totalCount_actualSubs).mDataArray[ newAssignment ]  += (*changeCountsPerSite).mRows.mDataArray[i].mDataArray[j];
			}
		}
	}
	
}

//---------------------------------------------------------------------------

inline
void
TreeNode::CountSimpleChangesOnSubtree( ObservedData& theData ) {
    int numStates = theData.getNumStates();
    
	if ((this == mRoot) && ( (options.m_nMethod == 3) || (options.m_nMethod == 5) || (options.m_nMethod == 6) ) ) {
        IntMatrix *ChangeCount = getChangeMatrix(this);
        
		if (ChangeCount->empty()) {
			ChangeCount->resize(numStates, numStates, 0);
		} else {
			ChangeCount->fill(0);
		}
	}
	
	if (Left() != NULL)
	{
		// not the assumed root node
        //	if ( Up()->Up() != this)
        CountSimpleChangesOnBranch(this, Left(), theData );
        //	else
        //		CountSimpleChangesOnBranch(Right(), Left(), inStateindex, minstate, numStates);
        
		Left()->CountSimpleChangesOnSubtree(theData ); 
	}
    
	if (Right() != NULL) 
	{ 
        //	if ( Up()->Up() != this)
        CountSimpleChangesOnBranch(this, Right(), theData );
        //	else
        //		CountSimpleChangesOnBranch(,Left(), Right(), inStateindex, minstate, numStates);
        
		Right()->CountSimpleChangesOnSubtree( theData );  
	}
}

//---------------------------------------------------------------------------

inline void
TreeNode::CountSimpleChangesOnBranch( TreeNode* node1, TreeNode* node2, ObservedData &theData) {
    int numStates = theData.getNumStates();
    
	IntMatrix *ChangeCount;
	ChangeCount = getChangeMatrix(node2);
	
	if (ChangeCount->empty()) {
		ChangeCount->resize(numStates, numStates, 0);
	} else {
		if (options.m_nMethod != 3) {
			ChangeCount->fill(0);
		}
	}
	
	if (node2->GetData().countchange.empty()) {
		node2->GetData().countchange.resize(numStates, numStates, 0);
	} else {
		node2->GetData().countchange.fill(0);
	}
    //	int totalCh = 0;
    
	if (node1 != NULL && node2 != NULL)
	{
		std::vector<int>& seq1 = node1->GetData().sequence; 
		std::vector<int>& seq2 = node2->GetData().sequence; 
		
		double& distance = node2->GetData().distance;
		
		if (seq1.size() && seq2.size() && distance != cInitDistance)
		{
			int	len = min(seq1.size(), seq2.size());
			
			int indx1, indx2;
			for (int s = 0; s < len; s++) {
                indx1 = seq1[s];
                indx2 = seq2[s];
                //SIndex(seq2.mImpl->mSequenceStr[s], inStateindex, minstate);
				//if (node2->IsLeaf())
//               cout << "site " << s << ", indx1 is " << indx1 << ", indx2 is " << indx2 << "; node1 " << node1->GetData().id << ", node2 " << node2->GetData().id << endl;
                
                // For some reason we got an indx1 == -1 at some point...adding extra conditional
				if ((indx1 < numStates) && (indx2 < numStates) && (indx1 >= 0) && (indx2 >= 0) ) {
                    //					std::cout << "indx1 " << indx1 << ", indx2 " << indx2 << "; numstates " << numStates << endl;
					(*ChangeCount).mRows.mDataArray[indx1].mDataArray[indx2] += 1; 
					node2->GetData().countchange.mRows.mDataArray[indx1].mDataArray[indx2] += 1;
                 //   cout << "Adding transition for " << indx1 << "-> " << indx2 << endl;
                    //					if (indx1 != indx2) {
                    //						totalCh++;
                    //						if ( (node1->Left() == node2) && (node1->Right() == NULL) ) {
                    //							std::cout << "		" << s << ": " << indx1 << " -> " << indx2 << endl;
                    //						}
                    //					}
					
				}
			}
            
            if (getChangeMatrix(node2)->empty())
                node2->GetData().countchange.resize(numStates, numStates, 0);
            
		//[block - why??]	
            node2->GetData().countchange = *ChangeCount;
		}
		
        /*	if ( (node1->Left() == node2) && (node1->Right() == NULL) ) {
         std::cout << "1/ Doing a branch change count for one mof my sub branches; count = " << totalCh << endl;
         }
         if ( (node1->Right() == node2) && (node1->Left() == NULL) ) {
         std::cout << "2/ Doing a branch change count for one mof my sub branches; count = " << totalCh << endl;
         }
         */	
		
	}
}




inline
void
TreeNode::PrintTreeChanges(
                           ofstream& outStream, 
                           TreeNode* inFrom) 
{
    if(IsLeaf())
    {
        // it is a terminal node.
        if(inFrom==NULL && Up()!=NULL)
        {	
            // don't want to begin with a terminal node.
            Up()->PrintTreeChanges(outStream, 0);
        }	
        else
            outStream << GetData().name;
    }
    else
    {	   //  it is an internal node.  
        outStream << "(";
        
        if (Up() != inFrom && Up() != NULL && Up()->Up() != this)
        {
            Up()->PrintTreeChanges(outStream, this);
            Up()->printchange(outStream, this);
        }
        
        if (Left() != inFrom && Left() != NULL)
        {
            if (Up() != inFrom && Up() !=NULL && Up()->Up() != this)
                outStream << ",";
            
            Left()->PrintTreeChanges(outStream, this);
            printchange(outStream, Left());
        }
        
        if (Right() != inFrom && Right() !=NULL)
        {
            if ((Up()!=inFrom && Up() !=NULL && Up()->Up() != this) || (Left()!=inFrom && Left()!=NULL)) 
                outStream << ",";
            
            Right()->PrintTreeChanges(outStream, this);
            printchange(outStream, Right());
        }
        
        outStream << ")";
        
        if(inFrom==NULL)
        {
            outStream << ";" << endl;
        }
    }
    
    outStream << flush;
}

// Obsolete stuff
//---------------------------------------------------------------------------

/*
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::printseq(
 ofstream& 	outStream,
 const Sequence&	inSeq,
 const char*	name,
 int		id,
 int		type) const
 {
 int j;
 
 outStream << NEWSEQ;
 
 if (type==PIR) 
 outStream << "P1;";
 
 if (name!=NULL && name[0]!=EOS) 
 outStream << name;
 else outStream << id;
 
 outStream << endl;
 
 if (type==PIR) 
 outStream << name << endl;
 
 // Modified by Jason: doing this is causing the bounds-check on Sequence_Impl::operator[] to take up way too much time!
 //	for(j=0;inSeq[j]!= ENDSEQ;j++)
 
 int seqSize = inSeq.size();
 for(j=0;j < seqSize;j++)
 {
 outStream << inSeq[j];
 
 if (((j+1) % BLOCK) == 0 ) 
 outStream << endl;
 }
 
 if (type==PIR) 
 outStream << ENDSEQ;
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::printseq(
 ofstream& 	outStream,
 const Sequence&	inSeq,
 const char*	name,
 int		id,
 int		type,
 const int& 	count) const
 {
 int j;
 
 outStream << NEWSEQ;
 
 if (type==PIR) 
 outStream << "P1;";
 
 if (name!=NULL && name[0]!=EOS) 
 outStream << name << "-" << count;
 else outStream << id << "-" << count;
 
 outStream << endl;
 
 if (type==PIR) 
 outStream << name << endl;
 
 // Modified by Jason: doing this is causing the bounds-check on Sequence_Impl::operator[] to take up way too much time!
 //	for(j=0;inSeq[j]!= ENDSEQ;j++)
 
 int seqSize = inSeq.size();
 for(j=0;j < seqSize;j++)
 {
 outStream << inSeq[j];
 
 if (((j+1) % BLOCK) == 0 ) 
 outStream << endl;
 }
 
 if (type==PIR) 
 outStream << ENDSEQ;
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::printseqcontent(
 ofstream& 	outStream,
 const Sequence&	inSeq,
 const IntVector& inStateindex,
 const int& minstate,
 const int& numStates) const
 {
 int	len = inSeq.size();
 
 if ( len >0 )
 {
 IntVector	Count(numStates, 0);
 
 for (int s = 0; s < len; s++)
 {
 int indx = SIndex(inSeq[s], inStateindex, minstate);
 
 if (indx < numStates)
 Count[indx] ++;
 }
 
 DoubleVector	Content(numStates, 0);
 
 for (unsigned int i = 0; i< numStates; i++)
 {
 Content[i] = (double)Count[i] / (double)len;
 outStream << setw(8) << setprecision(4) << Content[i] << setprecision(0) << setw(0) << " ";
 }	
 }
 }
 
 //---------------------------------------------------------------------------
 
 inline
 DoubleVector
 TreeNode::calculatenodecontent(
 const Sequence&	inSeq,
 const IntVector& inStateindex,
 const int& minstate,
 const int& numStates) const
 {
 int	len = inSeq.size();
 
 DoubleVector	Content(numStates, 0);
 
 if ( len >0 )
 {
 IntVector	Count(numStates, 0);
 
 for (int s = 0; s < len; s++)
 {
 int indx = SIndex(inSeq[s], inStateindex, minstate);
 
 if (indx < numStates)
 Count[indx] ++;
 }
 
 for (unsigned int i = 0; i< numStates; i++)
 Content[i] = (double)Count[i] / (double)len;
 }
 
 return(Content);
 }
 
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintTreeSequence(
 ofstream& outStream, 
 const TreeNode* inFrom) const
 {
 if (this != NULL)
 {
 if(GetData().sequence.size())
 {
 printseq(outStream, Sequence(GetData().sequence), GetData().name, GetData().id, FASTA);
 
 outStream << endl;
 }
 
 if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
 Left()->PrintTreeSequence(outStream, this);
 
 if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
 Right()->PrintTreeSequence(outStream, this);
 
 if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
 Up()->PrintTreeSequence(outStream, this);
 }
 
 outStream << flush;
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintTreeSequence(
 ofstream& outStream) const
 {
 PrintTreeSequence(outStream, 0);
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintTreeContent(
 ofstream& outStream, 
 const TreeNode* inFrom,
 const IntVector& inStateindex,
 const int& minstate,
 const int& numStates) const
 {
 if (this != NULL)
 {
 if(GetData().sequence.size())
 printseqcontent(outStream, Sequence(GetData().sequence), inStateindex, minstate, numStates);
 
 if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
 Left()->PrintTreeContent(outStream, this, inStateindex, minstate, numStates);
 
 if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
 Right()->PrintTreeContent(outStream, this, inStateindex, minstate, numStates);
 
 if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
 Up()->PrintTreeContent(outStream, this, inStateindex, minstate, numStates);
 }
 
 outStream << flush;
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintTreeContent(
 ofstream& outStream, 
 const IntVector& inStateindex,
 const int& minstate,
 const int& numStates) const
 {
 PrintTreeContent(outStream, 0, inStateindex, minstate, numStates);
 outStream << endl;
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::CalculateNodeContent(
 DoubleVector& internalContent,
 DoubleVector& leafContent,
 int& numInternalnode,
 int& numLeafnode,
 const TreeNode* inFrom,
 const IntVector& inStateindex,
 const int& minstate,
 const int& numStates) const
 {
 if (this != NULL)
 {
 if (Up()->Up() == this)
 {
 if (internalContent.empty())
 {
 internalContent.resize(numStates, 0);
 numInternalnode = 0;
 numLeafnode = 0;
 }
 else
 {
 internalContent.fill(0);
 numInternalnode = 0;
 numLeafnode = 0;
 }
 
 if (leafContent.empty())
 {
 leafContent.resize(numStates, 0);
 numInternalnode = 0;
 numLeafnode = 0;
 }
 else
 {
 leafContent.fill(0);
 numInternalnode = 0;
 numLeafnode = 0;
 }
 }
 else if ( (GetData().sequence.size()) && ((Left() != NULL) || (Right() != NULL)) )
 {
 internalContent += calculatenodecontent(Sequence(GetData().sequence), inStateindex, minstate, numStates);
 numInternalnode++;
 }
 else if ( (GetData().sequence.size()) && ((Left() == NULL) && (Right() == NULL)) )
 {
 leafContent += calculatenodecontent(Sequence(GetData().sequence), inStateindex, minstate, numStates);
 numLeafnode++;
 }
 
 if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
 Left()->CalculateNodeContent(internalContent, leafContent, numInternalnode, numLeafnode, this, inStateindex, minstate, numStates);
 
 if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
 Right()->CalculateNodeContent(internalContent, leafContent, numInternalnode, numLeafnode, this, inStateindex, minstate, numStates);
 
 if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
 Up()->CalculateNodeContent(internalContent, leafContent, numInternalnode, numLeafnode, this, inStateindex, minstate, numStates);
 }
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::CalculateNodeContent(
 DoubleVector& internalContent,
 DoubleVector& leafContent,
 int& numInternalnode,
 int& numLeafnode,
 const IntVector& inStateindex,
 const int& minstate,
 const int& numStates) const
 {
 CalculateNodeContent(internalContent, leafContent, numInternalnode, numLeafnode, 0, inStateindex, minstate, numStates);
 }
 
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintInternalNodeSequence(
 ofstream& outStream,
 const TreeNode* inFrom) const
 {
 if (this != NULL)
 {
 if ( (GetData().sequence.size()) && ((Left() != NULL) || (Right() != NULL)) )
 {
 printseq(outStream, Sequence(GetData().sequence), GetData().name, GetData().id, FASTA);
 
 outStream << endl;
 }
 
 if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
 Left()->PrintInternalNodeSequence(outStream, this);
 
 if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
 Right()->PrintInternalNodeSequence(outStream, this);
 
 if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
 Up()->PrintInternalNodeSequence(outStream, this);
 }
 
 outStream << flush;
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintInternalNodeSequence(
 ofstream& outStream) const
 {
 PrintInternalNodeSequence(outStream, 0);
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintLeafSequence(
 ofstream& outStream,
 const TreeNode* inFrom) const
 {
 if (this != NULL)
 {
 if ( (GetData().sequence.size()) && ((Left() == NULL) && (Right() == NULL)) )
 {
 printseq(outStream, Sequence(GetData().sequence), GetData().name, GetData().id, FASTA);
 
 outStream << endl;
 }
 
 if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
 Left()->PrintLeafSequence(outStream, this);
 
 if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
 Right()->PrintLeafSequence(outStream, this);
 
 if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
 Up()->PrintLeafSequence(outStream, this);
 }
 
 outStream << flush;
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintLeafSequence(
 ofstream& outStream) const
 {
 PrintLeafSequence(outStream, 0);
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintNodeSequence(
 ofstream& outStream,
 const TreeNode* inFrom,
 const int& node) const
 {
 if (this != NULL)
 {
 if ( (GetData().sequence.size()) && (GetData().id == node) )
 {
 printseq(outStream, Sequence(GetData().sequence), GetData().name, GetData().id, FASTA);
 
 outStream << endl;
 }
 
 if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
 Left()->PrintNodeSequence(outStream, this, node);
 
 if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
 Right()->PrintNodeSequence(outStream, this, node);
 
 if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
 Up()->PrintNodeSequence(outStream, this, node);
 }
 
 outStream << flush;
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintNodeSequence(
 ofstream& outStream,
 const int& node) const
 {
 PrintNodeSequence(outStream, 0, node);
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintNodeSequence(
 ofstream& outStream,
 const TreeNode* inFrom,
 const char* nodename) const
 {
 if (this != NULL)
 {
 if ( (GetData().sequence.size()) && (strcmp(GetData().name, nodename) == 0) )
 {
 printseq(outStream, Sequence(GetData().sequence), GetData().name, GetData().id, FASTA);
 
 outStream << endl;
 }
 
 if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
 Left()->PrintNodeSequence(outStream, this, nodename);
 
 if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
 Right()->PrintNodeSequence(outStream, this, nodename);
 
 if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
 Up()->PrintNodeSequence(outStream, this, nodename);
 }
 
 outStream << flush;
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintNodeSequence(
 ofstream& outStream,
 const char* nodename) const
 {
 PrintNodeSequence(outStream, 0, nodename);
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintInternalNodeSequence(
 ofstream& outStream,
 const TreeNode* inFrom,
 const int& count) const
 {
 if (this != NULL)
 {
 if ( (GetData().sequence.size()) && ((Left() != NULL) || (Right() != NULL)) )
 {
 printseq(outStream, Sequence(GetData().sequence), GetData().name, GetData().id, FASTA, count);
 
 outStream << endl;
 }
 
 if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
 Left()->PrintInternalNodeSequence(outStream, this, count);
 
 if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
 Right()->PrintInternalNodeSequence(outStream, this, count);
 
 if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
 Up()->PrintInternalNodeSequence(outStream, this, count);
 }
 
 outStream << flush;
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintInternalNodeSequence(
 ofstream& outStream,
 const int& count) const
 {
 PrintInternalNodeSequence(outStream, 0, count);
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintNodeSequence(
 ofstream& outStream,
 const TreeNode* inFrom,
 const int& node,
 const int& count) const
 {
 if (this != NULL)
 {
 if ( (GetData().sequence.size()) && (GetData().id == node) )
 {
 printseq(outStream, Sequence(GetData().sequence), GetData().name, GetData().id, FASTA, count);
 
 outStream << endl;
 }
 
 if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
 Left()->PrintNodeSequence(outStream, this, node, count);
 
 if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
 Right()->PrintNodeSequence(outStream, this, node, count);
 
 if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
 Up()->PrintNodeSequence(outStream, this, node, count);
 }
 
 outStream << flush;
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintNodeSequence(
 ofstream& outStream,
 const int& node,
 const int& count) const
 {
 PrintNodeSequence(outStream, 0, node, count);
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintNodeSequence(
 ofstream& outStream,
 const TreeNode* inFrom,
 const char* nodename,
 const int& count) const
 {
 if (this != NULL)
 {
 if ( (GetData().sequence.size()) && (strcmp(GetData().name, nodename) == 0) )
 {
 printseq(outStream, Sequence(GetData().sequence), GetData().name, GetData().id, FASTA, count);
 
 outStream << endl;
 }
 
 if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
 Left()->PrintNodeSequence(outStream, this, nodename, count);
 
 if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
 Right()->PrintNodeSequence(outStream, this, nodename, count);
 
 if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
 Up()->PrintNodeSequence(outStream, this, nodename, count);
 }
 
 outStream << flush;
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintNodeSequence(
 ofstream& outStream,
 const char* nodename,
 const int& count) const
 {
 PrintNodeSequence(outStream, 0, nodename, count);
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintInternalNodeSequence(
 StringMatrix& InternalnodeSeq, 
 const  TreeNode* inFrom,
 const int& m_nSampleSequence, 
 IntVector& NodeID) const
 {
 static int number;
 
 if (this != NULL)
 {
 if (Up()->Up() == this)
 number = 0;
 
 if ( (GetData().sequence.size()) && ((Left() != NULL) || (Right() != NULL)) )
 {
 Sequence tempSequence = GetData().sequence;
 
 InternalnodeSeq[number][m_nSampleSequence] = tempSequence.str();
 
 NodeID[number] = GetData().id;
 
 number++;
 }
 
 if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
 Left()->PrintInternalNodeSequence(InternalnodeSeq, this, m_nSampleSequence, NodeID);
 
 if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
 Right()->PrintInternalNodeSequence(InternalnodeSeq, this, m_nSampleSequence, NodeID);
 
 if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
 Up()->PrintInternalNodeSequence(InternalnodeSeq, this, m_nSampleSequence, NodeID);
 }
 }
 
 //---------------------------------------------------------------------------
 
 inline
 void
 TreeNode::PrintInternalNodeSequence(
 StringMatrix& InternalnodeSeq, 
 const int& m_nSampleSequence, 
 IntVector& NodeID) const
 {
 PrintInternalNodeSequence(InternalnodeSeq, 0, m_nSampleSequence, NodeID);
 }
 
 //---------------------------------------------------------------------------
 
 inline
 unsigned int
 TreeNode::SIndex(
 const char& state,
 const IntVector& inStateindex,
 const int& minstate)
 { 
 //	std::cout << "Sindex: " << state << ", size " << inStateindex.size() << endl;
 
 // Removed by Jason: calls to ispunct() were taking too much time.  Why is it there anyway??
 //	return (ispunct(state))
 //			?	inStateindex.size()
 //			:	inStateindex.mDataArray[static_cast<int>(state) - minstate]; 
 
 return inStateindex.mDataArray[static_cast<int>(state) - minstate]; 
 
 }
 */


//---------------------------------------------------------------------------

inline
void
TreeNode::NormalizeMatrix(
                          DoubleMatrix& m_probMatrix)
{
	// WHY?!?!
	
	for (int i = 0; i < m_probMatrix.rows(); i++)
	{
		double sum = 0;
		
		for (int j = 0; j< m_probMatrix.cols(); j++)
			sum += m_probMatrix.mRows.mDataArray[i].mDataArray[j];
        
		//std::cout << "CumProb calc, row " << i << ": sum = " << sum << endl;
		
		for (int j = 0; j< m_probMatrix.cols(); j++)
			m_probMatrix.mRows.mDataArray[i].mDataArray[j] = m_probMatrix.mRows.mDataArray[i].mDataArray[j]/sum;
	}
	
}	

//---------------------------------------------------------------------------

inline
void
TreeNode::NormalizeMatrix(
                          DoubleMatrix& m_probMatrix, int row)
{
	// WHY?!?!
	int i = row;
	//for (int i = 0; i < m_probMatrix.rows(); i++)
    double sum = 0;
    
    for (int j = 0; j< m_probMatrix.cols(); j++)
        sum += m_probMatrix.mRows.mDataArray[i].mDataArray[j];
    
    //std::cout << "CumProb calc, row " << i << ": sum = " << sum << endl;
    
    for (int j = 0; j< m_probMatrix.cols(); j++)
        m_probMatrix.mRows.mDataArray[i].mDataArray[j] = m_probMatrix.mRows.mDataArray[i].mDataArray[j]/sum;
	
}	
//---------------------------------------------------------------------------

inline
void
TreeNode::SetBranchBLProb(
                          TreeNode*	node,
                          int numStates,
                          DoubleMatrix3D& inRateMatrix3D)
{
	DoubleMatrix *ProbMatrix;
	unsigned int numModels = inRateMatrix3D.rows();
    
	if (node != NULL)
	{
        //		if (ProbMatrix->empty())
        //			ProbMatrix->resize(numStates, numStates, 0);
        //		else
        //			ProbMatrix->fill(0);
        //[debug]		std::cout << "doing b0prob for branch starting at " << node->GetData().id << endl;
        
		if (node->GetData().probmatrix3D.empty())
			node->GetData().probmatrix3D.resize(numModels, numStates, numStates, 0);
        //		else
        //			ProbMatrix->fill(0);
        
		for (unsigned int k = 0; k < numModels; k++)
		{
			const double& distance = node->GetData().distance;
            
			ProbMatrix = &(*getProbMatrix3D(node))[k]; //&node->GetData().probmatrix3D[k];
			
			if ((distance != cInitDistance) || (node->Up()->Up()->Up() == node->Up()))
			{
				// BL method - rate matrix to probability matrix;
				for (unsigned int i = 0; i < numStates; i++)
				{
					for (unsigned int j = 0; j < numStates; j++)
					{
						if (i != j) {
							(*ProbMatrix)[i][j] = inRateMatrix3D[k][i][j];
                            //	std::cout << "TEST: model " << k << ", i,j=" << i << ", " << j << " = " << (*ProbMatrix)[i][j] << endl;
						}
						else
							(*ProbMatrix)[i][j] = 1 + inRateMatrix3D[k][i][j];
					}
				}
                
			}
		}
	}
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetBranchBLProb(
                          TreeNode*	node,
                          int numStates,
                          DoubleMatrix3D& inRateMatrix3D, int dummyGibbs)
{
	// NO checking for substitutions...
	DoubleMatrix *ProbMatrix;
	unsigned int numModels = inRateMatrix3D.rows();
    
	if (node != NULL)
	{
        //		if (ProbMatrix->empty())
        //			ProbMatrix->resize(numStates, numStates, 0);
        //		else
        //			ProbMatrix->fill(0);
        //[debug]		std::cout << "doing b0prob for branch starting at " << node->GetData().id << endl;
        
		if (node->GetData().probmatrix3D.empty())
			node->GetData().probmatrix3D.resize(numModels, numStates, numStates, 0);
        //		else
        //			ProbMatrix->fill(0);
        
		for (unsigned int k = 0; k < numModels; k++)
		{
			const double& distance = node->GetData().distance;
            
			ProbMatrix = &(*getProbMatrix3D(node))[k]; //&node->GetData().probmatrix3D[k];
			
			if ((distance != cInitDistance) || (node->Up()->Up()->Up() == node->Up()))
			{
				// BL method - rate matrix to probability matrix;
				for (unsigned int i = 0; i < numStates; i++)
				{
					for (unsigned int j = 0; j < numStates; j++)
					{
						if (i != j) {
							(*ProbMatrix)[i][j] = inRateMatrix3D[k][i][j];
                            //	std::cout << "TEST: model " << k << ", i,j=" << i << ", " << j << " = " << (*ProbMatrix)[i][j] << endl;
						}
						else
							(*ProbMatrix)[i][j] = 1 + inRateMatrix3D[k][i][j];
					}
				}
			}
		}
	}
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetTreeBLProb(
                        int numStates,
                        DoubleMatrix3D& inRateMatrix3D)
{
	// Unless there is branch heterogeneity, this is the same for all branches.  Just do it once.
	SetBranchBLProb(mRoot, numStates, inRateMatrix3D);
    
    // disabling
    /*
     if (Left() != NULL)
     {
     SetBranchBLProb(Left(), numStates, inRateMatrix3D);
     
     Left()->SetTreeBLProb(numStates, inRateMatrix3D); 
     }
     
     if (Right() != NULL) 
     { 
     SetBranchBLProb(Right(), numStates, inRateMatrix3D);
     
     Right()->SetTreeBLProb(numStates, inRateMatrix3D);  
     }
     */
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetTreeBLProb(
                        int numStates,
                        DoubleMatrix3D& inRateMatrix3D, int dummyGibbs)
{
	SetBranchBLProb(mRoot, numStates, inRateMatrix3D, dummyGibbs);
    
    //disabling
    /*
     if (Left() != NULL)
     {
     SetBranchBLProb(Left(), numStates, inRateMatrix3D);
     
     Left()->SetTreeBLProb(numStates, inRateMatrix3D); 
     }
     
     if (Right() != NULL) 
     { 
     SetBranchBLProb(Right(), numStates, inRateMatrix3D);
     
     Right()->SetTreeBLProb(numStates, inRateMatrix3D);  
     }
     */
}
//---------------------------------------------------------------------------

inline
void
TreeNode::SetBranchB1Prob(
                          TreeNode*	node,
                          int numStates,
                          DoubleMatrix3D& inRateMatrix3D)
{
	SetBranchB1Prob( node, numStates, inRateMatrix3D, -1, -1 );
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetBranchB1Prob(
                          TreeNode*	node,
                          int numStates,
                          DoubleMatrix3D& inRateMatrix3D, int theModel, int row)
{
	DoubleMatrix3D *Pmat;
	DoubleMatrix *ProbMatrix;
    
	Pmat = getProbMatrix3D(node); //&node->GetData().probmatrix3D;
    //	IntMatrix3D *mModelCount3D = getChangeMatrix3D( node ); //&node->GetData().countModelchange3D;
    
	// [HUH???] if (mModelCount3D->empty()) return SetBranchB1Prob( node, numStates, inRateMatrix3D, 1 );
    
	unsigned int numModels = inRateMatrix3D.rows();
    
    //	std::cout << "about to check if pmat has been dimmd..." << endl;
	if (Pmat->empty()) {
		int theNumModels = numModels;
		if (mRoot->numDiscreteBLs != 0) theNumModels *= mRoot->numDiscreteBLs;
		if (options.allowASRV) theNumModels *= options.numCatASRV;
		Pmat->resize( theNumModels, numStates, numStates, 0.0);
	}
	
	int modelInit, modelLimit;
	int blInit, blLimit;
	int rateCatInit, rateCatLimit;
	
	if (node != NULL)
	{
		if (theModel != -1) { modelInit = theModel; modelLimit = modelInit+1; }
		else { modelInit = 0; modelLimit = numModels; }
		
		if (mRoot->numDiscreteBLs != 0) { blInit = 0; blLimit = mRoot->numDiscreteBLs; }
		else { blInit = 0; blLimit = 1; }
		
		if (options.allowASRV) { rateCatInit = 0; rateCatLimit = options.numCatASRV; }
		else { rateCatInit = 0; rateCatLimit = 1; }
        
		for (unsigned int k = modelInit; k < modelLimit; k++) {
			for (unsigned int k2 = blInit; k2 < blLimit; k2++) {
				for (unsigned int k3 = rateCatInit; k3 < rateCatLimit; k3++) {
					ProbMatrix = &(*getProbMatrix3D(node))[ num->returnModelIndex( k, k3, k2 ) ];
					// [old] ProbMatrix = &(*getProbMatrix3D(node))[   (k2*numModels) + k ]; //&node->GetData().probmatrix3D[k];
                    
					if (ProbMatrix->empty())
						ProbMatrix->resize(numStates, numStates, 0);
					
					double distance = node->GetData().distance;
					if (mRoot->numDiscreteBLs != 0) { distance = mRoot->discreteBLs[k2]; }
					
					if (distance != cInitDistance) {
						distance *= mRoot->GetData().siteCats->getRate( k3 );
                        if ( row == -1 ) {
                            // B1 method - rate matrix to probability matrix;
                            for (unsigned int i = 0; i < numStates; i++)
                            {
                                for (unsigned int j = 0; j < numStates; j++)
                                {	
                                    //				if ((*mModelCount3D).mRows.mDataArray[k].mRows.mDataArray[i].mDataArray[j] == 0) { continue; } // don't need to do it if this transition isn't observed
                                    if (options.HwangGreen == 1) {
                                        (*ProbMatrix).mRows.mDataArray[i].mDataArray[j] = inRateMatrix3D[k][i][j] * distance; //num->calculateB1prob( inRateMatrix3D[k], distance, i, j );
                                    } else {
                                        (*ProbMatrix).mRows.mDataArray[i].mDataArray[j] = num->calculateB1prob( inRateMatrix3D[k], distance, i, j );
                                    }
                                }
                            }
                            
                            if (options.HwangGreen == 1) {
                                for (unsigned int i = 0; i < numStates; i++)
                                {
                                    double sum = 0.0;
                                    for (unsigned int j = 0; j < numStates; j++)
                                    {	
                                        if (j != i) { sum += (*ProbMatrix).mRows.mDataArray[i].mDataArray[j]; }
                                    }
                                    (*ProbMatrix).mRows.mDataArray[i].mDataArray[i] = (1.0 - sum);
                                }
                            }
                        } else {
                            //	std::cout << "Here!" << endl;
                            //	int j=1;
                            //	(*ProbMatrix).mRows.mDataArray[row].mDataArray[j] = num->calculateB1prob( inRateMatrix3D[k], distance, row, j );
                            
                            for (unsigned int j = 0; j < numStates; j++)
                            {	
                                //				if ((*mModelCount3D).mRows.mDataArray[k].mRows.mDataArray[row].mDataArray[j] == 0) { continue; } // don't need to do it if this transition isn't observed
                                // Overriding with Hwang-Green...
                                if (options.HwangGreen == 1) {
                                    (*ProbMatrix).mRows.mDataArray[row].mDataArray[j] = inRateMatrix3D[k][row][j] * distance; //num->calculateB1prob( inRateMatrix3D[k], distance, i, j );
                                } else (*ProbMatrix).mRows.mDataArray[row].mDataArray[j] = num->calculateB1prob( inRateMatrix3D[k], distance, row, j );
                            }
                            
                            for (unsigned int j = 0; j < numStates; j++)
                            {	
                                //				if ((*mModelCount3D).mRows.mDataArray[k].mRows.mDataArray[j].mDataArray[row] == 0) { continue; } // don't need to do it if this transition isn't observed
                                // Overriding with Hwang-Green...
                                if (options.HwangGreen == 1) {
                                    (*ProbMatrix).mRows.mDataArray[j].mDataArray[row] = inRateMatrix3D[k][j][row] * distance; //num->calculateB1prob( inRateMatrix3D[k], distance, i, j );
                                } else (*ProbMatrix).mRows.mDataArray[j].mDataArray[row] = num->calculateB1prob( inRateMatrix3D[k], distance, j, row );
                                
                            }
                            
                            
                        }
                        NormalizeMatrix(*ProbMatrix);
                        
                    }
					
				}
			}
		}
	}
}

// --- Uniformized B1 method here ---
//---------------------------------------------------------------------------

inline
void
TreeNode::SetBranchB1uProb(
                           TreeNode*	node,
                           int numStates,
                           DoubleMatrix3D& inRateMatrix3D,
                           DoubleVector *uniformizationConstants)
{
	SetBranchB1uProb( node, numStates, inRateMatrix3D, uniformizationConstants, -1, -1 );
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetBranchB1uProb(
                           TreeNode*	node,
                           int numStates,
                           DoubleMatrix3D& inRateMatrix3D, DoubleVector *uniformizationConstants, int theModel, int row)
{
	DoubleMatrix3D *Pmat;
	DoubleMatrix *ProbMatrix;
	int rateCatInit, rateCatLimit;
	
	// TEMP OVERRIDE
	// mRoot->setCalculateFullProbabilityDistribution(1);
    
	if (mRoot->getCalculateFullProbabilityDistribution() == 1) {
		Pmat = getProbMatrix3D(node); //&node->GetData().probmatrix3D;
	} else {
		Pmat = getProbMatrix3D(mRoot); //&node->GetData().probmatrix3D;
	}
	
	unsigned int numModels = inRateMatrix3D.rows();

	if (Pmat->empty()) {
		int theNumModels = numModels;
		if (mRoot->numDiscreteBLs != 0) theNumModels *= mRoot->numDiscreteBLs;
		if (options.allowASRV) theNumModels *= options.numCatASRV;
		Pmat->resize( theNumModels, numStates, numStates, 0.0);
	}
	
	int modelInit, modelLimit;
	int blInit, blLimit;
	
	if (node != NULL)
	{
		
		if (theModel != -1) { modelInit = theModel; modelLimit = modelInit+1; }
		else { modelInit = 0; modelLimit = numModels; }
		//std::cout << "nummodels is " << numModels << "; modelinit = " << modelInit << ", ModelLimit = " << modelLimit << endl;
		
		if (mRoot->numDiscreteBLs != 0) { blInit = 0; blLimit = mRoot->numDiscreteBLs; }
		else { blInit = 0; blLimit = 1; }
        
		if (options.allowASRV) { rateCatInit = 0; rateCatLimit = options.numCatASRV; }
		else { rateCatInit = 0; rateCatLimit = 1; }
		
		for (unsigned int k = modelInit; k < modelLimit; k++) {
			for (unsigned int k2 = blInit; k2 < blLimit; k2++) {
				for (unsigned int k3 = rateCatInit; k3 < rateCatLimit; k3++) {
					if (mRoot->getCalculateFullProbabilityDistribution()) {
						ProbMatrix = &(*getProbMatrix3D(node)).mRows.mDataArray[ num->returnModelIndex( k, k3, k2 ) ]; //&node->GetData().probmatrix3D[k];
					} else {
						ProbMatrix = &(*getProbMatrix3D(mRoot)).mRows.mDataArray[ num->returnModelIndex( k, k3, k2 )  ]; //&node->GetData().probmatrix3D[k];
					}
					if (ProbMatrix == NULL) {
						std::cout << "ERROR: we failed to get a probMatrix..." << endl;
						exit(-1);
					}
					
					if (ProbMatrix->empty())
						ProbMatrix->resize(numStates, numStates, 0);
					
					double distance = node->GetData().distance;
					if (mRoot->numDiscreteBLs != 0) { distance = mRoot->discreteBLs[k2]; }
					
					// [DISABLING - TEST]
					//	if ((distance != cInitDistance) )  // || (node->Up()->Up()->Up() == node->Up()))
					{
						/*	if (distance == cInitDistance)
						 {
						 TreeNode* tempNode;
						 
						 if (node == node->Up()->Left())
						 tempNode = node->Up()->Right();
						 else
						 tempNode = node->Up()->Left();
						 
						 distance = tempNode->GetData().distance;
						 } */
						//	if ( node->Up() == mRoot ) { std::cout << "one from mRoot.  our bl is " << distance << "; left is " << node->Up()->Left()->GetData().distance << "; right is " << node->Up()->Right()->GetData().distance << endl; }
						if ( !( (mRoot->getCalculateFullProbabilityDistribution()) && (distance == cInitDistance) ) )
						{
							//	row = -1;
							if ( row == -1 ) {
								// B1 method - rate matrix to probability matrix;
								for (unsigned int i = 0; i < numStates; i++)
								{
									for (unsigned int j = 0; j < numStates; j++)
									{	
										// Overriding with Hwang-Green...
										// (*ProbMatrix).mRows.mDataArray[i].mDataArray[j] = inRateMatrix3D[k][i][j] * distance; //num->calculateB1prob( inRateMatrix3D[k], distance, i, j );

										if (mRoot->getCalculateFullProbabilityDistribution() == 1) {
											(*ProbMatrix).mRows.mDataArray[i].mDataArray[j] = num->calculateB1uprob( inRateMatrix3D.mRows.mDataArray[k], distance, i, j, (*uniformizationConstants).mDataArray[k], mRoot->GetData().siteCats->getRate( k3 ) );
										} else {
												(*ProbMatrix).mRows.mDataArray[i].mDataArray[j] = inRateMatrix3D.mRows.mDataArray[ k ].mRows.mDataArray[i].mDataArray[j];
										}
									}
								}
                                
							} else {
								for (unsigned int j = 0; j < numStates; j++)
								{	
 									if (mRoot->getCalculateFullProbabilityDistribution()) {
										(*ProbMatrix).mRows.mDataArray[row].mDataArray[j] = num->calculateB1uprob( inRateMatrix3D.mRows.mDataArray[k], distance, row, j, (*uniformizationConstants).mDataArray[k], mRoot->GetData().siteCats->getRate( k3 ) );
									} else {
										(*ProbMatrix).mRows.mDataArray[row].mDataArray[j] = inRateMatrix3D.mRows.mDataArray[ k ].mRows.mDataArray[row].mDataArray[j];
									}
								}
								
								for (unsigned int j = 0; j < numStates; j++)
								{	
									if (mRoot->getCalculateFullProbabilityDistribution()) {
										(*ProbMatrix).mRows.mDataArray[j].mDataArray[row] = num->calculateB1uprob( inRateMatrix3D[k], distance, j, row, (*uniformizationConstants)[k], mRoot->GetData().siteCats->getRate( k3 ) );
									} else {
										(*ProbMatrix).mRows.mDataArray[j].mDataArray[row] = inRateMatrix3D.mRows.mDataArray[ k ].mRows.mDataArray[j].mDataArray[row];
									}
								}
							}
						}
					}
				}
			} 
		}
	}
}


// ---

//---------------------------------------------------------------------------

inline
void
TreeNode::SetBranchBinfProb(
                            TreeNode*	node,
                            int numStates,
                            DoubleMatrix3D& inRateMatrix3D, DoubleMatrix *stateFreqs)
{
	// "Binf" -> B-infinity calculations...
	SetBranchBinfProb( node, numStates, inRateMatrix3D, stateFreqs, -1 );
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetBranchBinfProb(
                            TreeNode*	node,
                            int numStates,
                            DoubleMatrix3D& inRateMatrix3D, DoubleMatrix *stateFreqs, int theModel)
{
	int modelInit, modelLimit;
	int blInit, blLimit;
	int rateCatInit, rateCatLimit;
	
	static double *Qmat;
	static double *stateF;
	static int allocated = 0;
    
 	DoubleMatrix3D *Pmat;
	DoubleMatrix *ProbMatrix;
    
	Pmat = getProbMatrix3D(node); //&node->GetData().probmatrix3D;
     
	unsigned int numModels = inRateMatrix3D.rows();
    
 	if (Pmat->empty()) {
		int theNumModels = numModels;
		if (mRoot->numDiscreteBLs != 0) theNumModels *= mRoot->numDiscreteBLs;
		if (options.allowASRV) theNumModels *= options.numCatASRV;
		Pmat->resize( theNumModels, numStates, numStates, 0.0);
	}
	
	if (node != NULL)
	{
		if (theModel != -1) { modelInit = theModel; modelLimit = modelInit+1; }
		else { modelInit = 0; modelLimit = numModels; }
		
        //.mRows.mDataArray[k].mRows.mDataArray[i].mDataArray[j]
        
		if (allocated == 0) {
			//std::cout << "Allocating a qmat..." << endl;
			Qmat = new double[numStates*numStates];
			stateF = new double[numStates];
			allocated = 1;
		}
		
		// Allocate space for UVRoot matrices at the root node...
		if (mRoot->GetData().UVRallocated == 0) {
			mRoot->GetData().U = new double[numStates*numStates*modelLimit];
			mRoot->GetData().V = new double[numStates*numStates*modelLimit];
			mRoot->GetData().Root = new double[numStates*modelLimit];
			mRoot->GetData().UVRallocated = 1;
		}
        
		/* if (distance == cInitDistance)
         {
         TreeNode* tempNode;
         
         if (node == node->Up()->Left())
         tempNode = node->Up()->Right();
         else
         tempNode = node->Up()->Left();
         
         distance = tempNode->GetData().distance;
         } */
        
		if (node == mRoot) {
			// Do the spectral decomposition...
			for (unsigned int k = modelInit; k < modelLimit; k++) {
                for (int i=0;i<numStates;i++) { for (int j=0;j<numStates;j++) { Qmat[(i*numStates)+j] = inRateMatrix3D.mRows.mDataArray[k].mRows.mDataArray[i].mDataArray[j]; } }
                for (int i=0;i<numStates;i++) { stateF[i] = (double) stateFreqs->mRows.mDataArray[k].mDataArray[i]; } //->mRows.mDataArray[k].mDataArray[i]; }
                
				//	std::cout << "Calling PAML eigen routine..." << endl;
                num->PAML_eigenQREV(Qmat, stateF, numStates, &mRoot->GetData().Root[k*numStates], &mRoot->GetData().U[k*numStates*numStates], &mRoot->GetData().V[k*numStates*numStates]);
				//	std::cout << "Returned from PAML eigen routine..." << endl;
                
			}
		}
		
		if (mRoot->numDiscreteBLs != 0) { blInit = 0; blLimit = mRoot->numDiscreteBLs; }
		else { blInit = 0; blLimit = 1; }
		
		if (options.allowASRV) { rateCatInit = 0; rateCatLimit = options.numCatASRV; }
		else { rateCatInit = 0; rateCatLimit = 1; }
		
		double distance;
		for (unsigned int k = modelInit; k < modelLimit; k++) {
			for (unsigned int k2 = blInit; k2 < blLimit; k2++) {
				for (unsigned int k3 = rateCatInit; k3 < rateCatLimit; k3++) {
					ProbMatrix = &(*getProbMatrix3D(node))[ num->returnModelIndex( k, k3, k2 )  ];
					// [old] ProbMatrix = &(*getProbMatrix3D(node))[   (k2*numModels) + k ]; //&node->GetData().probmatrix3D[k];
					
					if (ProbMatrix->empty()) { ProbMatrix->resize(numStates, numStates, 0); }
					//	else
					//[not needed]		ProbMatrix->fill(0);
					
					distance = node->GetData().distance;
					if (distance != cInitDistance) {
						distance *= mRoot->GetData().siteCats->getRate( k3 );
						num->PAML_PMatUVRoot(ProbMatrix, distance, numStates, &mRoot->GetData().U[k*numStates*numStates], &mRoot->GetData().V[k*numStates*numStates],&mRoot->GetData().Root[k*numStates]);
					}
					
                    //		std::cout << "Just build Pmat for distance " <<  node->GetData().distance << " (x rate is): " << distance << endl;
					//int i,j;
					//for (i=0;i<numStates;i++) {
					//		for (j=0; j<numStates; j++) {
					//			std::cout << i << "," << j << ": " << ProbMatrix->mRows.mDataArray[i].mDataArray[j] << endl;
					//		}
					//	}
					
				}
			}
		}
	}
	
	//	free(Qmat);
    //	free(stateF);
    //	
}

//---------------------------------------------------------------------------

inline 
void
TreeNode::SetTreeB1xm_bl(double bl)
{
	std::cout << "OBSOLETE FUNCTION SetTreeB1xm_bl BEING CALLED!" << endl;
	/*
     // This function sets the branchlength segment size for B1xM to half the smallest bl on the tree.
     // The value is stored at mRoot->getData().distance
     
     if (Left() != NULL)
     {
     //		SetBranchB1Prob(Left(), numStates, inRateMatrix3D, theModel, x);
     if ( ( Left()->GetData().distance < bl ) && ( Left()->GetData().distance != cInitDistance ) ) { bl = Left()->GetData().distance; }
     
     Left()->SetTreeB1xm_bl(bl); 
     }
     
     if (Right() != NULL) 
     { 
     //		SetBranchB1Prob(Right(), numStates, inRateMatrix3D, theModel, x);
     if ( ( Right()->GetData().distance < bl) && ( Right()->GetData().distance != cInitDistance ) ) { bl = Right()->GetData().distance; }
     
     Right()->SetTreeB1xm_bl(bl);  
     }
     
     if (this == mRoot) { bl = 0.001; mRoot->GetData().distance = bl; std::cout << "the bl is set at root: " << bl << endl; }
     */
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetBranchB1Prob(
                          TreeNode*	node,
                          int numStates,
                          DoubleMatrix3D& inRateMatrix3D, int dummyGibbs)
{
	SetBranchB1Prob( node, numStates, inRateMatrix3D, dummyGibbs, -1, -1 );
}



inline
void
TreeNode::SetBranchB1Prob(
                          TreeNode*	node,
                          int numStates,
                          DoubleMatrix3D& inRateMatrix3D, int dummyGibbs, int theModel, int row)
{
	// NO CHECKING FOR SUBSTITUTIONS HERE...
	// (this version calculates the whole matrix, regardless of sub counts - needed for ancestral state sampling)
	
	DoubleMatrix3D *Pmat;
	DoubleMatrix *ProbMatrix;
    
	Pmat = getProbMatrix3D(node); //&node->GetData().probmatrix3D;
    
	unsigned int numModels = inRateMatrix3D.rows();
	int modelInit, modelLimit;
	int blInit, blLimit;
	int rateCatInit, rateCatLimit;
	
	if (Pmat->empty()) {
		int theNumModels = numModels;
		if (mRoot->numDiscreteBLs != 0) theNumModels *= mRoot->numDiscreteBLs;
		if (options.allowASRV) theNumModels *= options.numCatASRV;
		Pmat->resize( theNumModels, numStates, numStates, 0.0);
	}
	
	if (node != NULL)
	{
		if (theModel != -1) { modelInit = theModel; modelLimit = modelInit+1; }
		else { modelInit = 0; modelLimit = numModels; }
		
		if (mRoot->numDiscreteBLs != 0) { blInit = 0; blLimit = mRoot->numDiscreteBLs; }
		else { blInit = 0; blLimit = 1; }
		
		if (options.allowASRV) { rateCatInit = 0; rateCatLimit = options.numCatASRV; }
		else { rateCatInit = 0; rateCatLimit = 1; }
		
		for (unsigned int k = modelInit; k < modelLimit; k++) {
			for (unsigned int k2 = blInit; k2 < blLimit; k2++) {
				for (unsigned int k3 = rateCatInit; k3 < rateCatLimit; k3++) {
					ProbMatrix = &(*getProbMatrix3D(node))[ k * ( options.numCatASRV * mRoot->numDiscreteBLs ) + (k2 * options.numCatASRV) + k3 ];
					
					if (ProbMatrix->empty())
						ProbMatrix->resize(numStates, numStates, 0.0);
					else
						ProbMatrix->fill(0.0);
					
					double distance = node->GetData().distance;
					if (mRoot->numDiscreteBLs != 0) { distance = mRoot->discreteBLs[k2]; }
					
					if ((distance != cInitDistance) )  //|| (node->Up()->Up()->Up() == node->Up()))
					{
						/*		if (distance == cInitDistance)
						 {
						 TreeNode* tempNode;
						 
						 if (node == node->Up()->Left())
						 tempNode = node->Up()->Right();
						 else
						 tempNode = node->Up()->Left();
						 
						 distance = tempNode->GetData().distance;
						 }  */
						if (distance != cInitDistance)
						{
							distance *= mRoot->GetData().siteCats->getRate( k3 );
							
							// B1 method - rate matrix to probability matrix;
							if ( row == -1 ) {
								// B1 method - rate matrix to probability matrix;
								for (unsigned int i = 0; i < numStates; i++)
								{
									for (unsigned int j = 0; j < numStates; j++)
									{	
										// Overriding with Hwang-Green...
										if (options.HwangGreen == 1) {
											(*ProbMatrix).mRows.mDataArray[i].mDataArray[j] = inRateMatrix3D[k][i][j] * distance; //num->calculateB1prob( inRateMatrix3D[k], distance, i, j );
										} else (*ProbMatrix).mRows.mDataArray[i].mDataArray[j] = num->calculateB1prob( inRateMatrix3D.mRows.mDataArray[k], distance, i, j );
									}
								}
								
								
								// For Hwang-Green only:
								if (options.HwangGreen == 1) {
									for (unsigned int i = 0; i < numStates; i++)
									{
										double sum = 0.0;
										for (unsigned int j = 0; j < numStates; j++)
										{	
											if (j != i) { sum += (*ProbMatrix).mRows.mDataArray[i].mDataArray[j]; }
										}
										(*ProbMatrix).mRows.mDataArray[i].mDataArray[i] = (1.0 - sum);
									} 
								}
								
							} else {
								for (unsigned int j = 0; j < numStates; j++)
								{	
									// Overriding with Hwang-Green...
									if (options.HwangGreen == 1) {
										(*ProbMatrix).mRows.mDataArray[row].mDataArray[j] = inRateMatrix3D[k][row][j] * distance; //num->calculateB1prob( inRateMatrix3D[k], distance, i, j );
									} else (*ProbMatrix).mRows.mDataArray[row].mDataArray[j] = num->calculateB1prob( inRateMatrix3D.mRows.mDataArray[k], distance, row, j );
								}
								
								for (unsigned int j = 0; j < numStates; j++)
								{	
									// Overriding with Hwang-Green...
									if (options.HwangGreen == 1) {
										(*ProbMatrix).mRows.mDataArray[j].mDataArray[row] = inRateMatrix3D[k][j][row] * distance; //num->calculateB1prob( inRateMatrix3D[k], distance, i, j );
									} else (*ProbMatrix).mRows.mDataArray[j].mDataArray[row] = num->calculateB1prob( inRateMatrix3D.mRows.mDataArray[k], distance, j, row );
								}
								
							}
							NormalizeMatrix(*ProbMatrix);
							
						}
						
					}
				}
			}
		}
	}
}

// --- Uniformized B1 here -----
//---------------------------------------------------------------------------

inline
void
TreeNode::SetBranchB1uProb(
                           TreeNode*	node,
                           int numStates,
                           DoubleMatrix3D& inRateMatrix3D, DoubleVector *uniformizationConstants, int dummyGibbs)
{
	SetBranchB1uProb( node, numStates, inRateMatrix3D, uniformizationConstants, dummyGibbs, -1, -1 );
}



inline double
TreeNode::_calculateMarginalDwell( double siteRate, int x, int y, int whichState, int theModel, DoubleMatrix3D& inRateMatrix3D, double bl ) {
	double returnVal = 0.0;
	double LambdaX = (-inRateMatrix3D.mRows.mDataArray[theModel].mRows.mDataArray[x].mDataArray[x]) * siteRate;
	double LambdaY = (-inRateMatrix3D.mRows.mDataArray[theModel].mRows.mDataArray[y].mDataArray[y]) * siteRate;
	
	double oldLx, oldLy;
	if ( abs( (LambdaX - LambdaY) ) < 0.0001 ) {
		//	std:: cout << "	Before transform, the delta was " << abs(LambdaX - LambdaY) << endl;
		oldLx = LambdaX; oldLy = LambdaY;
		LambdaX = ( oldLx + oldLy ) / 2.0;
		LambdaY = LambdaX;
	}
	
	// get rate matrix...
	if ( (whichState != x) && (whichState != y) ) return 0.0;
	
	if ( LambdaX == LambdaY ) {
		returnVal = 0.5 * bl;
	} else { 
		returnVal = ( 1.0 / ( LambdaX - LambdaY ) ) + ( exp( (LambdaY * bl) ) * bl ) / (  exp( LambdaY * bl ) -  exp( LambdaX * bl ) );
	}
	
	if (returnVal > bl) { 
		std::cout << "An error was encountered in _calculateMarginalDwell().  The dwell time cannot exceed branch length!  Bl is " << bl << "; calc'd dwell time is " << returnVal << endl;
 		/*
         std::cout << "x, y, state = " << x << ", " << y << ", " << whichState << "; LambdaX = " << LambdaX << "; LambdaY = " << (LambdaY) << ", lambdaDiff = " << ((LambdaX) - (LambdaY)) << endl;
         
         std::cout << "bl is " << bl << endl;
         std::cout << "denom1 is " << ( 1.0 / ( LambdaX + inRateMatrix3D.mRows.mDataArray[theModel].mRows.mDataArray[y].mDataArray[y] ) ) << endl;
         
         std::cout << "numerator is " << ( exp( (LambdaY * bl) ) * bl ) << endl;
         std::cout << "denom2 is " << ((  exp( LambdaY * bl ) -  exp( LambdaX * bl ))) << endl;
         
         std::cout << "pt1 " << ( 1.0 / ( LambdaX + inRateMatrix3D.mRows.mDataArray[theModel].mRows.mDataArray[y].mDataArray[y] ) ) << endl;
         std::cout << "pt2 " << ( exp( (LambdaY * bl) ) * bl ) / (  exp( -inRateMatrix3D.mRows.mDataArray[theModel].mRows.mDataArray[y].mDataArray[y] * bl ) -  exp( LambdaX * bl ) ) << endl;
		 */
		exit(-1);
	}	
    
	return returnVal;
}

// ---

inline double
TreeNode::_calculateJointDwellTime( double siteRate1, double siteRate2, int x1, int x2, int y1, int y2, int state1, int state2, int theModel1, int theModel2, MixtureModel& mModel, double bl ) {
	double jointDwell = 0.0;
	double LambdaX, LambdaY;
	double LambdaX1, LambdaX2;
	double LambdaY1, LambdaY2;
	double finalTerm;
	DoubleMatrix3D& rateMatrix = *mModel.returnRateMatrix3D();
	
	LambdaX1 = -rateMatrix.mRows.mDataArray[theModel1].mRows.mDataArray[x1].mDataArray[x1]  * siteRate1;
	LambdaY1 = -rateMatrix.mRows.mDataArray[theModel1].mRows.mDataArray[y1].mDataArray[y1]  * siteRate1;
	LambdaX2 = -rateMatrix.mRows.mDataArray[theModel2].mRows.mDataArray[x2].mDataArray[x2]  * siteRate2;
	LambdaY2 = -rateMatrix.mRows.mDataArray[theModel2].mRows.mDataArray[y2].mDataArray[y2]  * siteRate2;
    
    // DIAGNOSTIC OUTPUT, CHECKING ON MODEL MIXING AND RATES...
    cout << "   Site 1 has model " << theModel1 << "; site 1 has rate " << siteRate1 << endl;
    cout << "   Site 2 has model " << theModel2 << "; site 2 has rate " << siteRate2 << endl;
    exit(-1);
    
	// Need to bypass the problem when the lambdas get small...
	double oldLx, oldLy;
	if ( abs( LambdaX1 - LambdaY1 ) < 0.0001 ) {
		oldLx = LambdaX1; oldLy = LambdaY1;
		//LambdaX1 = oldLx/(oldLx+oldLy) * 0.001;
		//LambdaY1 = oldLy/(oldLx+oldLy) * 0.001;
		LambdaX1 = ( oldLx + oldLy ) / 2.0;
		LambdaY1 = LambdaX1;
 	}
	
	if ( abs( LambdaX2 - LambdaY2 ) < 0.0001 ) {
		oldLx = LambdaX2; oldLy = LambdaY2;
		//LambdaX1 = oldLx/(oldLx+oldLy) * 0.001;
		//LambdaY1 = oldLy/(oldLx+oldLy) * 0.001;
		LambdaX2 = ( oldLx + oldLy ) / 2.0;
		LambdaY2 = LambdaX2;
	}
	
	
 	if ( ( (state1 == x1) && (x1 == y1 ) ) && ( ( state2 == x2) && (x2 == y2 ) ) ) {
		jointDwell += bl;
        //		std::cout << "SPECIAL CASE 1:  x1==y1==state1, x2==y2==state2; adding " << bl << endl;
	} else if (( (state1 == x1) && (x1 == y1 ) ) && !( ( state2 == x2) && (x2 == y2 ) ) ) {
		jointDwell += _calculateMarginalDwell( siteRate2, x2, y2, state2, theModel2, rateMatrix, bl );
		if (jointDwell > 10000) { std::cout << "ERROR CASE marg 1" << endl; }
		if ( jointDwell > bl ) {
			std::cout << jointDwell << ", " << bl << "; the marginal dwell time is too big!" << endl;
			std::cout << "bp here" << endl;
			std::cout << "bp here" << endl;
		}
        //	std::cout << "SPECIAL CASE 1a:  adding " << jointDwell << endl;
        
	} else if (!( (state1 == x1) && (x1 == y1 ) ) && ( ( state2 == x2) && (x2 == y2 ) ) ) {
		jointDwell += _calculateMarginalDwell( siteRate1, x1, y1, state1 , theModel1, rateMatrix, bl );
		if (jointDwell > 10000) { std::cout << "ERROR CASE marg 2" << endl; }
        //	std::cout << "SPECIAL CASE 1c:  adding " << jointDwell << endl;
        
	} 
	
	else if ( (state1 == x1) && ( state2 == x2 ) ) {
		// CASE 1
        //	std::cout << "	CASE 1" << endl;
        //	return;
		if ( (LambdaX1 == LambdaY1) && (LambdaX2 == LambdaY2) ) {
            //		if ( (rateMatrix.mRows.mDataArray[theModel1].mRows.mDataArray[x1].mDataArray[x1] == rateMatrix.mRows.mDataArray[theModel1].mRows.mDataArray[y1].mDataArray[y1]) && (rateMatrix.mRows.mDataArray[theModel2].mRows.mDataArray[x2].mDataArray[x2] == rateMatrix.mRows.mDataArray[theModel2].mRows.mDataArray[y2].mDataArray[y2] ) ) {
			jointDwell += bl / 3.0;
		} else if ( LambdaX1 == LambdaY1) { 
			//(rateMatrix.mRows.mDataArray[theModel1].mRows.mDataArray[x1].mDataArray[x1] == rateMatrix.mRows.mDataArray[theModel1].mRows.mDataArray[y1].mDataArray[y1] ) ) {
			LambdaX = LambdaX2;
			LambdaY = LambdaY2;
			
			// jointDwell += 1.0 / ( ( exp(LambdaX * bl) - exp(LambdaY * bl) ) * bl ) * ( ( exp(LambdaX*bl) * (LambdaX*bl - LambdaY*bl - 1) + exp(LambdaY*bl)) / ( (LambdaX - LambdaY) * (LambdaX - LambdaY ) ) - exp(LambdaY*bl)*bl*bl/2.0);
			
			// Fixed
			jointDwell += ( exp(LambdaY * bl) * ( 2-(bl*bl)*(LambdaX-LambdaY)*(LambdaX-LambdaY) ) + 2*exp(LambdaX*bl)*(-1+bl*LambdaX-bl*LambdaY) ) / ( 2*(exp(LambdaX*bl) - LambdaY*bl) * bl * (LambdaX-LambdaY)*(LambdaX-LambdaY) );
			if (jointDwell > 10000) { std::cout << "ERROR CASE 1b" << endl; }
            
		} else if ( LambdaX2 == LambdaY2) { 
			//(rateMatrix.mRows.mDataArray[theModel2].mRows.mDataArray[x2].mDataArray[x2] == rateMatrix.mRows.mDataArray[theModel2].mRows.mDataArray[y2].mDataArray[y2] ) ) {
			LambdaX = LambdaX1;
			LambdaY = LambdaY1;
			
			// jointDwell += 1.0 / ( ( exp(LambdaX * bl) - exp(LambdaY * bl) ) * bl ) * ( ( exp(LambdaX*bl) * (LambdaX*bl - LambdaY*bl - 1) + exp(LambdaY*bl) ) / ( (LambdaX - LambdaY) * (LambdaX - LambdaY ) ) - exp(LambdaY*bl)*bl*bl/2.0);
            
			// Fixed
			jointDwell += ( exp(LambdaY * bl) * ( 2-(bl*bl)*(LambdaX-LambdaY)*(LambdaX-LambdaY) ) + 2*exp(LambdaX*bl)*(-1+bl*LambdaX-bl*LambdaY) ) / ( 2*(exp(LambdaX*bl) - LambdaY*bl) * bl * (LambdaX-LambdaY)*(LambdaX-LambdaY) );
			if (jointDwell > 10000) { std::cout << "ERROR CASE 1c" << endl; }
			
		} else if ( (LambdaX2 == LambdaY1) && (LambdaY2 == LambdaX1) ) { 
            
			jointDwell += (-2 * exp( bl * LambdaX1+LambdaY1) * bl  +  ( exp(2*bl*LambdaX1) - exp(2 * bl * LambdaY1) ) / (LambdaX1 - LambdaY1) ) / ( (exp(LambdaX1*bl) - exp(LambdaY1*bl) ) * (exp(LambdaX1*bl) - exp(LambdaY1*bl) ) );
            
		} else {
			//finalTerm =( exp((LambdaY1+LambdaY2)*bl) - exp((LambdaX1+LambdaX2)*bl) ) / (LambdaX1+LambdaX2-LambdaY1-LambdaY2);
			//jointDwell += 1.0 / ( ( exp(LambdaX1 * bl) - exp(LambdaY1 * bl) ) * ( exp(LambdaX2 * bl) - exp(LambdaY2 * bl) ) ) *
			( ( exp((LambdaY1+LambdaY2)*bl)*bl + ( exp((LambdaY1+LambdaY2)*bl) - exp((LambdaX1+LambdaY2)*bl) ) / (LambdaX1 - LambdaY1) +
			   ( exp((LambdaY1+LambdaY2)*bl) - exp((LambdaX2+LambdaY1)*bl) ) / (LambdaX2 - LambdaY2) +
			   finalTerm ) );
			
			double numer = -exp(bl*(LambdaX1+LambdaY2))/(LambdaX1-LambdaY1) - exp(bl*(LambdaX2+LambdaY1))/(LambdaX2-LambdaY2) + ( exp(bl * (LambdaX1+LambdaX2)) / (LambdaX1+LambdaX2-LambdaY1-LambdaY2) ) + exp(bl*(LambdaY1+LambdaY2)) * ( bl + 1/(LambdaX1-LambdaY1) + 1/(LambdaX2-LambdaY2) + 1/(LambdaY1+LambdaY2-LambdaX1-LambdaX2) );
			jointDwell += numer / ( (exp(bl*LambdaX1)-exp(bl*LambdaY1)) * (exp(bl*LambdaX2) - exp(LambdaY2*bl) ) );
			
			if (jointDwell > 10000) { std::cout << "ERROR CASE 1c" << endl; }
		}
		
	} else if ( (state1 == y1) && ( state2 == x2 ) ) {
		// CASE 2
        //		std::cout << "	CASE 2" << endl;
		//return;
		if ( (LambdaX1 == LambdaY1) && (LambdaX2 == LambdaY2) ) {
			jointDwell += bl / 6.0;
		} else if ( LambdaX1 == LambdaY1 ) {
            //			std::cout << "case 2b" << endl;
			LambdaX = LambdaX2;
			LambdaY = LambdaY2;
			
			// jointDwell += ( 2.0 * exp(LambdaX * bl) - exp(LambdaY*bl) * (2.0 + bl * ( LambdaX-LambdaY )*(2.0 + bl*LambdaX - bl*LambdaY) ) )
			// / ( 2.0 * bl * ( exp(LambdaX*bl) - exp(LambdaY*bl) ) * (LambdaX-LambdaY)*(LambdaX-LambdaY) );
			
			// Fixed
			jointDwell += ( (-exp(LambdaY*bl)*bl*bl) + ( (2* (exp(LambdaX*bl) + exp(LambdaY*bl)*(-1-bl*LambdaX + bl*LambdaY)) ) / ((LambdaX-LambdaY)*(LambdaX-LambdaY)) ) ) / ( 2 * (exp(LambdaX*bl) - exp(LambdaY*bl)) * bl );
			if (jointDwell > 10000) { std::cout << "ERROR CASE 2b" << endl; }
			if (jointDwell > bl) {
				std::cout << "ERROR CASE 2b" << endl;
			}
            
		} else if ( ( LambdaX2 == LambdaY2) ) {
			//			std::cout << "case 2c" << endl;
			LambdaX = LambdaX1;
			LambdaY = LambdaY1;
			
            //	jointDwell += ( 2.0 * exp(LambdaX * bl) - exp(LambdaY*bl) * (2.0 + bl * ( LambdaX-LambdaY )*(2.0 + bl*LambdaX - bl*LambdaY) ) )
            //	/ ( 2.0 * bl * ( exp(LambdaX*bl) - exp(LambdaY*bl) ) * (LambdaX-LambdaY)*(LambdaX-LambdaY) );
            
			// Fixed
			jointDwell += ( (2/bl) + ( exp(LambdaX*bl)*(LambdaX-LambdaY)*(-2+bl*LambdaX-bl*LambdaY) ) / ( exp(LambdaX*bl) - exp(LambdaY*bl) ) ) / (2*(LambdaX-LambdaY)*(LambdaX-LambdaY));
			if (jointDwell > 10000) { std::cout << "ERROR CASE 2c" << endl; }
            
		} else if ( (LambdaX2 == LambdaY1) && (LambdaY2 == LambdaX1) ) { 
			
			jointDwell += (exp( 2 * bl * LambdaX1) * (-2 + bl*LambdaX1 - bl*LambdaY1) + exp(bl*(LambdaX1+LambdaY1)) * (2 + bl*LambdaX1 - bl*LambdaY1) ) / ( ( (exp(LambdaX1*bl) - exp(LambdaY1*bl) )*(exp(LambdaX1*bl) - exp(LambdaY1*bl) )) * (LambdaX1-LambdaY1) );
            
		} else {
			finalTerm =( exp((LambdaY1+LambdaY2)*bl) - exp((LambdaX1+LambdaX2)*bl) ) / (LambdaX1+LambdaX2-LambdaY1-LambdaY2);
			
			// Fixed
			jointDwell += ( 1.0 / ( ( exp(LambdaX1 * bl) - exp(LambdaY1 * bl) ) *( exp(LambdaX2 * bl) - exp(LambdaY2 * bl) ) ) ) *
			(-exp((LambdaX1+LambdaY2)*bl)*bl + ( exp((LambdaX1+LambdaY2)*bl) - exp((LambdaY1+LambdaY2)*bl) ) / (LambdaX1 - LambdaY1) +
			 ( exp((LambdaX1+LambdaX2)*bl) - exp((LambdaX1+LambdaY2)*bl) ) / (LambdaX2 - LambdaY2) +
			 finalTerm );
			
			if (jointDwell > 10000) { 
				std::cout << "ERROR CASE 2d; finalTerm = " << finalTerm << endl; 
				double term = (  ( ( exp(LambdaX1 * bl) - exp(LambdaY1 * bl) ) *( exp(LambdaX2 * bl) - exp(LambdaY2 * bl) ) ) );
				std::cout << "	term1 " << term << endl;
				
				term = 
				(-exp((LambdaX1+LambdaY2)*bl)*bl + ( exp((LambdaX1+LambdaY2)*bl) - exp((LambdaY1+LambdaY2)*bl) ) / (LambdaX1 - LambdaY1) +
				 ( exp((LambdaX1+LambdaX2)*bl) - exp((LambdaX1+LambdaY2)*bl) ) / (LambdaX2 - LambdaY2) +
				 finalTerm );
				
				std::cout << "	remaining term " << term << endl;
				
				term = exp(LambdaX1 * bl) - exp(LambdaY1 * bl);
				std::cout << "Term1 p1 " << term << endl;
				
				term = ( exp(LambdaX2 * bl) - exp(LambdaY2 * bl) );
				std::cout << "Term1 p2 " << term << endl;
				
				term = (exp(LambdaX1 * bl) - exp(LambdaY1 * bl)) * ( ( exp(LambdaX2 * bl) - exp(LambdaY2 * bl) ));
				std::cout << "Term1 prod " << term << endl;
				
				term = (-exp((LambdaX1+LambdaY2)*bl)*bl );
				std::cout << "	term2 " << term << endl;
				
				term = ( exp((LambdaX1+LambdaY2)*bl));
				std::cout << "	term3 " << term << endl;
				
				term = - exp((LambdaY1+LambdaY2)*bl);
				std::cout << "	term4 " << term << endl;
				
				term = (LambdaX1 - LambdaY1);
				std::cout << "	term5 " << term << endl;
				
				term = ( exp((LambdaX1+LambdaX2)*bl) - exp((LambdaX1+LambdaY2)*bl) ) / (LambdaX2 - LambdaY2);
				std::cout << "	term6 " << term << endl;
				
				term = finalTerm;
				
				
                
			}
            
		}
		
	} else if ( (state1 == x1) && ( state2 == y2 ) ) {
		// CASE 3
 		if ( (LambdaX1 == LambdaY1) && (LambdaX2 == LambdaY2) ) {
			jointDwell += bl / 6.0;
		} else if ( (LambdaX1 == LambdaY1) ) {
			// LambdaX1 == LambdaY1
			LambdaX = LambdaX2;
			LambdaY = LambdaY2;
			
            //	jointDwell += ( 2.0 * exp(LambdaX * bl) - exp(LambdaY*bl) * (2.0 + bl * ( LambdaX-LambdaY )*(2.0 + bl*LambdaX - bl*LambdaY) ) )
            //	/ ( 2.0 * bl * ( exp(LambdaX*bl) - exp(LambdaY*bl) ) * (LambdaX-LambdaY)*(LambdaX-LambdaY) );
            
			// Fixed
			jointDwell += ( (2/bl) + ( exp(LambdaX*bl)*(LambdaX-LambdaY)*(-2+bl*LambdaX-bl*LambdaY) ) / ( exp(LambdaX*bl) - exp(LambdaY*bl) ) ) / (2*(LambdaX-LambdaY)*(LambdaX-LambdaY));
			if (jointDwell > 10000) { std::cout << "ERROR CASE 3b" << endl; }
			if (jointDwell > bl) {
				std::cout << "ERROR CASE 3b" << endl;
			}
			
		} else if ( (LambdaX2 == LambdaY2) ) {
			LambdaX = LambdaX1;
			LambdaY = LambdaY1;
			
            //			jointDwell += ( 2.0 * exp(LambdaX * bl) - exp(LambdaY*bl) * (2.0 + bl * ( LambdaX-LambdaY )*(2.0 + bl*LambdaX - bl*LambdaY) ) )
            //			/ ( 2.0 * bl * ( exp(LambdaX*bl) - exp(LambdaY*bl) ) * (LambdaX-LambdaY)*(LambdaX-LambdaY) );
            
			// Fixed
			jointDwell += ( (-exp(LambdaY*bl)*bl*bl) + ( (2* (exp(LambdaX*bl)+exp(LambdaY*bl)*(-1-bl*LambdaX+bl*LambdaY)) ) / ((LambdaX-LambdaY)*(LambdaX-LambdaY)) ) ) / ( 2 * (exp(LambdaX*bl) - exp(LambdaY*bl)) * bl );
			if (jointDwell > 10000) { std::cout << "ERROR CASE 3c" << endl; }
            
		} else if ( (LambdaX2 == LambdaY1) && (LambdaY2 == LambdaX1) ) { 
			
			jointDwell += (exp( 2 * bl * LambdaY1) * (-2 - bl*LambdaX1 + bl*LambdaY1) + exp(bl*(LambdaX1+LambdaY1)) * (2 - bl*LambdaX1 + bl*LambdaY1) ) / ( ( (exp(LambdaX1*bl) - exp(LambdaY1*bl) )*(exp(LambdaX1*bl) - exp(LambdaY1*bl) )) * (-LambdaX1+LambdaY1) );
			
			
		} else {
            //			jointDwell += 1.0 / ( ( exp(LambdaX1 * bl) - exp(LambdaY1 * bl) ) *( exp(LambdaY2 * bl) - exp(LambdaX2 * bl) ) ) *
            //			(-exp((LambdaY1+LambdaY2)*bl)*bl - ( exp((LambdaY1+LambdaY2)*bl) - exp((LambdaX1+LambdaY2)*bl) ) / (LambdaX1 - LambdaY1) -( exp((LambdaY1+LambdaY2)*bl) - exp((LambdaX2+LambdaY1)*bl) ) / (LambdaX2 - LambdaY2) +
            //			   ( exp((LambdaX2+LambdaY1)*bl) - exp((LambdaX1+LambdaY2)*bl) ) / (LambdaX1-LambdaX2-LambdaY1+LambdaY2) );
            
			finalTerm =( exp((LambdaY1+LambdaY2)*bl) - exp((LambdaX1+LambdaX2)*bl) ) / (LambdaX1+LambdaX2-LambdaY1-LambdaY2);
			
			// Fixed
			jointDwell += ( 1.0 / ( ( exp(LambdaX1 * bl) - exp(LambdaY1 * bl) ) *( exp(LambdaX2 * bl) - exp(LambdaY2 * bl) ) ) ) *
			(-exp((LambdaX2+LambdaY1)*bl)*bl + ( exp((LambdaX1+LambdaX2)*bl) - exp((LambdaX2+LambdaY1)*bl) ) / (LambdaX1 - LambdaY1) +
			 ( exp((LambdaX2+LambdaY1)*bl) - exp((LambdaY1+LambdaY2)*bl) ) / (LambdaX2 - LambdaY2) +
			 finalTerm );
			if (jointDwell > 10000) { std::cout << "ERROR CASE 3d" << endl; }
            
		}
		
	} else if ( (state1 == y1) && ( state2 == y2 ) ) {
        //		std::cout << "	CASE 4" << endl;
		// CASE 4
		if ( (LambdaX1 == LambdaY1) && (LambdaX2 == LambdaY2) ) {
			jointDwell += bl / 3.0;
			
		} else if ( (rateMatrix.mRows.mDataArray[theModel1].mRows.mDataArray[x1].mDataArray[x1] == rateMatrix.mRows.mDataArray[theModel1].mRows.mDataArray[y1].mDataArray[y1] ) ) {
			LambdaX = LambdaX2;
			LambdaY = LambdaY2;
			
			// jointDwell += ( exp(LambdaY*bl) * ( 2.0 - (bl * bl) * (LambdaX -LambdaY )*(LambdaX -LambdaY ) ) + 2.0 * exp(LambdaX*bl)*(bl*LambdaX - bl*LambdaY - 1) )
			// / ( 2.0 * bl * ( exp(LambdaX*bl) - exp(LambdaY*bl) ) * (LambdaX-LambdaY)*(LambdaX-LambdaY) );
			
			// Fixed
			jointDwell += ( exp(LambdaY*bl) / ( (exp(LambdaX*bl) - exp(LambdaY*bl) ) * ( LambdaX - LambdaY ) ) ) - ( 1 / (bl * (LambdaX-LambdaY)*(LambdaX-LambdaY)) ) - ( bl / ( 2 * (exp( (LambdaY - LambdaX)*bl) - 1) ) );
			if (jointDwell > 10000) { std::cout << "ERROR CASE 4b" << endl; }
            
		} else if ( (LambdaX2 == LambdaY2) ) {
			LambdaX = LambdaX1;
			LambdaY = LambdaY1;
			
            //	jointDwell += ( exp(LambdaY*bl) * ( 2.0 - (bl * bl) * (LambdaX -LambdaY )*(LambdaX -LambdaY ) ) + 2.0 * exp(LambdaX*bl)*(bl*LambdaX - bl*LambdaY - 1) )
            //	/ ( 2.0 * bl * ( exp(LambdaX*bl) - exp(LambdaY*bl) ) * (LambdaX-LambdaY)*(LambdaX-LambdaY) );
            
			// Fixed
			jointDwell += ( exp(LambdaY*bl) / ( (exp(LambdaX*bl) - exp(LambdaY*bl) ) * ( LambdaX - LambdaY ) ) ) - ( 1 / (bl * (LambdaX-LambdaY)*(LambdaX-LambdaY)) ) - ( bl / ( 2 * (exp( (LambdaY - LambdaX)*bl) - 1) ) );
			if (jointDwell > 10000) { std::cout << "ERROR CASE 4c" << endl; }
            
		} else if ( (LambdaX2 == LambdaY1) && (LambdaY2 == LambdaX1) ) { 
			
			jointDwell += (exp( 2 * bl * LambdaX1) - exp(2*bl*LambdaY1) - 2*exp(bl*(LambdaX1+LambdaY1)) * bl * (LambdaX1-LambdaY1) ) / ( ( (exp(LambdaX1*bl) - exp(LambdaY1*bl) )*(exp(LambdaX1*bl) - exp(LambdaY1*bl) )) * (LambdaX1-LambdaY1) );
			
			
		} else {
             
			finalTerm =( exp((LambdaX1+LambdaX2)*bl) - exp((LambdaY1+LambdaY2)*bl) ) / (LambdaX1+LambdaX2-LambdaY1-LambdaY2);
            
			// Fixed
			jointDwell += 1.0 / ( ( exp(LambdaX1 * bl) - exp(LambdaY1 * bl) ) *( exp(LambdaX2 * bl) - exp(LambdaY2 * bl) ) ) *
			(( exp((LambdaX1+LambdaX2)*bl)*bl + ( exp((LambdaX2+LambdaY1)*bl) - exp((LambdaX1+LambdaX2)*bl) ) / (LambdaX1 - LambdaY1) +
			  ( exp((LambdaX1+LambdaY2)*bl) - exp((LambdaX1+LambdaX2)*bl) ) / (LambdaX2 - LambdaY2) +
			  finalTerm ));
			if (jointDwell > 10000) { std::cout << "ERROR CASE 4d" << endl; }
            
		}
		
	}
    
	if (jointDwell < 0.0) { 
		jointDwell = 0.0;
        //		std::cout << "THIS IS AN ERROR" << endl;
        //		std::cout << "jointDwell < 0: " << jointDwell << endl;
        //		std::cout << " Lambdas: " << LambdaX1 << ", " << LambdaX2 << ", " << LambdaY1 << ", " << LambdaY2 << ", bl " << bl << endl;
        //		std::cout << "simpler lambdas: " << LambdaX << ", " << LambdaY << endl;
	}
	
    //	jointDwell /= bl;
	if (jointDwell > bl) {
		std::cout << "	error condition: dwell time cannot exceed bl!! (dwell " << jointDwell << ", bl " << bl << ")" << endl; 
		std::cout << "  on branch " << x1 << " -> " << y1 << ", and " << x2 << " -> " << y2 << "; for states (" << state1 << ", " << state2 << ")" << endl;
		std::cout << "  Lambdas: " << LambdaX1 << ", " << LambdaY1 << "; " << LambdaX2 << ", " << LambdaY2 << endl;
		std::cout << "  Sums: " << (LambdaX1-LambdaX2) << ", " << (LambdaY1-LambdaX1) << ", " << (LambdaY2-LambdaX2) << ", " << (LambdaY2 - LambdaX1) << ", " << (LambdaY1-LambdaX2) << "; " << (LambdaX1+LambdaX2-LambdaY1-LambdaY2) << endl;
		std::cout << "BREAK POINT HERE"<< endl;
		std::cout << "BREAK POINT HERE"<< endl;
		
	}
	return jointDwell;
}	



inline
void
TreeNode::SetBranchB1uProb(
                           TreeNode*	node,
                           int numStates,
                           DoubleMatrix3D& inRateMatrix3D, DoubleVector *uniformizationConstants, int dummyGibbs, int theModel, int row)
{
	// NO CHECKING FOR SUBSTITUTIONS HERE...
	// (this version calculates the whole matrix, regardless of sub counts - needed for ancestral state sampling)
    
	DoubleMatrix3D *Pmat;
	DoubleMatrix *ProbMatrix;
    
	if (mRoot->getCalculateFullProbabilityDistribution()) {
		Pmat = getProbMatrix3D(node); //&node->GetData().probmatrix3D;
	} else {
 		Pmat = getProbMatrix3D(mRoot); //&node->GetData().probmatrix3D;
 	}
	
 	unsigned int numModels = inRateMatrix3D.rows();
	int modelInit, modelLimit;
	int blInit, blLimit;
	int rateCatInit, rateCatLimit;
	
	if (Pmat->empty()) {
		int theNumModels = numModels;
		if (mRoot->numDiscreteBLs != 0) theNumModels *= mRoot->numDiscreteBLs;
		if (options.allowASRV) theNumModels *= options.numCatASRV;
		Pmat->resize( theNumModels, numStates, numStates, 0.0);
	}
	
	if (node != NULL)
	{
		if (theModel != -1) { modelInit = theModel; modelLimit = modelInit+1; }
		else { modelInit = 0; modelLimit = numModels; }
		
		if (mRoot->numDiscreteBLs != 0) { blInit = 0; blLimit = mRoot->numDiscreteBLs; }
		else { blInit = 0; blLimit = 1; }
        
		if (options.allowASRV) { rateCatInit = 0; rateCatLimit = options.numCatASRV; }
		else { rateCatInit = 0; rateCatLimit = 1; }
        
		for (unsigned int k = modelInit; k < modelLimit; k++) {
			for (unsigned int k2 = blInit; k2 < blLimit; k2++) {
				for (unsigned int k3 = rateCatInit; k3 < rateCatLimit; k3++) {
					
					if (mRoot->getCalculateFullProbabilityDistribution()) {
						ProbMatrix = &(*getProbMatrix3D(node)).mRows.mDataArray[ num->returnModelIndex( k, k3, k2 ) ]; //&node->GetData().probmatrix3D[k];
					} else {
						ProbMatrix = &(*getProbMatrix3D(mRoot)).mRows.mDataArray[ num->returnModelIndex( k, k3, k2 ) ]; //&node->GetData().probmatrix3D[k];
					}
					if (ProbMatrix->empty())
						ProbMatrix->resize(numStates, numStates, 0.0);
					else
						ProbMatrix->fill(0.0);
					
					double distance = node->GetData().distance;
					if (mRoot->numDiscreteBLs != 0) { distance = mRoot->discreteBLs[k2]; }
					
					if ( !( (mRoot->getCalculateFullProbabilityDistribution()) && (distance == cInitDistance) ) )
					{
						//		if (distance != cInitDistance)
						{
							// B1 method - rate matrix to probability matrix;
							if ( row == -1 ) {
								// B1 method - rate matrix to probability matrix;
								for (unsigned int i = 0; i < numStates; i++)
								{
									for (unsigned int j = 0; j < numStates; j++)
									{	
										// Overriding with Hwang-Green...
										//(*ProbMatrix).mRows.mDataArray[i].mDataArray[j] = inRateMatrix3D[k][i][j] * distance; //num->calculateB1prob( inRateMatrix3D[k], distance, i, j );
										if (mRoot->getCalculateFullProbabilityDistribution()) {
											(*ProbMatrix).mRows.mDataArray[i].mDataArray[j] = num->calculateB1uprob( inRateMatrix3D.mRows.mDataArray[k], distance, i, j, (*uniformizationConstants).mDataArray[k], mRoot->GetData().siteCats->getRate( k3 ) );
										} else {
											//									std::cout << "SETTING IT..." << endl;
											(*ProbMatrix).mRows.mDataArray[i].mDataArray[j] = inRateMatrix3D.mRows.mDataArray[ k ].mRows.mDataArray[i].mDataArray[j];
										}
										
									}
								}
								
							} else {
								for (unsigned int j = 0; j < numStates; j++)
								{	
									if (mRoot->getCalculateFullProbabilityDistribution()) {
										(*ProbMatrix).mRows.mDataArray[row].mDataArray[j] = num->calculateB1uprob( inRateMatrix3D.mRows.mDataArray[k], distance, row, j, (*uniformizationConstants).mDataArray[k], mRoot->GetData().siteCats->getRate( k3 ) );
									} else {
										(*ProbMatrix).mRows.mDataArray[row].mDataArray[j] = inRateMatrix3D.mRows.mDataArray[ k ].mRows.mDataArray[row].mDataArray[j];
									}
									
								}
								
								for (unsigned int j = 0; j < numStates; j++)
								{	
									if (mRoot->getCalculateFullProbabilityDistribution()) {
										(*ProbMatrix).mRows.mDataArray[j].mDataArray[row] = num->calculateB1uprob( inRateMatrix3D.mRows.mDataArray[k], distance, j, row, (*uniformizationConstants).mDataArray[k], mRoot->GetData().siteCats->getRate( k3 ) );
									} else {
										(*ProbMatrix).mRows.mDataArray[j].mDataArray[row] = inRateMatrix3D.mRows.mDataArray[ k ].mRows.mDataArray[j].mDataArray[row];
									}
									
									
								}
								
							}
						}
					}
				}
			}
		}
	}
}
// ---

//---------------------------------------------------------------------------

inline
void
TreeNode::SetTreeB1Prob(
                        int numStates,
                        DoubleMatrix3D& inRateMatrix3D)
{
	SetTreeB1Prob( numStates, inRateMatrix3D, -1, -1 );
}

// JUST SET THE UNIQUEMODELFLAG...
inline void TreeNode::SetUniqueModelFlags(void) {
	if (Left() != NULL)
	{
		Left()->setUniqueModelFlag(1);
		Left()->SetUniqueModelFlags(); 
	}
    
	if (Right() != NULL) 
	{ 
		Right()->setUniqueModelFlag(1);
		Right()->SetUniqueModelFlags();  
	}
    
}

// STANDARD VERSION...

inline 
void
TreeNode::SetTreeB1Prob(
                        int numStates,
                        DoubleMatrix3D& inRateMatrix3D, int theModel, int x)
{
	if (options.m_nMethod == 3) {
        //	// should insert an index here to reference the proper pmat...
		return SetBranchB1Prob( mRoot, numStates, inRateMatrix3D, theModel, x);
	}
	
	SetBranchB1Prob( mRoot, numStates, inRateMatrix3D, theModel, x);
	
	if (Left() != NULL)
	{
		Left()->setUniqueModelFlag(1);
		SetBranchB1Prob(Left(), numStates, inRateMatrix3D, theModel, x);
		Left()->SetTreeB1Prob( numStates, inRateMatrix3D, theModel, x); 
	}
    
	if (Right() != NULL) 
	{ 
		Right()->setUniqueModelFlag(1);
		SetBranchB1Prob(Right(), numStates, inRateMatrix3D, theModel, x);
		
		Right()->SetTreeB1Prob( numStates, inRateMatrix3D, theModel, x);  
	}
    
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetTreeB1Prob(
                        int numStates,
                        DoubleMatrix3D& inRateMatrix3D, int dummyGibbs)
{
	SetTreeB1Prob( numStates, inRateMatrix3D, dummyGibbs, -1, -1 );
}


inline
void
TreeNode::SetTreeB1Prob(
                        int numStates,
                        DoubleMatrix3D& inRateMatrix3D, int dummyGibbs, int theModel, int x)
{
    
	if (options.m_nMethod == 3) {
		if ( this == mRoot ) { return SetBranchB1Prob(mRoot, numStates, inRateMatrix3D, dummyGibbs, theModel, x); }
		else { return; }
	}
    
	SetBranchB1Prob(mRoot, numStates, inRateMatrix3D, dummyGibbs, theModel, x);
    
	if (Left() != NULL)
	{
		Left()->setUniqueModelFlag(1);
		SetBranchB1Prob(Left(), numStates, inRateMatrix3D, dummyGibbs, theModel, x);
        
		Left()->SetTreeB1Prob( numStates, inRateMatrix3D, dummyGibbs, theModel, x); 
	}
    
	if (Right() != NULL) 
	{ 
		Right()->setUniqueModelFlag(1);
		SetBranchB1Prob(Right(), numStates, inRateMatrix3D, dummyGibbs, theModel, x);
        
		Right()->SetTreeB1Prob( numStates, inRateMatrix3D, dummyGibbs, theModel, x);  
	}
    
}

// --- Uniformized B1 method here ---
//---------------------------------------------------------------------------

inline
void
TreeNode::SetTreeB1uProb(
                         int numStates,
                         DoubleMatrix3D& inRateMatrix3D, DoubleVector* uniformizationConstants)
{
	SetTreeB1uProb( numStates, inRateMatrix3D, uniformizationConstants, -1, -1 );
}

// STANDARD VERSION...

inline 
void
TreeNode::SetTreeB1uProb(
                         int numStates,
                         DoubleMatrix3D& inRateMatrix3D, DoubleVector* uniformizationConstants, int theModel, int x)
{
	if ( (options.m_nMethod == 6) && (mRoot->getCalculateFullProbabilityDistribution() == 0) ) {
		return SetBranchB1uProb( mRoot, numStates, inRateMatrix3D, uniformizationConstants, theModel, x);
	}
	
	SetBranchB1uProb( mRoot, numStates, inRateMatrix3D, uniformizationConstants, theModel, x);
	
	if (Left() != NULL)
	{
		Left()->setUniqueModelFlag(1);
		SetBranchB1uProb(Left(), numStates, inRateMatrix3D,  uniformizationConstants, theModel, x);
		Left()->SetTreeB1uProb( numStates, inRateMatrix3D, uniformizationConstants,  theModel, x); 
	}
    
	if (Right() != NULL) 
	{ 
		Right()->setUniqueModelFlag(1);
		SetBranchB1uProb(Right(), numStates, inRateMatrix3D, uniformizationConstants,  theModel, x);
		
		Right()->SetTreeB1uProb( numStates, inRateMatrix3D, uniformizationConstants,  theModel, x);  
	}
    
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetTreeB1uProb(
                         int numStates,
                         DoubleMatrix3D& inRateMatrix3D, DoubleVector* uniformizationConstants, int dummyGibbs)
{
	SetTreeB1uProb( numStates, inRateMatrix3D, uniformizationConstants, dummyGibbs, -1, -1 );
}


inline
void
TreeNode::SetTreeB1uProb(
                         int numStates,
                         DoubleMatrix3D& inRateMatrix3D, DoubleVector* uniformizationConstants, int dummyGibbs, int theModel, int x)
{	
	if ( (options.m_nMethod == 6) && (mRoot->getCalculateFullProbabilityDistribution() == 0) ) {
		return SetBranchB1uProb( mRoot, numStates, inRateMatrix3D, uniformizationConstants, dummyGibbs, theModel, x);
	}
    
	SetBranchB1uProb(mRoot, numStates, inRateMatrix3D, uniformizationConstants, dummyGibbs, theModel, x);
    
	if (Left() != NULL)
	{
		Left()->setUniqueModelFlag(1);
		SetBranchB1uProb(Left(), numStates, inRateMatrix3D, uniformizationConstants, dummyGibbs, theModel, x);
        
		Left()->SetTreeB1uProb( numStates, inRateMatrix3D, uniformizationConstants, dummyGibbs, theModel, x); 
	}
    
	if (Right() != NULL) 
	{ 
		Right()->setUniqueModelFlag(1);
		SetBranchB1uProb(Right(), numStates, inRateMatrix3D, uniformizationConstants, dummyGibbs, theModel, x);
        
		Right()->SetTreeB1uProb(numStates, inRateMatrix3D, uniformizationConstants, dummyGibbs, theModel, x);  
	}
    
}

// --- Binf method here ---
//---------------------------------------------------------------------------

inline
void
TreeNode::SetTreeBinfProb(
                          int numStates,
                          DoubleMatrix3D& inRateMatrix3D,DoubleMatrix *stateFreqs)
{
	SetTreeBinfProb( numStates, inRateMatrix3D, stateFreqs, -1 );
}

// STANDARD VERSION...

inline 
void
TreeNode::SetTreeBinfProb(
                          int numStates,
                          DoubleMatrix3D& inRateMatrix3D, DoubleMatrix *stateFreqs, int theModel)
{
	if (options.m_nMethod == 5) {
		// should insert an index here to reference the proper pmat...
		return SetBranchBinfProb( mRoot, numStates, inRateMatrix3D, stateFreqs, theModel);
	}
    
	// Need to build U,V,R matrices first...
	SetBranchBinfProb( mRoot, numStates, inRateMatrix3D, stateFreqs, theModel);
	
	if (Left() != NULL)
	{
		Left()->setUniqueModelFlag(1);
		SetBranchBinfProb(Left(), numStates, inRateMatrix3D, stateFreqs, theModel);
		Left()->SetTreeBinfProb( numStates, inRateMatrix3D, stateFreqs,  theModel); 
	}
    
	if (Right() != NULL) 
	{ 
		Right()->setUniqueModelFlag(1);
		SetBranchBinfProb(Right(), numStates, inRateMatrix3D, stateFreqs,  theModel);
		
		Right()->SetTreeBinfProb( numStates, inRateMatrix3D, stateFreqs,  theModel);  
	}
    
}



//---------------------------------------------------------------------------

inline
void
TreeNode::SetBranchB2Prob(
                          TreeNode*	node,
                          int numStates,
                          DoubleMatrix3D& inRateMatrix3D)
{
	DoubleMatrix *ProbMatrix;
	static DoubleMatrix3D Prob2SubMatrix;
	DoubleMatrix3D *mModelCount3D = getChangeMatrix3D( node ); //&node->GetData().countModelchange3D;
    
	if (mModelCount3D->empty()) return SetBranchB2Prob( node, numStates, inRateMatrix3D, 1 );
	
	if (node != NULL)
	{
		unsigned int numModels = inRateMatrix3D.rows();
        
		if (node->GetData().probmatrix3D.empty())
			node->GetData().probmatrix3D.resize(numModels, numStates, numStates, 0);
        
		int rateCatInit, rateCatLimit;
		if (options.allowASRV) { rateCatInit = 0; rateCatLimit = options.numCatASRV; }
		else { rateCatInit = 0; rateCatLimit = 1; }
        
		for (unsigned int m = 0; m < numModels; m++)
		{
			for (unsigned int k3 = rateCatInit; k3 < rateCatLimit; k3++) {
                // NOTE: B2 models are not implemented to deal with discrete branchlengths!
                ProbMatrix = &(*getProbMatrix3D(node))[ num->returnModelIndex( m, k3, 0 ) ]; //&node->GetData().probmatrix3D[m] ;
                
                if (ProbMatrix->empty())
                    ProbMatrix->resize(numStates, numStates, 0.0);
                else
                    ProbMatrix->fill(0.0);
                
                if (Prob2SubMatrix.empty())
                    Prob2SubMatrix.resize(numStates, numStates, numStates, 0.0);
                else	
                    Prob2SubMatrix.fill(0.0);
                
                double distance = node->GetData().distance;
                
                if ((distance != cInitDistance) || (node->Up()->Up()->Up() == node->Up()))
                {
                    if (distance == cInitDistance)
                    {
                        TreeNode* tempNode;
                        
                        if (node == node->Up()->Left())
                            tempNode = node->Up()->Right();
                        else
                            tempNode = node->Up()->Left();
                        
                        distance = tempNode->GetData().distance;
                    }
                    
                    if (distance != cInitDistance)
                    {
                        distance *= mRoot->GetData().siteCats->getRate( k3 );
                        
                        // B2 method - rate matrix to probability matrix;
                        
                        // first set the probmatrix as the b1 method
                         for (unsigned int i = 0; i < numStates; i++)
                        {	
                            for (unsigned int j = 0; j < numStates; j++)
                            {	
                                (*ProbMatrix).mRows.mDataArray[i].mDataArray[j] = num->calculateB1prob( inRateMatrix3D.mRows.mDataArray[ num->returnModelIndex( m, k3, 0 ) ], distance, i, j );
                            }
  
                            for (unsigned int i = 0; i < numStates; i++)
                            {
                                for (unsigned int k = 0; k < numStates; k++)
                                {
                                    for (unsigned int j = 0; j < numStates; j++)
                                    {
                                        Prob2SubMatrix.mRows.mDataArray[i].mRows.mDataArray[j].mDataArray[k] = num->calculateB2prob( inRateMatrix3D.mRows.mDataArray[num->returnModelIndex( m, k3, 0 )], distance, i, j, k );
                                        
                                    }
                                }
                            }
                        }
                        
                        for (unsigned int i = 0; i < numStates; i++)
                        {
                            for (unsigned int k = 0; k < numStates; k++)
                            {
                                for (unsigned int j = 0; j < numStates; j++) {
                                    (*ProbMatrix).mRows.mDataArray[i].mDataArray[k] += Prob2SubMatrix.mRows.mDataArray[i].mRows.mDataArray[j].mDataArray[k];
                                }
                            }	
                        }
                        
                        
                        //[dont want to do this?!] 
                        NormalizeMatrix((*ProbMatrix));
                        
                     }	
                }
            }
        }
	}
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetBranchB2Prob(
                          TreeNode*	node,
                          int numStates,
                          DoubleMatrix3D& inRateMatrix3D, int dummyGibbs)
{
	// No checking if counts are set -- use this for Gibbs sampling...
	DoubleMatrix *ProbMatrix;
	static DoubleMatrix3D Prob2SubMatrix;
    
	if (node != NULL)
	{
		unsigned int numModels = inRateMatrix3D.rows();
        
		if (node->GetData().probmatrix3D.empty())
			node->GetData().probmatrix3D.resize( (numModels ), numStates, numStates, 0);
        
 		for (unsigned int m = 0; m < numModels; m++) {
            
			ProbMatrix = &(*getProbMatrix3D(node))[m]; //&node->GetData().probmatrix3D[m] ;
            

			if (ProbMatrix->empty())
				ProbMatrix->resize(numStates, numStates, 0);
			else
				ProbMatrix->fill(0);
            
			if (Prob2SubMatrix.empty())
				Prob2SubMatrix.resize(numStates, numStates, numStates, 0);
			else	
				Prob2SubMatrix.fill(0);
            
			double distance = node->GetData().distance;
            
			if ((distance != cInitDistance) || (node->Up()->Up()->Up() == node->Up()))
			{
				if (distance == cInitDistance)
				{
					TreeNode* tempNode;
                    
					if (node == node->Up()->Left())
						tempNode = node->Up()->Right();
					else
						tempNode = node->Up()->Left();
                    
					distance = tempNode->GetData().distance;
				}
                
				if (distance != cInitDistance)
				{
					// B2 method - rate matrix to probability matrix;
                    
					// first set the probmatrix as the b1 method
					for (unsigned int i = 0; i < numStates; i++)
					{	
						for (unsigned int j = 0; j < numStates; j++)
						{	
							if (i != j)
							{
								// cutoff set to avoid the calculation flip of exp calculation
								if (((inRateMatrix3D[m][i][i] - inRateMatrix3D[m][j][j]) > 0.0001) || ((inRateMatrix3D[m][j][j] - inRateMatrix3D[m][i][i]) > 0.0001))
									(*ProbMatrix)[i][j] = inRateMatrix3D[m][i][j] * (num->exponential(inRateMatrix3D[m][i][i] * distance) - num->exponential(inRateMatrix3D[m][j][j] * distance)) / (inRateMatrix3D[m][i][i] - inRateMatrix3D[m][j][j]);
								else
									(*ProbMatrix)[i][j] = inRateMatrix3D[m][i][j] * num->exponential(inRateMatrix3D[m][i][i] * distance) *  distance;
							}
                            (*ProbMatrix)[i][j] = 1.0;
						}
                        
						(*ProbMatrix)[i][i] = num->exponential(inRateMatrix3D[m][i][i] * distance);
						(*ProbMatrix)[i][i] = 1.0;
					}
					
					// secondly, calculate the probability with 2 substitutions of x !=y and y != z
					for (unsigned int i = 0; i < numStates; i++)
					{
						for (unsigned int k = 0; k < numStates; k++)
						{
							for (unsigned int j = 0; j < numStates; j++)
							{
								if (i == j || j == k) {	// only one substitution or no substitution
									Prob2SubMatrix[i][j][k] = 0;
								} else {
									if (((inRateMatrix3D[m][j][j]- inRateMatrix3D[m][k][k]) < 0.0001) || ((inRateMatrix3D[m][k][k] - inRateMatrix3D[m][j][j]) < 0.0001))
                                        // when rate[y] == rate[z]
									{
										if ( ((inRateMatrix3D[m][i][i]- inRateMatrix3D[m][k][k]) < 0.0001) || ((inRateMatrix3D[m][k][k] - inRateMatrix3D[m][i][i]) < 0.0001))
                                            // when rate[x] = rate[z]
											Prob2SubMatrix[i][j][k] = inRateMatrix3D[m][i][j] * inRateMatrix3D[m][j][k] * num->exponential(inRateMatrix3D[m][k][k] * distance) * distance * distance * 0.5;
										else
                                            // when rate[x] != rate[z]
											Prob2SubMatrix[i][j][k] = inRateMatrix3D[m][i][j] * inRateMatrix3D[m][j][k] * num->exponential(inRateMatrix3D[m][k][k] * distance) * (num->exponential((inRateMatrix3D[m][i][i] - inRateMatrix3D[m][k][k]) * distance) - 1 - distance * (inRateMatrix3D[m][i][i] - inRateMatrix3D[m][k][k])) / pow(inRateMatrix3D[m][i][i] - inRateMatrix3D[m][k][k], 2);
									}
									else
									{
										// when rate[y] != rate[z]
										if ( ((inRateMatrix3D[m][i][i]- inRateMatrix3D[m][j][j]) < 0.0001) || ((inRateMatrix3D[m][j][j] - inRateMatrix3D[m][i][i]) < 0.0001))
											// when rate[x] == rate[y]
											Prob2SubMatrix[i][j][k] = inRateMatrix3D[m][i][j] * inRateMatrix3D[m][j][k] * distance * num->exponential(inRateMatrix3D[m][j][j] * distance) / (inRateMatrix3D[m][j][j] - inRateMatrix3D[m][k][k]) -inRateMatrix3D[m][i][j] * inRateMatrix3D[m][j][k] * (num->exponential(inRateMatrix3D[m][i][i] * distance)-num->exponential(inRateMatrix3D[m][k][k] * distance))/((inRateMatrix3D[m][j][j] - inRateMatrix3D[m][k][k])*(inRateMatrix3D[m][i][i] - inRateMatrix3D[m][k][k]));
										else
										{
											// rate[x] != rate[y]
											if ( (inRateMatrix3D[m][i][i]- inRateMatrix3D[m][k][k]) < 0.0001 || ((inRateMatrix3D[m][k][k] - inRateMatrix3D[m][i][i]) < 0.0001))
												// rate[x] = rate[z]
												Prob2SubMatrix[i][j][k] = inRateMatrix3D[m][i][j] * inRateMatrix3D[m][j][k] * (num->exponential(inRateMatrix3D[m][i][i] * distance)-num->exponential(inRateMatrix3D[m][j][j] * distance))/((inRateMatrix3D[m][j][j] - inRateMatrix3D[m][k][k])*(inRateMatrix3D[m][i][i] - inRateMatrix3D[m][j][j])) - inRateMatrix3D[m][i][j] * inRateMatrix3D[m][j][k] * distance * num->exponential(inRateMatrix3D[m][k][k] * distance) / (inRateMatrix3D[m][j][j] -inRateMatrix3D[m][k][k]);
											else
												// rate[x] != rate[z]
												Prob2SubMatrix[i][j][k] = inRateMatrix3D[m][i][j] * inRateMatrix3D[m][j][k] * ((inRateMatrix3D[m][j][j] - inRateMatrix3D[m][k][k]) * num->exponential(inRateMatrix3D[m][i][i] * distance) + (inRateMatrix3D[m][k][k] - inRateMatrix3D[m][i][i]) * num->exponential(inRateMatrix3D[m][j][j] * distance) + (inRateMatrix3D[m][i][i] - inRateMatrix3D[m][j][j]) * num->exponential(inRateMatrix3D[m][k][k] * distance))/((inRateMatrix3D[m][j][j] - inRateMatrix3D[m][k][k]) * (inRateMatrix3D[m][i][i] - inRateMatrix3D[m][j][j]) * (inRateMatrix3D[m][i][i] - inRateMatrix3D[m][k][k]));
										}
									}
								}
							}
						}
					}
                    
					for (unsigned int i = 0; i < numStates; i++)
					{
						for (unsigned int k = 0; k < numStates; k++)
						{
							for (unsigned int j = 0; j < numStates; j++) {
								(*ProbMatrix)[i][k] += Prob2SubMatrix[i][j][k];
							}
						}	
					}
                    
                    
  				NormalizeMatrix((*ProbMatrix));
					
  				}	
			}
		} 
	}
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetTreeB2Prob(
                        int numStates,
                        DoubleMatrix3D& inRateMatrix3D)
{
	if (Left() != NULL)
	{
		Left()->setUniqueModelFlag(1);
		SetBranchB2Prob(Left(), numStates, inRateMatrix3D);
        
		Left()->SetTreeB2Prob(numStates, inRateMatrix3D); 
	}
    
	if (Right() != NULL) 
	{ 
		Right()->setUniqueModelFlag(1);
		SetBranchB2Prob(Right(), numStates, inRateMatrix3D);
        
		Right()->SetTreeB2Prob(numStates, inRateMatrix3D);  
	}
}

//---------------------------------------------------------------------------

inline
void
TreeNode::SetTreeB2Prob(
                        int numStates,
                        DoubleMatrix3D& inRateMatrix3D, int dummyGibbs)
{
	// This version does no checking for inferred substitutions...
	
	if (Left() != NULL)
	{
		Left()->setUniqueModelFlag(1);
		SetBranchB2Prob(Left(), numStates, inRateMatrix3D, dummyGibbs);
        
		Left()->SetTreeB2Prob(numStates, inRateMatrix3D, dummyGibbs); 
	}
    
	if (Right() != NULL) 
	{ 
		Right()->setUniqueModelFlag(1);
		SetBranchB2Prob(Right(), numStates, inRateMatrix3D, dummyGibbs);
        
		Right()->SetTreeB2Prob(numStates, inRateMatrix3D, dummyGibbs);  
	}
}

//---------------------------------------------------------------------------
// This likelihood function just uses the substitution probability matrices and the counts

inline
void
TreeNode::CalTreeLnLikelihood(
                              DoubleVector& inFreqVector,
                              double& logLikelihood)
{
	if (Up()->Up() == this)
		logLikelihood = 0;
	
	DoubleMatrix3D *mProb3D = getProbMatrix3D(this); //&GetData().probmatrix3D;
    
	IntMatrix *mCount = getChangeMatrix(this); //GetData().countchange;
	
	if (!mProb3D->empty() && !mCount->empty())
	{
		unsigned int numModels = inFreqVector.size();
        
		unsigned int numStates = mCount->rows();
		
		if (options.m_bSite)	//calculate the probability of mixture models when mixed with sites
		{
			for (unsigned int i = 0; i < numStates; i++)
				for (unsigned int j = 0; j < numStates; j++)
				{
					if ((*mCount)[i][j] == 0) continue;
					
					double prob = 0;
                    
					for (unsigned int k = 0; k < numModels; k++) {
						if (mRoot->numDiscreteBLs == 0) { 
							prob += inFreqVector[k] * (*mProb3D)[k][i][j];
						} else {
							prob += inFreqVector[k] * (*mProb3D)[(GetData().branchLengthIndex * numModels) +k][i][j];
						}
					}
                    
                    
					logLikelihood += (*mCount)[i][j] * num->logarithm(prob);
				}
		}
		else		// mixed with lineages [deprecated in current version]
		{
			DoubleVector logLikelihoodModel(numModels, 0);
			
			double logLikelihoodModelMax;
			
			for (unsigned int k = 0; k < numModels; k++)
			{
				for (unsigned int i = 0; i < numStates; i++)
					for (unsigned int j = 0; j < numStates; j++) {
						if ((*mCount)[i][j]==0) continue;
						
						if (mRoot->numDiscreteBLs == 0) { 
							logLikelihoodModel[k] += (*mCount)[i][j] * num->logarithm((*mProb3D)[k][i][j]);
						} else {
							logLikelihoodModel[k] += (*mCount)[i][j] * num->logarithm((*mProb3D)[(GetData().branchLengthIndex * numModels) +k][i][j]);
						}
                        
						
					}
				if (k == 0)
					logLikelihoodModelMax = logLikelihoodModel[k];
				else if (logLikelihoodModel[k] > logLikelihoodModelMax)
					logLikelihoodModelMax = logLikelihoodModel[k];
			}
			
			double likelihood = 0;
			
			for (unsigned int k = 0; k< numModels; k++)
			{
				if (logLikelihoodModel[k] - logLikelihoodModelMax >= -22)
					likelihood += inFreqVector[k] * num->exponential(logLikelihoodModel[k] - logLikelihoodModelMax);
			}
			
			logLikelihood += num->logarithm(likelihood) + logLikelihoodModelMax;
		}
	}
	
	if (options.m_nMethod == 3) return;
	
	if (Left() != NULL)
		Left()->CalTreeLnLikelihood( inFreqVector, logLikelihood ); 
    
	if (Right() != NULL) 
		Right()->CalTreeLnLikelihood( inFreqVector, logLikelihood );  
}

//---------------------------------------------------------------------------

inline
void
TreeNode::CalcLogLikelihood_branchDeltaB1(    MixtureModel& mModel,
                                          IntVector& m_vecSiteAssignment,
                                          double& logLikelihood, int theModel, int x)
{
	unsigned int i,j;
    
	if (Up()->Up() == this)
		logLikelihood =0;
    
	DoubleMatrix3D* mProb3D = getProbMatrix3D(this); //&GetData().probmatrix3D;
	int blInit, blLimit;
	
	if ( options.allowASRV && (options.numCatASRV > 1) ) { std::cout << "	ERROR!  UpdateBranchLengthDeltaB1() not yet adapted for ASRV..." << endl; exit(-1); }
    
	if (!mProb3D->empty())
	{
		unsigned int numModels = mProb3D->rows();
		unsigned int numStates = mProb3D->cols();
		double fh;
		
		if (mRoot->numDiscreteBLs != 0) { numModels /= mRoot->numDiscreteBLs; }
        
		if (options.m_bSite)	//calculate the probability of mixture models with assignments of model to each site
		{
			DoubleMatrix3D *mModelCount3D = getChangeMatrix3D( this ); //&GetData().countModelchange3D;
            
			if (mRoot->numDiscreteBLs != 0) { blInit = 0; blLimit = mRoot->numDiscreteBLs; }
			else { blInit = 0; blLimit = 1; }
            
			if (!mModelCount3D->empty())
			{
				fh = 0.0;
				for (i = 0; i < numStates; i++) {
					for (unsigned int k2 = blInit; k2 < blLimit; k2++) {
						if ((*mModelCount3D).mRows.mDataArray[(k2*numModels) + theModel].mRows.mDataArray[i].mDataArray[x] == 0) continue;
						fh = ((*mProb3D).mRows.mDataArray[(k2*numModels) + theModel].mRows.mDataArray[i].mDataArray[x]);
						if (fh <= 0) fh = 1e-70;
						logLikelihood += (*mModelCount3D).mRows.mDataArray[(k2*numModels) + theModel].mRows.mDataArray[i].mDataArray[x] * num->logarithm(fh);
					}
				}
				
				for (i = 0; i < numStates; i++) {
					if (i == x) continue;
					for (unsigned int k2 = blInit; k2 < blLimit; k2++) {
						if ((*mModelCount3D).mRows.mDataArray[(k2*numModels) + theModel].mRows.mDataArray[x].mDataArray[i] == 0) continue;
						fh = ((*mProb3D).mRows.mDataArray[(k2*numModels) + theModel].mRows.mDataArray[x].mDataArray[i]);
						if (fh <= 0) fh = 1e-70;
						logLikelihood += (*mModelCount3D).mRows.mDataArray[(k2*numModels) + theModel].mRows.mDataArray[x].mDataArray[i] * num->logarithm(fh);
					}
				}
			}
		}
        /*		else		//calculate the probability of mixture models with assignments of model to each lineage
         {
         std::cout << "ERROR!  Integration over branches not implmented in UpdateBranchLengthDeltaB1()" << endl; exit(-1);
         IntMatrix *mCount = getChangeMatrix(this); //GetData().countchange;
         
         if (!mCount->empty())
         {
         int model = GetData().sequence.GetAssignment();
         
         for (i = 0; i < numStates; i++)
         for (j = 0; j < numStates; j++) {
         fh = ((*mProb3D).mRows.mDataArray[model].mRows.mDataArray[i].mDataArray[j]);
         //if (fh <= 0) fh = 1e-70;
         logLikelihood += mCount->mRows.mDataArray[i].mDataArray[j] * num->logarithm(fh);
         }
         }
         }
         */
	} 
    // [still need to calculate likelihood contribution over tree]	if (options.m_nMethod == 3) { return; }
    
	// For B1xM: only need to do this once...
	if ( (options.m_nMethod == 3) && (this == mRoot) ) { return; }
	if ( (options.m_nMethod == 5) && (this == mRoot) ) { return; }
    
    
	if (Left() != NULL)
		Left()->CalcLogLikelihood_branchDeltaB1(  mModel, m_vecSiteAssignment, logLikelihood, theModel, x); 
    
	if (Right() != NULL) 
		Right()->CalcLogLikelihood_branchDeltaB1( mModel,  m_vecSiteAssignment, logLikelihood, theModel, x);  
}

//---------------------------------------------------------------------------
// --- B1 uniformized version --------


inline
double
TreeNode::CalcLogLikelihood_branchDeltaB1uni(   MixtureModel& mModel,
                                             //                                                 DoubleMatrix3D *rateMat3D,
                                             //                                               DoubleVector *uniformizationConstants,
                                             IntVector& m_vecSiteAssignment,
                                             double& logLikelihood, int theModel, int x, int y) {
    
	// This function returns the likelihood contribution of state transition (x,y) only (as logLikelihood).  Useful for change in likelihood evaluations, such as under RAR models
	// Returned value is 0, unless error
	DoubleMatrix3D* rateMat3D             = mModel.returnRateMatrix3D();
    DoubleVector* uniformizationConstants = mModel.returnUniformizationConstants();
    
	if (options.allowASRV) { std::cout << "UpdateBranchLengthDeltaB1uni() used or not?  not implemented scaling under ASRV yet." << endl; exit(-1); }
	
    //	unsigned int i,j;
	int blInit, blLimit;
	
	if (!rateMat3D->empty()) {
		unsigned int numModels = rateMat3D->rows();
		if (mRoot->numDiscreteBLs != 0) { numModels /= mRoot->numDiscreteBLs; }
        
		double term1, term2;
		
		// Calculate the probability of mixture models with assignments of model to each site
		if (options.m_bSite) {
			DoubleMatrix3D		*realChangeCount3D = getChangeMatrix3D( mRoot ); 
			DoubleMatrix		*augmentedCount    = getAugmentedCountMatrix( mRoot );
            
			if (mRoot->numDiscreteBLs != 0) { blInit = 0; blLimit = mRoot->numDiscreteBLs; }
			else { blInit = 0; blLimit = 1; }
            
			logLikelihood = 0.0;
			if (!realChangeCount3D->empty()) {
				term1 = 0.0;
				for (unsigned int k2 = blInit; k2 < blLimit; k2++) {
					term2 = uniformizationConstants->mDataArray[theModel] + (*rateMat3D).mRows.mDataArray[(k2*numModels) + theModel].mRows.mDataArray[x].mDataArray[x];
                    
					// ERROR CONDITION: mu is set too small
					if (term2 <=0) {
						if (!options.capUniformizationRate) {

							// Return the offending rowSum, and it will be used to reinit the uniformization constant
							return (*rateMat3D).mRows.mDataArray[(k2*numModels) + theModel].mRows.mDataArray[x].mDataArray[x];
						} else{
							logLikelihood = -std::numeric_limits<double>::max( );
							return -1;
						}
					}
                    
					// If there are no actual changes assigned to state transition (x,y), the change in likelihood will be zero
					if ((*realChangeCount3D).mRows.mDataArray[(k2*numModels) + theModel].mRows.mDataArray[x].mDataArray[y] == 0) continue;
					
					// term1 is the rate for (x,y)
					term1 = ((*rateMat3D).mRows.mDataArray[(k2*numModels) + theModel].mRows.mDataArray[x].mDataArray[y]);
					if (term1 <= 0) term1 = 1e-70;
					
					// Likelihood contribution for (x,y) - part 1
					logLikelihood += (*realChangeCount3D).mRows.mDataArray[(k2*numModels) + theModel].mRows.mDataArray[x].mDataArray[y] * num->logarithm(term1);
					
					// Likelihood contribution for (x,y) - part 2 (virtual subs)
					int rateClassLimit = options.numCatASRV;
					if ( options.allowASRV != 1 ) rateClassLimit = 1;
					for (int rateclass= 0; rateclass < rateClassLimit; rateclass++ ) {
						logLikelihood += (*augmentedCount).mRows.mDataArray[ num->returnModelIndex( theModel, rateclass, 0) ].mDataArray[x] * num->logarithm(term2);
					}
				}
			}
		}
		else		//calculate the probability of mixture models with assignments of model to each lineage
		{
			std::cout << "ERROR!  Integration over branches not implmented in UpdateBranchLengthDeltaB1uni()" << endl; exit(-1);
			/*		
             IntMatrix *mCount = getChangeMatrix(this); //GetData().countchange;
             if (!mCount->empty())
             {
             int model = GetData().sequence.GetAssignment();
             
             for (i = 0; i < numStates; i++)
             for (j = 0; j < numStates; j++) {
             term1 = ((*rateMat3D).mRows.mDataArray[model].mRows.mDataArray[i].mDataArray[j]);
             //if (fh <= 0) fh = 1e-70;
             logLikelihood += mCount->mRows.mDataArray[i].mDataArray[j] * num->logarithm(fh);
             }
             }
             */
		}
	}	
    
	return logLikelihood;
}


// ---
// --- B1 uniformized version --------


inline
double
TreeNode::CalcLogLikelihood_branchB1uni_bySite(
                                               MixtureModel& mModel,
                                               double* logLikelihood, int theModel, int site )
{
	// This returns a value proportional to the site-specific likelihood given model component and site...
	// * I think this is wrong when using mixture models
	// * In this case, the virtual substitution probabilities are different under each model
	// * Therefore easier (if only marginally more time consuming) to use the integrated likelihood (over virt subs)
	
	// Uses likelihood scaling instead of log-likelihood.  Check likelihoood funciton; this is probably only okay for rate variation, if that.
	
	DoubleMatrix3D* rateMat3D = mModel.returnRateMatrix3D();
	DoubleVector* uniformizationConstants = mModel.returnUniformizationConstants();
	
	double returnVal = 0;
	
	if (options.allowASRV) { std::cout << "UpdateBranchLengthB1uni_bySite() works for mixture models, not rate variation -- so far" << endl; exit(-1); }
	DoubleMatrix    *changeCountBySite = getChangeMatrix3D( mRoot, site );
	
	unsigned int x,y;
	if (GetData().id != mRoot->GetData().id) { std::cout << "ERROR: UpdateBranchLengthB1uni_bySite() was called at node " << GetData().id << endl; }
	
	if (!rateMat3D->empty()) {
		unsigned int numStates = rateMat3D->cols();
		double fh;
		
		long double tempL = ( numeric_limits<long double>::max( ) );
		
        *logLikelihood = 0.0;
        fh = 0.0;
        
        for ( x = 0; x<numStates; x++) {
            for ( y = 0; y<numStates; y++) {

                // Only adding in the effect of the x != y's ??
				if (y != x) {
                    returnVal += (*changeCountBySite).mRows.mDataArray[x].mDataArray[y];

					fh = ((*rateMat3D).mRows.mDataArray[theModel].mRows.mDataArray[x].mDataArray[y]);
                    if (fh <= 0) fh = 1e-70;
					
                    tempL *= pow( fh, (*changeCountBySite).mRows.mDataArray[x].mDataArray[y] );
					
					
                    fh = ((*rateMat3D).mRows.mDataArray[theModel].mRows.mDataArray[x].mDataArray[y]);
                    if (fh <= 0) fh = 1e-70;
					
                    double logger = uniformizationConstants->mDataArray[theModel] + (*rateMat3D).mRows.mDataArray[theModel].mRows.mDataArray[x].mDataArray[x];
                    if (logger <=0) logger = 1e-70;
					
					tempL *=  pow( logger,  (*changeCountBySite).mRows.mDataArray[x].mDataArray[x] );

				} else {

				}
            }
        }

		*logLikelihood = (double) ( log( tempL ) - log( ( numeric_limits<long double>::max( )  ) ) );

	}	
	return returnVal;
}

//---------------------------------------------------------------------------

inline
double 
TreeNode::rndu(void)
{
	return num->Random();
}

//---------------------------------------------------------------------------

inline
void
TreeNode::UpdateAncestralAndTransientStatesForSubtree_localUpdater(
                                                                   TreeNode*	head,
                                                                   TreeNode*	from,
                                                                   ObservedData& theData,
                                                                   DoubleVector& inFreqModels)
{
	int numStates = theData.getNumStates();
	
	double tempSum=0.0;
	DoubleVector probVec(numStates, 0);
	DoubleVector probNormalized(numStates, 0);
	
	if (head != NULL)
	{
		if (head->Up() != NULL && head->Left() != NULL && head->Right() != NULL)
		{
			if (head->Up()->Up() != head)
			{
				/*
                 get the sequence information of up, left and right
                 get the probability information of up node to current node, current node to left node, current node to right node.
                 calculate the probability distribution of each possible sequence state at each site
                 randomizly select the ancestral node state based on the probability distribution*/
				
				vector<int> seq1 = head->Left()->GetData().sequence;
				vector<int> seq2 = head->Right()->GetData().sequence;
				
				vector<int> seq3;
				
				if (head->Up()->Up()->Up()!= head->Up())
				{
					seq3 = head->Up()->GetData().sequence;
				}
				else
				{
					if (head->Up()->Left() == head)
						seq3 = head->Up()->Right()->GetData().sequence;
					else
						seq3 = head->Up()->Left()->GetData().sequence;
				}
                
				DoubleMatrix3D& probmatrix3D1 = *getProbMatrix3D(head->Left()); //head->Left()->GetData().probmatrix3D;
				DoubleMatrix3D& probmatrix3D2 = *getProbMatrix3D(head->Right()); //head->Right()->GetData().probmatrix3D;
				DoubleMatrix3D& probmatrix3D3 = *getProbMatrix3D(head); //head->GetData().probmatrix3D;
                
				DoubleMatrix probmatrix1(numStates, numStates, 0);
				DoubleMatrix probmatrix2(numStates, numStates, 0);
				DoubleMatrix probmatrix3(numStates, numStates, 0);
				
				unsigned int numModels = inFreqModels.size();
				
				for (unsigned int i = 0; i < numStates; i++)
					for (unsigned int j = 0; j < numStates; j++)
						for (unsigned int k = 0; k< numModels; k++) {
							if (mRoot->numDiscreteBLs == 0) { 
								probmatrix1[i][j] += inFreqModels[k] * probmatrix3D1[k][i][j];
							} else {
								probmatrix1[i][j] += inFreqModels[k] * probmatrix3D1[(head->Left()->GetData().branchLengthIndex * numModels) +k][i][j];
							}
						}
                
				for (unsigned int i = 0; i < numStates; i++)
					for (unsigned int j = 0; j < numStates; j++)
						for (unsigned int k = 0; k< numModels; k++) {
							if (mRoot->numDiscreteBLs == 0) { 
								probmatrix2[i][j] += inFreqModels[k] * probmatrix3D2[k][i][j];
							} else {
								probmatrix2[i][j] += inFreqModels[k] * probmatrix3D2[(head->Right()->GetData().branchLengthIndex * numModels) +k][i][j];
							}
						}
                
				for (unsigned int i = 0; i < numStates; i++)
					for (unsigned int j = 0; j < numStates; j++)
						for (unsigned int k = 0; k< numModels; k++) {
							if (mRoot->numDiscreteBLs == 0) { 
								probmatrix3[i][j] += inFreqModels[k] * probmatrix3D3[k][i][j];
							} else {
								probmatrix3[i][j] += inFreqModels[k] * probmatrix3D3[(head->GetData().branchLengthIndex * numModels) +k][i][j];
							}
						}
				
				// All 3 sequences should be the same size...
                unsigned int len = seq1.size(); //min(seq1.size(), seq2.size());
				//				len = min(len, seq3.size() );
				
				for (unsigned int s=0; s < len; s++)
				{
					int indx1 = seq1[s]; //SIndex(seq1.mImpl->mSequenceStr[s], inStateindex, minstate);
					int indx2 = seq2[s]; //SIndex(seq2.mImpl->mSequenceStr[s], inStateindex, minstate);
					int indx3 = seq3[s]; //SIndex(seq3.mImpl->mSequenceStr[s], inStateindex, minstate);
					
					tempSum = 0;
					
					for (unsigned int i = 0; i< numStates; i++)
					{
						probVec[i] = probmatrix1[i][indx1] * probmatrix2[i][indx2] * probmatrix3[indx3][i];
                        
						tempSum += probVec[i];
					}
                    
					for (unsigned int i = 0; i< numStates; i++)
						probNormalized[i] = probVec[i] / tempSum;
                    
					
                    
                    tempSum = 0;
                    
                    unsigned int i;
                    
                    for (i = 0; i < numStates; i++)
                    {
                        tempSum += probNormalized[i];
                        probNormalized[i] = tempSum;
                    }
                    
                    double test = rndu();
                    
                    for (i = 0; i< numStates; i++)
                    {
                        if (test < probNormalized[i])
                        {
                            head->GetData().sequence[s] = i; //SetAtIndex(s, inStateVector[i]);
                            break;
                        }
                    }
                    
                    if (i == numStates)
                        head->GetData().sequence[s] = ENDSEQ; //.SetAtIndex(s, ENDSEQ);
                }
            }
			
			
			if (head->Left() != from)
				UpdateAncestralAndTransientStatesForSubtree_localUpdater(head->Left(), head, theData, inFreqModels);
            
			if (head->Right() != from)
				UpdateAncestralAndTransientStatesForSubtree_localUpdater(head->Right(), head, theData, inFreqModels);
            
			if (head->Up() != from) 
				UpdateAncestralAndTransientStatesForSubtree_localUpdater(head->Up(), head, theData, inFreqModels);
		}
	}
}


//---------------------------------------------------------------------------

inline
void
TreeNode::findNodeForLocalUpdateOfAncestralAndTransientStates(
                                                              int&	rannode,
                                                              int&		found,
                                                              ObservedData& theData,
                                                              DoubleVector& inFreqModels)
{
	if ( this != NULL && found == 0)
	{
		if (Left() != NULL && Right() != NULL)
		{
			if (GetData().id == rannode)
			{
				found=1;
				UpdateAncestralAndTransientStatesForSubtree_localUpdater(this, NULL, theData, inFreqModels);
			}	
			
			if (Left() != NULL && found == 0)
				Left()->findNodeForLocalUpdateOfAncestralAndTransientStates(rannode, found, theData, inFreqModels);
            
			if (Right() != NULL && found == 0)
				Right()->findNodeForLocalUpdateOfAncestralAndTransientStates(rannode, found, theData, inFreqModels);
		}
		else
		{	
			// If it is a leaf node
			if (GetData().id == rannode)
			{
				found=1;
				
				UpdateAncestralAndTransientStatesForSubtree_localUpdater(Up(), NULL, theData, inFreqModels);
			}
		}
	}
}


inline
void
TreeNode::KEEP_UpdateBranchLength_fromStartNode( int startNode,
                                                Tree& mTree,
                                                int found,
                                                TreeNode* inFrom,
                                                double& windowSize,
                                                MixtureModel& theModel,
                                                ObservedData& theData,
                                                IntVector& m_vecSiteAssignment,
                                                int version, int& dirtyTree) {
    
    //	int dirtyTest = 0;
	if ( this != NULL && found == 0)
	{
		if (Left() != NULL || Right() != NULL)
		{
			if (GetData().id == startNode)
			{
				found=1;
				dirtyTree += KEEP_UpdateBranchLength(this, mTree, windowSize, theModel, theData, m_vecSiteAssignment, version);
			}	
			
			if (Left() != NULL && found == 0)
				Left()->KEEP_UpdateBranchLength_fromStartNode(startNode, mTree, found, inFrom, windowSize, theModel, theData, m_vecSiteAssignment, version, dirtyTree);
			
			if (Right() != NULL && found == 0)
				Right()->KEEP_UpdateBranchLength_fromStartNode(startNode, mTree, found, inFrom, windowSize, theModel, theData, m_vecSiteAssignment, version, dirtyTree);
		}
		else
		{	
			// If it is a leaf node
			if (GetData().id == startNode)
			{
				found=1;
				
				dirtyTree += KEEP_UpdateBranchLength(this, mTree, windowSize, theModel, theData, m_vecSiteAssignment, version);
			}
		}
	}
}

//---------------------------------------------------------------------------

inline
void
TreeNode::TransformBranchLength_B1u(TreeNode* inFrom, double rateFactor, double mu ) {
  	if (this != NULL) {
		if (Up()->Up() != this) {
			double branch = GetData().distance;
			//double newBranch = - ( rateFactor * branch ) / (rateFactor * mu * branch - mu * branch - 1.0 );
			// double newBranch   = rateFactor * branch / ( rateFactor + mu * (-1.0 + rateFactor) * branch );
			//double newBranch = branch / ( rateFactor + mu * (-1.0 + rateFactor) * branch );
			
            double newBranch = branch / rateFactor;
			
			if (newBranch > 0.0) {
			//	std::cout << "Transformed " << branch << " to " << newBranch << endl;
				GetData().distance = newBranch;
			} else {
				std::cout << "ERROR: Transformed " << branch << " is negative: " << newBranch << endl;
				std::cout << "		( rateFactor is " << rateFactor << ", mu is " << mu << ")" << endl;
				GetData().distance = newBranch;
			}
			
		}
		
		if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
			Left()->TransformBranchLength_B1u(this, rateFactor, mu);
		
		if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
			Right()->TransformBranchLength_B1u(this, rateFactor, mu);
		
		if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
			Up()->TransformBranchLength_B1u(this, rateFactor, mu);
	}
}

//---------------------------------------------------------------------------

inline
int
TreeNode::KEEP_UpdateBranchLength(
                                  TreeNode* inFrom,
                                  Tree& mTree,
                                  double& m_fWindowsize,
                                  MixtureModel& theModel,
                                  ObservedData& theData,
                                  IntVector& m_vecSiteAssignment,
                                  int version) {
	
	double branch;
	double mu = theModel.getUniformization(0);
	int dirty = 0;
	
	branch = GetData().distance;
	
	// Override for other methods...
	if (options.m_nMethod != 6) version = 0;
	
	if (this != NULL)
	{
		double changeamount = num->box_muller(0.0, m_fWindowsize); //(rndu() - 0.5) * m_fWindowsize;
		
		double proposal = branch + changeamount;
		
		if ( proposal <= 0.0 )
			proposal = branch - changeamount;
		
		GetData().distance = proposal;
        
		// Going to accept or reject here...
		if ( (version == 0) ) {
			// 1) Get branch-specific likelihood
			double likelihood1 = BranchSpecificLikelihood( theModel, theData, m_vecSiteAssignment );
			
			//GetData().distance = proposal;
			
			// 2) Re-calc substitution probability on branch
			if (options.m_nMethod == 0) {
				SetBranchBLProb( this, theData.getNumStates(), *theModel.returnRateMatrix3D() ); 
			} else if ( (options.m_nMethod == 1) || (options.m_nMethod == 3) ) {
				SetBranchB1Prob( this, theData.getNumStates(), *theModel.returnRateMatrix3D() );
			} else if ( options.m_nMethod == 4) {
				SetBranchBinfProb( this, theData.getNumStates(), *theModel.returnRateMatrix3D() , &theModel.returnStateFrequencies() ); 
			} else if (options.m_nMethod == 6) {
				SetBranchB1uProb( this, theData.getNumStates(), *theModel.returnRateMatrix3D() , theModel.returnUniformizationConstants() ); 
			} else if (options.m_nMethod == 2) {
				SetBranchB2Prob( this, theData.getNumStates(), *theModel.returnRateMatrix3D() ); 
			} else {
				std::cout << "	** ERROR: unknown probablity calculation method." << endl;
				exit(-1);
			}
			
			double likelihood2 = BranchSpecificLikelihood( theModel, theData, m_vecSiteAssignment );
			
			if ( num->evaluateProposal_logL( likelihood2, likelihood1 ) == 0 ) {
                GetData().distance = branch;
			}
		} else {
			if ( (options.m_nMethod == 6) && (version == 1) ) {
				double rescaled;
				rescaled = 0.0;
				int fail = 1;
				
				// This seems to be causing big problems with likelihood heating...
				while (fail == 1) {
                    
					// Values of 1 and 1 here give a uniform prior on t/(mu - mu t)
					double betaPrior_alpha = 1.0;
					double betaPrior_beta  = 1.0;
					
					// Going to use a transformed mean of 0.005 prior on the actual branchlength parameter, with an equal variance
					double desiredMean     = 0.005;
					double transformedMean = (mu * desiredMean)  / (1.0 + mu * desiredMean);
                    //					double var             = transformedMean;
                    
					// Set the desired mean on the beta prior, approximate a reasonable variance (looks like its okay unless mu is large):
					// betaPrior_alpha = (pow(transformedMean, 2) - pow(transformedMean, 3) - transformedMean * pow(var,2)) / pow(var,2);
					// betaPrior_beta  = (transformedMean - 2*pow(transformedMean,2) + pow(transformedMean,3) - pow(var,2) + transformedMean * pow(var, 2)) / pow(var, 2);
                    
					betaPrior_beta  = 5.0;
					betaPrior_alpha = betaPrior_beta * transformedMean / ( 1.0 - transformedMean ) ;
					
					if ( (betaPrior_alpha < 0 ) || ( betaPrior_beta < 0 ) ) {
						betaPrior_alpha = betaPrior_beta = 1.0;
					}
					
					double variate  = num->betaVariate( ( ((double)GetData().C1 * options.__globalLikelihoodTemp) + betaPrior_alpha), ( ((double)GetData().C0 * options.__globalLikelihoodTemp) + betaPrior_beta ));
					rescaled = variate / ( mu * (1.0 - variate) );
					
					fail = 0;
					if ( rescaled <= 0 ) { fail = 1; }
                    //	if ( rescaled >= 10.0 ) { fail = 1; }
				}
				GetData().distance = rescaled;
				
			} else if ( (options.m_nMethod == 6) && (version == 3) ) {
				// MLE for bl...
				GetData().distance = (GetData().C1 * options.__globalLikelihoodTemp) / ( ( GetData().C0 * options.__globalLikelihoodTemp) * mu );
			} else if ( (options.m_nMethod == 6) && (version == 2) ) {
				if (branch != cInitDistance) {
					// Checking if this version is being used...something is messing up bl updates under likelihood heating
					double LR  = (GetData().C0) * num->logarithm( ( 1.0 + (mu * branch) ) / ( 1.0 + (mu * proposal) ) ) + (GetData().C1) * num->logarithm(  (proposal * (1.0 + mu * branch)) / (branch * (1.0 + mu * proposal)) );
					
					if ( num->evaluateProposal_logL( 1.0, (1.0 - LR) ) == 1 ) {
						GetData().distance = proposal;
					} else {
						GetData().distance = branch;
					}
					
				}
			}
		}
		
		// ENFORCE MAXIMUM BL UNDER CAP-NUM-NODES...
		if ( (GetData().distance > options.maxBL) && (options.capNumTreeNodes) ) GetData().distance = branch;
		
		// ADD UP BRANCHES FOR TOTAL TREE LENGTH...
		if (GetData().distance > 0.0) mRoot->GetData().treeLength += GetData().distance;
		
		if ( ( GetData().distance > options.maxBL ) ) {
			dirty += 1;
            //		subdivideLongBranches( this, options.maxBL, 0 );
            //		std::cout << "returned succesffuly from subdivLongBranches()" << endl;
		}
		
		if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
			dirty += Left()->KEEP_UpdateBranchLength(this, mTree, m_fWindowsize, theModel, theData, m_vecSiteAssignment, version);			
		if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
			dirty += Right()->KEEP_UpdateBranchLength(this, mTree, m_fWindowsize, theModel, theData, m_vecSiteAssignment, version);
		
		if ( (Up()!= NULL) ) {
			if ( (Up() != inFrom) && (Up()->Up() != this) ) {
				dirty += Up()->KEEP_UpdateBranchLength(this, mTree, m_fWindowsize, theModel, theData, m_vecSiteAssignment, version);
			}
		}
	}
	
	return dirty;
}


//---------------------------------------------------------------------------

inline void
TreeNode::PrintBranchLabel(
                           TreeNode* inFrom,
                           ofstream& outStream)
{
  	if (this != NULL)
	{
		if (Up()->Up() != this)  // not the virtual root node
		{
			//print branch length to outstream
			double branch = GetData().distance;
			
			if (Up()->Up()->Up() == Up()) // left and right node of virtual node
			{
				if (branch != cInitDistance)	// not the ancestral node 
				{
					//print the label from id/name to id/name
					int fromid;
					
					if (Up()->Left() == this)
						fromid = Up()->Right()->GetData().id;
					else
						fromid = Up()->Left()->GetData().id;
					
					outStream << setw(8) << " " << setw(0);
                    
					if ((Left() == NULL) && (Right() == NULL))	//tonode is a leaf node
						outStream << fromid << "to" << GetData().name;
					else
						outStream << fromid << "to" << GetData().id;
				}
			}
			else
			{
				//print the label from id/name to id/name
				int fromid;
				
				fromid = Up()->GetData().id;
                
				outStream << setw(8) << " " << setw(0);
                
				if ((Left() == NULL) && (Right() == NULL))	//tonode is a leaf node
					outStream << fromid << "to" << GetData().name;
				else
					outStream << fromid << "to" << GetData().id;
			}
		}
        
		if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
			Left()->PrintBranchLabel(this, outStream);
        
		if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
			Right()->PrintBranchLabel(this, outStream);
		
		if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
			Up()->PrintBranchLabel(this, outStream);
	}
}

//---------------------------------------------------------------------------

inline void
TreeNode::PrintBranchLabel(
                           ofstream& outStream)
{
	PrintBranchLabel(0, outStream);
	
	outStream << endl;
}


//---------------------------------------------------------------------------

inline void
TreeNode::PrintBranchChangecount(
                                 TreeNode* inFrom,
                                 ofstream& outStream)
{
  	if (this != NULL)
	{
		if (Up()->Up() != this)  // not the virtual root node
		{
			//print branch length to outstream
			double branch = GetData().distance;
			
			if (Up()->Up()->Up() == Up()) // left and right node of virtual node
			{
				if (branch != cInitDistance)	// not the ancestral node 
				{
					int count = changeinmatrix(*getChangeMatrix(this)); //GetData().countchange);
                    
					outStream << setw(20) << count << setw(0);	//print the branch change count
				}
			}
			else
			{
				if (branch != cInitDistance)	 
				{
					int count = changeinmatrix(*getChangeMatrix(this)); //GetData().countchange);
                    
					outStream << setw(20) << count << setw(0);	//print the branch change count
				}
			}
		}
        
		if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
			Left()->PrintBranchChangecount(this, outStream);
        
		if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
			Right()->PrintBranchChangecount(this, outStream);
		
		if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
			Up()->PrintBranchChangecount(this, outStream);
	}
}

//---------------------------------------------------------------------------

inline void
TreeNode::PrintBranchChangecount(
                                 ofstream& outStream)
{
	PrintBranchChangecount(0, outStream);
	
	outStream << endl;
}

//---------------------------------------------------------------------------

inline void
TreeNode::PrintBranchLength(
                            TreeNode* inFrom,
                            ofstream& outStream)
{
  	if (this != NULL)
	{
		if (Up()->Up() != this)  // not the virtual root node
		{
			//print branch length to outstream
			double branch = GetData().distance;
			
			if (Up()->Up()->Up() == Up()) // left and right node of virtual node
			{
				if (branch != cInitDistance)	// not the ancestral node 
					outStream << setw(20) << setprecision(6) << branch << setw(0) << setprecision(0);	//print the branch length
			}
			else
			{
				if (branch != cInitDistance)	 
					outStream << setw(20) << setprecision(6) << branch << setw(0) << setprecision(0);	//print the branch length
			}
		}
        
		if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
			Left()->PrintBranchLength(this, outStream);
        
		if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
			Right()->PrintBranchLength(this, outStream);
		
		if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
			Up()->PrintBranchLength(this, outStream);
	}
}

//---------------------------------------------------------------------------

inline void
TreeNode::PrintBranchLength(
                            ofstream& outStream)
{
	PrintBranchLength(0, outStream);
	
	outStream << endl;
}

//---------------------------------------------------------------------------

inline void
TreeNode::CountSiteChangecount(
                               TreeNode*  inFrom,
                               TreeNode*  inTo,
                               IntVector& SiteCount)
{
	vector<int>& sequence1 = inFrom->GetData().sequence;
    
	vector<int>& sequence2 = inTo->GetData().sequence;
	
	unsigned int len1 = sequence1.size();
	
	unsigned int len2 = sequence2.size();
	
	assert(len1 == len2);
	
	if (SiteCount.empty())
		SiteCount.resize(len1, 0);
    
	for (unsigned int i = 0; i < len1; i++)
	{
		if (sequence1[i] != sequence2[i])
			SiteCount[i]++;
	}
}

//---------------------------------------------------------------------------

inline void
TreeNode::CountTreeSiteChangecount(
                                   IntVector& SiteCount)
{
  	if (this != NULL)
	{
		if (Left() != NULL)
		{
			// not the assumed root node
			if ( Up()->Up() != this)
				CountSiteChangecount(this, Left(), SiteCount);
			else
				CountSiteChangecount(Right(), Left(), SiteCount);
            
			Left()->CountTreeSiteChangecount(SiteCount); 
		}
        
		if (Right() != NULL) 
		{ 
			if ( Up()->Up() != this)
				CountSiteChangecount(this, Right(), SiteCount);
			else
				CountSiteChangecount(Left(), Right(), SiteCount);
            
			Right()->CountTreeSiteChangecount(SiteCount);  
		}
	}
}

//---------------------------------------------------------------------------

inline void
TreeNode::PrintSiteChangecount(
                               ofstream& outStream)
{
	IntVector m_vecSiteCount;
	
	TreeNode* temp = this;
	
	while (temp->Up()->Up() != temp)
		temp = temp->Up();
	
	temp->CountTreeSiteChangecount(m_vecSiteCount);
    
	outStream << setw(8) <<"Site" << "\t" << setw(8) <<"Changes" << endl;
    
	for (unsigned int i = 0; i < m_vecSiteCount.size(); i++)
		outStream << setw(8) << i+1 << "\t" << setw(8) << m_vecSiteCount[i] << setw(0) << endl;
	
	outStream << endl;
}

//---------------------------------------------------------------------------

inline void
TreeNode::CopyTree(
                   TreeNode* inFrom,
                   TreeNode* inTarget)
{
  	if (this != NULL)
	{
		// set the data to target data
		inTarget->mData = mData;
        
		if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
		{
			// new a Left node
			TreeNode* left = new TreeNode;
			
			left->SetUp(inTarget);
			
			inTarget->SetLeft(left);
			
			Left()->CopyTree(this, left);
		}
        
		if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
		{	
			// new a right node
			TreeNode* right = new TreeNode;
			
			right->SetUp(inTarget);
			
			inTarget->SetRight(right);
			
			Right()->CopyTree(this, right);
		}
		
		if ((Up() != NULL) && (Up()->Up() == this))  //virtual root node
		{
			// new a up node
			TreeNode* up = new TreeNode;
			
			inTarget->SetUp(up);
			
			up->mData = Up()->mData;
			
			up->SetUp(inTarget);
		}
		else if ( (Up()!= NULL) && (Up() != inFrom) )
		{
			// new a up node
			TreeNode* up = new TreeNode;
			
			Up()->CopyTree(this, up);
		}
	}
}

//---------------------------------------------------------------------------

inline void
TreeNode::CopyTree(
                   TreeNode* inTarget)
{
	CopyTree(0, inTarget);
}

//---------------------------------------------------------------------------

inline void
TreeNode::CopyTreeData(
                       TreeNode* inFrom,
                       TreeNode* inTarget)
{
  	if (this != NULL)
	{
		// set the data to target data
		inTarget->mData = mData;
        
		if ((Left()!= NULL) && (Left() != inFrom) && ( Left() != Up()))
			Left()->CopyTreeData(this, inTarget->Left());
        
		if ( (Right()!= NULL) && (Right()!= inFrom)  && (Right() != Up()) )
			Right()->CopyTreeData(this, inTarget->Right());
		
		if ( (Up()!= NULL) && (Up() != inFrom) && (Up()->Up() != this) )
			Up()->CopyTreeData(this, inTarget->Up());
	}
}

//---------------------------------------------------------------------------

inline void
TreeNode::CopyTreeData(
                       TreeNode* inTarget)
{
	CopyTreeData(0, inTarget);
}



inline double
TreeNode::BranchSpecificLikelihood(MixtureModel& theModel,
                                   ObservedData& theData,
                                   IntVector& m_vecSiteAssignment) {
	// This calculates the 'branch-specific' likelihood without using any cached counts at all
	// (we iterate over all sites, accumulating the substitution probability of all events)
	
	double returnLikelihood = 0.0;
	
	vector<int> *seq1, *seq2;
	double distance;
	DoubleMatrix3D *mProb3D = getProbMatrix3D(this); //&GetData().probmatrix3D;
    
	seq1 = &Up()->GetData().sequence; 
	seq2 = &GetData().sequence; 
    
	distance = GetData().distance;
	for (int site=0; site<m_vecSiteAssignment.size(); site++) {
		int modelassignment = m_vecSiteAssignment.mDataArray[site];
		int rateassignment = mRoot->GetData().siteCats->getAssignment( site );
		
		if (seq1->size() && seq2->size() && distance != cInitDistance) {
			// get the changes of each branch on that site
			int indx1 = (*seq1)[site];
			int indx2 = (*seq2)[site];
            
			if ( options.m_nMethod != 6 ) {
				returnLikelihood += num->logarithm(((*mProb3D).mRows.mDataArray[num->returnModelIndex( modelassignment, rateassignment, GetData().branchLengthIndex )].mRows.mDataArray[indx1].mDataArray[indx2]));
			} else {
				returnLikelihood += num->logarithm(((*mProb3D).mRows.mDataArray[num->returnModelIndex( modelassignment, rateassignment, GetData().branchLengthIndex )].mRows.mDataArray[indx1].mDataArray[indx2]));
			}
		}
	}
	
	return returnLikelihood;
}




inline double
TreeNode::CalculateSiteSpecificLnLikelihood_ASRV(           int& site,
                                                 MixtureModel& theModel,
                                                 ObservedData& theData,
                                                 IntVector& m_vecSiteAssignment,
                                                 DoubleVector& LnLikelihoodVector)
{
	// This version iterates over branches and calculates the probability of the current substitution history
	// (this probably also therefore requires calculation of all branch-specific substitution prob matrices)
	
	double returnVal = 0;
	double temp = 0;
	int numCat = options.numCatASRV;
	int modelassignment = m_vecSiteAssignment.mDataArray[site];
	
	if (Up()->Up() == this)	//root node
	{
		if (LnLikelihoodVector.empty())
			LnLikelihoodVector.resize(numCat, 0);
		else
			LnLikelihoodVector.fill(0);
	}
	else 
	{
		vector<int> *seq1, *seq2;
		double distance;
		DoubleMatrix3D *mProb3D = getProbMatrix3D(this); //&GetData().probmatrix3D;
		
		seq1 = &Up()->GetData().sequence; 
		seq2 = &GetData().sequence; 
		
		distance = GetData().distance;
		
		if (seq1->size() && seq2->size() && distance != cInitDistance)
		{
			// get the changes of each branch on that site
			
			int indx1 = (*seq1)[site];
			int indx2 = (*seq2)[site];
			if (indx2 != indx1) temp++;
			
			// Calculate the log likelihood of this site at this branch
			for (unsigned int i = 0; i< numCat; i++) {
					if ( options.m_nMethod != 6 ) {
					if (mRoot->numDiscreteBLs == 0) { 
						LnLikelihoodVector[i] += num->logarithm(((*mProb3D).mRows.mDataArray[num->returnModelIndex( modelassignment, i, 0 )].mRows.mDataArray[indx1].mDataArray[indx2])); //
					} else {
						LnLikelihoodVector[i] += num->logarithm(((*mProb3D).mRows.mDataArray[num->returnModelIndex( modelassignment, i, GetData().branchLengthIndex )].mRows.mDataArray[indx1].mDataArray[indx2])); //
					}
				} else {
					if (mRoot->numDiscreteBLs == 0) { 
						LnLikelihoodVector[i] += num->logarithm(((*mProb3D).mRows.mDataArray[num->returnModelIndex( modelassignment, i, 0 )].mRows.mDataArray[indx1].mDataArray[indx2])); //
					} else {
						LnLikelihoodVector[i] += num->logarithm(((*mProb3D).mRows.mDataArray[num->returnModelIndex( modelassignment, i, GetData().branchLengthIndex )].mRows.mDataArray[indx1].mDataArray[indx2])); //
					}
				}
			}
		}
	}
	returnVal += temp;
	
	// If B1xM, we don't have to recurse this over the tree...
	//	if (options.m_nMethod == 3) return;
	
	if (Left() != NULL)
		returnVal += Left()->CalculateSiteSpecificLnLikelihood_ASRV( site, theModel, theData, m_vecSiteAssignment, LnLikelihoodVector ); 
	
	if (Right() != NULL) 
		returnVal += Right()->CalculateSiteSpecificLnLikelihood_ASRV( site, theModel, theData, m_vecSiteAssignment, LnLikelihoodVector );  
	
	return returnVal;
}




inline void
TreeNode::CalculateSiteSpecificLnLikelihoodUni_ASRV(
                                                    int& site,
                                                    MixtureModel& theModel,
                                                    ObservedData& theData,
                                                    IntVector& m_vecSiteAssignment,
                                                    DoubleVector& LnLikelihoodVector)
{
	// This function returns the site specific log-likelihood vector for each rate category (assuming ASRV)
	// The equations are formulated assuming rate-scaling (not branch scaling)
	
	int numStates = theData.getNumStates();
	int numModels = theModel.GetNumModels();
	
	int numCat = options.numCatASRV;
	int modelassignment = m_vecSiteAssignment.mDataArray[site];
	DoubleMatrix *changeCountsPerSite;
	double mu = theModel.returnUniformizationConstants()->mDataArray[modelassignment];
	
	if (this != mRoot) { std::cout << "Error! CalculateSiteSpecificLnLikelihoodUni_ASRV() called somewhere NOT at the root node" << endl; return; }
	
	vector<int> *seq1;
	DoubleMatrix3D *mProb3D = getProbMatrix3D(mRoot); //&GetData().probmatrix3D;
	
	seq1 = &Up()->GetData().sequence; 
	
	changeCountsPerSite = getChangeMatrix3D( mRoot, site );
	double totalXY = 0;
	for (unsigned int i=0; i<numStates; i++) {
		for (unsigned int j=0; j<numStates; j++) {
			if (j != i) totalXY += (*changeCountsPerSite).mRows.mDataArray[i].mDataArray[j];
		}
	}
	
	for (unsigned int i = 0; i< numCat; i++) {
		LnLikelihoodVector[i] = totalXY * num->logarithm( mRoot->GetData().siteCats->getRate( i ) );		
		
		for (unsigned int j=0; j<numStates; j++) {
			double tempArg = ( mu - ( mRoot->GetData().siteCats->getRate( i ) * (-(*mProb3D).mRows.mDataArray[num->returnModelIndex( modelassignment, i, GetData().branchLengthIndex )].mRows.mDataArray[j].mDataArray[j])) );
            
			if (tempArg <= 0) {
				std::cout << "Error: B1u site specific logL, tempArv <= 0" << endl;
				exit(-1);
				tempArg = 1e-70;
			}
			LnLikelihoodVector[i] += (*changeCountsPerSite).mRows.mDataArray[j].mDataArray[j] * num->logarithm( tempArg );
		}		
	}
}



inline double
TreeNode::CalculateSiteSpecificLnLikelihood(
                                            int& numModels,
                                            int& site,
                                            DoubleVector& LnLikelihoodVector,
                                            ObservedData& theData)
{
	// This version is integrated over the virtual substitutions
	
 	double returnVal = 0;
	double temp = 0;
	
 	if (Up()->Up() == this)	//root node
	{
		if (LnLikelihoodVector.empty())
			LnLikelihoodVector.resize(numModels, 0);
		else
			LnLikelihoodVector.fill(0);
	}
	else 
	{
		vector<int> *seq1, *seq2;
		double distance;
		DoubleMatrix3D *mProb3D = getProbMatrix3D(this); //&GetData().probmatrix3D;
        
		seq1 = &Up()->GetData().sequence; 
		seq2 = &GetData().sequence; 
		
		distance = GetData().distance;
		
		if (seq1->size() && seq2->size() && distance != cInitDistance)
		{
			// get the changes of each branch on that site
            
			int indx1 = (*seq1)[site];
			int indx2 = (*seq2)[site];
			if (indx2 != indx1) temp++;
			
			// Calculate the log likelihood of this site at this branch
			for (unsigned int i = 0; i< numModels; i++) {
				//if (options.m_nMethod != 3 ) 
				if ( options.m_nMethod != 6 ) {
					if (mRoot->numDiscreteBLs == 0) { 
						LnLikelihoodVector[i] += num->logarithm(((*mProb3D).mRows.mDataArray[i].mRows.mDataArray[indx1].mDataArray[indx2])); //
					} else {
						LnLikelihoodVector[i] += num->logarithm(((*mProb3D).mRows.mDataArray[(GetData().branchLengthIndex * numModels)+i].mRows.mDataArray[indx1].mDataArray[indx2])); //
					}
				} else {
  					if (mRoot->numDiscreteBLs == 0) { 
						LnLikelihoodVector[i] += num->logarithm(((*mProb3D).mRows.mDataArray[i].mRows.mDataArray[indx1].mDataArray[indx2])); //
					} else {
						LnLikelihoodVector[i] += num->logarithm(((*mProb3D).mRows.mDataArray[(GetData().branchLengthIndex * numModels)+i].mRows.mDataArray[indx1].mDataArray[indx2])); //
					}
				}
			}
		}
	}
	returnVal += temp;
    
	// If B1xM, we don't have to recurse this over the tree...
	//	if (options.m_nMethod == 3) return;
	
	if (Left() != NULL)
		returnVal += Left()->CalculateSiteSpecificLnLikelihood( numModels, site, LnLikelihoodVector, theData ); 
    
	if (Right() != NULL) 
		returnVal += Right()->CalculateSiteSpecificLnLikelihood( numModels, site, LnLikelihoodVector, theData );  
    
	return returnVal;
}


inline double
TreeNode::CalculateSiteSpecificLnLikelihood_B1u(
												MixtureModel& mModel,
                                                int numModels,
                                                int site,
                                                DoubleVector* LnLikelihoodVector,
                                                ObservedData& theData ) {

	// THIS FUNCTION PROBABLY ONLY OKAY FOR ASRV.  NOT FOR MIXTURES (THE VIRTUAL SUBSTITUTION PROBABILITIES DEPEND ON MODEL)
	
	double returnVal = 0;
	
	if (LnLikelihoodVector->empty())
		LnLikelihoodVector->resize(numModels, 0);
	else
		LnLikelihoodVector->fill(0);
	
	double logLikelihood = 0;
	for (int theModel=0; theModel < numModels; theModel++) {
		returnVal = mRoot->CalcLogLikelihood_branchB1uni_bySite( mModel, &logLikelihood, theModel, site);
        //	std::cout << "model " << theModel << ", lik is " << logLikelihood << endl;
		(*LnLikelihoodVector)[theModel] = logLikelihood;
	}
	
	return returnVal;
}



inline void
TreeNode::CalculateSiteSpecificLikelihood(
                                          int& numModels,
                                          int& site,
                                          DoubleVector& LnLikelihoodVector,
                                          IntVector& inStateindex,
                                          int& minstate,
                                          int& numStates)
{
	if (Up()->Up() == this)	//root node
	{
		if (LnLikelihoodVector.empty())
			LnLikelihoodVector.resize(numModels, 1.0);
		else
			LnLikelihoodVector.fill(1.0);
	}
	else 
	{
		vector<int> *seq1, *seq2;
		double distance;

		DoubleMatrix3D *mProb3D = getProbMatrix3D(this); //&GetData().probmatrix3D;
        seq1 = &Up()->GetData().sequence; 
		seq2 = &GetData().sequence; 
		distance = GetData().distance;
		
		if (seq1->size() && seq2->size() && distance != cInitDistance)
		{
			// get the changes of each branch on that site
            
			int indx1 = (*seq1)[site];
			int indx2 = (*seq2)[site];
			
			// calculate the log likelihood of this site at this branch
			for (unsigned int i = 0; i< numModels; i++) {
				if (mRoot->numDiscreteBLs == 0) { 
					LnLikelihoodVector[i] *= (*mProb3D)[i][indx1][indx2];
				} else {
					LnLikelihoodVector[i] *= (*mProb3D)[(GetData().branchLengthIndex * numModels)+i][indx1][indx2];
				}
                
 			}
		}
	}
	
	if (Left() != NULL)
		Left()->CalculateSiteSpecificLikelihood(numModels, site, LnLikelihoodVector, inStateindex, minstate, numStates); 
    
	if (Right() != NULL) 
		Right()->CalculateSiteSpecificLikelihood(numModels, site, LnLikelihoodVector, inStateindex, minstate, numStates);  
}

// --- Uniformization method: counting of expected substitutions here ---------

inline
void
TreeNode::SetBranchModelExpectedSubsUni(
										TreeNode*	node1,
										TreeNode* node2,
										MixtureModel& theModel,
										ObservedData& theData,
										IntVector& m_vecSiteAssignment,
										double *siteLiklihoods)
{
	// WE SHOULD BE ABLE TO SIMPLIFY THIS SUBSTANTIALLY
	std::cout << "countExpectedSubstitutions() not fully implemented!  ASRV not accounted for.  Other stuff in progress." << endl; exit(-1);
	
	if (options.countExpectedSubstitutions != 1) { std::cout << "ERROR: called countExpectedSubstitutions (Uni) inappropriately.  Not counting expected subs..." << endl; exit(-1); }
	int numStates = theData.getNumStates();
	int numModels = theModel.GetNumModels();
	
	// Adding counts per site...
	DoubleMatrix	   *changeCountsPerSite;
	
	DoubleMatrix      *augmentedCount;
	DoubleMatrix   *augmentedLogSum;
	DoubleMatrix3D *ChangeCount3D;
	DoubleMatrix3D    *realChangeCount3D;
	DoubleMatrix3D *ChangeCount3D_scaling;
	DoubleMatrix *augmentedLogSum_scaling;
	
	double mu = 0.0;
	double logScalingFactor = log(1000);
	
	// the 'ChangeCount3D' matrix will hold different stuff for (x,y) and (x,x):
	// - for (x,y | x != y): SUM log Txy = SUM log ( t / (1 + mu x t) )
	// - for (x,x):                  Txx = SUM t
	
	// [these should both be for mRoot in this case (not node2)...]
	//	std::cout << "GETTING CHANGE MATRICES AT THE ROOT..." << endl;
	ChangeCount3D			= getChangeMatrix3Duni( mRoot ); //&node2->GetData().countModelchange3D; 
	realChangeCount3D		= getChangeMatrix3D( mRoot );
	augmentedCount			= getAugmentedCountMatrix( mRoot );
	augmentedLogSum			= getAugmentedLogSum( mRoot );
	DoubleMatrix3D* mProb3D = getProbMatrix3D(mRoot);
	
	if ( ( options.doB1uStar == 0 ) ) {
		augmentedLogSum_scaling	= getAugmentedLogSum_scaling( mRoot );
		ChangeCount3D_scaling	= getChangeMatrix3D_scaling( mRoot ); //&node2->GetData().countModelchange3D; 
	}
	
	// INITIALIZE MATRICES TO 0.0...
	if (ChangeCount3D->empty()) {
		// Accounting for discrete bl variation...
		if (mRoot->numDiscreteBLs != 0) { 
			if ( (options.m_nMethod != 6) || (options.doB1uStar == 1) ) {
				ChangeCount3D->resize((mRoot->numDiscreteBLs * numModels), numStates, numStates, 0.0);
			} else {
				ChangeCount3D->resize((mRoot->numDiscreteBLs * numModels), numStates, numStates, 1.0);
			}
		} else {
			if ( (options.m_nMethod != 6) || (options.doB1uStar == 1) ) {
				ChangeCount3D->resize(numModels, numStates, numStates, 0.0);
			} else {
				ChangeCount3D->resize(numModels, numStates, numStates, 1.0);
			}
		}
	}
	else {
		if ( (options.m_nMethod != 3) && (options.m_nMethod != 5) && (options.m_nMethod != 6) ) ChangeCount3D->fill(0.0);
	}
	
	// Aug count mat
	if (augmentedCount->empty()) {
		augmentedCount->resize( numModels, numStates, 0 );
	}
	if (augmentedLogSum->empty()) {
		if ( (options.m_nMethod != 6) || (options.doB1uStar == 1) ) {
			augmentedLogSum->resize( numModels, numStates, 0 );
		} else {
			augmentedLogSum->resize( numModels, numStates, 1.0 );
		}
	}
	
	
	if ( (options.m_nMethod == 6) && (options.doB1uStar == 0) ) {
		if (augmentedLogSum_scaling->empty()) {
			augmentedLogSum_scaling->resize( numModels, numStates, 0 );
		}
		if (ChangeCount3D_scaling->empty()) {
			ChangeCount3D_scaling->resize(numModels, numStates, numStates, 0.0);
		}
	}
	
	// INITIALIZE REAL CHANGE COUNT MATRIX...
	if (realChangeCount3D->empty()) {
		// Accounting for discrete bl variation...
		if (mRoot->numDiscreteBLs != 0) { 
			realChangeCount3D->resize((mRoot->numDiscreteBLs * numModels), numStates, numStates, 0);
		} else {
			realChangeCount3D->resize(numModels, numStates, numStates, 0);
		}
	}
	else {
		if ( (options.m_nMethod != 3) && (options.m_nMethod != 5) && (options.m_nMethod != 6) ) realChangeCount3D->fill(0);
	}
	
	
	// IF WE HAVE A BRANCH...
	if (node1 != NULL && node2 != NULL)
	{
		// Set sequence pointers...
		vector<int>& seq1 = node1->GetData().sequence; 
		vector<int>& seq2 = node2->GetData().sequence; 
		
		// Get branch length...
		double& distance = node2->GetData().distance;
		
		// If everything looks okay... [need to check the use of cInitDistance at the root]
		if (seq1.size() && seq2.size() && distance != cInitDistance)
		{
			int	len = min(seq1.size(), seq2.size());
			int modelassignment, indx1, indx2;
			int incr = 1;
			if (numStates == 64) incr = 3;
			
			int blInit, blLimit;
			
			// should this be node 1 or 2?
			unsigned int k2 = node1->GetData().branchLengthIndex;
			if (mRoot->numDiscreteBLs == 0) k2=0; 
			
			double isAugmented;
			for (int s = 0; s < len; s += incr) {
                // Get sequence states at site 's'...
                indx1 = seq1[s];
                indx2 = seq2[s];
				
				// Get mixture model assignment at site 's'...
				modelassignment = m_vecSiteAssignment.mDataArray[s];
				
				changeCountsPerSite = getChangeMatrix3D( mRoot, s );

				if (changeCountsPerSite->empty()) {
                    changeCountsPerSite->resize(numStates, numStates, 0.0);
				} else { 
					if (this == mRoot) {
						changeCountsPerSite->fill(0.0);
					}
				}
				
                
				// Get uniformization constant for model 'modelassignment...
				mu = theModel.returnUniformizationConstants()->mDataArray[modelassignment];
				
				int i,j;
				// [FOR EXPECTED SUBS]
				if (options.countExpectedSubstitutions == 1) {

					long double posteriorProbState_i = 0.0;
					long double totalSum = 0.0;
					for (i=0; i<numStates; i++) {
						posteriorProbState_i = (node1->GetData().siteLikelihoodBelow[(s*numStates) + i] * node1->GetData().siteLikelihoodAbove[(s*numStates) + i]); // / log(siteLiklihoods[s]);
						totalSum += posteriorProbState_i;
						double normalizingConstant_i = 0.0;

						// Pre-calculate the normalizing constantO(N^2)
						for (j=0; j<numStates; j++) {
							normalizingConstant_i += (*mProb3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[i].mDataArray[j] * node2->GetData().siteLikelihoodBelow[(s*numStates) + j];
						}
 
						// Calculate the substitution probabilitiesO(N^2)
						for (j=0; j<numStates; j++) {
							(*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += posteriorProbState_i * (*mProb3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[i].mDataArray[j] * node2->GetData().siteLikelihoodBelow[(s*numStates) + j] / normalizingConstant_i;
						}
						if (s == 100) std::cout << "site " << s << ", node is " << node1->GetData().id << "; updown prod " << (node1->GetData().siteLikelihoodBelow[(s*numStates) + i] * node1->GetData().siteLikelihoodAbove[(s*numStates) + i]) << "; log prod is " << log(node1->GetData().siteLikelihoodBelow[(s*numStates) + i] * node1->GetData().siteLikelihoodAbove[(s*numStates) + i]) << "; upLik " << log(node1->GetData().siteLikelihoodAbove[(s*numStates) + i]) << ", downLik " << log(node1->GetData().siteLikelihoodBelow[(s*numStates) + i]) << "; sitelik is " <<  log(siteLiklihoods[s]) << endl;
                        
					} // for i
					if (s == 100) std::cout << "tempSum = " << log(totalSum) << endl;
                    //					if (s == 100) std::cout << "Total upLik at site " << s << ", node is " << log(totalSum) << "; sitelik is " <<  log(siteLiklihoods[s]) << endl;
				}
				
				// If sequence states are valid...
				if ( (indx1 < numStates) && (indx2 < numStates)) {
					if (indx1 != indx2) {
						// Scaled product
						if (options.doB1uStar == 0) { 
							(*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] *= ( distance / (1.0 + mu * distance) );
							if ((*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] <= 0.0000000001) { (*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] *= 1000; (*ChangeCount3D_scaling).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += logScalingFactor; }
						} else {
							// Sum log of the function...
							(*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += num->logarithm( ( distance / (1.0 + mu * distance) ) );
						}
						// Increment the substitution count...
						(*realChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += 1;
						if (options.m_nMethod == 6) (*changeCountsPerSite).mRows.mDataArray[indx1].mDataArray[indx2] += 1.0;
					} else {
						if (options.doB1uStar == 1) {
							// Sum the time...[for approximate method]
							(*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += distance;
							
							// Increment the substitution count...
							(*realChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += 1;
						} else {
							// This is the probability of truly zero changes...
							isAugmented = 1.0 / (1.0 + (mu * distance) + (theModel.returnRateMatrix3D()->mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] * distance));
							
							// Decide if we're augmented or not...
							if (num->Random() > isAugmented) {
								// Yes...
								// Sum log of the function...
								//(*augmentedLogSum).mRows.mDataArray[(k2*numModels) + modelassignment].mDataArray[indx1] += num->logarithm( ( distance / (1.0 + mu * distance) ) );
								// Scaled product...
								(*augmentedLogSum).mRows.mDataArray[(k2*numModels) + modelassignment].mDataArray[indx1] *= ( distance / (1.0 + mu * distance) );
								if ((*augmentedLogSum).mRows.mDataArray[(k2*numModels) + modelassignment].mDataArray[indx1] <= 0.0000000001) { (*augmentedLogSum).mRows.mDataArray[(k2*numModels) + modelassignment].mDataArray[indx1] *= 1000; (*augmentedLogSum_scaling).mRows.mDataArray[(k2*numModels) + modelassignment].mDataArray[indx1] += logScalingFactor; }
								
								(*augmentedCount).mRows.mDataArray[(k2*numModels) + modelassignment].mDataArray[indx1] += 1;
								
								// Let's keep track of these on a site by site basis...
								if (options.m_nMethod == 6) (*changeCountsPerSite).mRows.mDataArray[indx1].mDataArray[indx1] += 1.0;
								
							} else {
								// No...(really no substitution)
	
								// Scaled product...
								(*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] *= ( 1.0 / (1.0 + mu * distance) );
								if ((*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] <= 0.0000000001) { (*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] *= 1000; (*ChangeCount3D_scaling).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += logScalingFactor; }
								
								// Increment the substitution count...
								(*realChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += 1;
								
								// When nothing happens, we have nothing to keep track of on a site by site basis.
							}
						}
					}
				} 
			}
			
			// [ should move a bunch of stuff out here - add up for every site, the number of subs and do the update once ]
			
			
		}
	}
} // [end of expected sub counting]


// --- Uniformization method: summing of substitution components here ---------

inline
int
TreeNode::_CalculateDwellTimeBranch(int s,
                                    double siteRate,
                                    TreeNode*	node1,
                                    TreeNode* node2,
                                    MixtureModel& theModel,
                                    ObservedData& theData,
                                    IntVector& m_vecSiteAssignment)
{
	// WE SHOULD BE ABLE TO SIMPLIFY THIS SUBSTANTIALLY

	int numSubs = 0;
	int numStates = theData.getNumStates();
	
	DoubleMatrix* marginalDwell = getMarginalDwell( node1 );
	
	if (node1 != NULL && node2 != NULL)
	{
		// Set sequence pointers...
		vector<int>& seq1 = node1->GetData().sequence; 
		vector<int>& seq2 = node2->GetData().sequence; 
		
		// Get branch length...
		double& distance = node2->GetData().distance;
		
		// If everything looks okay... [need to check the use of cInitDistance at the root]
		if (seq1.size() && seq2.size() && distance != cInitDistance)
		{
			int modelassignment, indx1, indx2;
			int incr = 1;
			if (numStates == 64) incr = 3;
			
			// should this be node 1 or 2?
			unsigned int k2 = node1->GetData().branchLengthIndex;
			if (mRoot->numDiscreteBLs == 0) k2=0; 
			
			// This stuff actually needs to be calculated only once per recursion over the tree; doesn't matter if it is post- or pre-order...
            
            // Get sequence states at site 's'...
            indx1 = seq1[s];
            indx2 = seq2[s];
            
			if (indx1 != indx2) numSubs++;
			
			// Get mixture model assignment at site 's'...
			modelassignment = m_vecSiteAssignment.mDataArray[s];
		
			// If sequence states are valid...
			if ( (indx1 < numStates) && (indx2 < numStates)) {
				marginalDwell->mRows.mDataArray[ 0 ].mDataArray[ indx1 ] += _calculateMarginalDwell( siteRate, indx1, indx2, indx1, modelassignment, *theModel.returnRateMatrix3D(), distance );
				marginalDwell->mRows.mDataArray[ 0 ].mDataArray[ indx2 ] += _calculateMarginalDwell( siteRate, indx1, indx2, indx2, modelassignment, *theModel.returnRateMatrix3D(), distance );
                
				// Need this to get the site's depth (sum of branchlengths)...
				mRoot->GetData().tempTreeDepth[ modelassignment ] += distance;
			}
			
			
			// Update observed frequency at the tips...
			if ( (node2->Left() == NULL) && (node2->Right() == NULL) && node2->Up() != NULL) {
				mRoot->GetData().observedFreqs.mRows.mDataArray[0].mDataArray[indx2]++;
			}
			
		}
	}
    
	return numSubs;
}



inline
void
TreeNode::_CalculateJointDwellTimeBranch(int s1, int s2,
                                         double siteRate1, double siteRate2,
                                         TreeNode*	node1,
                                         TreeNode* node2,
                                         MixtureModel& theModel,
                                         ObservedData& theData,
                                         IntVector& m_vecSiteAssignment)
{
	// WE SHOULD BE ABLE TO SIMPLIFY THIS SUBSTANTIALLY

	int numStates = theData.getNumStates();
	
	DoubleMatrix* jointDwell = getJointDwell( (TreeNode*)node1 );
	
	if (node1 != NULL && node2 != NULL)
	{
		// Set sequence pointers...
		vector<int>& seq1 = node1->GetData().sequence; 
		vector<int>& seq2 = node2->GetData().sequence; 
		
		// Get branch length...
		double& distance = node2->GetData().distance;
		
		// If everything looks okay... [need to check the use of cInitDistance at the root]
		if (seq1.size() && seq2.size() && distance != cInitDistance)
		{
            //	int	len = min(seq1.size(), seq2.size());
			int modelassignment1, modelassignment2, indx1, indx2, indx3, indx4;
			int incr = 1;
			if (numStates == 64) incr = 3;
			
            //			int blInit, blLimit;
			
			// should this be node 1 or 2?
			unsigned int k2 = node1->GetData().branchLengthIndex;
			if (mRoot->numDiscreteBLs == 0) k2=0; 
			
			// This stuff actually needs to be calculated only once per recursion over the tree; doesn't matter if it is post- or pre-order...
			
			// This part for codon models not yet implemented.  PRESENTLY OBSOLETE.
			if (numStates == 64) {
				std::cout << "ERROR: codon models not implemented in coevolution code yet." << endl;
				exit(-1);
  			} else {
				// Get sequence states at site 's'...
				indx1 = seq1[s1];
				indx2 = seq2[s1];
				indx3 = seq1[s2];
				indx4 = seq2[s2];
			}
			
			// Get mixture model assignment at site 's'...
			modelassignment1 = m_vecSiteAssignment.mDataArray[s1];
			modelassignment2 = m_vecSiteAssignment.mDataArray[s2];
			
			//			changeCountsPerSite = getChangeMatrix3D( mRoot, s );
			double total = 0.0;
			double temp = 0.0;
            
			// If sequence states are valid...
			if ( (indx1 < numStates) && (indx2 < numStates) && (indx3 < numStates) && (indx4 < numStates)) {
				
				int state1, state2;
				int theIndex;
				
				total = 0.0;
				temp = 0.0;
				// There are 4 non-zero cases: each state has to be in the set of {x,y}
                
				// xi && xj
				state1 = indx1; state2 = indx3;
				if (state2 > state1) {
					theIndex = (state1 * numStates) + state2;
				} else {
					theIndex = (state2 * numStates) + state1;
				}
				theIndex = (state1 * numStates) + state2;
				temp = _calculateJointDwellTime( siteRate1, siteRate2, indx1, indx3, indx2, indx4, state1, state2, modelassignment1, modelassignment2, theModel, distance  );
				jointDwell->mRows.mDataArray[ 0 ].mDataArray[ theIndex ] += temp ;
				total += temp;
                //	std::cout << "	Added " << temp << endl;
				
				if ( ( indx2 != indx1 ) ) {
					// yi && yj
					state1 = indx2; state2 = indx3;
					if (state2 > state1) {
						theIndex = (state1 * numStates) + state2;
					} else {
						theIndex = (state2 * numStates) + state1;
					}
					theIndex = (state1 * numStates) + state2;
					temp = _calculateJointDwellTime( siteRate1, siteRate2, indx1, indx3, indx2, indx4, state1, state2, modelassignment1, modelassignment2, theModel, distance  );
					jointDwell->mRows.mDataArray[ 0 ].mDataArray[ theIndex ] += temp ;
					total += temp;
                    //		std::cout << "	Added " << temp << endl;
                    
					// xi && yj
					if ( indx4 != indx3 ) {
						state1 = indx2; state2 = indx4;
						if (state2 > state1) {
							theIndex = (state1 * numStates) + state2;
						} else {
							theIndex = (state2 * numStates) + state1;
						}
						theIndex = (state1 * numStates) + state2;
						temp = _calculateJointDwellTime( siteRate1, siteRate2, indx1, indx3, indx2, indx4, state1, state2, modelassignment1, modelassignment2, theModel, distance  );
						jointDwell->mRows.mDataArray[ 0 ].mDataArray[ theIndex ] += temp ; 
						total += temp;
                        //			std::cout << "	Added " << temp << endl;
					}
				}
				
				if ( ( indx4 != indx3 ) ) {
					// yi && xj
					state1 = indx1; state2 = indx4;
					if (state2 > state1) {
						theIndex = (state1 * numStates) + state2;
					} else {
						theIndex = (state2 * numStates) + state1;
					}
					theIndex = (state1 * numStates) + state2;
					temp = _calculateJointDwellTime( siteRate1, siteRate2, indx1, indx3, indx2, indx4, state1, state2, modelassignment1, modelassignment2, theModel, distance  );
					jointDwell->mRows.mDataArray[ 0 ].mDataArray[ theIndex ] += temp ;
					total += temp;
                    //			std::cout << "	Added " << temp << endl;
				}
                
 			}
		}
	}
}


// Original version: augment the subs
// --- Uniformization method: summing of substitution components here ---------

inline
void
TreeNode::_SetChangesAlongBranchUni(
                                    TreeNode*	node1,
                                    TreeNode* node2,
                                    MixtureModel& theModel,
                                    ObservedData& theData,
                                    IntVector& m_vecSiteAssignment)
{
	// This function has been modified to allow two modes for ASRV: rate scaling or branch-length scaling
	// The reason for this is so that we can simply update the rate matrix (under branch-length scaling)
	
	// WE SHOULD BE ABLE TO SIMPLIFY THIS SUBSTANTIALLY
	
	// Adding counts per site...
	DoubleMatrix3D &inRateMatrix3D = *theModel.returnRateMatrix3D();
	DoubleMatrix   *changeCountsPerSite;
	
	DoubleMatrix      *augmentedCount;
	IntVector	   *totalCount_actualSubs;
	DoubleMatrix   *augmentedLogSum;
	DoubleMatrix3D *ChangeCount3D;
	DoubleMatrix3D    *realChangeCount3D;
	DoubleMatrix3D *ChangeCount3D_scaling;
	DoubleMatrix *augmentedLogSum_scaling;
    
	double mu = 0.0;
	double logScalingFactor = log(1000);
	int numStates = theData.getNumStates();
	
	// the 'ChangeCount3D' matrix will hold different stuff for (x,y) and (x,x):
	// - for (x,y | x != y): SUM log Txy = SUM log ( t / (1 + mu x t) )
	// - for (x,x):                  Txx = SUM t
	
	ChangeCount3D			= getChangeMatrix3Duni( mRoot ); //&node2->GetData().countModelchange3D;
	realChangeCount3D		= getChangeMatrix3D( mRoot );
	augmentedCount			= getAugmentedCountMatrix( mRoot );
	totalCount_actualSubs   = getTotalSubCountVector( mRoot );
	augmentedLogSum			= getAugmentedLogSum( mRoot );
    
	if ( ( options.doB1uStar == 0 ) ) {
		augmentedLogSum_scaling	= getAugmentedLogSum_scaling( mRoot );
		ChangeCount3D_scaling	= getChangeMatrix3D_scaling( mRoot ); //&node2->GetData().countModelchange3D;
	}
	
	// INITIALIZE MATRICES TO 0.0...
	int totalNumModels = theModel.GetNumModels();
	if ( options.numDiscreteBLs > 1) totalNumModels *= options.numDiscreteBLs;
	if (ChangeCount3D->empty()) {
        if ( (options.m_nMethod != 6) || (options.doB1uStar == 1) ) {
            ChangeCount3D->resize(totalNumModels, numStates, numStates, 0.0);
        } else {
            ChangeCount3D->resize(totalNumModels, numStates, numStates, 1.0);
        }
	}
	else {
		if ( (options.m_nMethod != 3) && (options.m_nMethod != 5) && (options.m_nMethod != 6) ) ChangeCount3D->fill(0.0);
	}
    
	// The augmentedCount has to be allocated for every ASRV class...
	int totalAugClasses = theModel.GetNumModels();
	if ( options.numCatASRV > 1) totalAugClasses *= options.numCatASRV;
	if ( options.numDiscreteBLs > 1) totalAugClasses *= options.numDiscreteBLs;
	
	if (augmentedCount->empty()) {
		augmentedCount->resize( totalAugClasses, numStates, 0 );
	}
	
	int numModels = theModel.GetNumModels();
	if (augmentedLogSum->empty()) {
		if ( (options.m_nMethod != 6) || (options.doB1uStar == 1) ) {
			augmentedLogSum->resize( numModels, numStates, 0 );
		} else {
			augmentedLogSum->resize( numModels, numStates, 1.0 );
		}
	}
    
	// for summing all x->y subs in a rate class...
	totalAugClasses = 1;
	if ( options.numCatASRV > 1) totalAugClasses *= options.numCatASRV;
	
	if (totalCount_actualSubs->empty()) {
		totalCount_actualSubs->resize( totalAugClasses, 0);
	}
    
	if ( (options.m_nMethod == 6) && (options.doB1uStar == 0) ) {
		if (augmentedLogSum_scaling->empty()) {
			augmentedLogSum_scaling->resize( numModels, numStates, 0 );
		}
		if (ChangeCount3D_scaling->empty()) {
			ChangeCount3D_scaling->resize(numModels, numStates, numStates, 0.0);
		}
	}
    
	// INITIALIZE REAL CHANGE COUNT MATRIX...
	if (realChangeCount3D->empty()) {
		// Accounting for discrete bl variation...
		if (mRoot->numDiscreteBLs != 0) { 
            //	std::cout << "dimming it for " << (mRoot->numDiscreteBLs * numModels) << endl;
			realChangeCount3D->resize((mRoot->numDiscreteBLs * numModels), numStates, numStates, 0);
		} else {
			realChangeCount3D->resize(numModels, numStates, numStates, 0);
		}
	}
	else {
		if ( (options.m_nMethod != 3) && (options.m_nMethod != 5) && (options.m_nMethod != 6) ) realChangeCount3D->fill(0);
	}
    
    
	// IF WE HAVE A BRANCH...
	if (node1 != NULL && node2 != NULL)
	{
		// Set sequence pointers...
		vector<int>& seq1 = node1->GetData().sequence; 
		vector<int>& seq2 = node2->GetData().sequence; 
		
		// Get branch length...
		double& distance = node2->GetData().distance;
		
		// If everything looks okay... [need to check the use of cInitDistance at the root]
		if (seq1.size() && seq2.size() && distance != cInitDistance)
		{
			int	len = min(seq1.size(), seq2.size());
			int modelassignment, rateClassAssignment, indx1, indx2;
			int incr = 1;
			if (numStates == 64) incr = 3;
            
  			// should this be node 1 or 2?
			unsigned int k2 = node1->GetData().branchLengthIndex;
			if (mRoot->numDiscreteBLs == 0) k2=0; 
            
			// This stuff actually needs to be calculated only once per recursion over the tree; doesn't matter if it is post- or pre-order...
			
			// Number of branch changes...
			node2->GetData().C0 = 0;
			node2->GetData().C1 = 0;
			
			double isAugmented;
			for (int s = 0; s < len; s += incr) {
                // Get sequence states at site 's'...
                indx1 = seq1[s];
                indx2 = seq2[s];
                
				// Get mixture model assignment at site 's'...
				modelassignment			= m_vecSiteAssignment.mDataArray[s];
				rateClassAssignment		= mRoot->GetData().siteCats->getAssignment( s );
				
				changeCountsPerSite = getChangeMatrix3D( mRoot, s );
				
				if (changeCountsPerSite->empty()) {
					changeCountsPerSite->resize(numStates, numStates, 0.0);
				} else { 
					if (this == mRoot) {
						changeCountsPerSite->fill(0.0);
					}
				}
				
				// Get uniformization constant for model 'modelassignment...
				mu = theModel.getUniformization(modelassignment);
				
				// If sequence states are valid...
				if ( (indx1 < numStates) && (indx2 < numStates)) {
					
					// Test the marginal dwell time stuff...
					// int theState = 0;
					// double dwellTest = _calculateMarginalDwell( indx1, indx2, theState, modelassignment, inRateMatrix3D, distance );
					// std::cout << "Site " << s << ", dwell in 0: " << dwellTest << endl;
					
					if (indx1 != indx2) {
						// Scaled product
						if (options.doB1uStar == 0) { 
							(*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] *= ( distance / (1.0 + mu * distance) );
							if ((*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] <= 0.0000000001) { (*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] *= 1000; (*ChangeCount3D_scaling).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += logScalingFactor; }
						} else {
							// Sum log of the function...
							(*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += num->logarithm( ( distance / (1.0 + mu * distance) ) );
						}
						// Increment the substitution count...
						(*realChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += 1;
						if (options.m_nMethod == 6) {
							(*totalCount_actualSubs).mDataArray[ rateClassAssignment ] += 1;
							(*changeCountsPerSite).mRows.mDataArray[indx1].mDataArray[indx2] += 1.0;
							node2->GetData().C1 += 1;
						}
					} else {
						if (options.doB1uStar == 1) {
							// Sum the time...[for approximate method]
							(*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += distance;
                            
							// Increment the substitution count...
							(*realChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += 1;
						} else {
							// This is the probability of truly zero changes...
							double rateMod = 1.0;
							if (options.allowASRV && (options.numCatASRV > 1) ) {
								rateMod = mRoot->GetData().siteCats->getRate( rateClassAssignment );
							}
							
							options.__ASRVmode = 1;
							
							if (options.__ASRVmode == 1) {
								// Assuming rate scaling...
								isAugmented = 1.0 / (1.0 + (mu * distance) + (inRateMatrix3D.mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] * rateMod * distance));
							} else {
								// Assuming branchlength scaling...
								isAugmented = 1.0 / (1.0 + (mu * rateMod * distance) + (inRateMatrix3D.mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] * rateMod * distance));
							}
							
    							// Decide if we're augmented or not...
							if (num->Random() > isAugmented) {
								// Yes...
								// Sum log of the function...
								//(*augmentedLogSum).mRows.mDataArray[(k2*numModels) + modelassignment].mDataArray[indx1] += num->logarithm( ( distance / (1.0 + mu * distance) ) );
								// Scaled product...
								(*augmentedLogSum).mRows.mDataArray[(k2*numModels) + modelassignment].mDataArray[indx1] *= ( distance / (1.0 + mu * distance) );
								if ((*augmentedLogSum).mRows.mDataArray[(k2*numModels) + modelassignment].mDataArray[indx1] <= 0.0000000001) { (*augmentedLogSum).mRows.mDataArray[(k2*numModels) + modelassignment].mDataArray[indx1] *= 1000; (*augmentedLogSum_scaling).mRows.mDataArray[(k2*numModels) + modelassignment].mDataArray[indx1] += logScalingFactor; }
								
								// Indexing based on rate class assignment etc.
								(*augmentedCount).mRows.mDataArray[ num->returnModelIndex( modelassignment, rateClassAssignment, k2) ].mDataArray[indx1] += 1;
                                
								// Let's keep track of these on a site by site basis...
								if (options.m_nMethod == 6) {
									(*changeCountsPerSite).mRows.mDataArray[indx1].mDataArray[indx1] += 1.0;
									node2->GetData().C1 += 1;
								}
							} else {
								// No...(really no substitution)
								// Log sum...
								// (*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += num->logarithm( ( 1.0 / (1.0 + mu * distance) ) );
								// Scaled product...
								(*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] *= ( 1.0 / (1.0 + mu * distance) );
								if ((*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] <= 0.0000000001) { (*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] *= 1000; (*ChangeCount3D_scaling).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += logScalingFactor; }
                                
								// Increment the substitution count...
								(*realChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += 1;
								if (options.m_nMethod == 6) node2->GetData().C0 += 1;
								// When nothing happens, we have nothing to keep track of on a site by site basis.
							}
						}
					}
				} 
			}
            
			// [ should move a bunch of stuff out here - add up for every site, the number of subs and do the update once ]
			
			
		}
	}
}

// This version may have a problem


inline
void
TreeNode::RecalcVirtualSubsAlongBranchUni(
										  
										  TreeNode*	node1,
										  TreeNode* node2,
										  MixtureModel& theModel,
										  ObservedData& theData,
										  IntVector& m_vecSiteAssignment) {
	// Adding counts per site...
	DoubleMatrix   *changeCountsPerSite;
	DoubleMatrix      *augmentedCount;
    //	DoubleMatrix3D *ChangeCount3D;
	DoubleMatrix3D    *realChangeCount3D;
    //	std::cout << "shouldnt be called yet - RecalcVirtualSubsAlongBranchUni() " << endl;
    //	exit(-1);
	double mu = 0.0;
	int numStates = theData.getNumStates();
	int numModels = theModel.GetNumModels();
	
	//	augmentedCount		= getAugmentedCountMatrix( mRoot );
	// changeCountsPerSite = getChangeMatrix3D( mRoot, site );
    
	realChangeCount3D		= getChangeMatrix3D( mRoot );
	augmentedCount			= getAugmentedCountMatrix( mRoot );
	
	// The augmentedCount has to be allocated for every ASRV class...
	int totalAugClasses = numModels;
	if ( options.numDiscreteBLs > 1) totalAugClasses *= options.numDiscreteBLs;
    
	if (this == mRoot) {
		// INITIALIZE REAL CHANGE COUNT MATRIX...
		for (int m=0; m<totalAugClasses; m++) {
			for (int i=0; i<numStates; i++) {
				// These are the genuine no-change events
				(*realChangeCount3D).mRows.mDataArray[m].mRows.mDataArray[i].mDataArray[i] =0.0;
			}
		}
		
		if ( options.numCatASRV > 1) totalAugClasses *= options.numCatASRV;
		if (augmentedCount->empty()) {
			augmentedCount->resize( totalAugClasses, numStates, 0.0 );
		} else {
			// These are the augmented counts across all model/rate classes
			augmentedCount->fill(0.0);
		}
	}
	
	// IF WE HAVE A BRANCH...
	if (node1 != NULL && node2 != NULL)
	{
		// Set sequence pointers...
		vector<int>& seq1 = node1->GetData().sequence; 
		vector<int>& seq2 = node2->GetData().sequence; 
		
		// Get branch length...
		double& distance = node2->GetData().distance;
		
		// If everything looks okay... [need to check the use of cInitDistance at the root]
		if (seq1.size() && seq2.size() && distance != cInitDistance)
		{
			int	len = min(seq1.size(), seq2.size());
			int modelassignment, rateClassAssignment, indx1, indx2;
			int incr = 1;
			if (numStates == 64) incr = 3;

			// should this be node 1 or 2?
			unsigned int k2 = node1->GetData().branchLengthIndex;
			if (mRoot->numDiscreteBLs == 0) k2=0; 
			
				if ( (this == mRoot) ) { //&& ( options.updatebls ) ) {
				for (int ii=0; ii<len; ii++) {
					changeCountsPerSite = getChangeMatrix3D( mRoot, ii );
					for (int xx=0; xx<numStates; xx++) {
						(*changeCountsPerSite).mRows.mDataArray[xx].mDataArray[xx] = 0;
					}
				}
			}
			
			double isAugmented;
			for (int s = 0; s < len; s += incr) {
                // Get sequence states at site 's'...
                indx1 = seq1[s];
                indx2 = seq2[s];
				
				// Get mixture model assignment at site 's'...
				modelassignment			= m_vecSiteAssignment.mDataArray[s];
				rateClassAssignment		= mRoot->GetData().siteCats->getAssignment( s );
				
				changeCountsPerSite = getChangeMatrix3D( mRoot, s );
				
				// Get uniformization constant for model 'modelassignment...
				mu = theModel.returnUniformizationConstants()->mDataArray[modelassignment];
				
				// If sequence states are valid...
				if ( (indx1 < numStates) && (indx2 < numStates)) {
					
					// Test the marginal dwell time stuff...
					// int theState = 0;
					// double dwellTest = _calculateMarginalDwell( indx1, indx2, theState, modelassignment, inRateMatrix3D, distance );
					// std::cout << "Site " << s << ", dwell in 0: " << dwellTest << endl;
					
					if (indx1 == indx2) {
						double rateMod = 1.0;
						if (options.allowASRV && (options.numCatASRV > 1) ) {
							rateMod = mRoot->GetData().siteCats->getRate( rateClassAssignment );
						}
                        
                        // Assuming rate scaling...
                        isAugmented = 1.0 / (1.0 + (mu * distance) + (theModel.returnRateMatrix3D()->mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] * rateMod * distance));

						if (num->Random() > isAugmented) {
							// Yes...
							(*augmentedCount).mRows.mDataArray[ num->returnModelIndex( modelassignment, rateClassAssignment, k2) ].mDataArray[indx1] += 1;
							
							// Let's keep track of these on a site by site basis...
							(*changeCountsPerSite).mRows.mDataArray[indx1].mDataArray[indx1] += 1.0;
                            //[test]							node2->GetData().C1 += 1;
						} else {
							// No...(really no substitution)
							(*realChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += 1;
                            //[test]							node2->GetData().C0 += 1;
							// When nothing happens, we have nothing to keep track of on a site by site basis.
						}
						
					}
				} 
			}
			
			// [ should move a bunch of stuff out here - add up for every site, the number of subs and do the update once ]
			
			
		}
	}
}



inline
DoubleMatrix *
TreeNode::getAugmentedLogSum_scaling( TreeNode *theNode ) {
	if (theNode != NULL) {
		if (theNode->getUniqueModelFlag()) {
            //			std::cout << "returning node changecountmat" << endl;
			return &theNode->GetData().augmentedLogSum_scaling;
		} else {
            //			std::cout << "returning ROOT changecountmat" << endl;
			return &mRoot->GetData().augmentedLogSum_scaling;
		}
	} 
    
    return NULL;
    //else {
	//	return &mRoot->GetData().countModelchange3D;
	//}
}


inline
DoubleMatrix3D *
TreeNode::getChangeMatrix3D_scaling( TreeNode *theNode ) {
	if (theNode != NULL) {
		if (theNode->getUniqueModelFlag()) {
            //			std::cout << "returning node changecountmat" << endl;
			return &theNode->GetData().ChangeCount3D_scaling;
		} else {
            //			std::cout << "returning ROOT changecountmat" << endl;
			return &mRoot->GetData().ChangeCount3D_scaling;
		}
	} 
    return NULL;
    //else {
	//	return &mRoot->GetData().countModelchange3D;
	//}
}


//---------------------------------------------------------------------------

inline int
TreeNode::calculateDwellTimeOverBranches(int s,
                                         double siteRate,
                                         MixtureModel& theModel,
                                         ObservedData& theData,
                                         IntVector& m_vecSiteAssignment)
{
	
	int numSubs = 0;
	int numStates = theData.getNumStates();
	int numModels = theModel.GetNumModels();
	
	// This initialization only affects the root -- need to modify this to allow subtree calcs...
	if ( (this == mRoot) || (Up()->Up() == Up()) ) { // (options.m_nMethod == 3) &&
		DoubleMatrix* marginalDwell = getMarginalDwell( this );
        
		if ( marginalDwell->empty() ) {
			// Not allocating for numModels - we integrate over the rate models
			// this should be expanded later for dealing with subtrees
			marginalDwell->resize( 1, numStates, 0.0 );
		} else {
			marginalDwell->fill(0.0);
		}
	}
	
	if (Left() != NULL)
	{
		numSubs += _CalculateDwellTimeBranch( s, siteRate, this, Left(), theModel, theData, m_vecSiteAssignment );
		numSubs += Left()->calculateDwellTimeOverBranches( s, siteRate, theModel, theData, m_vecSiteAssignment ); 
	}
	
	if (Right() != NULL) 
	{ 
		numSubs += _CalculateDwellTimeBranch( s, siteRate, this, Right(), theModel, theData, m_vecSiteAssignment );
		numSubs += Right()->calculateDwellTimeOverBranches(s, siteRate,  theModel, theData, m_vecSiteAssignment );  
	}
	
	return numSubs;
}

//---------------------------------------------------------------------------

inline void
TreeNode::calculateJointDwellTimeOverBranches(int s1, int s2,
                                              double siteRate1, double siteRate2,
                                              MixtureModel& theModel,
                                              ObservedData& theData,
                                              IntVector& m_vecSiteAssignment)
{
	int numModels = theModel.GetNumModels();
	int numStates = theData.getNumStates();
	
	// This initialization only affects the root -- need to modify this to allow subtree calcs...
	if ( (this == mRoot) || (Up()->Up() == Up()) ) { // (options.m_nMethod == 3) &&
		DoubleMatrix* jointDwell = getJointDwell( this );
		
		if ( jointDwell->empty() ) {
			jointDwell->resize( numModels, numStates * numStates, 0.0 );
		} else {
			jointDwell->fill(0.0);
		}
	}
	
	if (Left() != NULL)
	{
		_CalculateJointDwellTimeBranch( s1, s2, siteRate1, siteRate2, this, Left(), theModel, theData, m_vecSiteAssignment );
		Left()->calculateJointDwellTimeOverBranches( s1, s2, siteRate1, siteRate2, theModel, theData, m_vecSiteAssignment ); 
	}
	
	if (Right() != NULL) 
	{ 
		_CalculateJointDwellTimeBranch( s1, s2, siteRate1, siteRate2, this, Right(), theModel, theData, m_vecSiteAssignment );
		Right()->calculateJointDwellTimeOverBranches( s1, s2, siteRate1, siteRate2,  theModel, theData, m_vecSiteAssignment );  
	}
}


//---------------------------------------------------------------------------

inline void
TreeNode::countChangesAlongBranchUni(
									 MixtureModel& theModel,
									 ObservedData& theData,
									 IntVector& m_vecSiteAssignment,
									 int version)
{
	int numStates = theData.getNumStates();
	int numModels = theModel.GetNumModels();
	
	if ( ((this == mRoot) || (Up()->Up() == Up())) && (version==0) ) { // (options.m_nMethod == 3) &&
        
		if (options.m_nMethod == 6) {
			if (options.doB1uStar == 0) {
				getChangeMatrix3Duni( mRoot )->fill(1.0); // 0.0
				getAugmentedLogSum( mRoot )->fill(1.0); //0.0
				getAugmentedLogSum_scaling( mRoot )->fill(0);
				getChangeMatrix3D_scaling( mRoot )->fill(0);
				getTotalSubCountVector( mRoot )->fill(0);
			} else {
				getChangeMatrix3Duni( mRoot )->fill(0.0); 
				getAugmentedLogSum( mRoot )->fill(0.0);
			}
		} else {
			getChangeMatrix3Duni( mRoot )->fill(0.0); 
			getAugmentedLogSum( mRoot )->fill(0.0);
		}
		getChangeMatrix3D( mRoot )->fill(0.0);
		getAugmentedCountMatrix( mRoot )->fill(0.0);
        
		// Bringing back change counts per site...
		vector<int>& seq1 = mRoot->GetData().sequence; 
		int	len = seq1.size();
		if (mRoot->GetData().countModelchange_perSite.empty() ) {
			mRoot->GetData().countModelchange_perSite.resize(len);
		}
		
        
        //	std::cout << "INITIALIZING CHANGEMATRIX3D per SITE..." << endl;
		for (int s = 0; s < len; s++) {
			if ( getChangeMatrix3D( mRoot, s )->empty() ) {
				getChangeMatrix3D( mRoot, s )->resize(numStates, numStates, 0);
			} else {
                //		std::cout << "RESIZING CHANGEMATRIX3D per SITE..." << endl;
				for (int ii=0; ii< numModels; ii++) {
					if ( getChangeMatrix3D( mRoot, s )->empty() ) {
						getChangeMatrix3D( mRoot, s )->resize( numStates, numStates, 0 );
					} else {
						getChangeMatrix3D( mRoot, s )->fill( 0 );
					}
				}
			}
		}
        //std::cout << "DONE..." << endl;
		// end of changecount per site
	}
    
	if (Left() != NULL)
	{
		// not the assumed root node
        //	if ( Up()->Up() != this)
		if (version==0) {
			_SetChangesAlongBranchUni( this, Left(), theModel, theData, m_vecSiteAssignment );
		} else {
			RecalcVirtualSubsAlongBranchUni( this, Left(), theModel, theData, m_vecSiteAssignment );
		}
		//	else
        //		SetChangesAlongBranch(Right(), Left(), m_vecSiteAssignment, numModels, uniformizationConstants, theData, options.m_nMethod);
        
		Left()->countChangesAlongBranchUni( theModel, theData, m_vecSiteAssignment, version); 
	}
    
	if (Right() != NULL) 
	{ 
        //	if ( Up()->Up() != this)
		if (version == 0) {
			_SetChangesAlongBranchUni( this, Right(), theModel, theData, m_vecSiteAssignment );
		}else{
			RecalcVirtualSubsAlongBranchUni( this, Right(), theModel, theData, m_vecSiteAssignment );
		}
		//	else
        //		SetChangesAlongBranch(Left(), Right(), m_vecSiteAssignment, numModels, uniformizationConstants, theData, options.m_nMethod);
        
		Right()->countChangesAlongBranchUni( theModel, theData, m_vecSiteAssignment, version );  
	}
}

//---------------------------------------------------------------------------

inline void
TreeNode::CalculateBranchModelExpectedSubsUni(
                                              MixtureModel& theModel,
                                              ObservedData& theData,
                                              IntVector& m_vecSiteAssignment )
{
	static double *siteLikelihoods;
	static int allocatedSLs = 0;
	
	int numStates = theData.getNumStates();
	int numModels = theModel.GetNumModels();
	
	if (mRoot->GetData().siteLikelihoodBelow == NULL) {
		std::cout << "ERROR: CalculateBranchModelExpectedSubsUni() called before pruning step..." << endl;
		return;
	}
    
	if ( (this == mRoot) || (Up()->Up() == Up()) ) { // (options.m_nMethod == 3) &&
		if (options.m_nMethod == 6) {
			if (options.doB1uStar == 0) {
				getChangeMatrix3Duni( mRoot )->fill(1.0); // 0.0
				getAugmentedLogSum( mRoot )->fill(1.0); //0.0
				getAugmentedLogSum_scaling( mRoot )->fill(0);
				getChangeMatrix3D_scaling( mRoot )->fill(0);
			} else {
				getChangeMatrix3Duni( mRoot )->fill(0.0); 
				getAugmentedLogSum( mRoot )->fill(0.0);
			}
		} else {
			getChangeMatrix3Duni( mRoot )->fill(0.0); 
			getAugmentedLogSum( mRoot )->fill(0.0);
		}
		getChangeMatrix3D( mRoot )->fill(0);
		getAugmentedCountMatrix( mRoot )->fill(0);
		
		// Bringing back change counts per site...
		vector<int>& seq1 = mRoot->GetData().sequence; 
		int	len = seq1.size();
		if (mRoot->GetData().countModelchange_perSite.empty() ) {
			mRoot->GetData().countModelchange_perSite.resize(len);
		}
		
		
		//	std::cout << "INITIALIZING CHANGEMATRIX3D per SITE..." << endl;
		for (int s = 0; s < len; s++) {
			if ( getChangeMatrix3D( mRoot, s )->empty() ) {
				getChangeMatrix3D( mRoot, s )->resize(numStates, numStates, 0);
			} else {
				//		std::cout << "RESIZING CHANGEMATRIX3D per SITE..." << endl;
				for (int ii=0; ii< numModels; ii++) {
					if ( getChangeMatrix3D( mRoot, s )->empty() ) {
						getChangeMatrix3D( mRoot, s )->resize( numStates, numStates, 0 );
					} else {
						getChangeMatrix3D( mRoot, s )->fill( 0 );
					}
				}
			}
		}
		
		// This stuff actually needs to be calculated only once per recursion over the tree; doesn't matter if it is post- or pre-order...
		
		if ( (options.countExpectedSubstitutions == 1) ) {
			if (allocatedSLs == 0) {
				// Allocation step...
				siteLikelihoods = new double[len];
				allocatedSLs = 1;
			} else {
				// Update site likelihood step...
				for (int i=0; i<len; i++) {
					siteLikelihoods[i] = 0.0;
					for (int j=0; j<numStates; j++) {
						siteLikelihoods[i] += mRoot->GetData().siteLikelihoodBelow[(i*numStates) + j]; //+ mRoot->Right()->GetData().siteLikelihoodBelow[(i*numStates) + j];
					}
					// What about scaling at different sites...?
					
					if (siteLikelihoods[i] <= 0) { siteLikelihoods[i] = 1e-70; }
					siteLikelihoods[i] = log(siteLikelihoods[i]);
					
					// std::cout << "Site likelihood for site " << i << " is " << log(siteLikelihoods[i]) << endl;
				}
			}
		}
		
		//std::cout << "DONE..." << endl;
		// end of changecount per site
	}
	
	if (Left() != NULL)
	{
		// not the assumed root node
		//	if ( Up()->Up() != this)
		SetBranchModelExpectedSubsUni(this, Left(), theModel, theData, m_vecSiteAssignment, siteLikelihoods);
		//	else
		//		SetBranchModelExpectedSubs(Right(), Left(), m_vecSiteAssignment, numModels, uniformizationConstants, inStateindex, minstate, numStates, options.m_nMethod);
		
		Left()->CalculateBranchModelExpectedSubsUni( theModel, theData, m_vecSiteAssignment ); 
	}
	
	if (Right() != NULL) 
	{ 
		//	if ( Up()->Up() != this)
		SetBranchModelExpectedSubsUni( this, Right(), theModel, theData, m_vecSiteAssignment, siteLikelihoods );
		//	else
		//		SetBranchModelExpectedSubs(Left(), Right(), m_vecSiteAssignment, numModels, uniformizationConstants, inStateindex, minstate, numStates, options.m_nMethod);
		
		Right()->CalculateBranchModelExpectedSubsUni( theModel, theData, m_vecSiteAssignment  );  
	}
}

// ----


inline
void
TreeNode::_SetChangesAlongBranch(
                                 TreeNode*	node1,
                                 TreeNode* node2,
                                 MixtureModel& theModel,
                                 ObservedData& theData,
                                 IntVector& m_vecSiteAssignment) {
    
	int numModels = theModel.GetNumModels();
	int numStates = theData.getNumStates();
	
	DoubleMatrix3D *ChangeCount3D;
    //	IntMatrix3D *ChangeCount3D_perSite;
	
	ChangeCount3D			= getChangeMatrix3D( node2 ); 
	
	if (ChangeCount3D->empty()) {
		// Accounting for discrete bl variation...
		int dimensions = numModels;
		if ( options.numCatASRV >0 ) dimensions *=  options.numCatASRV;
		if ( mRoot->numDiscreteBLs >0 ) dimensions *= options.numDiscreteBLs;
		ChangeCount3D->resize(dimensions, numStates, numStates, 0);
	} else {
		if ( (options.m_nMethod != 3) && (options.m_nMethod != 5) ) ChangeCount3D->fill(0);
	}
	
	// Not sure if these matrices are still being used: branch-specific changes?
	if (node2->GetData().countchange.empty()) {
		node2->GetData().countchange.resize(numStates, numStates, 0);
	} else {
		node2->GetData().countchange.fill(0);
	}
	
	if (node1 != NULL && node2 != NULL)
	{
		vector<int>& seq1 = node1->GetData().sequence; 
		vector<int>& seq2 = node2->GetData().sequence; 
		
		double& distance = node2->GetData().distance;
		
		if (seq1.size() && seq2.size() && distance != cInitDistance)
		{
			int	len = min(seq1.size(), seq2.size());
			int modelassignment, indx1, indx2;
			int incr = 1;
			if (numStates == 64) incr = 3;
			
            //			int blInit, blLimit;
			
			// should this be node 1 or 2?
			unsigned int k2 = node1->GetData().branchLengthIndex;
			if (mRoot->numDiscreteBLs == 0) k2=0; 
			
			for (int s = 0; s < len; s += incr) {
				//					if (options.m_nMethod == 3 && 1==2 )	ChangeCount3D_perSite = getChangeMatrix3D( node2, s );
                indx1 = seq1[s];
                indx2 = seq2[s];
                
				modelassignment = m_vecSiteAssignment.mDataArray[s];
				unsigned int k3 = mRoot->GetData().siteCats->getAssignment( s );
				
				//	modelassignment = node2->GetData().marked; // temp for temporal model
				if ( (indx1 < numStates) && (indx2 < numStates)) {
					// [before adding ASRV]			(*ChangeCount3D).mRows.mDataArray[(k2*numModels) + modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += 1;
					(*ChangeCount3D).mRows.mDataArray[ num->returnModelIndex( modelassignment, k3, k2) ].mRows.mDataArray[indx1].mDataArray[indx2] += 1;
					
					node2->GetData().countchange.mRows.mDataArray[indx1].mDataArray[indx2] += 1;
					//		if (  (numModels > 1) && (options.m_nMethod==3) && 1==2) (*ChangeCount3D_perSite).mRows.mDataArray[modelassignment].mRows.mDataArray[indx1].mDataArray[indx2] += 1;
				} 
			}
		}
	}
}



//---------------------------------------------------------------------------

inline void
TreeNode::countChangesAlongBranch(
                                  MixtureModel& theModel,
                                  ObservedData& theData,
                                  IntVector& m_vecSiteAssignment)
{
    if ( (this == mRoot) || (Up()->Up() == Up()) ) { // (options.m_nMethod == 3) &&
        //	std::cout << "CLEARING THE COUNT MATRIX..." << endl;
		getChangeMatrix3D( mRoot )->fill(0);
		
 	}
    
	if (Left() != NULL)
	{
		// not the assumed root node
        //	if ( Up()->Up() != this)
        _SetChangesAlongBranch(this, Left(), theModel, theData, m_vecSiteAssignment );
        //	else
        //		SetChangesAlongBranch(Right(), Left(), m_vecSiteAssignment, numModels, inStateindex, minstate, numStates, options.m_nMethod);
        
		Left()->countChangesAlongBranch( theModel, theData, m_vecSiteAssignment ); 
	}
    
	if (Right() != NULL) 
	{ 
        //	if ( Up()->Up() != this)
        _SetChangesAlongBranch(this, Right(), theModel, theData, m_vecSiteAssignment );
        //	else
        //		SetChangesAlongBranch(Left(), Right(), m_vecSiteAssignment, numModels, inStateindex, minstate, numStates, options.m_nMethod);
        
		Right()->countChangesAlongBranch(  theModel, theData, m_vecSiteAssignment );  
	}
}


//---------------------------------------------------------------------------

inline
void
TreeNode::CalcLogLikelihood_branch(
                                   MixtureModel& theModel,													 
                                   IntVector& m_vecSiteAssignment,
                                   double& logLikelihood) {
	
	CalcLogLikelihood_branch( theModel, m_vecSiteAssignment, logLikelihood, 0);
}


inline
void
TreeNode::CalcLogLikelihood_branch(
                                   MixtureModel& theModel,
                                   IntVector& m_vecSiteAssignment,
                                   double& logLikelihood,
                                   int version )
{
	// For uniformized B1, we'd like three versions: BRANCH, RATE, ALL [these will return either the full (ALL) likleihood, or just the parts that depend on rates or branches]
    //	const int BOTH    = 0;
	const int RATE    = 1;
	const int BRANCH  = 2;
	
	int numModels = theModel.GetNumModels();
	
	DoubleVector *uniConstants = theModel.returnUniformizationConstants();
	
	if ( (uniConstants == NULL) && (options.m_nMethod == 6))  { std::cout << "	ERROR: CalTreeGibbsSamplingLnLikleihood called for method #6 without uniformization constants..." << endl; exit(-1); }
	
	unsigned int m,i,j;
	int blInit, blLimit;
    
	if (Up()->Up() == this) {
		logLikelihood =0;
	}
    
	if ( logLikelihood == -std::numeric_limits<double>::max( )) return;
    
	DoubleMatrix3D* mProb3D = getProbMatrix3D(this); //&GetData().probmatrix3D;
    
    if (!mProb3D->empty()) {
        unsigned int numStates = mProb3D->cols();
        double fh;
        
        if (options.m_bSite) {	//calculate the probability of mixture models with assignments of model to each site
            DoubleMatrix3D *uniformizedMethodCount3D;
            DoubleMatrix3D *mModelCount3D = getChangeMatrix3D( this ); //&GetData().countModelchange3D;
            DoubleMatrix    *augmentedCount;
            DoubleMatrix *augmentedLogSum;
            IntVector   *totalCount_actualSubs;
            
            // Get a pointer to the uniformized method matrix...
            if (options.m_nMethod == 6 ) { 
                uniformizedMethodCount3D = getChangeMatrix3Duni( mRoot );
                augmentedCount		 	 = getAugmentedCountMatrix( mRoot );
                augmentedLogSum		 	 = getAugmentedLogSum( mRoot );
                totalCount_actualSubs   = getTotalSubCountVector( mRoot );
                mProb3D = getProbMatrix3D(mRoot);
            }
            
            int totalNumModels = numModels;
            if ( options.numDiscreteBLs > 1) totalNumModels *= options.numDiscreteBLs;
            if ( options.numCatASRV > 1)     totalNumModels *= options.numCatASRV;
            
            if (!mModelCount3D->empty()) {
                if (mRoot->numDiscreteBLs != 0) { blInit = 0; blLimit = mRoot->numDiscreteBLs; }
                else { blInit = 0; blLimit = 1; }
                
                if (options.m_nMethod != 6) {
                    for (m = 0; m < totalNumModels; m++) {
                        //			for (unsigned int k2 = blInit; k2 < blLimit; k2++) {
                        for (i = 0; i < numStates; i++) {
                            for (j = 0; j < numStates; j++) {
                                fh = ((*mProb3D).mRows.mDataArray[m].mRows.mDataArray[i].mDataArray[j]);
                                if (fh <= 0) fh = 1e-70;
                                logLikelihood += (*mModelCount3D).mRows.mDataArray[ m ].mRows.mDataArray[i].mDataArray[j] * num->logarithm(fh);
                            } // states (j)
                        } // states (i)
                        //			} // discrete branchlengths
                    } // models
                } else {
                    // This version for the uniformized B1 with subdivisioning method...
                    for (m = 0; m < numModels; m++) {
                        if (options.allowASRV && (options.numCatASRV > 1) ) {
                            for (unsigned int rateclass=0; rateclass < options.numCatASRV; rateclass++) {
                                logLikelihood += totalCount_actualSubs->mDataArray[ rateclass ] * num->logarithm( mRoot->GetData().siteCats->getRate( rateclass) );
                            }
                        }		
                        
                        for (unsigned int k2 = blInit; k2 < blLimit; k2++) {
                            for (i = 0; i < numStates; i++) {
                                for (j = 0; j < numStates; j++) {
                                    
                                    if (j != i) {
                                        fh = ((*mProb3D).mRows.mDataArray[(k2*numModels) + m].mRows.mDataArray[i].mDataArray[j]);
                                        if (fh <= 0) fh = 1e-70;
                                        if ( version != BRANCH ) logLikelihood += (*mModelCount3D).mRows.mDataArray[ (k2*numModels) +m].mRows.mDataArray[i].mDataArray[j] * num->logarithm(fh);
                                        if ( version != RATE   ) logLikelihood += (*uniformizedMethodCount3D).mRows.mDataArray[ (k2*numModels) +m].mRows.mDataArray[i].mDataArray[j];
                                    } else {
                                        // Full version...
                                        if ( (options.doB1uStar == 0) ) {
                                            fh = -((*mProb3D).mRows.mDataArray[(k2*numModels) + m].mRows.mDataArray[i].mDataArray[j]);
                                            if ( version != BRANCH ) {
                                                // Now do the virtual substitutions (may be different across rates classes)...
                                                int rateClassLimit = options.numCatASRV;
                                                if ( options.allowASRV != 1 ) rateClassLimit = 1;
                                                for (int rateclass= 0; rateclass < rateClassLimit; rateclass++ ) {
                                                    double tempVal = (*uniConstants)[ (k2*numModels) +m] - (fh * mRoot->GetData().siteCats->getRate( rateclass ) );
                                                    if (tempVal < 0) {
                                                      //  std::cout << "	Row " << i << ", fh: " << fh << ", rateclass " << rateclass << ", rate " << mRoot->GetData().siteCats->getRate( rateclass ) << ", uni is " << (*uniConstants)[ (k2*numModels) +m] << endl;
                                                      //  std::cout << " --it shouldn't happen that fh " << fh  << ", rateclass (" << rateclass << "), rate " << mRoot->GetData().siteCats->getRate( rateclass ) << ", " << (fh * mRoot->GetData().siteCats->getRate( rateclass ) ) << " > uniconst " << (*uniConstants)[ (k2*numModels) +m] << ", with index " << (k2*numModels) +m << endl;
                                                        // exit(-1);
                                                    }
                                                    if (tempVal <= 0) {
                                                        tempVal = 1e-70;
                                                    }
                                                    logLikelihood += (*augmentedCount).mRows.mDataArray[ num->returnModelIndex( m, rateclass, k2) ].mDataArray[i] * num->logarithm( ( tempVal ) );
                                                }
                                                
                                            }
                                            if ( version != RATE ) logLikelihood += (*uniformizedMethodCount3D).mRows.mDataArray[ (k2*numModels) +m].mRows.mDataArray[i].mDataArray[j] + (*augmentedLogSum).mRows.mDataArray[ (k2*numModels) +m].mDataArray[i];
                                        } else {
											// Approximate method here...
                                            std::cout << "B1u* method is deprecated!" << endl;
                                            exit(-1);
                                            //												fh = -((*mProb3D).mRows.mDataArray[(k2*numModels) + m].mRows.mDataArray[i].mDataArray[j]);
                                            //												logLikelihood += num->logarithm( ( 1.0 / (1.0 + (fh * (*uniformizedMethodCount3D).mRows.mDataArray[ (k2*numModels) +m].mRows.mDataArray[i].mDataArray[j])))); 
                                        }
                                    }
                                } // states (j)
                            } // states (i)
                        } // discrete branchlengths
                    } // models
                } // uniformized B1 method
                
            }
            
        }
        
    } else {
        //std::cout << "ERROR!  probMatrix is empty! Node: " << this << "; root node is " << mRoot << endl;
    }
    
    if ( (options.m_nMethod==3) && (this == mRoot) ) return;
    if ( (options.m_nMethod==5) && (this == mRoot) ) return;
    if ( (options.m_nMethod==6) && (this == mRoot) ) return;
	
	if (Left() != NULL)
		Left()->CalcLogLikelihood_branch( theModel, m_vecSiteAssignment, logLikelihood, version); 
    
	if (Right() != NULL) 
		Right()->CalcLogLikelihood_branch( theModel, m_vecSiteAssignment, logLikelihood, version);  
}



inline
int
TreeNode::UpdateAncestralAndTransientStatesForSubtree_localUpdaterForB1xM(
                                                                          TreeNode*	head,
                                                                          TreeNode*	from,
                                                                          ObservedData& theData,
                                                                          IntVector& m_vecSiteAssignment,
                                                                          DoubleMatrix& stateFreqMatrix)
{
	int numStates = theData.getNumStates();
	
	int numUpdated = 0;
	double tempSum=0.0;
	DoubleVector probVec(numStates, 0);
	DoubleVector probNormalized(numStates, 0);
	vector<int> *seq1, *seq2, *seq3;
	DoubleMatrix3D *probmatrix3D1, *probmatrix3D2, *probmatrix3D3;
	int indx1, indx2, indx3;
	
	DoubleMatrix *probmatrix1; //(numStates, numStates, 0);
	DoubleMatrix *probmatrix2; //(numStates, numStates, 0);
	DoubleMatrix *probmatrix3; //(numStates, numStates, 0);
	
	unsigned int numModels = getProbMatrix3D(mRoot)->rows();
	if (mRoot->numDiscreteBLs != 0) { numModels /= mRoot->numDiscreteBLs; }
    
	double maxProb;
	int bestState;
	
  	if (head != NULL)
	{
		if (head->Up() != NULL && ( head->Left() != NULL && head->Right() != NULL) )
		{
            
 			if ( head != mRoot) //->Up()->Up() != head->Up())
			{
                
                
				seq1 = &head->Left()->GetData().sequence;
				seq2 = &head->Right()->GetData().sequence;
                
                 seq3 = &head->Up()->GetData().sequence;

				probmatrix3D1 = getProbMatrix3D(head->Left()); //&head->Left()->GetData().probmatrix3D;
				probmatrix3D2 = getProbMatrix3D(head->Right()); //&head->Right()->GetData().probmatrix3D;
				probmatrix3D3 = getProbMatrix3D(head); //&head->GetData().probmatrix3D;

				
				unsigned int model1, model2, model3;
                /*		
                 if (!m_bSite)
                 {
                 model1 = head->Left()->GetData().sequence.GetAssignment();
                 
                 model2 = head->Right()->GetData().sequence.GetAssignment();
                 
                 model3 = head->GetData().sequence.GetAssignment();
                 
                 if (mRoot->numDiscreteBLs == 0) { 
                 probmatrix1 = &(*probmatrix3D1).mRows.mDataArray[model1];
                 probmatrix2 = &(*probmatrix3D2).mRows.mDataArray[model2];
                 probmatrix3 = &(*probmatrix3D3).mRows.mDataArray[model3];
                 } else {
                 probmatrix1 = &(*probmatrix3D1).mRows.mDataArray[(head->Left()->GetData().branchLengthIndex * numModels)+model1];
                 probmatrix2 = &(*probmatrix3D2).mRows.mDataArray[(head->Right()->GetData().branchLengthIndex * numModels)+model2];
                 probmatrix3 = &(*probmatrix3D3).mRows.mDataArray[(head->GetData().branchLengthIndex * numModels)+model3];
                 }
                 
                 }
                 */		
				unsigned int len = min(seq1->size(), seq2->size());
				
				for (unsigned int s=0; s < len; s++)
				{
					if (options.m_bSite)
					{
						model1 = model2 = model3 = m_vecSiteAssignment.mDataArray[s];
						//model1 = model2 = model3 = GetData().marked; // temp for temporal model
                        
						if (mRoot->numDiscreteBLs == 0) { 
							probmatrix1 = &(*probmatrix3D1).mRows.mDataArray[model1];
							probmatrix2 = &(*probmatrix3D2).mRows.mDataArray[model2];
							probmatrix3 = &(*probmatrix3D3).mRows.mDataArray[model3];
						} else {
							probmatrix1 = &(*probmatrix3D1).mRows.mDataArray[(head->Left()->GetData().branchLengthIndex * numModels)+model1];
							probmatrix2 = &(*probmatrix3D2).mRows.mDataArray[(head->Right()->GetData().branchLengthIndex * numModels)+model2];
							probmatrix3 = &(*probmatrix3D3).mRows.mDataArray[(head->GetData().branchLengthIndex * numModels)+model3];
						}
					}
					
					indx1 = (*seq1)[s];
					indx2 = (*seq2)[s];
					indx3 = (*seq3)[s];
					
					tempSum = 0;
					
					maxProb = 0.0;
					bestState = -1;
					
					if (probmatrix3 == NULL ) { //|| head == mRoot) { 
						for (unsigned int i = 0; i< numStates; i++)
						{
							probVec.mDataArray[i] = (*probmatrix1).mRows.mDataArray[i].mDataArray[indx1] * (*probmatrix2).mRows.mDataArray[i].mDataArray[indx2];
							tempSum += probVec.mDataArray[i];
						}
					} else {
						for (unsigned int i = 0; i< numStates; i++)
						{
							probVec.mDataArray[i] = (*probmatrix1).mRows.mDataArray[i].mDataArray[indx1] * (*probmatrix2).mRows.mDataArray[i].mDataArray[indx2] * (*probmatrix3).mRows.mDataArray[indx3].mDataArray[i];
							tempSum += probVec.mDataArray[i];
						}
					}
					
					
					
					for (unsigned int i = 0; i< numStates; i++)
						probNormalized.mDataArray[i] = probVec.mDataArray[i] / tempSum;
					
					
                    tempSum = 0;
                    
                    unsigned int i;
                    
                    for (i = 0; i < numStates; i++)
                    {
                        tempSum += probNormalized.mDataArray[i];
                        probNormalized.mDataArray[i] = tempSum;
                    }
                    
                    double test = rndu();
                    for (i = 0; i< numStates; i++)
                    {
                        if (test < probNormalized.mDataArray[i])
                        {
                            head->GetData().sequence[s] = i; //SetAtIndex(s, inStateVector[i]);
                            break;
                        }
                    }
                    
                    // FORCE BEST ASSIGNMENT
                    //	i = bestState;
                    //	std::cout << "Best state is " << bestState << " (with maxProb " << maxProb << ")" << endl;
                    //head->GetData().sequence.SetAtIndex(s, inStateVector[bestState]);
                    
                    numUpdated++;
                    
                    if (i == numStates)
                        head->GetData().sequence[s] = ENDSEQ; //SetAtIndex(s, ENDSEQ);
                }
                
                
			}
            
			else {
				// We must be at the root...
				seq1 = &head->Left()->GetData().sequence;
				seq2 = &head->Right()->GetData().sequence;
 				
				probmatrix3D1 = getProbMatrix3D(head->Left()); //&head->Left()->GetData().probmatrix3D;
				probmatrix3D2 = getProbMatrix3D(head->Right()); //&head->Right()->GetData().probmatrix3D;
 				
				unsigned int model1, model2;
				
				/*
                 if (!m_bSite)
                 {
                 model1 = head->Left()->GetData().sequence.GetAssignment();
                 
                 model2 = head->Right()->GetData().sequence.GetAssignment();
                 
                 if (mRoot->numDiscreteBLs == 0) { 
                 probmatrix1 = &(*probmatrix3D1).mRows.mDataArray[model1];
                 probmatrix2 = &(*probmatrix3D2).mRows.mDataArray[model2];
                 } else {
                 probmatrix1 = &(*probmatrix3D1).mRows.mDataArray[(head->Left()->GetData().branchLengthIndex * numModels)+model1];
                 probmatrix2 = &(*probmatrix3D2).mRows.mDataArray[(head->Right()->GetData().branchLengthIndex * numModels)+model2];
                 }
                 }
                 */
				unsigned int len = min(seq1->size(), seq2->size());
				
				for (unsigned int s=0; s < len; s++)
				{
					if (options.m_bSite)
					{
						model1 = model2 = m_vecSiteAssignment.mDataArray[s];
						//model1 = model2 = GetData().marked; // temp for temporal model
                        
						probmatrix1 = &(*probmatrix3D1).mRows.mDataArray[model1];
						
						probmatrix2 = &(*probmatrix3D2).mRows.mDataArray[model2];
					}
					
					indx1 = (*seq1)[s];
					indx2 = (*seq2)[s];
					
					tempSum = 0;
					
					maxProb = 0.0;
					bestState = -1;
					
					if (head == mRoot) { 
						for (unsigned int i = 0; i< numStates; i++)
						{
							probVec.mDataArray[i] = stateFreqMatrix.mRows.mDataArray[model1].mDataArray[i] * (*probmatrix1).mRows.mDataArray[i].mDataArray[indx1] * (*probmatrix2).mRows.mDataArray[i].mDataArray[indx2];
							tempSum += probVec.mDataArray[i];
						}
					}
					
					for (unsigned int i = 0; i< numStates; i++)
						probNormalized.mDataArray[i] = probVec.mDataArray[i] / tempSum;
                    
                    tempSum = 0;
                    
                    unsigned int i;
                    
                    for (i = 0; i < numStates; i++)
                    {
                        tempSum += probNormalized.mDataArray[i];
                        probNormalized.mDataArray[i] = tempSum;
                    }
                    
                    double test = rndu();
                    for (i = 0; i< numStates; i++)
                    {
                        if (test < probNormalized.mDataArray[i])
                        {
                            head->GetData().sequence[s] = i; //SetAtIndex(s, inStateVector[i]);
                            break;
                        }
                    }
                    
                    // FORCE BEST ASSIGNMENT
                    //	i = bestState;
                    //	std::cout << "Best state is " << bestState << " (with maxProb " << maxProb << ")" << endl;
                    //head->GetData().sequence.SetAtIndex(s, inStateVector[bestState]);
                    
                    numUpdated++;
                    
                    if (i == numStates)
                        head->GetData().sequence[s] = ENDSEQ; //SetAtIndex(s, ENDSEQ);
                }
				
                
			} 
		}
		else if ( (head->Up() != NULL) && ( head->Left() != NULL || head->Right() != NULL) && !(( head->Left() == NULL && head->Right() == NULL) ) ) {
			TreeNode *tempNode;
			if (head->Left() == NULL) { tempNode = head->Right(); }
			else { tempNode = head->Left(); }
			
			if (tempNode == NULL) {
				std::cout << "ERROR! tempNode is NULL!" << endl;
				exit(-1);
			}
			
			if (head->Up()->Up() != head->Up())
			{
                
				seq1 = &tempNode->GetData().sequence;
				
                seq3 = &head->Up()->GetData().sequence;

				probmatrix3D1 = getProbMatrix3D(tempNode); //&head->Left()->GetData().probmatrix3D;
				probmatrix3D3 = getProbMatrix3D(head); //&head->GetData().probmatrix3D;
				if (probmatrix3D3 == NULL) {
					std::cout << "ERROR! probmatrix1 is NULL!" << endl;
					exit(-1);
				}

				unsigned int model1, model2, model3;
				
				/*
                 if (!m_bSite)
                 {
                 model1 = tempNode->GetData().sequence.GetAssignment();
                 
                 // Fixed.
                 model3 = head->GetData().sequence.GetAssignment();
                 
                 if (mRoot->numDiscreteBLs == 0) { 
                 probmatrix1 = &(*probmatrix3D1).mRows.mDataArray[model1];
                 probmatrix3 = &(*probmatrix3D3).mRows.mDataArray[model3];
                 } else {
                 probmatrix1 = &(*probmatrix3D1).mRows.mDataArray[(head->Left()->GetData().branchLengthIndex * numModels)+model1];
                 probmatrix3 = &(*probmatrix3D3).mRows.mDataArray[(head->GetData().branchLengthIndex * numModels)+model3];
                 }
                 }
                 */
				
				unsigned int len = seq1->size();
				
				len = min(seq1->size(), seq3->size());
				
 				for (unsigned int s=0; s < len; s++)
				{
 					if (options.m_bSite)
					{
						model1 = model3 = m_vecSiteAssignment.mDataArray[s];
						//model1 = model3 = GetData().marked; // temp for temporal model
                        
						probmatrix1 = &(*probmatrix3D1).mRows.mDataArray[model1];
						
						probmatrix3 = &(*probmatrix3D3).mRows.mDataArray[model3];
					}
					
					indx1 = (*seq1)[s];
					indx3 = (*seq3)[s];
					
					tempSum = 0;
					
					maxProb = 0.0;
					bestState = -1;
					
                    //	if (s==0) std::cout << "site " << s << ", internal node: start state is " << indx1 << ", end at " << indx3 << endl;
                    //		std::cout << "TEST: probVec is " << probVec.size() << endl;
					
					for (unsigned int i = 0; i< numStates; i++)
					{
						probVec.mDataArray[i] = (*probmatrix1).mRows.mDataArray[i].mDataArray[indx1] * (*probmatrix3).mRows.mDataArray[indx3].mDataArray[i];
                        //			std::cout << "state " << i << ", val " << probVec.mDataArray[i] << endl;
						tempSum += probVec.mDataArray[i];
					}
					
					for (unsigned int i = 0; i< numStates; i++)
						probNormalized.mDataArray[i] = probVec.mDataArray[i] / tempSum;
					
					
					
                    tempSum = 0;
                    
                    unsigned int i;
                    
                    for (i = 0; i < numStates; i++)
                    {
                        tempSum += probNormalized.mDataArray[i];
                        probNormalized.mDataArray[i] = tempSum;
                    }
                    
                    double test = rndu();
                     for (i = 0; i< numStates; i++)
                    {
                        if (test <= probNormalized.mDataArray[i])
                        {
                             head->GetData().sequence[s] = i; //SetAtIndex(s, inStateVector[i]);
                            break;
                        }
                    }
                    
                    // FORCE BEST ASSIGNMENT
                    //	i = bestState;
                    //	std::cout << "Best state is " << bestState << " (with maxProb " << maxProb << ")" << endl;
                    //head->GetData().sequence.SetAtIndex(s, inStateVector[bestState]);
                    
                    numUpdated++;
                    
                    if (i == numStates)
                        head->GetData().sequence[s] = ENDSEQ; //SetAtIndex(s, ENDSEQ);
                }
				
                
			} 
			else {
				std::cout << "THIS CONDITION SHOULDNT HAPPEN: the ROOT MUST HAVE A LEFT() AND RIGHT() NO MATTER WHAT!" << endl; exit(-1);
				
			}
		}
		//	else {
		//		std::cout << "Head->Up() is " << head->Up() << ", this " << head << ", left " << head->Left() << ", right " << head->Right() << endl;
		//		std::cout << "THIS IS AN ERROR!  This condition should never happen.  At root and one of left() or right() is null" << endl; exit(-1);
		//	}
        
		if ( (head->Left() != from) && (head->Left() != NULL) )
			numUpdated += UpdateAncestralAndTransientStatesForSubtree_localUpdaterForB1xM(head->Left(), head, theData, m_vecSiteAssignment, stateFreqMatrix );
		
		if ( (head->Right() != from) && (head->Right() != NULL) )
			numUpdated += UpdateAncestralAndTransientStatesForSubtree_localUpdaterForB1xM(head->Right(), head, theData, m_vecSiteAssignment, stateFreqMatrix );
		
		if ( (head->Up() != from) && (head->Up() != NULL) && (from != NULL) ) 
			numUpdated += UpdateAncestralAndTransientStatesForSubtree_localUpdaterForB1xM(head->Up(), head, theData, m_vecSiteAssignment, stateFreqMatrix );
		
	}
	
	return numUpdated;
}


inline
int
TreeNode::_resampleAncestralAndTransientStates_local_fromNode(
																		TreeNode*	head,
																		TreeNode*	from,
																		MixtureModel& theModel,
																		ObservedData& theData,
																		IntVector& m_vecSiteAssignment,
																		DoubleMatrix& stateFreqMatrix)
{
	int numStates = theData.getNumStates();
	
	int numUpdated = 0;
	double tempSum=0.0;
	static DoubleVector probVec(numStates, 0);
	static DoubleVector probNormalized(numStates, 0);
	probVec.fill(0);
	probNormalized.fill(0);
	
	vector<int> *seq1, *seq2, *seq3;
	DoubleMatrix3D *probmatrix3D1, *probmatrix3D2, *probmatrix3D3;
	int indx1, indx2, indx3;
	
	DoubleMatrix *probmatrix1; //(numStates, numStates, 0);
	DoubleMatrix *probmatrix2; //(numStates, numStates, 0);
	DoubleMatrix *probmatrix3; //(numStates, numStates, 0);
	
	double maxProb;
	int bestState;
	unsigned int numModels = getProbMatrix3D(mRoot)->rows();
	if (mRoot->numDiscreteBLs != 0) { numModels /= mRoot->numDiscreteBLs; }
	

	if (head != NULL) {
		if (head->Up() != NULL && ( head->Left() != NULL && head->Right() != NULL) ) {
			if ( head != mRoot) {
				
				seq1 = &head->Left()->GetData().sequence;
				seq2 = &head->Right()->GetData().sequence;
				seq3 = &head->Up()->GetData().sequence;
				
				probmatrix3D1 = getProbMatrix3D(head->Left());
				probmatrix3D2 = getProbMatrix3D(head->Right()); 
				probmatrix3D3 = getProbMatrix3D(head);
				
				unsigned int model1, model2, model3;
				unsigned int len = min(seq1->size(), seq2->size());

				len = min(len, (unsigned int)seq3->size());
				
				for (unsigned int s=0; s < len; s++)
				{
					model1 = model2 = model3 = m_vecSiteAssignment.mDataArray[s];
					//model1 = model2 = model3 = GetData().marked; // temp for temporal model
					
					probmatrix1 =  &(*probmatrix3D1).mRows.mDataArray[ num->returnModelIndex( model1, mRoot->GetData().siteCats->getAssignment( s ), head->Left()->GetData().branchLengthIndex) ];
					probmatrix2 =  &(*probmatrix3D2).mRows.mDataArray[ num->returnModelIndex( model2, mRoot->GetData().siteCats->getAssignment( s ), head->Right()->GetData().branchLengthIndex) ];
					probmatrix3 =  &(*probmatrix3D3).mRows.mDataArray[ num->returnModelIndex( model3, mRoot->GetData().siteCats->getAssignment( s ), head->GetData().branchLengthIndex) ];
					
					indx1 = (*seq1)[s];
					indx2 = (*seq2)[s];
					indx3 = (*seq3)[s];
					
					tempSum = 0;
					maxProb = 0.0;
					bestState = -1;
					
					double temp;
					if (probmatrix3 == NULL ) { //|| head == mRoot) { 
						for (unsigned int i = 0; i< numStates; i++)
						{
							temp = (*probmatrix1).mRows.mDataArray[i].mDataArray[indx1] * (*probmatrix2).mRows.mDataArray[i].mDataArray[indx2];
							if ( options.__globalLikelihoodTemp != 1.0 ) {
								probVec.mDataArray[i] = pow( temp, options.__globalLikelihoodTemp );
							} else {
								probVec.mDataArray[i] = temp;
							}
							
							tempSum += probVec.mDataArray[i];
							probNormalized.mDataArray[i] = tempSum;
						}
					} else {
						for (unsigned int i = 0; i< numStates; i++)
						{
							temp = (*probmatrix1).mRows.mDataArray[i].mDataArray[indx1] * (*probmatrix2).mRows.mDataArray[i].mDataArray[indx2] * (*probmatrix3).mRows.mDataArray[indx3].mDataArray[i];
							
							if ( options.__globalLikelihoodTemp != 1.0 ) {
								probVec.mDataArray[i] = pow( temp, options.__globalLikelihoodTemp );
							} else {
								probVec.mDataArray[i] = temp;
							}
							
							tempSum += probVec.mDataArray[i];
							probNormalized.mDataArray[i] = tempSum;
						}
					}

						unsigned int i;
						double test = rndu() * tempSum;

						for (i = 0; i< numStates; i++)
						{
							if (test < probNormalized.mDataArray[i])
							{
								head->GetData().sequence[s] = i;
								break;
							}
						}
						
						// FORCE BEST ASSIGNMENT
						//	i = bestState;
						//	std::cout << "Best state is " << bestState << " (with maxProb " << maxProb << ")" << endl;
						//head->GetData().sequence.SetAtIndex(s, inStateVector[bestState]);
						
						numUpdated++;
						
						if (i == numStates)
							head->GetData().sequence[s] = i;
				}
			} else if ( (head != NULL)  ) {
				// We must be at the root...
				seq1 = &head->Left()->GetData().sequence;
				seq2 = &head->Right()->GetData().sequence;
				
				probmatrix3D1 = getProbMatrix3D(head->Left()); //&head->Left()->GetData().probmatrix3D;
				probmatrix3D2 = getProbMatrix3D(head->Right()); //&head->Right()->GetData().probmatrix3D;
				
				unsigned int model1, model2;
				unsigned int len = min(seq1->size(), seq2->size());
				
				for (unsigned int s=0; s < len; s++)
				{
					model1 = model2 = m_vecSiteAssignment.mDataArray[s];
					//model1 = model2 = GetData().marked; // temp for temporal model
					
					probmatrix1 =  &(*probmatrix3D1).mRows.mDataArray[ num->returnModelIndex( model1, mRoot->GetData().siteCats->getAssignment( s ), head->Left()->GetData().branchLengthIndex) ];
					probmatrix2 =  &(*probmatrix3D2).mRows.mDataArray[ num->returnModelIndex( model2, mRoot->GetData().siteCats->getAssignment( s ), head->Right()->GetData().branchLengthIndex) ];
					
					indx1 = (*seq1)[s];
					indx2 = (*seq2)[s];
					
					tempSum = 0;
					maxProb = 0.0;
					bestState = -1;
					double temp;
					///if (head == mRoot) { 
					for (unsigned int i = 0; i< numStates; i++)
					{
						temp = stateFreqMatrix.mRows.mDataArray[model1].mDataArray[i] * (*probmatrix1).mRows.mDataArray[i].mDataArray[indx1] * (*probmatrix2).mRows.mDataArray[i].mDataArray[indx2];
						
						if ( options.__globalLikelihoodTemp != 1.0 ) {
							probVec.mDataArray[i] = pow( temp, options.__globalLikelihoodTemp );
						} else {
							probVec.mDataArray[i] = temp;
						}
						
						tempSum += probVec.mDataArray[i];
						probNormalized.mDataArray[i] = tempSum;
					}
					//}
					
					unsigned int i;
					double test = rndu() * tempSum;
					for (i = 0; i< numStates; i++)
					{
						if (test < probNormalized.mDataArray[i]) {
							head->GetData().sequence[s] = i; //SetAtIndex(s, inStateVector[i]);
							break;
						}
					}
					
					// FORCE BEST ASSIGNMENT
					//	i = bestState;
					//	std::cout << "Best state is " << bestState << " (with maxProb " << maxProb << ")" << endl;
					//head->GetData().sequence.SetAtIndex(s, inStateVector[bestState]);
					
					numUpdated++;
					
					if (i == numStates)
						head->GetData().sequence[s] = i; //SetAtIndex(s, ENDSEQ);
					//	}
				}
				
			} 
		}
		else if ( (head->Up() != NULL) && ( head->Left() != NULL || head->Right() != NULL) && !(( head->Left() == NULL && head->Right() == NULL) ) ) {
			
			// [Sep 1]  There are a bunch of refs in here to ->Left() that should be tempNode
			
			TreeNode *tempNode;
			if (head->Left() == NULL) { tempNode = head->Right(); }
			else { tempNode = head->Left(); }
			
			if (tempNode == NULL) {
				std::cout << "ERROR! tempNode is NULL!" << endl;
				exit(-1);
			}
			
			if (head->Up()->Up() != head->Up()) {
                // For branches without two children (i.e., transient sampling points along branches)
                
				seq1 = &tempNode->GetData().sequence;
				seq3 = &head->Up()->GetData().sequence;
				
				probmatrix3D1 = getProbMatrix3D(tempNode); //&head->Left()->GetData().probmatrix3D;
				probmatrix3D3 = getProbMatrix3D(head); // added Up();  //&head->GetData().probmatrix3D;
				
				if (probmatrix3D3 == NULL) {
					std::cout << "ERROR! probmatrix1 is NULL!" << endl;
					exit(-1);
				}
				
				unsigned int model1, model3;
				unsigned int len = seq1->size();
				len = min(seq1->size(), seq3->size());
				
				for (unsigned int s=0; s < len; s++)
				{
					model1 = model3 = m_vecSiteAssignment.mDataArray[s];
					//model1 = model3 = GetData().marked; // temp for temporal model
					
					probmatrix1 =  &(*probmatrix3D1).mRows.mDataArray[ num->returnModelIndex( model1, mRoot->GetData().siteCats->getAssignment( s ), tempNode->GetData().branchLengthIndex) ];
					probmatrix3 =  &(*probmatrix3D3).mRows.mDataArray[ num->returnModelIndex( model3, mRoot->GetData().siteCats->getAssignment( s ), head->GetData().branchLengthIndex) ];

					indx1 = (*seq1)[s];
					indx3 = (*seq3)[s];
					
					tempSum = 0;
					
					maxProb = 0.0;
					bestState = -1;
					double temp;
					
					for (unsigned int i = 0; i< numStates; i++)
					{
                        temp = (*probmatrix1).mRows.mDataArray[i].mDataArray[indx1] * (*probmatrix3).mRows.mDataArray[indx3].mDataArray[i];
						
						if ( options.__globalLikelihoodTemp != 1.0 ) {
							probVec.mDataArray[i] = pow( temp, options.__globalLikelihoodTemp );
						} else {
							probVec.mDataArray[i] = temp;
						}
						
						tempSum += probVec.mDataArray[i];
						probNormalized.mDataArray[i] =tempSum;
					}

					unsigned int i;
						
						double test = rndu() * tempSum;

						for (i = 0; i< numStates; i++)
						{
							if (test <= probNormalized.mDataArray[i])
							{
								head->GetData().sequence[s] = i; //SetAtIndex(s, inStateVector[i]);
								break;
							}
						}
						
						// FORCE BEST ASSIGNMENT
						//	i = bestState;
						//	std::cout << "Best state is " << bestState << " (with maxProb " << maxProb << ")" << endl;
						//head->GetData().sequence.SetAtIndex(s, inStateVector[bestState]);
						
						numUpdated++;
						
						if (i == numStates)
							head->GetData().sequence[s] = i; //ENDSEQ; //SetAtIndex(s, ENDSEQ);

				}
				
			} 
			else {
				std::cout << "THIS CONDITION SHOULDNT HAPPEN: the ROOT MUST HAVE A LEFT() AND RIGHT() NO MATTER WHAT!" << endl; exit(-1);
				
			}
		}
		
		if ( (head->Left() != from) && (head->Left() != NULL) ) {
			numUpdated += _resampleAncestralAndTransientStates_local_fromNode(head->Left(), head, theModel, theData, m_vecSiteAssignment, stateFreqMatrix );
		}
		
		if ( (head->Right() != from) && (head->Right() != NULL) ) {
			numUpdated += _resampleAncestralAndTransientStates_local_fromNode(head->Right(), head, theModel, theData, m_vecSiteAssignment, stateFreqMatrix );
		}
		
		if ( (head->Up() != from) && (head->Up() != NULL) && (head != mRoot) ) { //&& (from != NULL) ) {
			//	std::cout << "Calling updateGibbs Up()..." << endl;
			numUpdated += _resampleAncestralAndTransientStates_local_fromNode(head->Up(), head,  theModel, theData, m_vecSiteAssignment, stateFreqMatrix );
		}
	}
	
	return numUpdated;
	// return count;
}



//---------------------------------------------------------------------------

inline
int
TreeNode::ResampleAncestralAndTransientStates_local_fromNode(
                                                             int&	rannode,
                                                             int&		found,
                                                             MixtureModel& theModel,
                                                             ObservedData& theData,
                                                             IntVector& m_vecSiteAssignment,
                                                             DoubleMatrix& stateFreqs
                                                             )
{  
	int numUpdated = 0;
    
	
	if ( this != NULL && found == 0)
	{
		if (Left() != NULL || Right() != NULL) // [THIS CONDITIONAL PROBLEMATIC!  FORMERLY the '&&' CAUSED PROBLEMS FOR ROOTED TREES]
		{
			if (GetData().id == rannode)
			{
				found=1;
				numUpdated += _resampleAncestralAndTransientStates_local_fromNode(this, NULL, theModel, theData, m_vecSiteAssignment, theModel.returnStateFrequencies() );
			}	
			
			if (Left() != NULL && found == 0)
				numUpdated += Left()->ResampleAncestralAndTransientStates_local_fromNode(rannode, found, theModel, theData, m_vecSiteAssignment, theModel.returnStateFrequencies());
            
			if (Right() != NULL && found == 0)
				numUpdated += Right()->ResampleAncestralAndTransientStates_local_fromNode(rannode, found,theModel, theData, m_vecSiteAssignment, theModel.returnStateFrequencies());
		}
		else
		{	
			// If it is a leaf node
			if (GetData().id == rannode)
			{
				found=1;               
				numUpdated += _resampleAncestralAndTransientStates_local_fromNode(Up(), NULL,theModel, theData, m_vecSiteAssignment, theModel.returnStateFrequencies());
			}
		}
	}
    
	return numUpdated;
}

//---------------------------------------------------------------------------
/*
 inline
 void
 TreeNode::markNodesBelow(
 int&	theNode,
 int&		found,
 IntVector& m_vecSiteAssignment,
 bool&	m_bSite,
 IntVector& inStateindex,
 CharVector& inStateVector,
 int& minstate,
 int& numStates)
 {
 
 //[debug]		std::cout << "	treeNode ResampleAncestralAndTransientStates_local_fromNode() called for rannode " << rannode << ", this node is " << GetData().id << ", name: " << GetData().name << endl;
 //[debug]		std::cout << "DEBUG: Root node has id " << mRoot->GetData().id << endl;// exit(-2);
 int numUpdated = 0;
 if (this == mRoot || Up() == Up()->Up() ) { GetData().marked = 0; found = 0;}
 
 if ( this != NULL )
 {
 if (Left() != NULL || Right() != NULL) // [THIS CONDITIONAL PROBLEMATIC. FORMERLY the '&&' CAUSED PROBLEMS FOR ROOTED TREES]
 {
 if (GetData().id == theNode)
 {
 found=1;
 GetData().marked = 1;
 std::cout << "THIS IS IT.  Marked node " << GetData().id << ", branch: " << Up()->GetData().id << ".." << GetData().id << " (" << GetData().name << ") " << " to 1" << endl;
 } else {
 if ( Up()->GetData().marked != 1) {
 GetData().marked = 0;
 std::cout << "upMarked is ZERO.  Marked node " << GetData().id << ", branch: " << Up()->GetData().id << ".." << GetData().id << " (" << GetData().name << ") " << " to ZERO" << endl;
 } else {
 GetData().marked = 1;
 std::cout << "upMarked not ZERO.  Marked node " << GetData().id << ", branch: " << Up()->GetData().id << ".." << GetData().id << " (" << GetData().name << ") " << " to 1" << endl;
 }
 }
 
 if (Left() != NULL)
 Left()->markNodesBelow(theNode, found, m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates);
 
 if (Right() != NULL)
 Right()->markNodesBelow(theNode, found, m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates);
 }
 else
 {	
 // If it is a leaf node
 if (GetData().id == theNode)
 {
 found=1;
 GetData().marked = 1;
 std::cout << "marked node " << GetData().id << ", branch: " << Up()->GetData().id << ".." << GetData().id << " (" << GetData().name << ") " << " to 1" << endl;
 } else {
 if ( Up()->GetData().marked != 1) {
 GetData().marked = 0;
 std::cout << "marked node " << GetData().id << ", branch: " << Up()->GetData().id << ".." << GetData().id << " (" << GetData().name << ") " << " to ZERO" << endl;
 } else {
 GetData().marked = 1;
 std::cout << "marked node " << GetData().id << ", branch: " << Up()->GetData().id << ".." << GetData().id << " (" << GetData().name << ") " << " to 1" << endl;
 }
 }
 }
 } 
 
 //	return numUpdated;
 }
 */

//---------------------------------------------------------------------------
inline void TreeNode::initSiteLikelihoods(
                                          TreeNode*	theNode,
                                          IntVector& m_vecSiteAssignment,
                                          int& numStates) {
	
	int length = m_vecSiteAssignment.size();
    
	initSiteLikelihoods( theNode, m_vecSiteAssignment, numStates, 0, length );
}

inline void TreeNode::initSiteLikelihoods(
                                          TreeNode*	theNode,
                                          IntVector& m_vecSiteAssignment,
                                          int& numStates,
                                          int startSite,
                                          int endSite) {
	
	// Initialize conditional site likelihoods from site startSite to endSite...
	vector<int> *theSeq;
	theSeq = &theNode->GetData().sequence;
    
	if ( this != NULL) {
		if (Left() != NULL) {
			Left()->initSiteLikelihoods(Left(), m_vecSiteAssignment, numStates, startSite, endSite); // inStateindex, inStateVector, minstate, 
		}
		
		if (Right() != NULL) {
			Right()->initSiteLikelihoods(Right(), m_vecSiteAssignment, numStates, startSite, endSite); // inStateindex, inStateVector, minstate,
		}
	
		// Only select scaling factors once (for first site)
		if ( (this != mRoot) && (startSite == 0) && (mRoot->GetData().doScaling == 0) ) {
			if (rndu() <= 0.1) { GetData().doScaling = 1; std::cout << "		[DOING SCALING AT NODE " << GetData().id << "]" << endl; }
		}
		
		// Calculate the conditional site likelihoods here...	
		theNode->GetData().allocateForSiteLikelihoods( numStates, m_vecSiteAssignment.size() );
        
		int i, s;
		double *siteLikelihoods = theNode->GetData().siteLikelihoodBelow;
		
        //	pragma omp parallel num_threads( options.numThreads ) shared( siteLikelihoods ) private(s, i)
        //	{
        for (s=startSite; s<endSite; s++) {
            for (i=0; i<numStates; i++) {
                siteLikelihoods[(s*numStates) + i] = 1.0;
                //[block] theNode->GetData().siteLikelihoodBelow_andAbove[(s*numStates) + i] = 0.0;
            }
        }
        //	}  // end of parallel section
        
	}
}

//---------------------------------------------------------------------------

inline
DoubleMatrix3D *
TreeNode::getProbMatrix3D( TreeNode *theNode ) {
	if (theNode != NULL) {
		if (theNode->getUniqueModelFlag()) {
			return &theNode->GetData().probmatrix3D;
		} else {
			return &mRoot->GetData().probmatrix3D;
		}
	} else {
		return &mRoot->GetData().probmatrix3D;
	}
}


inline
IntVector *
TreeNode::getTotalSubCountVector( TreeNode *theNode ) {
	if (theNode != NULL) {
		if (theNode->getUniqueModelFlag()) {
			//			std::cout << "returning node changecountmat" << endl;
			return &theNode->GetData().totalCount_actualSubs;
		} else {
			//			std::cout << "returning ROOT changecountmat" << endl;
			return &mRoot->GetData().totalCount_actualSubs;
		}
	}
    return NULL;
    //else {
	//	return &mRoot->GetData().countModelchange3D;
	//}
}



inline
DoubleMatrix *
TreeNode::getAugmentedLogSum( TreeNode *theNode ) {
	if (theNode != NULL) {
		if (theNode->getUniqueModelFlag()) {
            //			std::cout << "returning node changecountmat" << endl;
			return &theNode->GetData().augmentedLogSum;
		} else {
            //			std::cout << "returning ROOT changecountmat" << endl;
			return &mRoot->GetData().augmentedLogSum;
		}
	}
    return NULL;
    //else {
	//	return &mRoot->GetData().countModelchange3D;
	//}
}



inline
DoubleMatrix3D *
TreeNode::getChangeMatrix3Duni( TreeNode *theNode ) {
	if (theNode != NULL) {
		if (theNode->getUniqueModelFlag()) {
            //			std::cout << "returning node changecountmat" << endl;
			return &theNode->GetData().countModelchange3Duni;
		} else {
            //			std::cout << "returning ROOT changecountmat" << endl;
			return &mRoot->GetData().countModelchange3Duni;
		}
	}
    return NULL;
    //else {
	//	return &mRoot->GetData().countModelchange3D;
	//}
}

//---------------------------------------------------------------------------

inline
void
TreeNode::restoreLastSequenceStates(
                                    IntVector& m_vecSiteAssignment,
                                    CharVector& inStateVector,
                                    int& minstate,
                                    int &numStates) {
    
	int isTip;
	
	if ( this != NULL) {
		if (Left() != NULL)
			Left()->restoreLastSequenceStates( m_vecSiteAssignment, inStateVector, minstate, numStates );
        
		if (Right() != NULL)
			Right()->restoreLastSequenceStates( m_vecSiteAssignment, inStateVector, minstate, numStates );
        
		// --------------------------
        vector<int> *theSeq;
        
        theSeq = &GetData().sequence;
        if (( Left() == NULL) && (Right() == NULL) && (Up() != NULL) ) {
            // Tip node...
            isTip = 1;
        } else {
            isTip = 0;
        }
        
        int length = theSeq->size();
        
        for (int s=0; s < m_vecSiteAssignment.size(); s++) {
            if (isTip == 0) {
                restoreSeqSite( s, inStateVector, minstate);
            } //not tip
        } // all sites
	} // node is not null
}


// Cleaned up version.  USING LOGS!  ..why not?
//---------------------------------------------------------------------------


inline
void
TreeNode::setSiteLikelihoods_downPass(
                                      TreeNode*	theNode,
                                      MixtureModel& theModel,
                                      ObservedData& theData,
                                      IntVector& m_vecSiteAssignment,
                                      DoubleVector& modelFreqVector,
                                      int& theMode) {
	
	setSiteLikelihoods_downPass( theNode, theModel, theData, m_vecSiteAssignment, modelFreqVector, theMode, 0, m_vecSiteAssignment.size() );
}


inline void TreeNode::setSiteLikelihoods_downPass(
                                           TreeNode*	theNode,
                                           MixtureModel& mModel,
                                           ObservedData& theData,
                                           IntVector& m_vecSiteAssignment,
                                           DoubleVector& modelFreqVector,
                                           int& theMode,
                                           int startSite,
                                           int endSite) {
    
	// theMode = 0: normal site likelihoods
	// theMode = 1: Pupko et al. type
    
	// *** NEED TO REWRITE THIS TO ALLOW BETTER PARALLELIZATION ***
	
	double maxVal;
	int from, to, isTip;
	int numStates = theData.getNumStates();
	
	int i, s, theModel; //, from, to;
	DoubleMatrix3D *probmatrix3D;
	probmatrix3D = getProbMatrix3D(theNode); //&theNode->Right()->GetData().probmatrix3D;
	double temp, tempVal;
	int chunk = int((endSite - startSite) / options.numThreads);
	
	double *siteLikelihoods = theNode->GetData().siteLikelihoodBelow;
	DoubleVector *scalingFactors  = & mRoot->GetData().scalingFactor;
	
	
	if ( this != NULL) {
		if (Left() != NULL)
			Left()->setSiteLikelihoods_downPass( Left(), mModel, theData, m_vecSiteAssignment, modelFreqVector, theMode, startSite, endSite);
        
		if (Right() != NULL)
			Right()->setSiteLikelihoods_downPass( Right(), mModel, theData, m_vecSiteAssignment,  modelFreqVector, theMode, startSite, endSite);
    
    // SUSPECT OPENMP PARALLELIZATION IS CAUSING TROUBLE EVEN WITH JUST 1 THREAD
    //pragma omp parallel num_threads( options.numThreads ) private(s, i, from, to, maxVal, temp, tempVal, theModel )
		{
            
			vector<int> *theSeq;
			int theModel;
			
			DoubleMatrix *theProbMatrix;
			
			if (theNode != NULL) {
				theSeq = &theNode->GetData().sequence;
				if (( theNode->Left() == NULL) && (theNode->Right() == NULL) && (theNode->Up() != NULL) ) {
					isTip = 1;
				} else {
					isTip = 0;
				}
                
				probmatrix3D = getProbMatrix3D(theNode);
				int length = theSeq->size();
				
				if (theMode == 1) {
					if (theNode->conditionalState.empty()) {
						theNode->conditionalState.resize( numStates*length, 0 );
					}
				}				
				
				if (theNode == mRoot) {
					probmatrix3D = getProbMatrix3D(theNode);
#pragma omp for schedule(static, chunk)
					for ( s=startSite; s < endSite; s++) {
						theModel = m_vecSiteAssignment.mDataArray[s];
						//	theModel = theNode->GetData().marked; // temporal model override
                        
						theProbMatrix = &(*probmatrix3D).mRows.mDataArray[ num->returnModelIndex( theModel, mRoot->GetData().siteCats->getAssignment(s), theNode->GetData().branchLengthIndex ) ];
						if (theMode == 1) {
							// For Pupko algorithm...
                            probmatrix3D = getProbMatrix3D( theNode->Left() );
							
                            theProbMatrix = &(*probmatrix3D).mRows.mDataArray[ num->returnModelIndex( theModel, mRoot->GetData().siteCats->getAssignment(s), theNode->Left()->GetData().branchLengthIndex ) ];
                            
                            tempVal = 0.0;
                            for (from=0; from<numStates; from++) {
                                tempVal = mModel.returnStateFrequencies().mRows.mDataArray[theModel].mDataArray[from];
                                if (theNode->Left() != NULL) tempVal *= theNode->Left()->GetData().siteLikelihoodBelow[(s*numStates) + from];
                                if (theNode->Right() != NULL) tempVal *= theNode->Right()->GetData().siteLikelihoodBelow[(s*numStates) + from];
                                siteLikelihoods[(s*numStates) + from] = tempVal;
                            }
                            
						} else {
							// Standard algorithm...
							// LEFT TREE BELOW...
							if (theNode->Left() != NULL) {
                                // [diagnostic]  if (s==232) cout << "site " << s << ", processing left subtree" << endl;
								probmatrix3D = getProbMatrix3D( theNode->Left() ); //&theNode->Left()->GetData().probmatrix3D;
								theProbMatrix = &(*probmatrix3D).mRows.mDataArray[ num->returnModelIndex( theModel, mRoot->GetData().siteCats->getAssignment(s), theNode->Left()->GetData().branchLengthIndex ) ];
                                
								temp = 0;
                                maxVal = -999999999999999;

                                for (from=0; from<numStates; from++) {
									temp = 0.0;
									for ( to=0; to<numStates; to++) { //stateFreqMatrix.mRows.mDataArray[theModel].mDataArray[from] * 
										temp += (*theProbMatrix).mRows.mDataArray[from].mDataArray[to] * theNode->Left()->GetData().siteLikelihoodBelow[(s*numStates) + to];
									}
									// Multiply in the statef...[June 2009]
									// temp *= stateFreqMatrix.mRows.mDataArray[theModel].mDataArray[from];
                                    
									theNode->GetData().siteLikelihoodBelow[(s*numStates) + from] = temp;
									
                                    // [Don't rescale here!] if (theNode->Right() == NULL) siteLikelihoods[(s*numStates) + from] *= exp((*scalingFactors)[s]);
                                   
                                    // [diagnostic]  
                                    /*
                                    if (temp == 0) { cout << "    [downpass] Got 0 siteL for site " << s << "; left subtree root; siteL = " << siteLikelihoods[(s*numStates) + from] << endl; }
                                    else {
                                    // [diagnostic]      if (s==232) {
                                     // [diagnostic]         cout <<"    [downpass 232] siteL is " << theNode->GetData().siteLikelihoodBelow[(s * numStates)+from] << endl;
                                      // [diagnostic]    }
                                    }
                                     */
								}
							}
							
							double temp;
							if (theNode->Right() != NULL) {
                               // [diagnostic]   if (s==232) cout << "site " << s << ", processing right subtree" << endl;
								probmatrix3D = getProbMatrix3D(theNode->Right());
								
								theProbMatrix = &(*probmatrix3D).mRows.mDataArray[ num->returnModelIndex( theModel, mRoot->GetData().siteCats->getAssignment(s), theNode->Right()->GetData().branchLengthIndex ) ];
                                
                                maxVal = -999999999999999;

                                for ( from=0; from<numStates; from++) {
									temp = 0.0;
									for ( to=0; to<numStates; to++) { 
										temp += (*theProbMatrix).mRows.mDataArray[from].mDataArray[to] * theNode->Right()->GetData().siteLikelihoodBelow[(s*numStates) + to];
									}
									// Multiply in the statef... [June 2009]
									//temp *= stateFreqMatrix.mRows.mDataArray[theModel].mDataArray[from];
// [diagnostic]  
                                    /*
                                    if (temp == 0) { cout << "  Got 0 siteL for site " << s << "; right subtree root; siteL = " << siteLikelihoods[(s*numStates) + from] << endl; }
                                    else {
                                   // [diagnostic]       if (s==232) {
                                    // [diagnostic]         cout <<"    [downpass 232] siteL is " << siteLikelihoods[(s * numStates)+from] << endl;
                                     // [diagnostic]     }
                                    }
*/
									siteLikelihoods[(s*numStates) + from] *= temp;
                                    // [Don't rescale here!] siteLikelihoods[(s*numStates) + from] *= exp((*scalingFactors)[s]);
                                    
                               // [diagnostic]     if (s==232) cout << "after scaling: " << siteLikelihoods[(s*numStates) + from] << "; scalingfactor is " << exp((*scalingFactors)[s]) << endl;
                                    
								} // for from
							}
						}
					} // s all sites
                    //	} // parallelization
					
				} else {
					// If not root...

					probmatrix3D = getProbMatrix3D(theNode);	
#pragma omp for schedule(static, chunk) 
					for (s=startSite; s < endSite; s++) {
						theModel = m_vecSiteAssignment.mDataArray[s];
						
						DoubleMatrix *theProbMatrix;
						theProbMatrix = &(*probmatrix3D).mRows.mDataArray[ num->returnModelIndex( theModel, mRoot->GetData().siteCats->getAssignment(s), theNode->GetData().branchLengthIndex ) ];
						
						if (isTip) {
							to = (*theSeq)[s];
							for (i=0; i<numStates; i++) {
								if (theMode==1) {
									// For Pupko algorithm...
									theNode->conditionalState[(s*numStates) + i] = to;
									siteLikelihoods[(s*numStates) + i] = (*theProbMatrix).mRows.mDataArray[i].mDataArray[to];
								} else {
									// For standard site likelihood...
									if (i == to) { 
										siteLikelihoods[(s*numStates) + i] = 1.0;
									} else {
										siteLikelihoods[(s*numStates) + i] = 0.0;
									}
								}
							}
						} else {
							// Not a tip...
							if (theMode == 1) {
								// Pupko algorithm...
								double currentBest = 0.0;
								tempVal = 0.0;
								
								for (int from=0; from<numStates; from++) {
									currentBest = 0.0; tempVal = 1.0;
									theNode->conditionalState[(s*numStates) + from] = -1;
									for (int to=0; to<numStates; to++) {
										tempVal = (*theProbMatrix).mRows.mDataArray[from].mDataArray[to];
										if (theNode->Left() != NULL) tempVal *= theNode->Left()->GetData().siteLikelihoodBelow[(s*numStates) + to];
										if (theNode->Right() != NULL) tempVal *= theNode->Right()->GetData().siteLikelihoodBelow[(s*numStates) + to];
										if (tempVal > currentBest) { currentBest = tempVal; theNode->conditionalState[(s*numStates) + from] = to; }
									}
									siteLikelihoods[(s*numStates) + from] = currentBest;	
								}
								
							} else {
								// Standard algorithm...
                                
								// LEFT TREE BELOW...
								if (theNode->Left() != NULL) {
									probmatrix3D = getProbMatrix3D(theNode->Left());
									theProbMatrix = &(*probmatrix3D).mRows.mDataArray[ num->returnModelIndex( theModel, mRoot->GetData().siteCats->getAssignment(s), theNode->Left()->GetData().branchLengthIndex ) ];
									
									temp = 0;
									maxVal = -999999999999999;
									
									// Alexandros:  
									// pragma omp parallel for private(from, to) reduction(+:temp)
									for (from=0; from<numStates; from++) {
										temp = 0.0;
										for (to=0; to<numStates; to++) {
											temp += (*theProbMatrix).mRows.mDataArray[from].mDataArray[to] * theNode->Left()->GetData().siteLikelihoodBelow[(s*numStates) + to];
										}
										siteLikelihoods[(s*numStates) + from] *= temp;
										if ((GetData().doScaling)  && (theNode->Right() == NULL) && (siteLikelihoods[(s*numStates) + from]>maxVal)) { maxVal = siteLikelihoods[(s*numStates) + from]; }
									}
									
									if (GetData().doScaling && (theNode->Right() == NULL)) {
                                     // [diagnostic]   if (log(maxVal) == 0) {
                                     // [diagnostic]       cout << "ERROR: log(maxval) is 0! " << maxVal << endl;
                                     // [diagnostic]   }
										for (from=0; from<numStates; from++) {
											siteLikelihoods[(s*numStates) + from] /= maxVal;
                                        }
										(*scalingFactors)[s] += log(maxVal);
                                   // [diagnostic]       if (s==232) cout << "   [temp 232] scalingF = " << exp((*scalingFactors)[s]) << endl;

									}
                                    
								}
                                
								
								if (theNode->Right() != NULL) {
									probmatrix3D = getProbMatrix3D(theNode->Right()); //&theNode->Right()->GetData().probmatrix3D;
									
									theProbMatrix = &(*probmatrix3D).mRows.mDataArray[ num->returnModelIndex( theModel, mRoot->GetData().siteCats->getAssignment(s), theNode->Right()->GetData().branchLengthIndex ) ];
                                    
                                    maxVal = -999999999999999;

                                    for (from=0; from<numStates; from++) {
										temp = 0.0;
										for (to=0; to<numStates; to++) {
											temp += (*theProbMatrix).mRows.mDataArray[from].mDataArray[to] * theNode->Right()->GetData().siteLikelihoodBelow[(s*numStates) + to];
										}
                                        
										siteLikelihoods[(s*numStates) + from] *= temp;
                                        
										if (GetData().doScaling && (siteLikelihoods[(s*numStates) + from] > maxVal)) { maxVal = siteLikelihoods[(s*numStates) + from]; }
									}
									
									if (GetData().doScaling) {
                                      // [diagnostic]  if (log(maxVal) == 0) {
                                       // [diagnostic]     cout << "ERROR: log(maxval) is 0! " << maxVal << endl;

                                       // [diagnostic] }

										for (from=0; from<numStates; from++) {
											siteLikelihoods[(s*numStates) + from] /= maxVal;
										}
										(*scalingFactors)[s] += log(maxVal);
                                     // [diagnostic]     if (s==232) cout << "   [temp 232] scalingF = " << exp((*scalingFactors)[s]) << endl;
									}
                                    
								}
							}
						} //not tip
					} // all sites
                    
                    //	} // end of parallelization
					
				} // not root
			} // node is not null
		}
        
	} 
}

//---------------------------------------------------------------------------

inline
void
TreeNode::setSiteLikelihoods_upPass(
                                    TreeNode*	theNode,
                                    DoubleMatrix& stateFreqMatrix,
                                    DoubleVector& modelFreqVector,
                                    IntVector& m_vecSiteAssignment,
                                    IntVector& inStateindex,
                                    CharVector& inStateVector,
                                    int& minstate,
                                    int& numStates,
                                    int startSite,
                                    int endSite)
{
    
    // [experimental]
    
	vector<int> *theSeq;
	theSeq = &theNode->GetData().sequence;
	int length = theSeq->size();
	int theModel;
    
	DoubleMatrix3D *probmatrix3D1, *probmatrix3D2;
	DoubleMatrix *theProbMatrix1, *theProbMatrix2;
	unsigned int numModels = getProbMatrix3D(mRoot)->rows();
	if (mRoot->numDiscreteBLs != 0) { numModels /= mRoot->numDiscreteBLs; }
    
	if ( theNode == mRoot ) {			
		//std::cout << "At the root node for uppass...seq size is " << length << endl;
		double test = 0.0;
		double testLL = 0.0;
		for ( int s=startSite; s < endSite; s++) {
			theModel = m_vecSiteAssignment.mDataArray[s];
            //	theModel = theNode->GetData().marked; // temporal model override
            //	std::cout << "Setting freqs at root node for up siteL's, site " << s << "..." << endl;
			test = 0.0;
			for (int i=0; i<numStates; i++) {
				theNode->GetData().siteLikelihoodAbove[(s*numStates) + i] = stateFreqMatrix.mRows.mDataArray[theModel].mDataArray[i] * modelFreqVector.mDataArray[theModel];

				test += (stateFreqMatrix[theModel][i] * modelFreqVector[theModel] * theNode->Left()->GetData().siteLikelihoodBelow_andAbove[(s*numStates) + i] * theNode->Right()->GetData().siteLikelihoodBelow_andAbove[(s*numStates) + i]);
			}
			testLL += log(test);
		}
		std::cout << "Test of Left/Right likelihoods: " << testLL << endl;
        
	} else {
		probmatrix3D1 = getProbMatrix3D(theNode); //->GetData().probmatrix3D;
 
		if ((theNode->Up() != NULL) && (theNode != NULL)) { //(theNode->Up() != theNode->Up()->Up())
            //	std::cout << "Up() gives node " << theNode->Up()->GetData().id << endl;
			for ( int s=startSite; s < endSite; s++) {
				double temp1, temp2;
				theModel = m_vecSiteAssignment.mDataArray[s];
                //		theModel = theNode->GetData().marked; // temporal model override
                //		std::cout << "theModel is " << theModel << endl;
				
				if (mRoot->numDiscreteBLs == 0) { 
					theProbMatrix1 = &(*probmatrix3D1).mRows.mDataArray[theModel];
				} else {
					theProbMatrix1 = &(*probmatrix3D1).mRows.mDataArray[(theNode->Left()->GetData().branchLengthIndex * numModels)+theModel];
				}
            	
				for (int theState=0; theState<numStates; theState++) {
					double temp = 0.0; double finalTerm;
					for (int from=0; from<numStates; from++) {
                        if (theNode->Sibling() != NULL) {
                            finalTerm = theNode->Sibling()->GetData().siteLikelihoodBelow_andAbove[(s*numStates) + from];
                        } else {
                            finalTerm = 1.0;
                        }
                        temp += theNode->Up()->GetData().siteLikelihoodAbove[(s*numStates) + from] * (*theProbMatrix1).mRows.mDataArray[from].mDataArray[theState] * finalTerm ; 
					} // from
					theNode->GetData().siteLikelihoodAbove[(s*numStates) + theState] = temp;
				} //theState
			} // over sites 's'
		} // Up not null...

        
	} // not root
	
	if ( this != NULL) {
		if (Left() != NULL)
			Left()->setSiteLikelihoods_upPass(Left(), stateFreqMatrix, modelFreqVector, m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates, startSite, endSite);
        
		if (Right() != NULL)
			Right()->setSiteLikelihoods_upPass(Right(), stateFreqMatrix, modelFreqVector, m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates, startSite, endSite);
	}
	
}

//---------------------------------------------------------------------------

inline
void
TreeNode::setSiteLikelihoods_doAssignment(
                                          TreeNode*	theNode,
										  MixtureModel& theModel,
										  ObservedData& theData,
										  IntVector& m_vecSiteAssignment,
                                          DoubleVector& modelFreqVector)
{
	// Default to MAP assignment...
	setSiteLikelihoods_doAssignment(theNode, theModel, theData, m_vecSiteAssignment, modelFreqVector, 0, 0, theData.getAlignmentLength() );
}



inline
void
TreeNode::setSiteLikelihoods_doAssignment(
										  TreeNode*	theNode,
										  MixtureModel& mModel,
										  ObservedData& theData,
										  IntVector& m_vecSiteAssignment,
                                          DoubleVector& modelFreqVector,
										  int theMode, int startSite, int endSite)
{
    
	// theMode == 0: assign the MAP ancestral state
	// theMode == 1: sample from the posterior distribution
	
    int numStates = theData.getNumStates();
	
	vector<int> *theSeq;
	theSeq = &theNode->GetData().sequence;
	int theModel;
    
    // Blocked the condition that ignored the root node; we do want to sample a state there...
    if ( (theNode != NULL) && ( ! ((theNode->Left() == NULL) && (theNode->Right() == NULL)) ) ) {/////// && theNode != mRoot) { //(theNode->Up() != theNode->Up()->Up())
		//	std::cout << "Up() gives node " << theNode->Up()->GetData().id << endl;
        double siteLikelihood, cumulative[(int)numStates];
        for ( int s=startSite; s < endSite; s++) {
            double temp1, temp2;
            theModel = m_vecSiteAssignment.mDataArray[s];
			//	theModel = theNode->GetData().marked; // temporal model override
            siteLikelihood = 0.0;
            for (int theState=0; theState<numStates; theState++) {
                if (theNode != mRoot) {
                    cumulative[theState] = theNode->GetData().siteLikelihoodAbove[(s*numStates) + theState]  * theNode->GetData().siteLikelihoodBelow[(s*numStates) + theState]; 
                } else {
                    cumulative[theState] = mRoot->GetData().siteLikelihoodBelow[(s*numStates) + theState]  * modelFreqVector.mDataArray[theModel] * mModel.returnStateFrequencies().mRows.mDataArray[theModel].mDataArray[theState];
                }
                siteLikelihood += cumulative[theState];
            }
            
            
            double max = 0.0;
            int maxState = 0;
            for (int theState=0; theState<numStates; theState++) {
                if (siteLikelihood != 0.0)cumulative[theState]/=siteLikelihood;
                
                // [for maximization]
                if (theMode == 0) {
                    if (cumulative[theState] > max) { max = cumulative[theState]; maxState = theState; }
                }						
                // [for sampling] 
                else if (theMode == 1) {
                    if (theState>0) { cumulative[theState] += cumulative[theState-1]; }
                }
            }
            
            if (theMode == 0) {
                // [for maximization]
                theNode->GetData().sequence[s] = maxState; //SetAtIndex(s, inStateVector[maxState]);
            } else if (theMode == 1) {
                // [for sampling]
                double rand = rndu();
                
                for (int theState=0; theState<numStates; theState++) {
                    if (rand <= cumulative[theState]) {
                        // set
                         theNode->GetData().sequence[s] = theState; //SetAtIndex(s, inStateVector[theState]);
                        break;
                    }
                }
            }
	        
        } // site 's'
    } // node not null
    
	/*	double lnltest = 0.0;
     double tempSum = 0.0;
     for ( int s=0; s < m_vecSiteAssignment.size(); s++) {
     theModel = m_vecSiteAssignment.mDataArray[s];
     tempSum = 0.0;
     for (int x=0; x<numStates; x++) {
     //	std::cout << "Test of sitelikelihood up x down for state " << x << ", site0: " << (theNode->GetData().siteLikelihoodAbove[(0) + x] * theNode->GetData().siteLikelihoodBelow[(0) + x] ) << endl;
     //	std::cout << "	just up(): " << (theNode->GetData().siteLikelihoodAbove[(0) + x]) << "; just down(): " << (theNode->GetData().siteLikelihoodBelow[(0) + x]) << endl;
     tempSum += ( theNode->GetData().siteLikelihoodAbove[((s*numStates)) + x] * theNode->GetData().siteLikelihoodBelow[((s*numStates)) + x] );
     }
     lnltest += log(tempSum);
     }
     std::cout << "lnltest is " << lnltest << ", for node " << theNode->GetData().id << endl;
     */
	if ( this != NULL) {
		if (Left() != NULL)
			Left()->setSiteLikelihoods_doAssignment(Left(), mModel, theData, m_vecSiteAssignment, modelFreqVector, theMode, startSite, endSite);
        
		if (Right() != NULL)
			Right()->setSiteLikelihoods_doAssignment(Right(), mModel, theData, m_vecSiteAssignment, modelFreqVector, theMode, startSite, endSite);
	}
	
}



inline
void
TreeNode::posteriorPredictive_getSite(
                                      int theSite,
                                      TreeNode*	theNode,
                                      DoubleMatrix& stateFreqMatrix,
                                      DoubleVector& modelFreqVector,
                                      IntVector& m_vecSiteAssignment,
                                      IntVector& inStateindex,
                                      CharVector& inStateVector,
                                      int& minstate,
                                      int& numStates, std::vector<int>& tipStates)
{
	static int tipCount;
	static int modelAssignment;
    
	double siteLikelihood;
	double cumulative[(int)numStates];
	int numModels = modelFreqVector.size();
	double modelCumulative[(int)numModels];
	double sum = 0.0;
    
	//int numStates = stateFreqMatrix->rows();
	
	if (theNode == mRoot) {
		tipCount = 0;
		//tipStates.clear();
		
		modelAssignment = m_vecSiteAssignment.mDataArray[ theSite ];
        //	modelAssignment = theNode->GetData().marked; // temp for temporal model
        
	} else {
		int isTip;
		
		if ( (theNode->Left() == NULL) && (theNode->Right() == NULL) && theNode->Up() != NULL) {
			// Tip node...
			isTip = 1;
			
			vector<int>& seq1 = theNode->GetData().sequence; 
			int indx1 = seq1[theSite];
			tipStates[tipCount++] = indx1;
		} else {
			isTip = 0;
		}
	}
	
	if ( this != NULL) {
		if (Left() != NULL)
			Left()->posteriorPredictive_getSite(theSite, Left(), stateFreqMatrix, modelFreqVector, m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates, tipStates);
        
		if (Right() != NULL)
			Right()->posteriorPredictive_getSite(theSite, Right(), stateFreqMatrix, modelFreqVector, m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates, tipStates);
	}
}


inline
void
TreeNode::posteriorPredictive_doSimpleAssignment(
                                                 TreeNode*	theNode,
                                                 DoubleMatrix& stateFreqMatrix,
                                                 DoubleVector& modelFreqVector,
                                                 IntVector& m_vecSiteAssignment,
                                                 IntVector& inStateindex,
                                                 CharVector& inStateVector,
                                                 int& minstate,
                                                 int& numStates, std::vector<int>& tipStates)
{
    // Need mModel.GetRateMatrix3D()
    
	static int tipCount;
	static int modelAssignment = 0;
    
	double siteLikelihood;
	double cumulative[(int)numStates];
	int numModels = modelFreqVector.size();
	double modelCumulative[(int)numModels];
	double sum = 0.0;
    
	//int numStates = stateFreqMatrix->rows();
	
	if (theNode == mRoot) {
		tipCount = 0;
		//tipStates.clear();
		
		for (int theModel=0; theModel<numModels; theModel++) {
			modelCumulative[theModel] = modelFreqVector[theModel];
			sum += modelCumulative[theModel];
		}
		
		for (int theModel=0; theModel<numModels; theModel++) {
			modelCumulative[theModel] /= sum;
			if (theModel>0) modelCumulative[theModel] += modelCumulative[theModel-1];
		}
        
		// Sample...
		double rand = rndu();
		for (int theModel=0; theModel<numModels; theModel++) {
			if (rand <= modelCumulative[theModel]) {
				modelAssignment = theModel;
				break;
			}
		}
        
		// Model for the simulated site is now made
		// std::cout << "Posterior predictive site is going to be drawn from mixture component: " << modelAssignment << endl;
		
		// Now sample character state...
		sum = 0.0;
		for (int theState=0; theState<numStates; theState++) {
			cumulative[theState] = stateFreqMatrix.mRows.mDataArray[modelAssignment].mDataArray[theState];
			sum += cumulative[theState];
		}
		for (int theState=0; theState<numStates; theState++) {
			cumulative[theState] /= sum;
			if (theState > 0) cumulative[theState] += cumulative[theState-1];
		}
		
		// Sample...
		int theState;
		rand = rndu();
		for (theState=0; theState<numStates; theState++) {
			if (rand <= cumulative[theState]) {
				theNode->GetData().posteriorPredictiveAssignment = theState;
				break;
			}
		}
		
        //	std::cout << "Sampled posterior predictive site at root: " << theState << endl;
	} else {
        
		DoubleMatrix3D *probmatrix3D;
		DoubleMatrix *theProbMatrix;
		int isTip;
		int from, to;
		
		// Want the Pmat ending on the current node...
		probmatrix3D = getProbMatrix3D(theNode); //&theNode->GetData().probmatrix3D;
		
		if ( (theNode->Left() == NULL) && (theNode->Right() == NULL) && theNode->Up() != NULL) {
			// Tip node...
			isTip = 1;
		} else {
			isTip = 0;
		}
        
		if ( theNode != NULL ) {
			from = theNode->Up()->GetData().posteriorPredictiveAssignment;
            
			if (mRoot->numDiscreteBLs == 0) { 
				theProbMatrix = &(*probmatrix3D).mRows.mDataArray[modelAssignment];
			} else {
				theProbMatrix = &(*probmatrix3D).mRows.mDataArray[ (theNode->GetData().branchLengthIndex * numModels) + modelAssignment];
			}
            
			sum = 0.0;
			for (int theState=0; theState<numStates; theState++) {
				cumulative[theState] = theProbMatrix->mRows.mDataArray[from].mDataArray[theState]; 
				sum += cumulative[theState];
			}
            
			for (int theState=0; theState<numStates; theState++) {
				cumulative[theState] /= sum;
				if (theState>0) { cumulative[theState] += cumulative[theState-1]; }
			}
			
			double rand = rndu();
			for (int theState=0; theState<numStates; theState++) {
				if (rand <= cumulative[theState]) {
					theNode->GetData().posteriorPredictiveAssignment = theState;
					//std::cout << "Setting posterior predictive state at node " << theNode->GetData().id << " to " << theState << endl;
					break;
				}
			}
            
			if (isTip == 1) { tipStates[tipCount++] = theNode->GetData().posteriorPredictiveAssignment; }
            
		} // node not null
	}
	
	if ( this != NULL) {
		if (Left() != NULL)
			Left()->posteriorPredictive_doSimpleAssignment(Left(), stateFreqMatrix, modelFreqVector, m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates, tipStates);
        
		if (Right() != NULL)
			Right()->posteriorPredictive_doSimpleAssignment(Right(), stateFreqMatrix, modelFreqVector, m_vecSiteAssignment, inStateindex, inStateVector, minstate, numStates, tipStates);
	}
}



inline
void
TreeNode::backupSeqSite(int theSite, 	IntVector& inStateindex, int& minstate) {
	vector<int>& seq1 = GetData().sequence;
	int indx1 = seq1[theSite];
	
	if (GetData().seqBackup.empty()) {
		GetData().seqBackup.resize( GetData().sequence.size() , 0 );
	}
	
	GetData().seqBackup.mDataArray[ theSite ] = indx1;
}


inline
void
TreeNode::restoreSeqSite(int theSite, CharVector& inStateVector, int& minstate) {
	{
		GetData().sequence[theSite] = GetData().seqBackup.mDataArray[ theSite ]; //SetAtIndex(theSite, inStateVector[GetData().seqBackup.mDataArray[ theSite ]] );
	}
    
}


inline int TreeNode::setSiteLikelihoods_doSimpleAssignment(
                                                           TreeNode*	theNode,
                                                           MixtureModel& theModel,
                                                           ObservedData& theData,
                                                           IntVector& m_vecSiteAssignment,
                                                           DoubleVector& modelFreqVector,
                                                           int theMode) {
	return setSiteLikelihoods_doSimpleAssignment( theNode, theModel, theData, m_vecSiteAssignment, modelFreqVector, theMode, 0, m_vecSiteAssignment.size() );								
}

inline int TreeNode::setSiteLikelihoods_doSimpleAssignment(
                                                           TreeNode*	theNode,
                                                           MixtureModel& mModel,
                                                           ObservedData& theData,
                                                           IntVector& m_vecSiteAssignment,
                                                           DoubleVector& modelFreqVector,
                                                           int theMode,
                                                           int startSite, int endSite) {

	// [this function has been modified to deal with options.__globalLikelihoodHead]
	
    // Modes 0 and 2 are mostly deprecated.
    
	// theMode == 0: assign the MAP ancestral state
	// theMode == 1: sample from the posterior distribution
	// theMode == 2: used in conjunction with downpass mode==1 for Pupko et al joint reconstruction
    //	std::cout << "Do simple assignment is at node " << theNode->GetData().id << endl;
	
	DoubleMatrix3D *probmatrix3D;
	DoubleMatrix *theProbMatrix;
	unsigned int numModels = getProbMatrix3D(mRoot)->rows();
	if (mRoot->numDiscreteBLs != 0) { numModels /= mRoot->numDiscreteBLs; }
    
	int numStates = theData.getNumStates();
    
	int theModel;
	int from; //, to;
	int numUpdates = 0;
	
	if (theNode == mRoot ) {
		// Sampling at the root depends on the down site likelihoods and the equilbrium distribution...
		double siteLikelihood, cumulative[(int)numStates];
		for ( int s=startSite; s < endSite; s++) {
			theModel = m_vecSiteAssignment.mDataArray[s];
            //		theModel = theNode->GetData().marked; // temporal model override
            
			siteLikelihood = 0.0;
			double temp;
			for (int theState=0; theState<numStates; theState++) { //
				if (options.scaleByFrequencies) {
					temp = mModel.returnStateFrequencies().mRows.mDataArray[theModel].mDataArray[theState] * modelFreqVector.mDataArray[theModel]  * theNode->GetData().siteLikelihoodBelow[(s*numStates) + theState] ; 
				} else {
					temp =   mModel.returnStateFrequencies().mRows.mDataArray[theModel].mDataArray[theState] * modelFreqVector.mDataArray[theModel]  * theNode->GetData().siteLikelihoodBelow[(s*numStates) + theState] ;
				}

				if (options.__globalLikelihoodTemp != 1.0) {
					cumulative[theState] = pow( temp, options.__globalLikelihoodTemp );
				} else {
					cumulative[theState] = temp;
				}
				
              // [diagnostic]
                /*if (s==232) {
                    cout << "stateF for " << theState << " is " << mModel.returnStateFrequencies().mRows.mDataArray[theModel].mDataArray[theState] << ", modelFreqVec " << modelFreqVector.mDataArray[theModel] << ", sitelbelow " << theNode->GetData().siteLikelihoodBelow[(s*numStates) + theState] << endl;
                    
                }
                */
                
				siteLikelihood += cumulative[theState]; //mRoot->GetData().siteLikelihoodBelow[(s*numStates) + theState] * stateFreqMatrix.mRows.mDataArray[theModel].mDataArray[theState] * modelFreqVector.mDataArray[theModel];
			}
            
           // [diagnostic] if (s==232) { cout << " [doSimpleAssign] total siteL for site " << s << ": " << siteLikelihood << endl;}
            
            double max = 0.0;
            int maxState = 0;
            
            for (int theState=0; theState<numStates; theState++) {
                if (siteLikelihood != 0.0) cumulative[theState]/=siteLikelihood;
                //					std::cout << "		state " << theState << ": cumulative = " << cumulative[theState] << endl;
                
                // [for maximization]
                if (theMode == 0 || theMode == 2) {
                    if (cumulative[theState] > max) { max = cumulative[theState]; maxState = theState; }
                }						
                // [for sampling] 
                else if (theMode == 1) {
                    if (theState>0) { cumulative[theState] += cumulative[theState-1]; }
                }
            }
            //std::cout << endl;
            
            if (theMode == 0 || theMode == 2) {
                // [for maximization]
                theNode->GetData().sequence[s] = maxState; //.SetAtIndex(s, inStateVector[maxState]);
            } else if (theMode == 1) {
                // [for sampling]
                // our standard dsfmt random number generator is not thread safe!
                double u = num->Random();
                
                double ccnt=0;
                // Will this solve the race condition...?
                while ( (u<0) || (u>1) || (u!=u) ) {
                    u = num->Random();
                    ccnt++;
                }	
                
                int isSet = 0;
                for (int theState=0; theState<numStates; theState++) {
                    if (u <= cumulative[theState]) {
                        theNode->GetData().sequence[s] = theState; //SetAtIndex(s, inStateVector[theState]);
                        isSet = 1;
                        break;
                    }
                }
                if (isSet == 0) {
                    int temp = (int)(num->Random() * (numStates) );
                    theNode->GetData().sequence[s] = temp;
                    std::cout << "		[ Numerical error calculating small site-likelihoods for site " << s << "! Randomly chose state " << temp << "]" << endl;
                    
                    // Let's track this down...
             // [diagnostic]
                    /*cout << "   Random # was " << u << endl;
                    cout << "   Node is " << theNode->GetData().id << "; mRoot id says: " << mRoot->GetData().id << endl;
                    cout << "   cumulative dist: ";
                    for (int ii=0; ii<numStates; ii++) {
                        cout << cumulative[ii] << ", ";
                    }
                    cout << endl;
                    */
                }
                numUpdates++;
            }
		} // site 's'
		
	}
	else {
		// not root

		// Want the Pmat ending on the current node...
		probmatrix3D = getProbMatrix3D(theNode); //&theNode->GetData().probmatrix3D;
		int isTip;
		
		// And the parental sequence...
		vector<int> *theSeq;
		theSeq = &theNode->Up()->GetData().sequence;
 		
		if ( (theNode->Left() == NULL) && (theNode->Right() == NULL) && theNode->Up() != NULL) {
			// Tip node...
			isTip = 1;
		} else {
			isTip = 0;
		}
        
		if ( ( isTip != 1) && (theNode != NULL) ) { //(theNode->Up() != theNode->Up()->Up())
			double siteLikelihood, cumulative[(int)numStates];
			
			for ( int s=startSite; s < endSite; s++) {
				from = (*theSeq)[s];
				theModel = m_vecSiteAssignment.mDataArray[s];
                //	theModel = theNode->GetData().marked; // temporal model override
                
				theProbMatrix = &(*probmatrix3D).mRows.mDataArray[ num->returnModelIndex( theModel, mRoot->GetData().siteCats->getAssignment( s ), theNode->GetData().branchLengthIndex) ];
                
				siteLikelihood = 0.0;
				double temp;
				
				for (int theState=0; theState<numStates; theState++) {
					temp = theProbMatrix->mRows.mDataArray[from].mDataArray[theState] * theNode->GetData().siteLikelihoodBelow[(s*numStates) + theState]; // 
					
					if (options.__globalLikelihoodTemp != 1.0) {
 						cumulative[theState] = pow( temp, options.__globalLikelihoodTemp );
					} else {
						cumulative[theState] = temp;
					}
					siteLikelihood += cumulative[theState];
				}
				
                double max = 0.0;
                int maxState = 0;
                for (int theState=0; theState<numStates; theState++) {
                    if (siteLikelihood != 0.0) cumulative[theState]/=siteLikelihood;

                    // [for maximization]
                    if (theMode == 0) {
                        if (cumulative[theState] > max) { max = cumulative[theState]; maxState = theState; }
                    }						
                    // [for sampling] 
                    else if (theMode == 1) {
                        if (theState>0) { cumulative[theState] += cumulative[theState-1]; }
                    }
                    else if (theMode == 2) {
                        maxState = theNode->conditionalState[ (s * numStates) + from ];
                    }
                }
                
                if (theMode == 0 || theMode == 2) {
                    // [for maximization]
                    theNode->GetData().sequence[s] = maxState;
                    
                } else if (theMode == 1) {
                    // [for sampling]
                    // our standard dsfmt random number generator is not thread safe
                    double u = num->Random();
                    // Will this solve the race condition?
                    double ccnt = 0;
                    while ( (u<0) || (u>1) || (u!=u)) {
                        u = num->Random();
                        ccnt++;
                    }	

                    int isSet = 0;
                    for (int theState=0; theState<numStates; theState++) {
                        if (u <= cumulative[theState]) {
                            theNode->GetData().sequence[s] = theState; //.SetAtIndex(s, inStateVector[theState]);
                            isSet = 1;
                            break;
                        }
                    }
                    if (isSet == 0) {
                        int temp = (int)(num->Random() * (numStates) );
                        theNode->GetData().sequence[s] = temp;
                        std::cout << "		[ Numerical error calculating small site-likelihoods for site " << s << "! Randomly chose state " << temp << "]" << endl;
                    }
                }
                numUpdates++;
                //			} // sharedtransaction?
			} // site 's'
		} // node not null
	}

	if ( this != NULL) {
		if (Left() != NULL)
			numUpdates += Left()->setSiteLikelihoods_doSimpleAssignment( Left(), mModel, theData, m_vecSiteAssignment, modelFreqVector, theMode, startSite, endSite );
        
		if (Right() != NULL)
			numUpdates += Right()->setSiteLikelihoods_doSimpleAssignment( Right(), mModel, theData, m_vecSiteAssignment, modelFreqVector, theMode, startSite, endSite );
	}
	
	return numUpdates;
}

//---------------------------------------------------------------------------

inline
double
TreeNode::incompleteDataLikelihood(
                                   TreeNode*	theNode,
                                   DoubleMatrix& stateFreqMatrix,
                                   DoubleVector& modelFreqVector,
                                   IntVector& m_vecSiteAssignment,
                                   ObservedData &theData)
{
	double ll = 0.0;

	unsigned int numModels = getProbMatrix3D(mRoot)->rows();
	if (mRoot->numDiscreteBLs != 0) { numModels /= mRoot->numDiscreteBLs; }
    
    int theModel;
    DoubleMatrix3D *probmatrix3D1, *probmatrix3D2;
    
    DoubleMatrix *theProbMatrix1, *theProbMatrix2;
    DoubleVector *scalingFactors  = & mRoot->GetData().scalingFactor;
    
    probmatrix3D1 = getProbMatrix3D(theNode->Left()); //&theNode->Left()->GetData().probmatrix3D;
    probmatrix3D2 = getProbMatrix3D(theNode->Right()); //&theNode->Right()->GetData().probmatrix3D;
    
    int length = m_vecSiteAssignment.size();
    
    for (unsigned int s=0; s < length; s++)
    {
        theModel = m_vecSiteAssignment.mDataArray[s];
        //		theModel = theNode->GetData().marked; // temporal model override
        
        theProbMatrix1 = &(*probmatrix3D1).mRows.mDataArray[ num->returnModelIndex( theModel, mRoot->GetData().siteCats->getAssignment( s ), theNode->GetData().branchLengthIndex) ];
        theProbMatrix2 = &(*probmatrix3D2).mRows.mDataArray[ num->returnModelIndex( theModel, mRoot->GetData().siteCats->getAssignment( s ), theNode->GetData().branchLengthIndex) ];
        
        double temp = 0.0;
        for (int from=0; from<theData.getNumStates(); from++) {
            if (options.scaleByFrequencies) {
                temp += exp((*scalingFactors)[s]) * theNode->GetData().siteLikelihoodBelow[(s*theData.getNumStates()) + from]* modelFreqVector.mDataArray[theModel] * stateFreqMatrix.mRows[theModel].mDataArray[from];
            } else {
                temp += exp((*scalingFactors)[s]) * theNode->GetData().siteLikelihoodBelow[(s*theData.getNumStates()) + from]* modelFreqVector.mDataArray[theModel]; // * stateFreqMatrix.mRows[theModel].mDataArray[from];
            }
        }
        
        ll += log(temp);
    } // all sites

	return ll;
}





#endif // TreeNode_h__

